#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "pasteup.h"

/********************************************************************
Automatic Gain Control Function.
  -- this function was made with reference to SU's sugain.c --
--------------------------------------------------------------------
inputs: 
  a         : wave amplitude array
  n         : dimension of a.
  sampling  : sampling rate (Hz)
  agc_period: AGC period (sec)
  scale     : amplitude scale factor.
********************************************************************/
void agc(float *a, int n, double sampling, double agc_period, double scale)
{
    /********** Local Variables *********/
    register int i;
    int n2;   /** Number of samples during half of agc_period **/
    int nwin; /** Number of samples in agc window **/
    float *g;
    register float val;
    register double sum, rms;
    
    /********** Main *********/
    if (NULL == (g = (float *) malloc(maxsample * sizeof(float)))) exit(1);
    n2 = (sampling*agc_period/2.0);
    if (n2 < 0) {
	/** invalid agc period **/
	fprintf(stderr, "Invalid agc period. %f\n", agc_period);
    }
    if(n2>n) n2 = n;
    /********** AGC processing **********/
    /* compute initial window for first datum */
    sum = 0.0;
    for (i=0; i<=n2; i++) {
	val = a[i];
	sum += val*val;
    }
    nwin = n2 + 1;
    rms = sum/nwin;
    g[0] = (!rms) ? 0.0 : scale*a[0]/sqrt(rms);

    /* ramping on */
    for (i=1;i<=n2;i++) {
	val = a[i+n2];
	sum += val*val;
	nwin++;
	rms = sum/nwin;
	g[i] = (!rms) ? 0.0 : scale*a[i]/sqrt(rms);
    }
    /* middle range -- full rms window */
    for (i=n2+1; i<=n-n2; i++) {
	val = a[i+n2];
	sum += val*val;
	val = a[i-n2-1];
	sum -= val*val; /* rounding could make sum negative! */
	rms = sum/nwin;
	g[i] = (rms <= 0.0) ? 0.0 : scale*a[i]/sqrt(rms);
    }
    /* ramping off */
    for (i=n-n2+1; i<n;i++) {
	val = a[i-n2-1];
	sum -= val*val; /* rounding could make sum negative! */
	--nwin;
	rms = sum/nwin;
	g[i] = (rms <= 0.0) ? 0.0 : scale*a[i]/sqrt(rms);
    }
    /** copy memory **/
    for(i=0;i<n;i++) a[i] = g[i];
    free(g);
}

