# --------------------------------------------------------------------
# Execute amplitude to accuracy computation
# --------------------------------------------------------------------
proc amp2acc {} {
   # global --------------------
   global wk paste TMPDIR file_name gbl

   # main --------------------------------------------------
   if { [check_su_mode] == 0 } {return}
   if {$gbl(data,type) != 2 && $gbl(data,type) < 5} { 
      tk_messageBox -message "This function is valid for SU or SEG-Y data."\
	     -type ok -icon error -parent .
      return
   }

   # check parameters ----------
   set sn [regsub -all \[\\s\] $paste(amp2acc,sn) ""]
   set acc [regsub -all \[\\s\] $paste(amp2acc,acc) ""]
   set sn_array [split $sn ,]
   set acc_array [split $acc ,]

   # check dimension of arrays
   if { [llength $sn_array] != [llength $acc_array] } {
      tk_messageBox -message "Array dimension is not same."\
	  -type ok -icon error -parent .
      return
   }
   if { [llength $sn_array] < 2 } { 
      tk_messageBox -message "Array dimension is too small. (Do you use \",\" for a delimitter?)"\
	  -type ok -icon error -parent .
      return
   }
   # sn must be descendant
   set large [lindex $sn_array 0]
   for {set i 1} { $i < [llength $sn_array] } { incr i} {
      set small [lindex $sn_array $i]
      if { $small > $large } {
	 tk_messageBox -message "S/N array must be decreasing."\
	     -type ok -icon error -parent .
	 return
      }
      set large $small
   }

   # execute ----------
   set mark_modification 0

   # temporary pick file.
   set tmp1 [format "%s/pasteup.%s.pick1" $TMPDIR [pid]]
   set tmp2 [format "%s/pasteup.%s.pick2" $TMPDIR [pid]]
   # write out to temporary file
   write_pasteup_pick $tmp1
   # execute SU command
   set com [make_su_command 0\
		[format "| supickamp2acc pfile=%s tafter=%g tbefore=%g\
                           mode=%d sn=%s acc=%s"\
		     $tmp1 $paste(amp2acc,tafter) $paste(amp2acc,tbefore)\
		     $paste(amp2acc,mode) $sn $acc]\
		$file_name(wave) $tmp2 "" ""]
   set stat [exec_external_command $com "SUPICKAM2ACC"]
   if { $stat == 1 } { return }
   
   # read from result file
   read_pasteup_pick $tmp2

   # clear temporary files
   file delete -force -- $tmp1 $tmp2
   
   # redisplay
   plot_pick_marks 0

   set mark_modification 1
}
