#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "pasteup.h"

/********************************************************************
Compute maximum absolute value of array "a" (dimension "n").
********************************************************************/
static float search_maxamp(int n, float *a)
{
    /***** Local Variables *****/
    int i;
    float max;
    
    /********** Main **********/
    max = fabs(a[0]);
    for(i=1;i<n;i++) if(max < fabs(a[i])) max = fabs(a[i]);
    return(max);
}

/********************************************************************
Amplitude correction; AGC or Geometric or Relative
--------------------------------------------------------------------
[arguments]
 a          : array of amplitude (raw data)
 count      : array size.
 sampling   : sampling frequency in Hz
 amp_correct: amplitude correction mode, "geometric" "relative" "none"
 agc_period : Auto Gain Control period.
 scale      : Scale factor
 kika_coef  : Geometric attenuation correction value
********************************************************************/
void amplitude_correct(float *a, int count, double sampling,
		       int amp_correct, double agc_period,
		       double scale, double kika_coef)
{
    /********** Local Variables **********/
    register int i;
    register float c;
    float max_amp;
    
    /********** Local Function **********/
    void agc(float *a, int n, double sampling, double agc_p, double scale);
    
    /********** Main **********/
    /***** Time axis amplitude correction *****/
    
    /** now not implemented **/
    
    /***** AGC/Geometric/Relative *****/
    if (amp_correct == 1 && agc_period > 0.0) {
	/**********  AGC with scaling **********/
	agc(a, count, sampling, agc_period, scale);
    } else {
	/***** Relative and Geometric correction with scaling *****/
	switch (amp_correct) {
	    case 3: /********** Geometric Attenuation **********/
		c = kika_coef * scale;
		break;
	    case 2: /********** Relative **********/
		max_amp = search_maxamp(count, a);
		c = (max_amp <= 0.0) ? scale : 32768.0/max_amp*scale;
		break;
	    default:
		c = scale;
		break;
	}
	for(i=0;i<count;i++) a[i] *= c;
    }
}

/********************************************************************
 Compute amplification factor
 ********************************************************************/
float compute_amp_coefficient(int mode)
{
    /********** Main **********/
    if( mode > 10 ) return(1.0);  /** VD, TwoTone **/
    return(100.0 / 32768);        /** VA, wiggle  **/
}

/********************************************************************
 Amplitude step to change colors in VD mode
 ********************************************************************/
int compute_amp_step(int bfmt, int mode, int amp_step)
{
    /********** Local Variables **********/
    int val;
    
    /********** Main **********/
    val = (mode == 15) ? NRAINBOW : NCOLOR;
    if( mode > 10 ) {
	/*** VD and TWO_TONE (except RAINBOW) ***/
	if( bfmt == 2 ) {
	    val = 16384/val;   /* 2^14/NCOLORS */
	    if (val < 1) val = 1;
	    return(val);
	}
	return(1);
    }
    /*** Wiggle/VA ***/
    return(amp_step);
}

/********************************************************************
Clip Amplitude
--------------------------------------------------------------------
[arguments]
 a      : array of amplitude (raw data)
 count  : array size.
 clip   : Clip value in pixel.
********************************************************************/
void amp_clip(float *a, int count, int clip)
{
    /********** Local Variables **********/
    register int i;
    register float plus_clip, minus_clip;
    
    /********** Main **********/
    if (clip<=0) return;
    plus_clip = clip;
    minus_clip = -plus_clip;
    for(i=0;i<count;i++){
	if(a[i]>plus_clip) a[i] = plus_clip;
	else if(a[i] < minus_clip) a[i] = minus_clip;
    }
}

/********************************************************************
Clip Amplitude for float value
--------------------------------------------------------------------
[arguments]
 a          : array of amplitude (raw data)
 count      : array size.
 clip       : clip amplitude
********************************************************************/
void amp_clip_float(float *a, int count, float clip)
{
    /********** Local Variables **********/
    register int i;
    register float plus_clip, minus_clip;
    
    /********** Main **********/
    if (clip<0.0) return;
    plus_clip = (float)clip;
    minus_clip = - plus_clip;
    for(i=0;i<count;i++){
	if(a[i]>plus_clip) a[i] = plus_clip;
	else if(a[i] < minus_clip) a[i] = minus_clip;
    }
}
