# --------------------------------------------------------------------
# Change amplitude scale
# --------------------------------------------------------------------
# [arguments]
# a  : scale factor
# --------------------------------------------------------------------
proc amplify {a} {
    # global variable
    global gbl paste

    # main ------------------------------
    # Confirmation window
    set ans [tk_dialog .amplify_or_not "Amplify or not?"\
		 [format "The amplitude gain will be modified into %g.\n\nRedraw record secion?"\
		      [expr $paste(amp,amplify)*$a]] question 0 Yes No]
    if {$ans==1} {
	print_message "Amplification was canceled.\n"
	return
    }
    set paste(amp,amplify) [expr $paste(amp,amplify) * $a]
    draw_pasteup
}

# --------------------------------------------------------------------
# Determine maximum amplitude in the specified area
# --------------------------------------------------------------------
proc max_amplitude {x1 x2 t1 t2} {
   # global variable ------------------------------
   global gbl file_name filt paste TMPDIR wk
   
   # Main ------------------------------
   update_amp_correct

   # x-axis manipulation
   set xaxis $gbl(data,xaxis)
   if { $gbl(data,type) == 3 } {
      set xaxis $gbl(obs,direc)
   }
   set wk(condition) [check_same_draw_condition]
   return [getamp $x1 $x2 $t1 $t2]
}

# --------------------------------------------------------------------
# Change amplitude scale : Only valid for SU and SEG-Y
# --------------------------------------------------------------------
# [arguments]
# ratio  :  amplitude larger than maxamp * ratio(%) will be saturated.
# --------------------------------------------------------------------
proc auto_scale_factor {} {
   # global variable ------------------------------
   global gbl amp_rect paste wk
   
   # Main ------------------------------
   # Get maximum amplitude of all the viewable record section.
   if { $paste(amp,correct_label) == "Geometric" } {
      set max -1.0
      if { [expr abs($paste(left)) ] > [expr abs($paste(right))] } {
	 set max [max_amplitude $paste(left) 0 $paste(top) $paste(bottom)]
      } else {
	 set max [max_amplitude 0 $paste(right) $paste(top) $paste(bottom)]
      }
      if { $max <= 0.0 } {
	 set max [max_amplitude $paste(left) $paste(right)\
		      $paste(top) $paste(bottom)]
      }
   } else {
      set max [max_amplitude $paste(left) $paste(right)\
		   $paste(top) $paste(bottom)]
   }
   if {$max <= 0.0} {
      print_message "AMPLITUDE ERROR: No trace data."
      return
   }
   #
   if { $gbl(mode) >= 10 } {
      # when VD style
      if { $gbl(vdcolor) == $wk(VD_COLOR,TwoTone) } {
	 set paste(amp,amplify)\
	     [expr 2 * 100.0 / ($max * $gbl(option,scaling_factor))]
      } else {
	 switch -- [expr abs($gbl(data,byte))] {
	    2 { # 2byte integer
	       set paste(amp,amplify)\
		   [expr 16384*100.0 / ($max * $gbl(option,scaling_factor))]
	    }
	    default {
	       set paste(amp,amplify)\
		   [expr 256.0*100.0 / ($max * $gbl(option,scaling_factor))]
	    }
	 }
      }
   } else {
      # VA/Wiggle style
      switch -- [expr abs($gbl(data,byte))] {
	 2 { # 2byte integer
	    set paste(amp,amplify)\
		[expr 32768*100.0/($max * $gbl(option,scaling_factor) *20)]
	 }
	 default {
	    set paste(amp,amplify)\
		[expr 32768*100.0/($max*$gbl(option,scaling_factor)*10)]
	 }
      }
   }
   # significant digit
   set a $paste(amp,amplify)
   set b [expr pow(10, int(log10($a)) - 3)]
   set paste(amp,amplify) [format "%g" [expr int($a/$b)*$b]]

   # report the result
   print_message "set scale factor automatically ($paste(amp,amplify))\n"
}

# --------------------------------------------------------------------
# configuration for Geometrical spreading correction
# --------------------------------------------------------------------
proc geometric_configure_window {} {
   # Global variables ----------
   global paste gbl wk
   
   # Main ------------------------------
   set w .geocfg
   if [raise_window $w] { return } 
   toplevel $w
   wm title $w "PasteUp: Corrections for Geometrical spreading"
   
   # binding ------------------------------
   basic_bindings $w

   # title frame --------------------
   set tf [title_frame $w.title_frame "Pasteup: Geometrical Spreading"]
   
   set pf [frame $w.param_frame -padx 15 -pady 10 -relief groove -bd 2]

   label $pf.explanation -text "Amp. Correct. Coef. = (offset / L)**a"
   grid $pf.explanation - -sticky news
   make_entry_right $pf stan_dist "Standard distance (L)"\
       paste(amp,geometric,standard) "km"
   make_entry_right $pf power "power (a)"  paste(amp,geometric,power) ""

   # Button frame --------------------
   set bf [button_frame $w $w.button_frame]
   
   # Packing --------------------
   pack $tf $pf $bf -side top -fill x
}
