# --------------------------------------------------------------------
# analysis_window ----------------------------------------------------
#    invoke analysis window
# --------------------------------------------------------------------
proc analysis_window {} {
   # Global variable
   global gbl wk PREF env custom
   
   # Main --------------------
   set aw .analysis_window
   if [raise_window $aw] { return }
   toplevel $aw
   wm title $aw "PasteUp: Analysis Window"
   
   # title frame --------------------
   set titlef [title_frame $aw.title_frame "Pasteup: Analysis"]
   
   # tabset ------------------------------
   set ts $aw.tabset
   tabset $ts -pagewidth 580 -relief flat
   $ts conf -borderwidth 1 -highlightthickness 0 -side top
   $ts insert end "mapping" -text "Mapping"
   $ts insert end "raytrace" -text "RayTrace"
   $ts insert end "multiples" -text "Multiples"
   $ts insert end "aux" -text "Aux."
   $ts insert end "position" -text "Position"
   $ts insert end "correction" -text "Correct."
   $ts insert end "velocity" -text "Vel."
   $ts insert end "taup" -text "Tau-P"

   # tabset balloon help
   foreach {tab mes}\
       {mapping "Traveltime Mapping"\
	    raytrace "Synthetic traveltime computation by ray-tracing"\
	    multiples "Plot sea surface multiples"\
	    aux "Plot water waves and 0-level..."\
	    position "OBS Position Fix"\
	    velocity "Apparent Velocity data handling"\
	    correction "Automatic shot time corrections"\
	    taup "Tau-P and Tau-Sum analysis"\
	 } {
	    $ts bind $tab <Enter> "balloon \"$mes\" %X %Y"
	    $ts bind $tab <Leave> {cancel_balloon}
	 }
   
   # --------------------------------------------------
   # Mapping tab --------------------------------------

   set pf $ts.mapping
   frame $pf
   $ts tab conf "mapping" -window $pf -fill both

   # mapping tabset ------------------------------
   set mts $pf.tabset
   tabset $mts -pagewidth 480 -relief flat -rotate 90 -side left
   $mts conf -borderwidth 1 -highlightthickness 0
   $mts insert end "model" -text "Model"
   $mts insert end "pick" -text "Pick"
   $mts insert end "condition" -text "Condition"

   # tabset balloon help
   foreach {tab mes} {model "Velocity model"\
			  pick "Traveltime picks / binning"\
			  condition "Traveltime fields and Imaging conditions"\
			  viewer "Mapping results viewer settings"
		       } {
      $mts bind $tab <Enter> "balloon \"$mes\" %X %Y"
      $mts bind $tab <Leave> {cancel_balloon}
   }

   # model tab ----------
   set mf $mts.model
   frame $mf
   $mts tab conf "model" -window $mf -fill both

   # netCDF
   set cdf [radio_labelframe $mf netcdf "netCDF" gbl(vmodel,format) 1]
   filename_hscroll_frame $cdf netcdf "netCDF filename"\
       gbl(vmodel,netcdf) 35 load netcdf
   
   # ray86 + gm
   set raygm [radio_labelframe $mf raygm "Ray86 + gm" gbl(vmodel,format) 0]
   filename_hscroll_frame $raygm ray86 "Ray86 filename"\
       gbl(vmodel,ray86) 35 load ray86
   filename_hscroll_frame $raygm gm "gm filename"\
       gbl(vmodel,gm) 35 load gm
   make_spinbox $raygm mb "Boundary"\
       gbl(vmodel,mboundary) 0 20 1 "(between ray86 and gm)"
   update_radio_frames $mf gbl(vmodel,format)

   # viewer related
   set view [labelframe $mf.viewer -text "CPT file for viewing velocity model"]
   filename_hscroll_frame $view velcpt "CPT Filename"\
       gbl(vmodel,cptfile) 35 load cpt

   # packing
   pack $cdf $raygm $view -side top -expand 1 -fill x

   # pick tab ----------
   set mf $mts.pick
   frame $mf
   $mts tab conf "pick" -window $mf -fill both

   # pick type
   set pt [labelframe $mf.pick_type -text "Pick types for mapping" -padx 30 -pady 30]
   make_entry_com $pt "Pick type list" type gbl(mapping,types)\
       "ListUp" "mapping_listup_picktypes"

   # binning
   set bin [labelframe $mf.binning -padx 30 -pady 30]
   checkbutton $mf.bincheck -text "Binning of shots" -var gbl(mapping,renum)\
       -command "labelframe_checkbutton_status $bin gbl(mapping,renum)"
   $bin conf -labelw $mf.bincheck

   # x-origin
   make_entry_right $bin xorigin "X-origin for binning" gbl(mapping,xorg) "km"
   # bin width
   make_entry_right $bin bin "bin width" gbl(mapping,bin) "km"

   labelframe_checkbutton_status $bin gbl(mapping,renum)
   pack $pt $bin -side top -expand 1 -fill x


   # condition tab ----------
   set mf $mts.cond
   frame $mf
   $mts tab conf "condition" -window $mf -fill both

   # traveltime fields
   set tf [labelframe $mf.tfield -text "Traveltime fields configuration"\
	      -pady 10]
   filename_hscroll_frame $tf tfield "Directory"\
       gbl(mapping,tfdir) 35 dir netcdf
   make_entry_right $tf dx "grid spacing" gbl(mapping,dx) "km"
   make_menu_com $tf byte "format(byte)" gbl(mapping,byte) ""\
       short 2 float 4 double 8
   make_entry_right $tf tbase "Time-base" gbl(mapping,tbase) "(coefficient)"
   define_balloon $tf.entry(tbase)\
       "Valid only for short format.\nTo obtaine actual traveltime,\nTime-base must be multiplied to the traveltime field"

   # obs position
   set pos [labelframe $mf.pos -text "OBS position"]
   make_entry_right $pos xpos "x-position" gbl(obs,x) "km"
   make_entry_right $pos zpos "depth" gbl(obs,depth) "km"

   # pick type
   set pt [labelframe $mf.type -text "Pick type list for mapping"]
   make_entry_com $pt "Pick list" type gbl(mapping,types)\
       "ListUp" "mapping_listup_picktypes"

   # imaging conditions
   set ic [labelframe $mf.imaging_condition -text "Imaging conditions"]
   make_menu_com $ic wmode "Weight mode" gbl(mapping,wmode) ""\
       "(0)pick accuracy" 0\
       "(1)pick + x-pos" 1\
       "(2)box window" 2\
       "(3)Gaussian window" 3\
       "(4)2*sigma Gaussian" 4\
       "(5)angle aperture" 5

   make_optionMenu_button $ic wmodesign "Mapping area" gbl(mapping,wsign)\
       "deeper" "shallower"
   make_entry_right $ic wpick "Pick acc. weight" gbl(mapping,wpick) ""
   make_entry_right $ic alpha "Aperture(alpha)" gbl(mapping,alpha) ""
   make_entry_right $ic aperture "Aperture angle" gbl(mapping,angle) "degree"
   make_entry_right $ic smute "Stretch mute" gbl(mapping,smute) ""
   make_menu_com $ic normalize "Normalization" gbl(mapping,normalize) ""\
       "(0)none" 0\
       "(1)by # of picks" 1\
       "(2)by max value" 2
   grid $tf - -sticky news -pady 5 -padx 5
   grid $pos $ic -sticky news -pady 5 -padx 2
   grid $pt ^ -sticky news -pady 5 -padx 2

   # buttons ------------------------------
   set bf [button_frame "" $pf.button_frame\
	       "View Vel. Model" VModel::invoke_window\
	       "Make T-fields" mapping_make_tfield\
	       "   Mapping   " mapping_pick_image]
   pack $mts $bf -side top -fill x

   # --------------------------------------------------
   # Raytrace tab --------------------------------------

   set pf $ts.raytrace
   frame $pf
   $ts tab conf "raytrace" -window $pf -fill both

   # raytrace tabset ------------------------------
   set mts $pf.tabset
   tabset $mts -pagewidth 480 -relief flat -rotate 90 -side left
   $mts conf -borderwidth 1 -highlightthickness 0
   $mts insert end "model" -text "Model"
   $mts insert end "conditions" -text "conditions"

   # tabset balloon help
   foreach {tab mes} {model "Velocity model"\
			  conditions "OBS pos / Ray types / output filename"\
			  viewer "Velocity model viewer configuration"\
		       } {
      $mts bind $tab <Enter> "balloon \"$mes\" %X %Y"
      $mts bind $tab <Leave> {cancel_balloon}
   }

   # model tab ----------
   set mf $mts.model
   frame $mf
   $mts tab conf "model" -window $mf -fill both

   # netCDF
   set cdf [radio_labelframe $mf netcdf "netCDF" gbl(vmodel,format) 1]
   filename_hscroll_frame $cdf netcdf "netCDF filename"\
       gbl(vmodel,netcdf) 35 load netcdf
   
   # ray86 + gm
   set raygm [radio_labelframe $mf raygm "Ray86 + gm" gbl(vmodel,format) 0]
   filename_hscroll_frame $raygm ray86 "Ray86 filename"\
       gbl(vmodel,ray86) 35 load ray86
   filename_hscroll_frame $raygm gm "gm filename"\
       gbl(vmodel,gm) 35 load gm
   make_spinbox $raygm mb "Boundary"\
       gbl(vmodel,mboundary) 0 20 1 "(between ray86 and gm)"
   update_radio_frames $mf gbl(vmodel,format)

   # viewer related
   set view [labelframe $mf.viewer -text "CPT file for velocity model"]
   filename_hscroll_frame $view velcpt "CPT Filename"\
       gbl(vmodel,cptfile) 35 load cpt

   # packing
   pack $cdf $raygm $view -side top -expand 1 -fill x

   # conditions tab ----------
   set mf $mts.conditions
   frame $mf
   $mts tab conf "conditions" -window $mf -fill both

   set pos [labelframe $mf.pos -text "OBS position"]
   make_entry_right $pos xpos "x-position" gbl(obs,x) "km"
   make_entry_right $pos zpos "depth" gbl(obs,depth) "km"

   set acc [labelframe $mf.acc -text "Accuracy"]
   make_entry_right $acc acc "Accuracy" gbl(hamen,accuracy) "km"

   set code [labelframe $mf.raycode -text "Type of rays"]
   make_entry $code code "Ray Code" gbl(hamen,raycode) ""
   define_balloon $mf.raycode.entry(code)\
       "--- Example ---\n0: first arrival\n3: reflection from 3rd reflector"

   # synthetic file
   set tfn [labelframe $mf.tfilename -text "Synthetic traveltime file"]
   filename_hscroll_frame $tfn fn "Output filename"\
       gbl(hamen,filename) 35 save "pick"
   make_menu_plus_button $tfn preserve "After compuation" \
       gbl(option,raytrace,preserve) "" \
       "clear cache" raytrace_delete_preserved_data \
       "preserve in cache" 1 "don't make cache" 0
   

   # two-point raytracing
   set twopo [radio_labelframe $mf twopo "Two-point raytracing"\
		  gbl(hamen,twopo) 1]
   make_entry $twopo type "Pick type" gbl(hamen,twopo,type) ""
   make_spinbox $twopo rout "Ray decimation" gbl(hamen,rout) 0 100 1 ""
   define_balloon $twopo.spinbox(rout)\
       "--- Example ---\n0: no rays.\n1: all rays for all picks.\n2: one ray per two picks."
   
   # shot points
   set shotpoint [radio_labelframe $mf shotpoint "Shooting to surface"\
		      gbl(hamen,twopo) 0]
   make_spinbox $shotpoint rout "Receiver spacing" gbl(hamen,rout) 0 100 1\
       "grid(s)"
   define_balloon $shotpoint.spinbox(rout)\
       "--- Example ---\n0: no rays.\n1: receivers are placed at every grid.\n2: receivers are placed at every 2 grids."
   update_radio_frames $mf gbl(hamen,twopo)

   # raypath computation
   set rp [labelframe $mf.raypath]
   checkbutton $mf.raycheck -text "Calc. and Plot raypaths"\
       -var gbl(hamen,raypath)\
       -command "labelframe_checkbutton_status $rp gbl(hamen,raypath)"
   $rp conf -labelwidget $mf.raycheck
   filename_hscroll_frame $rp rays "Raypath filename"\
       gbl(hamen,rayfilename) 35 save ""
   labelframe_checkbutton_status $rp gbl(hamen,raypath)

   grid $pos $acc -sticky news -padx 5
   grid $code - -sticky news -padx 5
   grid $tfn - -sticky news -padx 5
   grid $twopo - -sticky news -padx 5
   grid $shotpoint - -sticky news -padx 5
   grid $rp - -sticky news -padx 5

   # buttons ------------------------------
   set bf [button_frame "" $pf.button_frame\
	       "Exec Raytracing" raytrace_exec\
	       "View Vel. Model" VModel::invoke_window]
   pack $mts $bf -side top -fill x

   # --------------------------------------------------
   # Multiples --------------------------------------

   set tf [frame $ts.multiples]
   $ts tab conf "multiples" -window $tf -fill x
   
   set pf [labelframe $tf.param -text "Sea surface multiple marks" -pady 20]

   # direction and colors
   make_many_radio $pf "Direction" prior\
       wk(multiple,direc) Prior -1 Posterior 1
   make_color_button $pf color "Color"\
       "Multiple" $wk(color,multiple) wk(color,multiple)\
       "setColor $aw $pf.button(color) wk(color,multiple); multiple_plot_all"

   # water depth / water vel.
   set wf [labelframe $tf.waterparam -text "Water layer" -pady 20]
   foreach {i j k m}\
       {water_depth "Depth" gbl(obs,depth) "(km)"\
	    vel_water "Sound velocity" gbl(water_v) "(km/sec)"} {
	       make_entry_right $wf $i $j $k $m
	    }

   set bf [button_frame "" $tf.button_frame\
	       "Plot" multiple_plot_all "Erase" clear_multiple]

   grid $pf $wf -sticky news -padx 5
   grid $bf - -sticky nws
   
   # --------------------------------------------------
   # Auxiliary information ----------------------------
   set pf $ts.aux
   frame $pf
   $ts tab conf "aux" -window $pf -fill both
   
   # water wave ----------
   set wv [labelframe $pf.waterwave -padx 60 -pady 30]
   checkbutton $pf.check_water -text "Water wave" -var wk(plot,water_wave)\
       -command "labelframe_checkbutton_status $wv wk(plot,water_wave)"
   $wv conf -labelw $pf.check_water

   # color of water wave line
   make_color_button $wv color "Color"\
       "Water wave" $wk(color,water) wk(color,water)\
       "setColor $aw $pf.button(color_wl) wk(color,water) ; draw_auxliary"
   
   foreach {i j k m}\
       {water_depth "Water Depth" gbl(obs,depth) "(km)"\
	    vel_water "Sound velocity in water" gbl(water_v) "(km/sec)"} {
	       make_entry_right $wv $i $j $k $m
	    }

   # auxiliary lines ----------
   set zl [labelframe $pf.zerolevel -pady 30]
   checkbutton $pf.check_zero -text "Zero level line" -var wk(plot,auxline)\
       -command "labelframe_checkbutton_status $zl wk(plot,auxline)"
   $zl conf -labelw $pf.check_zero

   make_color_button $zl color_zl "Color of zero level"\
       "Zero level" $wk(color,auxline) wk(color,auxline)\
       "setColor $aw $pf.button(color_zl) wk(color,auxline); draw_auxliary"

   # button frame ----------
   set bf [button_frame "" $pf.button_frame "update" draw_auxiliary]

   labelframe_checkbutton_status $wv wk(plot,water_wave)
   labelframe_checkbutton_status $zl wk(plot,auxline)
   grid $wv -sticky news
   grid $zl -sticky news
   grid $bf -sticky news

   # --------------------------------------------------
   # Position fix tab ---------------------------------

   set pf [frame $ts.position]
   $ts tab conf "position" -window $pf -fill both

   # auto pick related parameters ----------
   set apf [labelframe $pf.autopick -pady 25 -text "Auto Pick by AR model"]

   # range
   label $apf.label(range) -text "Pick Range(km)" -anchor e
   entry $apf.entry(min) -width 12 -textvar gbl(pos,min) -justify right
   label $apf.label(to) -text "to"
   entry $apf.entry(max) -width 12 -textvar gbl(pos,max) -justify right
   grid $apf.label(range) $apf.entry(min) $apf.label(to) $apf.entry(max)\
       -sticky news
   # AR-model order
   make_entry_right $apf ar "Max AR-model" gbl(pos,maxorder) "samples"
   make_check_label $apf "Apply SU-filter before autopick" filt\
       gbl(corpick,applyfilter) ""

   # position fix related parameters ----------
   set pff [labelframe $pf.posfix -pady 25\
		-text "Inversion parameters for position fix"]

   # water depth / velocity / inversion limit
   foreach {i j k m}\
       {water_depth "Water Depth" gbl(obs,depth) "(km)"\
	    vel_water "Water velocity" gbl(water_v) "(km/sec)"\
	    limit "Convergence limit" gbl(pos,limit) "" } {
	       make_entry_right $pff $i $j $k $m
	    }

   # uncertainties
   label $pff.luncertain -text "Uncertainties"
   set pfu $pff.funcertain
   frame $pfu
   label $pfu.ldw -text "Vel(km/s)"
   label $pfu.ldh -text "XY(km)"
   label $pfu.ldd -text "Depth(km)"
   label $pfu.ldt -text "Clock(s)"
   entry $pfu.edw -width 8 -textvar gbl(pos,dw) -justify right
   entry $pfu.edh -width 8 -textvar gbl(pos,dh) -justify right
   entry $pfu.edd -width 8 -textvar gbl(pos,dd) -justify right
   entry $pfu.edt -width 8 -textvar gbl(pos,dt) -justify right
   grid $pfu.ldw $pfu.ldh $pfu.ldd $pfu.ldt -sticky news
   grid $pfu.edw $pfu.edh $pfu.edd $pfu.edt -sticky news
   grid $pff.luncertain $pfu - - - - -sticky nws

   # direction
   make_many_radio $pff "Positive direction" direc gbl(pos,direc)\
       "East" 1 "West" -1 "North" 2 "South" -2

   # exec buttons ----------
   button_frame "" $pf.exec "AutoPick(AR)" "auto_pick_AR"\
       "Position Fix" position_fix "Recover data" su_recover_data

   pack $apf $pff $pf.exec -side top -fill x -expand 1 -padx 30

   # --------------------------------------------------
   # Velocity tab -------------------------------------

   set pf $ts.vel
   frame $pf
   $ts tab conf "velocity" -window $pf -fill both

   # listbox frame
   set lf [labelframe $pf.listbox_frame -text "Apparent velocities"]
   label $lf.legend -font small_fixed -anchor w -just left\
       -text [format "%-5s %-7s %-7s %-7s %-7s %-7s %-7s"\
		  " No." "  X1" "  X2" "  T1" "  T2" "  Vel." "  Int."]
   set lb $lf.listbox
   set sb $lf.scroll
   set wk(vline,listbox) $lb
   listbox $lb -yscrollcommand "$sb set" -selectmode extended\
       -selectbackground black -selectforeground white -width 52\
       -font small_fixed
   scrollbar $sb -command "$lb yview"
   grid $lf.legend -column 0 -columnspan 2 -sticky news
   grid $lb $sb -sticky news

   # button frame
   set bf1 [button_frame "" $pf.sort_buttons "Sort by x1" "vline_sort x1"\
		"Sort by intercept" "vline_sort intercept"]
   set bf2 [button_frame "" $pf.data_buttons\
		"Delete selected" delete_selected_vline\
		"Save" exec_save_vline]

   # packing
   grid $lf -sticky news -ipadx 30 -ipady 30 -pady 10 -padx 20
   grid $bf1 -sticky news
   grid $bf2 -sticky news
   update_velocity_listbox

   # --------------------------------------------------
   # Corrections --------------------------------------
   set pf [frame $ts.correction]
   $ts tab conf "correction" -window $pf -fill both

   # check buttons
   set ch [labelframe $pf.check -text "Shot time corrections"]
   
   make_radio_vertical $ch shot_correct paste(correct,shot_mode)\
       "None" 0 "Shots move to sea floor" 2   "Flat seafloor correction" 1
   
   # conditions
   set cond [labelframe $pf.condition -text "Conditions"]
   foreach {i j k m}\
       {vel_water "Sound velocity in water" gbl(water_v) "(km/sec)"\
	    vel_sediment "P-wave Velocity in sediments" gbl(sediment_v)\
	    "(km/sec)"} {
	       make_entry_right $cond $i $j $k $m
	    }
   grid $ch -sticky news -ipady 30 -pady 10
   grid $cond -sticky news -ipady 30 -ipadx 25 -pady 10

   # --------------------------------------------------
   # Tau-P tab ----------------------------------------

   set pf [frame $ts.taup]
   $ts tab conf "taup" -window $pf -fill both

   # VZ-graph ----------
   set wg [labelframe $pf.graph -text "V-Z graph"]
   set g [graph $wg.vzgraph -height 2i]
   $g axis conf x -hide true -min 0.0 -max 9.0
   $g axis conf x2 -hide false -min 0.0 -max 9.0 -title "Velocity(km/sec)"
   $g axis conf y -min 0.0 -max 15.0 -title "Depth(km)" -desc 1
   $g grid conf -hide false
   $g legend conf -hide yes

   $g element create vzgraph -xdata vecv -ydata vecz -mapx x2 -pixel 0
   Blt_ZoomStack $g
   Blt_Crosshairs $g

   # show the sea floor depth for reference
   $g element create vzseafloor -mapx x2 -pixel 0 -outline gray
   update_tausum_depth

   # show position
   set pos [label $wg.pos -text "" -relief flat]
   # button
   set wg_bf [button_frame "" $wg.save "Save VZ-graph" tausum_save]
   pack $g $pos $wg_bf -side top -fill both -expand 1

   # binding
   bind $g <Any-Motion> "tausum_show_position $g $pos %x %y"

   # file frame ----------
   set fn [labelframe $pf.filename -text "Tau-P pick data" -padx 15]
   filename_hscroll_frame $fn dat "" taup(file) 50 load dat
   set fn_bf [button_frame "" $fn.buttons \
		  "load" taup_load_pick \
		  "save" taup_save_pick \
		  "clear" taup_clear_pick ]
   grid $fn_bf - -sticky news

   # parameters ----------
   set wp [labelframe $pf.param -text ""]

   make_many_radio_com $wp \
       "Draw water wave" taup_water_wave gbl(taup,water_wave) \
       "$wk(w,canvas,c) delete taup_water ; draw_taup_waterwave"\
       on 1 off 0

   # conditions ----------
   foreach {i j k m} \
       {water_depth "Water Depth" gbl(obs,depth) "(km)"\
	    vel_water "Sound velocity in water" gbl(water_v) "(km/sec)"} {
	       make_entry_right $wp $i $j $k $m
	    }

   pack $wg $fn $wp -side top -fill x -expand 1

   # --------------------------------------------------
   # Button frame -------------------------------------
   set bf [button_frame $aw $aw.button_frame]

   # --------------------------------------------------
   # Packing ------------------------------------------
   pack $titlef $ts $bf -side top -fill x -expand 1

   # binding
   basic_bindings $aw

   # disabled tab ------------------------------
   if {$PREF(com,hamenpath) == ""} {
      # disabled
      $ts move 1 after end
      $ts tab config "raytrace" -state disabled
   }
   if {$PREF(com,tmmpath) == "" || $PREF(com,viewgrd) == "" } {
      # disabled
      $ts move 0 after end
      $ts tab config "mapping" -state disabled
      update
   }
}
