# --------------------------------------------------------------------
# Check command line arguments
# --------------------------------------------------------------------
proc check_argument {} {
   # Global variables --------------------
   global argc argv HOMEDIR wk paste gbl file_name CPU_ARCHTECTURE
   
   # Main --------------------
   set i 0
   while {$i<$argc} {
      # get option
      set opt [lindex $argv $i]
      if [regexp ^- $opt] {
	 set opt [string range $opt 1 end]
	 switch -- $opt {
	    -view {# Automatic VIEW mode
	       set wk(invoke,autogain) 20        ; # auto amp 20%
	       set wk(invoke,autorange) 1        ; # auto range on
	       set wk(invoke,range_width) ""     ; # all over the traces
	       set wk(invoke,autodraw) 1         ; # auto draw
	       set wk(invoke,logo_duration) -1   ; # no logo
	       set wk(invoke,ampc) AGC           ; # AGC
	       set wk(invoke,agc) 4              ; # 4sec AGC window
	       set wk(invoke,style) "VD"         ; # VD
	       set wk(invoke,style2) 2           ; # Interpolate
	       set wk(invoke,usedb) 0            ; # use db off
	    }
	    a -
	    -auto {# Auto draw after invoke
	       set wk(invoke,autodraw) 1
	    }
	    p -
	    -ps {# Auto generate PostScript and quit
	       set wk(invoke,autoPS) 1
	    }
	    l -
	    -logo {# Suppress opening logo
	       set wk(invoke,logo_duration) -1
	    }
	    q -
	    -quit {# Auto quit
	       set wk(invoke,autoquit) 1
	    }
	    -noparam {# Don't read default parameter file
	       set wk(invoke,mode) noparam
	    }
	    -vanilla {# Don't read parameter, theme and preferences files
	       set wk(invoke,mode) vanilla
	    }
	    h -
	    -help {# Show command line help
	       argument_help
	       exit
	    }
	    v -
	    -version {# Version info
	       puts stdout "Pasteup Application Version $wk(VERSION)."
	       exit
	    }
	    d -
	    -debug {# Debug mode
	       set wk(debug) 1
	    }
	    -agc {# set agc period
	       incr i
	       set wk(invoke,agc) [get_option_argument $i $argc $argv]
	    }
	    -ampc {# set amplitude correction mode
	       incr i
	       set wk(invoke,ampc) [get_option_argument $i $argc $argv]
	    }
	    -analysis {# invoke analysis window
	       set wk(invoke,analysis) 1
	    }
	    -autodb {# create DB automatically
	       set wk(invoke,autoDB) 1
	    }
	    -causal {# set causality of the filter
	       incr i
	       set wk(invoke,causal) [get_option_argument $i $argc $argv]
	    }
	    -clip {# set clip amplitude
	       incr i
	       set wk(invoke,clip) [get_option_argument $i $argc $argv]
	    }
	    -db {# whether use data base or not
	       incr i
	       set wk(invoke,usedb) [get_option_argument $i $argc $argv]
	    }
	    -dpi {# set image dpi
	       incr i
	       set wk(invoke,dpi) [get_option_argument $i $argc $argv]
	    }
	    -every {# set wave_density
	       incr i
	       set wk(invoke,density) [get_option_argument $i $argc $argv]
	    }
	    -filter {# set filter type
	       incr i
	       set wk(invoke,filter) [get_option_argument $i $argc $argv]
	    }
	    -freq {# set filter frequencies
	       incr i
	       set wk(invoke,freq) [get_option_argument $i $argc $argv]
	    }
	    -maxsample {# set maximum sample number per a trace
	       incr i
	       set wk(invoke,maxsample) [get_option_argument $i $argc $argv]
	    }
	    -maxtrace {# set maximum trace number
	       incr i
	       set wk(invoke,maxtrace) [get_option_argument $i $argc $argv]
	    }
	    -plabel {# plot pick mark labels
	       set wk(invoke,plabel) 1
	    }
	    -pick {# give a pick file name
	       incr i
	       set wk(invoke,havepick) 1
	       set wk(invoke,pickfn) [get_option_argument $i $argc $argv]
	    }
	    -range {# determine offset range automatically
	       incr i
	       set wk(invoke,autorange) 1
	       set wk(invoke,range_width) [get_option_argument $i $argc $argv]
	       if { $wk(invoke,range_width) <= 0 } {
		  set wk(invoke,range_width) ""
	       }
	    }
	    -rsize  {# offset edge reference size
	       incr i
	       set wk(invoke,autorsize) [get_option_argument $i $argc $argv]
	    }
	    -rv {# set reduction velocity
	       incr i
	       set wk(invoke,rvel) [get_option_argument $i $argc $argv]
	    }
	    -scale {# set scale factor automatically
	       incr i
	       set wk(invoke,autogain) [get_option_argument $i $argc $argv]
	    }
	    -style {# set drawing style
	       incr i
	       set wk(invoke,style) [get_option_argument $i $argc $argv]
	    }
	    -style2 {# set drawing style2
	       incr i
	       set wk(invoke,style2) [get_option_argument $i $argc $argv]
	    }
	    -su {# use sufilter
	       incr i
	       set wk(invoke,su) [get_option_argument $i $argc $argv]
	    }
	    -sufilter {# sufilter file name
	       incr i
	       set wk(invoke,sufilter) [get_option_argument $i $argc $argv]
	    }
	    -synthe {# give a synthetic file name
	       incr i
	       set wk(invoke,havesynthe) 1
	       set wk(invoke,synthefn) [get_option_argument $i $argc $argv]
	    }
	    -taup {# invoke taup/tausum analysis mode
	       set wk(invoke,taup) 1
	    }
	    -thinout {# set thinning out interval
	       incr i
	       set wk(invoke,thinout) [get_option_argument $i $argc $argv]
	    }
	    -tmin {# set tmin
	       incr i
	       set wk(invoke,tmin) [get_option_argument $i $argc $argv]
	    }
	    -tmax {# set tmax
	       incr i
	       set wk(invoke,tmax) [get_option_argument $i $argc $argv]
	    }
	    -ttic {# set ttic interval
	       incr i
	       set wk(invoke,ttic) [get_option_argument $i $argc $argv]
	    }
	    -title {# set figura title
	       incr i
	       set wk(invoke,title) [get_option_argument $i $argc $argv]
	    }
	    -tmp {# set temporary disk space
	       incr i
	       set wk(invoke,tmp) [get_option_argument $i $argc $argv]
	    }
	    -txratio {# set txratio
	       incr i
	       set wk(invoke,txratio) [get_option_argument $i $argc $argv]
	    }
	    -water {# draw water wave
	       set wk(invoke,drawwater) 1
	    }
	    -Xdpi {# set DPI for X server
	       incr i
	       set wk(invoke,Xdpi) [get_option_argument $i $argc $argv]
	    }
	    -xaxis {# set xaxis header entry
	       incr i
	       set wk(invoke,xaxis) [get_option_argument $i $argc $argv]
	    }
	    -xmax {# set xmax
	       incr i
	       set wk(invoke,xmax) [get_option_argument $i $argc $argv]
	    }
	    -xmin {# set xmin
	       incr i
	       set wk(invoke,xmin) [get_option_argument $i $argc $argv]
	    }
	    -xtic {# set xtic interval
	       incr i
	       set wk(invoke,xtic) [get_option_argument $i $argc $argv]
	    }
	    f {# Read parameters from a specified file
	       incr i
	       set wk(invoke,mode) specified
	       set wk(resource_file) [get_option_argument $i $argc $argv]
	       if { ![file readable $wk(resource_file)] } {
		  puts stderr "ERROR:\nCan't read \"$wk(resource_file)\".\n"
		  exit
	       }
	    }
	    s {# Save parameters to a specified file
	       incr i
	       set wk(invoke,autoquit) 1
	       set wk(invoke,saveparafn) [get_option_argument $i $argc $argv]
	    }
	    default {
	       puts stderr "ERROR:\nInvalid option, \"-$opt\"."
	       argument_help
	       exit
	    }
	 }
      } else {
	 # a file is given, auto identification is necessary...
	 auto_type_info [lindex $argv $i]
      }
      incr i
   }

   # Adjust option status --------------------------------------------
   # autoPS has the top precedence 
   if { $wk(invoke,autoPS) || $wk(invoke,autoquit) } {
      set wk(invoke,logo_duration) -1
      set wk(invoke,autodraw) 0
   }
}
# --------------------------------------------------------------------
# Show usage
# --------------------------------------------------------------------
proc argument_help {} {
   global DATADIR
   # Main --------------------
   puts stdout\
       "Usage:\
	    \n % pasteup \[option\] \[file\]\
            \n\nFile\
            \n  Parameter file, SU, SEGY file.\
            \n  File format will be automatically determined.\
            \n\nOptions\
            \n --view       : View SU/SEGY file automatically,\
                              see below for details.\
            \n -a,--auto    : Auto drawing after startup.\
            \n -p,--ps      : Auto drawing, sending PostScript to STDOUT, and quit.\
            \n --scale \[w\]  : Set scale factor automatically. (w%)\
            \n --range \[w\]  : Set horizontal range automatically. w is horizontal width(km).\
            \n                if w=0, all range is adopted.\n\
            \n --rsize \[dx\] : Adjust both side margin.  Only effective with --range option.\
            \n --agc \[sec\]  : Set agc window (second or km).\
            \n --ampc \[mode\]: Set amp correction mode,\
                                \"None\", \"AGC\", \"Relative\" or \"Geometric\"\
            \n --analysis   : Invoke analysis window.\
            \n --autodb     : Create database automatically.\
            \n --causal \[n\] : Set filter causality. (1:causal, 2:uncausal)\
            \n --clip \[p\]   : Set clip amplitude for wiggel, VA mode (pixel).\
            \n --db \[n\]       : 1:use data base, 0:don't use data base.\
            \n --dpi \[n\]    : Output image (PNG/JPEG) resolution (dpi).
            \n --every \[n\]  : Display every \"n\" trace.
            \n --filter \[n\] : Set filter type,\
            \n                  0:None, 1:LowPass, 2:HighPass, 3:BandPass, 4:BandStop,\
            \n                  5:(B)LowPass, 6:(B)HighPass, 7:(B)BandPass\
            \n --freq \[str\] : Set frequencies for filter (Hz),\
                                see below for details.\
            \n --plabel       : Plot pick mark labels (must use with --pick).\
            \n --rv \[v\]     : Set reduction velocity in km/sec.\
            \n --style \[st\] : Set drawing style, specify one of the following\
            \n                \"Wiggle\", \"VA\", \"VA(R&B)\", \"VA(w/o wiggle)\",\
            \n                \"VD\", \"Wiggle on VD\", \"VA on VD\"\
            \n --style2 \[st\]: Set drawing style,\
            \n                  0:vector, 1:image, 2:interpolate\
            \n --su      \[x\]: 1:use sufilter,  0:none
            \n --sufilter \[fn\]: specify a SU fileter file.\
            \n --taup       : taup/tausum analysis mode.\
            \n --thinout \[x\]: Set thinning out interval of traces (km).\
            \n --tmax \[t\]   : Set maximum of vertical axis (second or km).\
            \n --tmin \[t\]   : Set minimum of vertical axis (second or km).\
            \n --ttic \[t\]   : Set vertical axis tic interval (second or km).\
            \n --title \[str\]: Set figure title to \"str\"\
            \n --tmp \[path\] : Set path of temporary disk space.\
            \n --txratio \[r\]: Set Time/Distance axis length ratio,\
            \n                and resize figure(canvas) size.\
            \n --water      : Draw direct water waves.\
            \n --Xdpi \[dpi\] : Set dpi for X server.\ 
            \n --xaxis \[str\]: Set x-axis header entry, one of\
                                offset, sx, gx, tracl, CDP\
            \n --xmax \[x\]   : Set maximum of horizontal axis (km).\
            \n --xmin \[x\]   : Set minimum of horizontal axis (km).\
            \n --xtic \[x\]   : Set tic interval of horizontal axis (km).\n\
            \n --pick \[file\]: A pick filename for read.\
            \n --synthe \[file\]: A synthetic traveltime filename for read.\
	    \n -f \[file\]    : A parameter filename for read.\
            \n -s \[file\]    : A parameter filename for write.\
            \n -l,--logo    : Suppress opening logo.\
	    \n -d,--debug   : Debug mode.\
            \n -q,--quit    : Quit without drawing(read and save parameters).\
	    \n -v,--version : Show version of Pasteup.\
	    \n -h,--help    : Show this help.\
            \n --noparam    : Don't restore previous plotting parameters.\
	    \n --vanilla    : Completely reset to default.\
            \n --maxsample \[n\]: Set maximum number of samples per a trace.\
            \n                      Default is 4000 samples.\
            \n --maxtrace \[n\] : Set maximum number of traces per a file.\
            \n                      Default is 10000 traces.\
            \n\nNotes --- \
            \n\"view\" option is equivalent to the following options,\
            \n  --scale 20 --range 0 --stlye VD --style2 2 --ampc AGC --agc 4 --logo --auto \
            \n\"ps\" option has the top precedence.\
            \n\"freq\" option examples are, \"1,2,10,12\" or \"1,10\".\
            \n   These values correspond to f1,f2,f3,f4 for each filter.\
            \n   Chebyshev and Butterworth Low/HighPass filter read only f1,f2\
            \n   Chebyshev BandPass/Stop filter read only f2,f3,f4.\
            \n   Butterworth BandPass filter reads f1,f2,f3,f4.\
            \n\
            \n  More detailed documents in HTML format are found in\
            \n  $DATADIR/help/index.html
"
}

# --------------------------------------------------------------------
# identify_file : Identify file format among 
#                          parameter file, SU, SEG-Y, unknown
# --------------------------------------------------------------------
# [Arguments]
#  fn     : filename
# --------------------------------------------------------------------
proc identify_file {fn} {
   # Readable ?? ------------------------------
   if ![file readable $fn] {
      # Cannot read the file
      puts stderr "ERROR:\nCan't read \"$fn\".\n"
      exit
   }
   # Identify by extension ------------------------------
   set ext [string toupper [file extension $fn]]
   switch $ext {
      ".SU" {
	 return SU
      }
      ".SGY" -
      ".SEGY" {
	 return SEGY
      }
      ".RC" {
	 return RC
      }
      ".TT" -
      ".MACRAY" -
      ".PICK" {
	 return PICK
      }
   }
   # Binary or ASCII ?? ------------------------------
   set fp [open $fn]
   binary scan [read $fp 40] c* dat ; # read first 40 bytes
   set bflag 0
   foreach c "$dat" {
      if { ( $c != 10 && $c < 32 ) || $c > 126} {
	 set bflag 1
      }
   }
   seek $fp 0
   if $bflag {
      # BINARY: Is it a SU file?
      if [su_or_not $fp [file size $fn]] {
	 close $fp
	 return SU
      }
   } else {
      # ASCII:  Is it a parameter file?
      set com [get_line $fp]
      if { [lindex $com 0] == "set" } {
	 close $fp
	 return RC
      }
      # ASCII: Is it a pick file (MACRAY-format)?
      set tmp [split $com]
      if {[llength $tmp] == 4 && [lindex $tmp 3] == 1.0} {
	 close $fp
	 return PICK
      }
   }
   close $fp
   return UNKNOWN
}

# --------------------------------------------------------------------
# su_or_not : Is it a SU file?
# --------------------------------------------------------------------
# [Arguments]
#  fp     : file pointer
#  size   : file size in bytes
# --------------------------------------------------------------------
proc su_or_not {fp size} {
   # read ns header (114-th byte, short int)
   seek $fp 114
   set nsbin [read $fp 2] 
   # little endian
   binary scan $nsbin s ns
   if { [expr $size % ($ns * 4 + 240)] == 0 } {
      return 1
   }
   # Big endian
   binary scan $nsbin S ns
   if { [expr $size % ($ns * 4 + 240)] == 0 } {
      return 1
   }
   return 0
}

# --------------------------------------------------------------------
# Get option argument from the command line argument
# --------------------------------------------------------------------
# [Arguments]
#  index  : index of the argument
#  argc   : number of command line arguments
#  argv   : command line arguments
# --------------------------------------------------------------------
proc get_option_argument {index argc argv} {
   if {$index < $argc} {
      return [lindex $argv $index]
   } else {
      puts stderr "ERROR:\nInvalid option usage."
      exit
   }
}