# -----------------------------------------------------------------
# draw auxiliary lines (water waves, center lines)
# -----------------------------------------------------------------
proc draw_auxiliary {} {
   global wk

   # clear
   clear_water_wave $wk(w,current)
   clear_zero_level $wk(w,current)

   # draw direct wave arrivals
   if {$wk(plot,water_wave)} {
      draw_water_wave $wk(w,current)
   }

   # draw zero level
   if {$wk(plot,auxline)} {
      draw_zero_level $wk(w,current)
   }
}



# --------------------------------------------------------------------
# draw zero level line
# --------------------------------------------------------------------
proc draw_zero_level {w} {
   # global --------------------
   global paste wk
   
   # main --------------------
   clear_zero_level $w
   set wk(condition) [check_same_rh_condition]
   set xlist {}
   foreach d [getrh range] {
      lappend xlist [lindex $d 1]
   }
   plot_zero_level $w $xlist $wk(color,auxline)
}
# --------------------------------------------------------------------
# draw water wave
# --------------------------------------------------------------------
proc draw_water_wave {w} {
   # Variable --------------------
   global paste gbl wk
   
   # Main --------------------
   clear_water_wave $w
   
   # spacing
   set dx [expr double($paste(right) - $paste(left)) / 200]
   set dx [expr { $dx > 0.2 } ? 0.2 : $dx]

   # coordinate
   set x $paste(left)
   set depth2 [expr $gbl(obs,depth) - 0.010]   ; # airgun shot depth is 10meter
   set depth2 [expr $depth2 * $depth2]
   set coord {}
   while { $x <= $paste(right) } {
      set t [coord tt2rt $x [expr sqrt($x*$x + $depth2) / $gbl(water_v)]]
      if {$paste(top) <= $t && $paste(bottom) >= $t} {
	 lappend coord $x $t
      } else {
	 plot_water_wave $w $coord $wk(color,water)
	 set coord {}
      }
      set x [expr $x + $dx]
   }
   # plot
   plot_water_wave $w $coord $wk(color,water)
}
