# --------------------------------------------------------------------
# set binding for each mode
# --------------------------------------------------------------------
proc binding {} {
   # global --------------------
   global wk gbl
   
   # -----------------------------------------------------------------
   # main ------------------------------------------------------------
   # -----------------------------------------------------------------

   set c $wk(w,canvas,c)
   set g $wk(w,canvas,g)

   # bindings on the canvas window
   basic_bindings .cw
   bind all <F10> {}

   # -----------------------------------------------------------------
   # bindings both for the NormalCanvas and GraphCanvas---------------
   # -----------------------------------------------------------------

   foreach w "$c $g" {
      bindfe $w Control-Button 2 "draw_pasteup"
      bindfe $w Control m {toggle_message_window}
      bindfe $w Control r {Synthetic::reread}
      bindfe $w Control t {Synthetic::toggle}
      bindfe $w Control v {toggle_show_uncertainty}
      bindfe $w Control u {undo_pop}
      bindfe $w Control z {undo_pop}
      bindfe $w Meta u    {redo_pop}
      bindfe $w Meta b    {toggle_show_picks}
      bindfe $w Meta L    {pick_label_plot}
      bind $w <Enter> "focus $w"
      bind $w <Escape>    {cancel_line_pick %W}
   }

   # -----------------------------------------------------------------
   # bindings ONLY for the NormalCanvas ------------------------------
   # -----------------------------------------------------------------

   bind $c <Right> "hscroll_canvas $c 1"
   bind $c <Left>  "hscroll_canvas $c -1"
   bind $c <Down>  "vscroll_canvas $c 1"
   bind $c <Up>    "vscroll_canvas $c -1"
   bindfe $c Control-Button 4 "hscroll_canvas $c -1"
   bindfe $c Control-Button 5 "hscroll_canvas $c  1"
   bindfe $c Button 4  "vscroll_canvas $c -1"
   bindfe $c Button 5  "vscroll_canvas $c  1"
   bindfe $c Control-Button 1 "zoom_start %x %y"
   bindfe $c Control-Button 3 "zoom_out"

   bindfe $c Control 2 {shiftsection -1 0}
   bindfe $c Control 3 {shiftsection 1 0}
   bindfe $c Control 4 {shiftsection 0 -1}
   bindfe $c Control 5 {shiftsection 0 +1}

   bind $c <Double-Button-1> {double_click1 %W %x %y}

   # drag section
   bind $c <Meta-Button-3> {drag_section_start %x %y}
   bind $c <Alt-Button-3>  {drag_section_start %x %y}
   bind $c <Meta-Button-1> {drag_section_horizontal_start %x %y}
   bind $c <Alt-Button-1>  {drag_section_horizontal_start %x %y}
   bind $c <Meta-Button-2> {drag_section_vertical_start %x %y}
   bind $c <Alt-Button-2>  {drag_section_vertical_start %x %y}

   # -----------------------------------------------------------------
   # binding ONLY for GraphCanvas -----------------------------------------
   # -----------------------------------------------------------------

   bind $g <Control-Button-1> "GraphCanvas::dummy"
   bind $g <Control-Button-3> "GraphCanvas::dummy"
   bind $g <Double-Button-1> {double_click1 %W %x %y}

   # -----------------------------------------------------------------
   # mode depending bindings -----------------------------------------
   # -----------------------------------------------------------------

   foreach i {"%d" "Shift-%d"\
		  "ButtonRelease-%d" "Button%d-Motion"} {
      foreach j {1 2 3} {
	 bind $c [format "<$i>" $j] {}
      }
   }
   if {$gbl(bind) == "Analysis Mode"} {
      foreach w "$c $g" {
	 bind $w <Button-1> {show_position_shot %W %x %y %X %Y}
	 bind $w <Button-2> {vel_line_start %W %x %y}
	 bind $w <Button-3> {clear_last_vel_line}
	 bind $w <Shift-Button-3> {clear_multiple}
      }
      bind $c <Shift-Button-1> {multiple_point %x %y}
      bind $c <Shift-Button-2> {multiple_line_start %x %y}
      binding_info "position" "Measure velocity" "Erase vel-line"\
	  "Multiple(Point)" "Multiple(Line)" "Erase Multiples"\
	  "Zoom region" "(Re)Draw" "Zomm out"
   } elseif {$gbl(bind) == "Pick Mode"} {
      foreach w "$c $g" {
	 bind $w <Button-1> {show_position_shot %W %x %y %X %Y}
	 bind $w <Button-2> {pick_start %W %x %y}
	 bind $w <Button-3> {delete_closest_pick %W %x %y}
	 bind $w <Shift-Button-1> {show_pick_info %W %x %y %X %Y}
	 bind $w <Shift-Button-2> {magnify_trace %W %x %y}
	 bind $w <Shift-Button-3> {change_pick_start %W %x %y}
      }
      binding_info "position" "Pick" "Erase pick"\
	  "Pick mark info" "Trace window" "Select region"\
	  "Zoom region" "(Re)Draw" "Zoom out"
   } elseif {$gbl(bind) == "Line Pick Mode"} {
      foreach w "$c $g" {
	 bind $w <Button-1> {show_position_shot %W %x %y %X %Y}
	 bind $w <Button-2> {line_pick_start %W %x %y}
	 bind $w <Button-3> {delete_closest_pick %W %x %y}
	 bind $w <Shift-Button-1> {show_pick_info %W %x %y %X %Y}
	 bind $w <Shift-Button-2> {magnify_trace %W %x %y}
	 bind $w <Shift-Button-3> {change_pick_start %W %x %y}
      }
      binding_info "position" "Line pick" "Erase pick"\
	  "Pick mark info" "Trace window" "Select region"\
	  "Zoom region" "(Re)Draw" "Zoom out"
   } elseif {$gbl(bind) == "Amplitude Mode"} {
      foreach w "$c $g" {
	 bind $w <Button-1> {show_position_shot %W %x %y %X %Y}
	 bind $w <Shift-Button-1> {amplify 2.0}
	 bind $w <Shift-Button-2> {magnify_trace %W %x %y}
	 bind $w <Shift-Button-3> {amplify 0.5}
      }
      bind $c <Button-2> {amp_start %x %y}
      bind $c <Button-3> {clear_amp_rect}
      binding_info "position" "Select region" "Erase rectangle"\
	  "Amp * 2" "Trace window" "Amp / 2"\
	  "Zoom region" "(Re)Draw" "Zoom out"
   } elseif {$gbl(bind) == "Tau-P Mode"} {
      bind $c <Button-1> {mouse_position %W %x %y}
      bind $c <Button-2> {pick_taup_pick %x %y}
      bind $c <Button-3> {clear_nearest_taup %x %y}
      binding_info "position" "Tau-P pick" "Clear pick"\
	  "-" "-" "-"\
	  "Zoom region" "(Re)Draw" "Zomm out"
   } else {
      # View mode
      foreach w "$c $g" {
	 bind $w <Button-1> {show_position_shot %W %x %y %X %Y}
	 bind $w <Shift-Button-1> {amplify 2.0}
	 bind $w <Shift-Button-2> {magnify_trace %W %x %y}
	 bind $w <Shift-Button-3> {amplify 0.5}
      }
      bind $c <Button-2> {crosshair_cursor %W %x %y}
      bind $c <Button-3> {clear_crosshair_cursor %W}
      binding_info "position" "Plot crosshair" "Erase crosshair"\
	  "Amp * 2" "Trace window" "Amp / 2"\
	  "Zoom region" "(Re)Draw" "Zoom out"
   }

   # show position binding
   bind_show_position

   # update menus depending on the bindings
   menus_depend_binding

   # function keys
   customize_function_key 
}

# --------------------------------------------------------------------
# show binding help
# --------------------------------------------------------------------
# [args]
# b1, b2, b3    :  explanations for button 1, 2 and 3
# sb1,sb1,sb3   :  explanations for shift+button 1, 2 and 3
# cb1,cb1,cb3   :  explanations for control+button 1, 2 and 3
# --------------------------------------------------------------------
proc binding_info {b1 b2 b3 sb1 sb2 sb3 cb1 cb2 cb3} {
    # global
    global gbl
    # main ------------------------------
    set b $gbl(widget,bind_info).help
    $b.up_b1 conf -text $b1
    $b.up_b2 conf -text $b2
    $b.up_b3 conf -text $b3
    $b.mid_b1 conf -text $sb1
    $b.mid_b2 conf -text $sb2
    $b.mid_b3 conf -text $sb3
    $b.down_b1 conf -text $cb1
    $b.down_b2 conf -text $cb2
    $b.down_b3 conf -text $cb3
}
# --------------------------------------------------------------------
# basic(common) bindings
# --------------------------------------------------------------------
# [arguments]
# w   : widget name for setting bindings
# --------------------------------------------------------------------
proc basic_bindings {w} {
   # global
   global gbl
   # main ------------------------------
   bindfe $w Control w "wm iconify $w"
   bindfe $w Control q {quit_application}
   bindfe $w Control c {pasteup_console}
   bindfe $w Control p {pick_window}
   bindfe $w Control l {draw_pasteup}
   bindfe $w Control o {layout_window}
   bindfe $w Control D {data_set_window}
   bindfe $w Control x {analysis_window}
   bindfe $w Control s {save_pick}
   bindfe $w Meta p    {print_pasteup}
   bindfe $w Meta z    {customize_window}
   bindfe $w Meta i    {su_read_auxiliary_info}
   bindfe $w Control 1 {su_read_auxiliary_info}
   bindfe $w Meta c    {duplicate_canvas}
   bindfe $w Control Prior {change_data_set -1}
   bindfe $w Control Next {change_data_set +1}
   bindfe $w Meta s    {auto_scale_factor}
   bindfe $w Meta d    {auto_offset_range ""}
   bindfe $w Meta r    {resize_canvas_size_by_ratio}
   bindfe $w Meta h    {headerwords_window}
   bindfe $w Meta l    {clear_data_cache}
}

proc bind_show_position {} {
   # global
   global gbl wk

   # main ------------------------------
   set c $wk(w,canvas,c)
   set g $wk(w,canvas,g)
   bind $c <Leave> "hide_position_window"
   bind $g <Leave> "hide_position_window"
   switch $gbl(option,showposition) {
      3 {
	 # show always
	 bind $c <Any-Motion> "show_position %W %x %y %X %Y"
	 bind $g <Any-Motion> "show_position %W %x %y %X %Y"
      }
      1 {
	 # when required
	 bind $c <Any-Motion> "hide_position_window"
	 bind $g <Any-Motion> "hide_position_window"
      }
      2 -
      0 {
	 # stay long/never
	 hide_position_window
	 bind $c <Any-Motion> ""
	 bind $g <Any-Motion> ""
      }
   }
}

proc menus_depend_binding {} {
   global gbl wk pick

   # main ------------------------------
   # init
   set f $wk(widget,binding)

   # clear existing contents
   set lst [winfo children $f]
   foreach w $lst {
      destroy $w
   }
   switch $gbl(bind) {
      "Tau-P Mode" -
      "View Mode" -
      "Amplitude Mode" -
      "Analysis Mode" {
	 return
      }
      default {
	 # pick, line pick mode
	 # create new contents
	 label $f.lpick -text "PickType :"
	 picktype_selector_widget $f ptype pick(current_type) pick(range)
	 
	 # pick accuracy
	 label $f.lacc -text "accuracy"
	 label $f.lunit -text "sec"
	 set bak $pick(default_accuracy)
	 spinbox $f.spin -width 7 -textv pick(default_accuracy) -just right\
	     -values $wk(accuracy,default_list)
	 set pick(default_accuracy) $bak
	 
	 # undo/redo button
	 label $f.separator -text "     "
	 button $f.undo -text "undo" -command "undo_pop"
	 button $f.redo -text "redo" -command "redo_pop"
	 
	 # placing
	 grid $f.lpick $f.mb(ptype) $f.sb(ptype) $f.lacc $f.spin $f.lunit\
	     $f.separator $f.undo $f.redo\
	     -sticky news
	 
	 bind $f.spin <Button-4> "$f.spin invoke buttonup"
	 bind $f.spin <Button-5> "$f.spin invoke buttondown"
	 bind $f.spin <Enter> {balloon "Default pick accuracy" %X %Y}
	 bind $f.spin <Leave> {cancel_balloon}
	 
      }
   }
}

# --------------------------------------------------------------------
# Wheel-bindings for the mouse-binding-menu
# --------------------------------------------------------------------
# [argument]
# w    :  widget name of menu button
# direc:  direction of change (next or prev)
# --------------------------------------------------------------------
proc wheel_binding_menu {w direc} {
   # global ------------------------------
   global gbl

   # main ------------------------------
   set modelist {"Pick Mode" "Line Pick Mode" "Analysis Mode" "View Mode"\
		     "Amplitude Mode" "Tau-P Mode" }
   set i [lsearch $modelist $gbl(bind)]
   switch $direc {
      prev {
	 # previous menu
	 if { $i < 1 } { return }
	 incr i -1
      }
      next {
	 # next menu
	 if { $i >= [expr [llength $modelist] -1] } { return }
	 incr i
      }
   }
   # invoke the menu
   $w invoke $i
}

# --------------------------------------------------------------------
# Front-End of bind
# --------------------------------------------------------------------
# [argument]
# w    :  tag name for bind (usually, top level window)
# mod  :  modifier (such as, shift, alt, control)
# key  :  key
# com  :  command to associate with
# --------------------------------------------------------------------
proc bindfe {w mod key com} {
   # global ------------------------------
   global gbl

   # main ------------------------------
   if {$mod == "Meta"} {
      bindfe $w Alt $key $com
      set keysym [format "<%s-KeyPress-%s>" $mod $key]
   } elseif { [string match *Button* $mod] } {
      # mouse button
      set keysym [format "<%s-%s>" $mod $key]
   } elseif { $mod == "KeyPress" } {
      # keypress with out modifier
      set keysym [format "<%s-%s>"  $mod $key]
   } else {
      # keys
      set keysym [format "<%s-KeyPress-%s>" $mod $key]
   }
   eval bind $w $keysym {$com}
}
# --------------------------------------------------------------------
# cancel button-2 bindings
# --------------------------------------------------------------------
proc cancel_button2 {w} {
   bind $w <B2-Motion> {}
   bind $w <ButtonRelease-2> {}
}