# --------------------------------------------------------------------
#
#
#                       NormalCanvas widget module
#
#
# --------------------------------------------------------------------

namespace eval NormalCanvas {
   namespace export *

   # -----------------------------------------------------------------
   # use normal canvas widget for drawing
   # -----------------------------------------------------------------
   proc setup_canvas {} {
      # global --------------------------------------------------
      global wk
      
      # main --------------------------------------------------
      # clear
      clear_slaves $wk(w,canvas)

      # placement
      grid $wk(w,canvas,c) -in $wk(w,canvas) -row 0 -column 0\
	  -rowspan 1 -columnspan 1 -sticky news
      grid $wk(w,canvas).v -row 0 -column 1 -rowspan 1 -columnspan 1 -stic news
      grid $wk(w,canvas).h -row 1 -column 0 -rowspan 1 -columnspan 1 -stic news
      grid $wk(w,canvas).b -row 1 -column 1 -rowspan 1 -columnspan 1 -stic news
      grid rowconfig    $wk(w,canvas) 0 -weight 1 -minsize 0
      grid columnconfig $wk(w,canvas) 0 -weight 1 -minsize 0
   }
   
   # -----------------------------------------------------------------
   # make NormalCanvas widget
   # -----------------------------------------------------------------
   proc make_canvas_widget {} {
      # global --------------------------------------------------
      global wk paste
      
      # main --------------------------------------------------
      set c $wk(w,canvas,c)
      canvas $c -relief groove -background white -highlightthickn 0\
	  -width $paste(canvas_width) -height $paste(canvas_height) \
	  -scrollregion "0 0 $paste(canvas_width) $paste(canvas_height)"\
	  -xscrollcom "$wk(w,canvas).h set" -yscrollcom "$wk(w,canvas).v set"
      scrollbar $wk(w,canvas).h -ori horiz -com "$c xview" -width 10
      scrollbar $wk(w,canvas).v -command "$c yview" -width 10
      button $wk(w,canvas).b -command duplicate_canvas -padx 0 -pady 0\
	  -image copyicon -background white
      bind $wk(w,canvas).b <Enter> {balloon "duplicate Current figure" %X %Y}
      bind $wk(w,canvas).b <Leave> {cancel_balloon}
   }
   # -----------------------------------------------------------------
   # draw axis and subplots
   # -----------------------------------------------------------------
   proc draw_axis {c} {
      # global --------------------
      global paste gbl file_name filt wk fontmap
      
      # main--------------------------------------------------
      # clear
      catch {$c delete frame}
      catch {$c delete stack_hyperbola}

      # check tick interval
      if {$gbl(t_tic_int) <= 0 || $gbl(xtic_int) <= 0} {
	 put_error "Invalid tick invervals."
	 return
      }

      # definitions
      set xl $paste(left_margin)
      set xr [expr $xl + $paste(graph_width)]
      set tt(top)    $paste(top_margin) 
      set tt(bottom) [expr $tt(top) + $paste(graph_height)]
      set lw 0.5 ; # line width

      # frame (rectangle)
      $c create rectangle $xl $tt(top) $xr $tt(bottom) -width $lw -tags frame
      
      # vertical tic mark
      set t [expr int($paste(top)/$gbl(t_tic_int) - 1)*$gbl(t_tic_int)]
      while {1} {
	 if { $t > $paste(bottom) } { break }
	 # major tic
	 plot_time_tic $c $t 1 $lw $xl $xr
	 # sub tics
	 for {set i 0} {$i < $gbl(t_sub_tic)} {incr i} {
	    plot_time_tic $c\
		[expr $t+($i+1)*$gbl(t_tic_int)/($gbl(t_sub_tic)+1.0)]\
		0 $lw $xl $xr
	 }
	 set t [expr $t+$gbl(t_tic_int)]
      }

      # horizontal-tic mark
      foreach side {top bottom} {
	 if { $gbl(x_axis,$side) != 1 } {
	    # x-axis is ABSOLUTE POSITION
	    set xmin [expr $paste(left) + $gbl(obs,x)]
	    set xmax [expr $paste(right) + $gbl(obs,x)]
	 } else {
	    # x-axis is OFFSET
	    set xmin $paste(left)
	    set xmax $paste(right)
	 }
	 set x [expr int($xmin/$gbl(xtic_int) - 1)*$gbl(xtic_int)]
	 while {1} {
	    if { $x > $xmax } { break }
	    # major tic
	    plot_dist_tic $c $x 1 $lw $tt($side) $xmin $xmax $side
	    # sub tics
	    for {set i 0} {$i < $gbl(x_sub_tic)} {incr i} {
	       plot_dist_tic $c\
		   [expr $x+($i+1)*$gbl(xtic_int)/($gbl(x_sub_tic)+1.0)]\
		   0 $lw $tt($side) $xmin $xmax $side
	    }
	    set x [expr $x + $gbl(xtic_int) ]
	 }
      }

      # show unit
      switch $paste(string,unit) {
	 "tx" { # tx
	    set vunit "(sec)"
	    set hunit "(km)"
	 }
	 "xz" { # xz
	    set vunit "(km)"
	    set hunit "(km)"
	 }
	 default {
	    set vunit ""
	    set hunit ""
	 }
      }

      # show horizontal axis unit
      if {$paste(string,toptick)} {
	 $c create text $xr [expr $tt(top) - $paste(tic_size)\
				 - $paste(tick_font_height)]\
	     -text $hunit -anchor sw -font $paste(fn,unit) -tags frame
      }
      if {$paste(string,bottomtick)} {
	 $c create text $xr\
	     [expr $tt(bottom) + $paste(tic_size) + $paste(tick_font_height)]\
	     -text $hunit -anchor nw -font $paste(fn,unit) -tags frame
      }

      # show vertical axis unit
      $c create text [expr $xl - $paste(tic_size) - $paste(tick_font_width)]\
	  [expr $tt(bottom) + $paste(tic_size)]\
	  -text $vunit -anchor ne -font $paste(fn,unit) -tags frame

      # sub plot
      draw_sub_plot_frame $c
      draw_sub_plot $c
   }

   # -----------------------------------------------------------------
   # draw labels
   # -----------------------------------------------------------------
   proc draw_labels {c} {
      # global --------------------
      global paste gbl file_name filt wk fontmap
      
      # main--------------------------------------------------
      # clear
      catch {$c delete labels}
      
      # plot title
      if { [string length $paste(label,title)] > 0 } {
	 $c create text [expr $paste(canvas_width)/2] 1 -anch n\
	     -font $paste(fn,title) -tags labels -text "$paste(label,title)" 
      }
      # plot top axis label
      if { $paste(check,toplabel) } {
	 $c create text [expr $paste(canvas_width)/2]\
	     [expr $tt(top) - $paste(tic_size) - $paste(tick_font_height)]\
	     -anch s\
	     -font $paste(fn,axis) -tags labels -text "$paste(label,topaxis)" 
      }
      # plot bottom axis label
      if { $paste(check,bottomlabel) } {
	 set y [expr $paste(top_margin) + $paste(graph_height) + $paste(tic_size)]
	 if {$paste(string,bottomtick)} {
	    set y [expr $y + $paste(tick_font_height)]
	 }
	 $c create text [expr $paste(canvas_width)/2] $y -anch n\
	     -font $paste(fn,axis) -tags labels -text "$paste(label,bottomaxis)"
      }
      # show info at the bottom
      plot_bottom_label $c
   }

   # --------------------------------------------------------------------
   # Plot bottom label
   # --------------------------------------------------------------------
   proc plot_bottom_label {c} {
      global paste gbl file_name

      set str ""
      # filename
      if {$paste(string,filename)} {
	 set file_list [split $file_name(wave) \/]
	 set file_base_name [lindex $file_list [expr [llength $file_list] - 1]]
	 lappend str [format "Data File: %s" $file_base_name]
      }
      # filters
      set flabel [filter_explanation]
      if {$paste(string,filter)} {
	 if { $gbl(check,sufilter) } {
	    lappend str "SU filters  "
	 } else {
	    lappend str [format "Filter: %s "  $flabel]
	    if { $gbl(check,stack) == 1 } {
	       lappend str \
		   [format "Stack(Range: %.3fkm, Vel: %.3fkm/sec. Tau: %.2fsec.)"\
			$paste(stack,range) $paste(stack,vel)\
			$paste(stack,time)]
	    }
	 }
      }
      # reduction
      if {$paste(string,reduction)} {
	 lappend str [format "Reduction Velocity: %4.2f km/sec."\
			  $paste(reduction)]
      }
      # plot
      if {$str == ""} { return }
      set x [expr $paste(left_margin) + $paste(graph_width)\
		 - $paste(label_font_width)]
      set y [expr $paste(canvas_height) - $paste(label_font_height)]
      $c create text $x $y -tag frame\
	  -anchor ne -text [join $str "     "] -font $paste(fn,unit)
   }
   # --------------------------------------------------------------------
   # Convert time of data into y-coordinate of the graph
   # --------------------------------------------------------------------
   proc time_to_graph {time} {
      # global --------------------
      global paste gbl

      # main --------------------
      if {$gbl(t_axis) > 0} {
	 # downward t-axis
	 return [expr $paste(top_margin) +\
		     $paste(graph_height)*($time - $paste(top))\
		     /($paste(bottom) - $paste(top))]
      } else {
	 # upward t-axis
	 return [expr $paste(top_margin) + $paste(graph_height)\
		     - $paste(graph_height)*($time - $paste(top))\
		     /($paste(bottom) - $paste(top))]
      }
   }
   # --------------------------------------------------------------------
   # Plot a vertical tick
   # --------------------------------------------------------------------
   # c    : canvas widget
   # t    : time of tick
   # mode : 1:major tic,   0:minor tic)
   # lw   : line width
   # xl   : x-position of left end of graph
   # xr   : x-position of right end of graph
   # --------------------------------------------------------------------
   proc plot_time_tic {c t mode lw xl xr} {
      # global variables ------------------------------
      global paste
      
      # main ------------------------------
      if { $t < $paste(top) || $t > $paste(bottom) } {
	 # out of range.
	 return
      }
      if {$mode} {
	 # major mode tic ----------
	 set tic_size $paste(tic_size)
      } else {
	 # minor mode tic ----------
	 set tic_size $paste(sub_tic_size)
	 set lw [expr $lw / 2.]
      }
      set t_graph [time_to_graph $t]
      $c create line $xl $t_graph\
	  [expr $xl - $tic_size] $t_graph -width $lw -tags frame
      $c create line $xr $t_graph	[expr $xr + $tic_size + 1]\
	  $t_graph -width $lw -tags frame
      # plot tic label
      if {$paste(string,lefttick) && $mode} {
	 $c create text [expr $xl - $tic_size - 6] $t_graph\
	     -text "$t" -anchor e -font $paste(fn,tick) -tags frame
      }
      if {$paste(string,righttick) && $mode} {
	 $c create text [expr $xr + $tic_size + 6] $t_graph\
	     -text "$t" -anchor w -font $paste(fn,tick) -tags frame
      }
   }

   # --------------------------------------------------------------------
   # plot a horizontal tick mark
   # --------------------------------------------------------------------
   # c    : canvas widget name
   # x    : x position of tick
   # mode : 1:majour tick, 0:minor tick
   # lw   : line width
   # t    : z-position of top/bottom of the graph
   # xmin : x must be 
   # xmax :     $xmin <= $x <= $xmax
   # side : top or bottom
   # --------------------------------------------------------------------
   proc plot_dist_tic {c x mode lw t xmin xmax side} {
      # global variables ------------------------------
      global paste wk gbl
      
      # main ------------------------------
      # inside range ?
      if { $x > $xmax || $x < $xmin } { return }
      # convert  data-x into graph-x
      if { $gbl(data,xaxis_direction) == 0 } {
	 # normal direction
	 set x_graph [expr $paste(left_margin) +\
			  $paste(graph_width)*($x - $xmin) / ($xmax - $xmin)]
      } else {
	 # reverse direction
	 set x_graph [expr $paste(graph_width)*($xmax - $x) / ($xmax - $xmin) \
			  + $paste(left_margin)]
      }
      set ts [expr $mode ? { $paste(tic_size) } : { $paste(sub_tic_size) } ]
      # plot tick mark and label
      if {$side == "top"} {
	 # top
	 $c create line $x_graph $t $x_graph\
	     [expr $t - $ts] -width $lw -tags frame
	 if {$paste(string,toptick) && $mode} {
	    $c create text $x_graph [expr $t - $ts - 0]\
		-text "$x" -anchor s -font $paste(fn,tick) -tags frame
	 }
      } else {
	 # bottom
	 $c create line $x_graph $t $x_graph [expr $t + $ts + 1]\
	     -width $lw -tags frame
	 if {$paste(string,bottomtick) && $mode} {
	    $c create text $x_graph [expr $t + $ts + 2]\
		-text "$x" -anchor n -font $paste(fn,tick) -tags frame
	 }
      }
   }
   # -----------------------------------------------------------------
   # Put a pasteup image
   # -----------------------------------------------------------------
   proc put_pasteupimage {c} {
      global paste
      $c create image $paste(left_margin) $paste(top_margin)\
	  -image pasteupimage -anchor nw -tag pasteup_image
      $c lower pasteup_image
   }
   # -----------------------------------------------------------------
   # clear canvas
   # -----------------------------------------------------------------
   proc delete_all {c} {
      $c delete all
   }
   # -----------------------------------------------------------------
   # Delete all pick marks
   # -----------------------------------------------------------------
   proc delete_pick_marks {} {
      global wk
      pick_label_erase
      catch {$wk(w,canvas,c) delete pk acc trace_pick trace_pick_accuracy}
   }
   # -----------------------------------------------------------------
   # Plot synthetics
   # -----------------------------------------------------------------
   proc plot_synthetic {w} {
      # global --------------------
      global macray
      
      # main --------------------
      # Plot Style ----------
      switch -- [string trim $macray(style) " "] {
	 "Symbol" {
	    # Plot with Symbol
	    for {set i 0} {$i < $macray(n)} {incr i} {
	       if [coord inout $macray(x,$i) $macray(t,$i)] {
		  set x [coord gx2cx $macray(x,$i)]
		  set t [coord tt2cy $macray(x,$i) $macray(t,$i)]
		  plot_symbol $w $x $t [Synthetic::color $macray(m,$i)]
	       }
	    }
	 }
	 "Line" {
	    # Plot with Line
	    set coord ""
	    set cmode 10000
	    set prevx 0
	    for {set i 0} {$i < $macray(n)} {incr i} {
	       if { $cmode < 1 || $cmode != $macray(m,$i)\
			|| [expr $macray(x,$i) * $prevx] < 0.0 } {
		  draw_line $w [Synthetic::color $cmode] $coord
		  set coord ""
		  set cmode $macray(m,$i)
	       }
	       if [coord inout $macray(x,$i) $macray(t,$i)] {
		  set x [coord gx2cx $macray(x,$i)]
		  set t [coord tt2cy $macray(x,$i) $macray(t,$i)]
		  set coord "$coord $x $t"
	       }
	       set prevx $macray(x,$i)
	    }
	    draw_line $w [Synthetic::color $cmode] $coord
	 }
	 default -
	 "Line&Symbol" {
	    # Plot With Line and Symbols
	    set coord ""
	    set cmode 10000
	    set current_color [Synthetic::color $cmode]
	    set prevx 0
	    for {set i 0} {$i < $macray(n)} {incr i} {
	       if { $cmode < 1 || $cmode != $macray(m,$i) \
			|| [expr $macray(x,$i) * $prevx] < 0.0 } {
		  draw_line $w [Synthetic::color $cmode] $coord
		  set coord ""
		  set cmode $macray(m,$i)
		  set current_color [Synthetic::color $cmode]
	       }
	       if [coord inout $macray(x,$i) $macray(t,$i)] {
		  set x [coord gx2cx $macray(x,$i)]
		  set t [coord tt2cy $macray(x,$i) $macray(t,$i)]
		  plot_symbol $w $x $t $current_color
		  set coord "$coord $x $t"
	    }
	       set prevx $macray(x,$i)
	    }
	    draw_line $w $current_color $coord
	 }
      }
   }
   # -----------------------------------------------------------------
   # Plot a single synthetic mark in the MacRay format.
   # -----------------------------------------------------------------
   proc plot_symbol {w x t c} {
      # global --------------------
      global macray
      
      # main --------------------
      set s $macray(symbol_size)
      switch -- $macray(symbol_style) {
	 "oval" {
	    $w create oval  [expr $x-$s] [expr $t - $s]\
		[expr $x+$s] [expr $t + $s] -fill $c -outline "" -tags macray
	 }
	 "cross" {
	    # cross mark
	    $w create line  [expr $x-$s] [expr $t - $s]\
		[expr $x+$s] [expr $t + $s]\
		-fill $c -tags macray -width $macray(width)
	    $w create line  [expr $x+$s] [expr $t - $s]\
		[expr $x-$s] [expr $t + $s]\
		-fill $c -tags macray -width $macray(width)
	 }
      }
   }
   # -----------------------------------------------------------------
   # Draw synthetic a line segment in the MacRay format
   # -----------------------------------------------------------------
   proc draw_line {w color xt} {
      # global --------------------
      global macray
      
      # main --------------------
      if { [llength $xt] < 4} { return }
      switch -- $macray(line_style) {
	 "solid" {
	    eval {$w create line} $xt {-tags macray}\
		-fill $color -width $macray(width)
	 }
	 "dashed" {
	    eval {$w create line} $xt {-tags macray}\
		-fill $color -width $macray(width) -dash {"12 6"}
	 }
	 default {
	    # dotted
	    eval {$w create line} $xt {-tags macray}\
		-fill $color -width $macray(width) -dash {"3 3"}
	 }
      }
   }
   
   # -----------------------------------------------------------------
   # Delete synthetics
   # -----------------------------------------------------------------
   proc delete_synthetic {w} {
      $w delete macray
   }
   # -----------------------------------------------------------------
   # Delete the closest pick mark
   # -----------------------------------------------------------------
   proc delete_closest_pick {w mx my} {
      set x [coord cx2gx [$w canvasx $mx]]
      set my [$w canvasy $my]
      delete_single_pick $x [coord cy2gt $my]
   }
   # -----------------------------------------------------------------
   # start to a single pick
   # -----------------------------------------------------------------
   proc pick_start {w pick_x pick_y} {
      # global --------------------
      global pick gbl

      # main --------------------
      # searching closest trace
      scan [getrh nearest [coord cx2gx [$w canvasx $pick_x]]] "%d%g%d%d"\
	  pick(near_no) pick(dist) pick(obs) pick(depth)
      if {$pick(near_no) < 0} { return }
      # store to the temporary array
      set pick(status) 1
      set pick(cy0) [$w canvasy $pick_y]
      set x [coord gx2cx $pick(dist)]
      set pick(cx0) [expr $x - $gbl(mark_size)]
      set pick(cx1) [expr $x + $gbl(mark_size)]
      set pick(start_t) [coord cy2tt $pick(dist) [$w canvasy $pick_y]]

      # temporary binding
      bind $w <B2-Motion> "pick_motion $w %x %y"
      bind $w <ButtonRelease-2> "pick_end $w %x %y"
   }

   # -----------------------------------------------------------------
   # while picking... (determining accuracy)
   # -----------------------------------------------------------------
   proc pick_motion {w mx my} {
      # global --------------------
      global pick gbl

      # main --------------------
      if { !$pick(status) } {
	 cancel_button2
	 return;
      }
      catch {$w delete temp_mark}
      set cy1 [$w canvasy $my]
      set cy [expr ($cy1 + $pick(cy0)) / 2.0]
      # draw
      $w create rectangle $pick(cx0) $pick(cy0) $pick(cx1) $cy1\
	  -outline gray -tags temp_mark -width 1
      $w create line  $pick(cx0) $cy $pick(cx1) $cy\
	  -fill $pick(color,$pick(current_type)) -tags temp_mark -width 1
   }
   # -----------------------------------------------------------------
   # end pick (confirmation)
   # -----------------------------------------------------------------
   proc pick_end {w mx my} {
      # global --------------------
      global pick gbl mark_modification paste pm undopm

      # main --------------------
      # draw temporary pick range
      $w delete temp_mark
      cancel_button2 $w
      set pick(status) 0

      # pick info ----------
      set end_t [coord cy2tt $pick(dist) [$w canvasy $my]]
      set pt [expr ($pick(start_t) + $end_t)/2.0]
      # determine pick accuracy
      set acc [expr abs($pick(start_t) - $end_t) / 2.0]
      set acc [expr {$acc < $pick(default_accuracy)} ?\
		   $pick(default_accuracy) : $acc ]

      # store it in the pm list
      undo_stack
      switch $paste(correct,shot_mode) {
	 0 {
	    # none
	 }
	 2 {
	    # shots move to sea floor
	    set pt [expr $pt + $pick(depth) / ($gbl(water_v) * 1000.0)]
	 }
	 1 {
	    # flat sea floor correction
	    set pt [expr $pt + ($gbl(obs,depth) - $pick(depth))\
			* (1.0/($gbl(sediment_v)*1000.0)\
			       - 1.0/($gbl(water_v)*1000.0)) ]
	 }
      }
      lappend pm "$pick(near_no) $pick(dist) $pt $acc $pick(current_type)"
      set mark_modification 1

      # draw a pick mark ----------
      plot_single_pick $pick(dist) $pt $acc $pick(current_type)

      # report pick info
      print_message [format "Shot #%4d (OBS %2d) : Location(%fkm, %fsec), Accuracy %fsec, Pick Type%d\n"\
			 $pick(near_no) $pick(obs) $pick(dist) $pt $acc $pick(current_type)]
   }

   # -----------------------------------------------------------------
   # Start line pick
   # -----------------------------------------------------------------
   proc line_pick_start {w mx my} {
      # global --------------------
      global pick

      # main --------------------
      if { $pick(line,status) == 1 } { return }
      set pick(line,status) 1
      set pick(line,start_x) [$w canvasx $mx]
      set pick(line,start_y) [$w canvasy $my]
      # temporal binding
      bind $w <B2-Motion> "line_pick_move $w %x %y %X %Y"
      bind $w <ButtonRelease-2> "line_pick_end $w %x %y %X %Y"
   }

   # -----------------------------------------------------------------
   # while line picking...
   # -----------------------------------------------------------------
   proc line_pick_move {w mx my RX RY} {
      # global --------------------
      global pick
      
      # main --------------------
      catch {$w delete line_pick_tmp_tag}
      if { $pick(line,status) == 0 } { 
	 return 
	 cancel_button2 $w
      }
      $w create line $pick(line,start_x) $pick(line,start_y)\
	  [$w canvasx $mx] [$w canvasy $my] \
	  -tag line_pick_tmp_tag -fill green -width 1
      show_position $w $mx $my $RX $RY
   }

   # -----------------------------------------------------------------
   # end line pick
   # -----------------------------------------------------------------
   proc line_pick_end {w mx my RX RY} {
      # global --------------------
      global pick
      
      # main --------------------
      # erase a temporal line
      catch {$w delete line_pick_tmp_tag}
      cancel_button2 $w
      if { $pick(line,status) == 0 } { return }

      # Convert canvas coord into graph coord
      set x0 [coord cx2gx $pick(line,start_x)]
      set t0 [coord cy2gt $pick(line,start_y)]
      set x1 [coord cx2gx [$w canvasx $mx]]
      set t1 [coord cy2gt [$w canvasy $my]]
      
      # exec line pick 
      undo_stack
      exec_line_pick $x0 $t0 $x1 $t1 $RX $RY
      set pick(line,status) 0
   }

   # -----------------------------------------------------------------
   # Cancel line pick
   # -----------------------------------------------------------------
   proc cancel_line_pick {w} {
      # global --------------------
      global pick

      # main --------------------
      set pick(line,status) 0
      catch {$w delete line_pick_tmp_tag}
      cancel_button2 $w
   }

   # -----------------------------------------------------------------
   # Show mouse position with balloon
   # -----------------------------------------------------------------
   proc show_position {w mx my RX RY} {
      # global --------------------
      global gbl wk

      # Main --------------------
      set x [format "%.3f" [coord cx2gx [$w canvasx $mx]]]
      set t [format "%.3f" [coord cy2gt [$w canvasy $my]]]
      if ![coord inout2 $x $t] { return }
      set x2 [expr ( $gbl(x_axis,top) == 1 ) ? $x : [expr $x + $gbl(obs,x)] ]
      show_position_balloon [format "x=%.3f (%.3f), t=%.3f" $x $x2 $t] $RX $RY
   }
   # -----------------------------------------------------------------
   # Show position balloon with shot number
   # -----------------------------------------------------------------
   proc show_position_shot {w mx my RX RY} {
      set gx [coord cx2gx [$w canvasx $mx]]
      set cy [$w canvasy $my]
      set rt [coord cy2gt $cy]
      if { [coord inout2 $gx $rt] == 1 } { return }
      set tt [coord cy2tt $gx $cy]
      scan [getrh nearest $gx] "%d%*s%d" sn obsn
      report_position_shot $gx $tt $rt $sn $obsn $RX $RY
   }
   # --------------------------------------------------------------------
   # show pick mark info
   # --------------------------------------------------------------------
   proc show_pick_info {w mx my PX PY} {
      set gx [coord cx2gx [$w canvasx $mx]]
      set my [$w canvasy $my]
      set rt [coord cy2gt $my]
      report_pick_info $gx $my $rt $PX $PY
   }
   # --------------------------------------------------------------------
   # Select a region for pick property
   # --------------------------------------------------------------------
   proc change_pick_start {w mx my} {
      # global --------------------
      global change_rect

      # main --------------------
      set change_rect(status) 1
      set change_rect(x1) [$w canvasx $mx]
      set change_rect(y1) [$w canvasy $my]

      # temporal binding
      bind $w <B3-Motion> {
	 change_pick_motion %W %x %y
      }
      bind $w <ButtonRelease-3> {
	 change_pick_end %W %x %y %X %Y
      }
   }

   # --------------------------------------------------------------------
   # while selecting a region for pick property
   # --------------------------------------------------------------------
   proc change_pick_motion {w x y} {
      # global --------------------
      global change_rect

      # main --------------------
      if { !$change_rect(status) } {
	 bind $w <B3-Motion> ""
	 return
      }
      # show a temporal region
      catch {$w delete temp_mark}
      $w create rectangle $change_rect(x1) $change_rect(y1)\
	  [$w canvasx $x]  [$w canvasy $y] -outline red -tags temp_mark -width 2
   }
   # --------------------------------------------------------------------
   # end selecting a region for pick property
   # --------------------------------------------------------------------
   proc change_pick_end {w mx my RX RY} {
      # global --------------------
      global change_rect

      # main --------------------
      set x2 [$w canvasx $mx]
      set y2 [$w canvasy $my]
      
      # show the region
      catch {$w delete temp_mark}
      $w create rectangle $change_rect(x1) $change_rect(y1) $x2 $y2\
	  -outline green -tags temp_mark -width 2

      # invoke pick property window
      change_pick_property $w $change_rect(x1) $x2 $change_rect(y1) $y2 $RX $RY

      # erase the region
      catch { tkwait window .change_pick_property}
      $w delete temp_mark
   }
   # -----------------------------------------------------------------
   # Plot all the multiple marks
   # -----------------------------------------------------------------
   proc plot_all_multiple {dt} {
      # global --------------------
      global wk pm gbl
      
      # main --------------------
      clear_multiple
      foreach d $pm { 
	 scan $d "%d%s%s%*s%*d" sn px pt
	 set y [pick tt2cy $sn $px [expr $dt + $pt]]
	 set x [coord gx2cx $px]
	 $wk(w,canvas,c) create line [expr $x - $gbl(mark_size)] $y\
	     [expr $x + $gbl(mark_size)] $y\
	     -fill $wk(color,multiple) -tags multiple -width 2
      }
   }
   # -----------------------------------------------------------------
   # Erase multiple marks
   # -----------------------------------------------------------------
   proc clear_multiple {} {
      global wk
      $wk(w,canvas,c) delete multiple
   
   }
   # --------------------------------------------------------------------
   # Measure apprarent velocity (start to draw a line)
   # --------------------------------------------------------------------
   proc vel_line_start {w mx my} {
      # global --------------------
      global wk
      
      # Main --------------------
      set wk(vline,x0) [coord cx2gx [$w canvasx $mx]]
      set wk(vline,t0) [coord cy2tt $wk(vline,x0) [$w canvasy $my]]
      bind $w <B2-Motion> "vel_line_move $w %x %y"
      bind $w <ButtonRelease-2> "vel_line_end $w %x %y %X %Y"
   }

   # --------------------------------------------------------------------
   # Drawing a line (searching for end point)
   # --------------------------------------------------------------------
   proc vel_line_move {w mx my} {
      # global --------------------
      global wk
      
      # Main --------------------
      catch {$w delete vline_tmp_tag}
      set x2 [coord cx2gx [$w canvasx $mx]]
      set t2 [coord cy2tt $x2 [$w canvasy $my]]
      draw_single_vline $w $wk(vline,x0) $x2 $wk(vline,t0) $t2\
	  vline_tmp_tag 1 green
   }

   # --------------------------------------------------------------------
   # Determine an end point of a line, show corresponding velocity
   # --------------------------------------------------------------------
   proc vel_line_end {w mx my RX RY} {
      # global --------------------
      global wk

      # Main --------------------
      # clear temporary line
      catch {$w delete vline_tmp_tag}

      # detach binding
      bind $w <B2-Motion> {}
      bind $w <ButtonRelease-2> {}

      # compute coordinate
      set x2 [coord cx2gx [$w canvasx $mx]]
      set t2 [coord cy2tt $x2 [$w canvasy $my]]
      if { [expr $wk(vline,x0)  * $x2] < 0.0 } {
	 set t2 [expr $wk(vline,t0) - ($t2 - $wk(vline,t0))* $wk(vline,x0)\
		     /($x2 - $wk(vline,x0)) ]
	 set x2 0.0
      }
      # draw 
      draw_single_vline $w $wk(vline,x0) $x2 $wk(vline,t0) $t2\
	  vline_tag 2 green
      # result
      vel_line_result $wk(vline,x0) $wk(vline,t0) $x2 $t2 $RX $RY
   }
   # --------------------------------------------------------------------
   # Draw all the velocity line
   # --------------------------------------------------------------------
   proc draw_all_vline {} {
      # global --------------------
      global wk
      
      # Main --------------------
      catch {$wk(w,canvas,c) delete vline_tag}
      foreach d $wk(vline,data) {
	 scan $d "%s %s %s %s %s %s" x1 x2 t1 t2 vel t
	 draw_single_vline $wk(w,canvas,c) $x1 $x2 $t1 $t2 vline_tag 2 green
      }
   }
   # --------------------------------------------------------------------
   # Draw single velocity line
   #   x1, x2 are x graph coordinates
   #   t1, t2 are trut times
   # --------------------------------------------------------------------
   proc draw_single_vline {w x1 x2 t1 t2 tag width color} {
      # convert coordinates into canvas coordinates
      set t1 [coord tt2cy $x1 $t1]
      set t2 [coord tt2cy $x2 $t2]
      set x1 [coord gx2cx $x1]
      set x2 [coord gx2cx $x2]
      $w create line $x1 $t1 $x2 $t2 -tag $tag -fill $color -width $width
   }
   # -----------------------------------------------------------------
   # Make Print command
   # -----------------------------------------------------------------
   proc make_print_command {outfn} {
      global paste ps_page_long ps_page_short print fontmap wk
      update_paper_size
      if { $print(normalize) == "h" } {
	 # normalized by x
	 if {$print(orientation) == "Landscape"} {
	    set orientation 1
	    set w [format "%sc" [expr $ps_page_long * 0.95]]
	 } else {
	    set orientation 0
	    set w [format "%sc" [expr $ps_page_short * 0.95]]
	 }
	 set normalize "-pagewidth $w"
      } else {
	 # normalized by t
	 if {$print(orientation) == "Landscape"} {
	    set orientation 1
	    set h [format "%sc" [expr $ps_page_short * 0.95]]
	 } else {
	    set orientation 0
	    set h [format "%sc" [expr $ps_page_long * 0.95]]
	 }
	 set normalize "-pageheight $h"
      }
      set pagex [format "%sc" [expr $ps_page_short/2.0]]
      set pagey [format "%sc" [expr $ps_page_long/2.0]]
      set com "$wk(w,canvas,c) postscript -fontmap fontmap\
          -colormode $print(mode) $normalize -rotate $orientation\
	  -width $paste(canvas_width) -x 0 -pagex $pagex\
	  -height $paste(canvas_height) -y 0 -pagey $pagey"
      if { $outfn == "" } {
	 return $com
      }
      return "$com -file $outfn"
   }
   # -----------------------------------------------------------------
   # clear zero level
   # -----------------------------------------------------------------
   proc clear_zero_level {c} {
      $c delete -tags aux_line
   }
   # -----------------------------------------------------------------
   # plot zero level
   # -----------------------------------------------------------------
   proc plot_zero_level {c xlist color} {
      # global --------------------
      global paste wk
      
      # main --------------------
      set cy [expr $paste(top_margin) + $paste(graph_height)]
      foreach x $xlist {
	 set cx [coord gx2cx $x]
	 $c create line $cx $cy $cx $paste(top_margin)\
	     -width 1 -tags aux_line -fill $color
      }
   }
   # -----------------------------------------------------------------
   # clear water wave
   # -----------------------------------------------------------------
   proc clear_water_wave {c} {
      $c delete waterwave
   }
   # -----------------------------------------------------------------
   # plot water wave
   # -----------------------------------------------------------------
   proc plot_water_wave {c coord color} {
      if {[llength $coord] <= 3} { return }
      set xy {}
      foreach {x t} $coord {
	 lappend xy [coord gx2cx $x] [coord tt2cy $x [coord rt2tt $x $t]]
      }
      eval {$c create line} $xy {-width 2 -tags waterwave -fill} $color
   }
   # -----------------------------------------------------------------
   # Magnify single trace.
   # -----------------------------------------------------------------
   proc magnify_trace {w pick_x pick_y} {
      # global --------------------
      global paste wk
      
      # main --------------------
      # coordinates
      set gx [coord cx2gx [$w canvasx $pick_x]]
      set gt [coord cy2gt [$w canvasy $pick_y]]
      set near_rh [getrh nearest $gx]
      set wk(trace,current,dist) [lindex $near_rh 1]
      
      # valid ranve?
      if [coord inout2 $gx $gt] {
	 print_message "Warning: Out of record section.\n"
      }
      # invoke trace window
      trace_window
      
      # compute coordinates
      trace_coordinate
      
      # show the trace
      display_trace 0
   }
   # -----------------------------------------------------------------
   # clear strings
   # -----------------------------------------------------------------
   proc clear_string {c} {
      $c delete str
   }
   # -----------------------------------------------------------------
   # plot a string
   # -----------------------------------------------------------------
   proc plot_string {c x t str anc fn col shadow shadow_col} {
      set t [coord tt2cy $x $t]
      set x [coord gx2cx $x]
      if { $shadow > 0 } {
	 $c create text [expr $x + $shadow] [expr $t + $shadow]\
	     -text $str -fill $shadow_col -font $fn -anc $anc -tag str
      }
      $c create text $x $t -text $str -fill $col -font $fn -anc $anc -tag str
   }
   # -----------------------------------------------------------------
   # draw hyperbola
   # -----------------------------------------------------------------
   proc draw_hyperbola {} {
      # global variable
      global paste wk
      
      # main ------------------------------
      # Clear
      catch {$wk(w,canvas,c) delete stack_hyperbola}
      # Compute hyperbola coordiantes
      set xy {}
      set PLOT_NO 100
      set v2 [expr $paste(stack,vel)*$paste(stack,vel)]
      set t2 [expr $paste(stack,time)*$paste(stack,time)]
      for {set i 0} {$i <= $PLOT_NO} {incr i} {
	 set gx [expr $paste(left) + $i*($paste(right) - $paste(left))\
		     /double($PLOT_NO)]
	 set x [coord gx2cx $gx]
	 set t [expr sqrt($t2 + $gx*$gx/$v2)]
	 set y [coord tt2cy $gx $t]
	 if {$paste(top_margin) < $y && \
		 [expr $paste(top_margin)+$paste(graph_height)] > $y} {
	    set xy [concat $xy $x $y]
	 }
      }
      # Draw it!
      if {[llength $xy] > 3} {
	 eval {$wk(w,canvas,c) create line} $xy \
	     {-width 2 -tags stack_hyperbola -fill} green
      }
   }
   # -----------------------------------------------------------------
   # sub plot origin
   # -----------------------------------------------------------------
   proc sub_plot_origin {} {
      global paste
      set x0 $paste(left_margin)
      set y0 [expr { $paste(sub,position) == "top" } ?\
		  {$paste(title_font_height) + $paste(sub,height)\
		       + $paste(tic_size)} :\
		  {$paste(canvas_height) - $paste(label_font_height) \
		       - $paste(tic_size)}]
      set x1 [expr $x0 + $paste(graph_width)]
      set y1 [expr $y0 - $paste(sub,height)]
      return "$x0 $y0 $x1 $y1"
   }
   # -----------------------------------------------------------------
   # draw frame of sub plot
   # -----------------------------------------------------------------
   proc draw_sub_plot_frame {c} {
      # global variable
      global paste gbl
      
      # main ------------------------------
      if { ! $paste(sub,check) } { return }

      # coordinates
      scan [sub_plot_origin] "%g%g%g%g" x0 y0 x1 y1
      
      # draw axis ----------
      $c create rectangle $x0 $y0 $x1 $y1 -width 0.5 -tags frame
      
      # horizontal tick mark ----------
      if { $gbl(x_axis,$paste(sub,position)) != 1 } {
	 # x-axis is ABSOLUTE POSITION
	 set xmin [expr $paste(left) + $gbl(obs,x)]
	 set xmax [expr $paste(right) + $gbl(obs,x)]
      } else {
	 # x-axis is OFFSET
	 set xmin $paste(left)
	 set xmax $paste(right)
      }
      set tmp(bottom) $paste(string,bottomtick)
      set tmp(top) $paste(string,toptick)
      set paste(string,bottomtick) 0
      set paste(string,toptick) 0
      foreach {side y} "top $y1 bottom $y0" {
	 set x [expr int($xmin/$gbl(xtic_int) - 1)*$gbl(xtic_int)]
	 while {1} {
	    if { $x > $xmax } { break }
	    # major tic
	    plot_dist_tic $c $x 1 0.5 $y $xmin $xmax $side
	    # sub tics
	    for {set i 0} {$i < $gbl(x_sub_tic)} {incr i} {
	       plot_dist_tic $c\
		   [expr $x+($i+1)*$gbl(xtic_int)/($gbl(x_sub_tic)+1.0)]\
		   0 0.5 $y $xmin $xmax $side
	    }
	    set x [expr $x + $gbl(xtic_int) ]
	 }
      }
      set paste(string,bottomtick) $tmp(bottom)
      set paste(string,toptick) $tmp(top)

      # vertical tick mark and label ----------
      set yrange [expr ($paste(sub,max) - $paste(sub,min)) * 1.0]
      if { $paste(sub,dy) == "" || $paste(sub,dy) < [expr $yrange/20.0] } {
	 set paste(sub,dy) [expr $yrange/4.0]
      }
      foreach {x} "$x0 [expr $x1+$paste(tic_size)]" {
	 set y $paste(sub,min)
	 set i 0
	 while { $y <= $paste(sub,max) } {
	    # plot
	    if { $paste(sub,axis) } {
	       # downward
	       set gy [expr $y1 + \
			   ($y - $paste(sub,min))/$yrange * $paste(sub,height)]
	    } else {
	       # upward
	       set gy [expr $y0 -\
			   ($y - $paste(sub,min))/$yrange * $paste(sub,height)]
	    }
	    $c create line $x $gy [expr $x - $paste(tic_size)] $gy\
				      -width 0.5 -tags frame
	    if { $x <= $x0 } {
	       $c create text [expr $x - $paste(tic_size)] $gy\
		   -text $y -font $paste(fn,tick) -anc e -tag frame
	    }
	    # for next step
	    incr i
	    set y [expr $paste(sub,min) + $i * $paste(sub,dy)]
	 }
      }
      
   }
   # -----------------------------------------------------------------
   # check combination of data format and sub plot settings
   # -----------------------------------------------------------------
   proc sub_plot_check_format {} {
      # global variable
      global paste
      
      # main ------------------------------
      proc warning {} {
	 global gbl
	 if {$gbl(data,type) != 2 && $gbl(data,type) < 5} {
	    tk_messageBox -type ok -icon error -message \
		"Cannot plot sub graph for this data type."
	    return 1
	 }
	 return 0
      }

      switch -- $paste(sub,data) {
	 "swdep" { # water depth
	    set paste(sub,data_format) 0
	 }
	 "tracl" { # shot number
	    set paste(sub,data_format) 1
	 }
	 "tracr" {
	    set paste(sub,data_format) 2
	    return [warning]
	 }
	 "tracf" {
	    set paste(sub,data_format) 3
	    return [warning]
	 }
	 "fldr" {
	    set paste(sub,data_format) 4
	    return [warning]
	 }
	 "ep" {
	    set paste(sub,data_format) 5
	    return [warning]
	 }
	 "delrt" { # delay recording time
	    set paste(sub,data_format) 10
	    return [warning]
	 }
      }
      return 0
   }
   # -----------------------------------------------------------------
   # draw sub plot body
   # -----------------------------------------------------------------
   proc draw_sub_plot {c} {
      # global variable
      global paste
      
      # main ------------------------------
      if { ! $paste(sub,check) } { return }
      
      # coordiantes
      scan [sub_plot_origin] "%g%g%g%g" x0 y0 x1 y1

      # check sub plot data and data format
      if { [sub_plot_check_format] } { return }

      # draw graph
      subplot $paste(sub,data_format) $x0 $y0 $x1 $y1\
	  $paste(sub,min) $paste(sub,max) $paste(sub,height) $paste(sub,axis)
   }

   # --------------------------------------------------------------------
   # Double click button 1
   # --------------------------------------------------------------------
   proc double_click1 {w mx my} {
      global paste
      set mx [$w canvasx $mx]
      set my [$w canvasy $my]
      if ![coord inout2 [coord cx2gx $mx] [coord cy2gt $my]] {
	 # inside record section
	 return
      }
      if { $mx < $paste(left_margin) } {
	 shiftsection -1 0
      }
      if { $mx > [expr $paste(left_margin) + $paste(graph_width)] } {
	 shiftsection 1 0
      }
      if { $my < $paste(top_margin) } {
	 shiftsection 0 -1
      }
      if { $my > [expr $paste(top_margin) + $paste(graph_height)] } {
	 shiftsection 0 1
      }
   }
}
