# --------------------------------------------------------------------
# PasteUP Console 
# --------------------------------------------------------------------
proc pasteup_console {} {
   # Global variable ------------------------------
   global paste gbl wk file_name filt PREF

   # Main --------------------------------------------------

   # Create window -----------------------------------------
   set c $wk(w,console)
   if [raise_window $c] { return }
   toplevel $c
   wm title $c "PasteUp:Console"
   wm resizable $c 1 1
   wm withdraw $c
   wm protocol $c WM_DELETE_WINDOW "quit_application"
   if { $wk(invoke,autoPS) } {
      wm geometry $c +0+0
   }

   # binding ------------------------------
   basic_bindings $c

   # -----------------------------------------------------
   # Menu frame ------------------------------------------
   set mf [menu $c.menu_frame -tearoff 0]
   
   # File menu ----------
   set m [menu $mf.file -tearoff 0]
   $mf add cascade -label "File" -menu $m -underline 0

   # Load/Save resource file
   $m add command -label "Load parameter file..." -command "load_resource" \
       -compound left -image open_icon
   $m add command -label "Save parameter as..." -command "save_resource_as" \
       -compound left -image save_icon

   # Change data set
   $m add separator
   $m add command -label "Select next data set..." \
       -command "change_data_set +1" -accel Ctrl+Meta+Right \
       -compound left -image right_icon
   $m add command -label "Select previous data set..." \
       -command "change_data_set -1" -accel Ctrl+Meta+Left \
       -compound left -image left_icon

   # Print
   $m add separator
   $m add command -label "Print..." -command "print_pasteup" \
       -compound left -image printer_icon -accelerator Meta+p

   # Preferences
   $m add separator
   $m add command -label "PASTEUP Preferences..." -compound left \
       -image config_icon\
       -command "customize_window"

   # Separator
   $m add separator
   # Exit application
   $m add command -label "Quit..." -command "quit_application"\
       -compound left -image exit_icon \
       -accelerator Ctl+q

   # Pick related menu ----------
   set m [menu $mf.pick -tearoff 0]
   $mf add cascade -label "Pick" -menu $m
   $m add command -label "Load pick" -command load_pick
   $m add command -label "Save pick" -command save_pick \
       -accelerator Ctrl+s
   $m add command -label "Save pick as" -command save_as_pick
   $m add separator

   # external program?
   catch {set stat [exec which pick_edit]}
   if { [info vars stat] != "" } {
      $m add command -label "Pick File Editor (External Program)"\
	  -command "exec_external_program pick_edit"
      $m add separator
   }
   $m add command -label "Read synthetic traveltimes" -command Synthetic::load
   $m add command -label "Re-read synthetics" -command Synthetic::reread\
       -accelerator Ctrl+r
   $m add command -label "Toggle showing synthetics"\
       -command Synthetic::toggle -accelerator Ctrl+t

   # Window list menu ----------
   set m [menu $mf.window -tearoff 0]
   $mf add cascade -label "Window" -menu $m -underline 0

   $m add command -label "Pick..." -command "pick_window"\
       -compound left -image pick_icon -accelerator Ctrl+p
   $m add command -label "Layout and Format..." -command "layout_window"\
       -compound left -image layout_icon -accelerator Ctrl+o
   $m add command -label "Analysis..." -command "analysis_window"\
       -compound left -image analysis_icon -accelerator Ctrl+x
   $m add command -label "Print..." -command "print_pasteup"\
       -compound left -image printer_icon -accelerator Meta+p
   $m add command -label "Data set database..." -command "data_set_window"\
       -compound left -image db_icon -accelerator Ctrl+D
   $m add command -label "Header words..." -command "headerwords_window"\
       -accelerator Meta+h
   $m add command -label "PASTEUP Preferences..." -command "customize_window"\
       -compound left -image config_icon -accelerator Meta+z

   # Exec menu ----------
   set m [menu $mf.exec -tearoff 0]
   $mf add cascade -label "Exec" -menu $m -underline 0

   # Draw!
   $m add command -label "Draw Pasteup" -command "draw_pasteup"\
       -accelerator Ctl+l
   # Clear cache
   $m add command -label "Clear data cache" -command {set wk(prev,wave) ""}\
       -accelerator Meta+l
   # Auto range (Horizontal)
   $m add command -label "Set distance-range automatically"\
       -command {auto_offset_range ""} -accelerator Meta+d
   # Auto scale
   $m add command -label "Set scale factor automatically"\
       -command {auto_scale_factor} -accelerator Meta+s
   # Read auxiliary info from SU file
   $m add command -label "Read auxiliary info from SU file"\
       -command {su_read_auxiliary_info} -accelerator Meta+i
   # Set canvas size by t/x-ratio
   $m add command -label "Resize figure size by t/x ratio"\
       -command {resize_canvas_size_by_ratio} -accelerator Meta+r
   # Duplicate current canvas image
   $m add command -label "Duplicate current figure image"\
       -command {duplicate_canvas} -accelerator Meta+c
   # Save SU/SEG-Y files as a new SU file after applying sufilter
   $m add command -label "Save as a new SU file after SUFILTER"\
       -command {SUfilter::save_filtered_su}

   # Convert raw-data into SU
   $m add command -label "Convert raw waveform data into SU" \
       -command {convert_data_into_su}

   # toggle message frame
   $m add command -label "Toggle message frame"\
       -command "toggle_message_window"

   # Help menu ----------
   set m [menu $mf.help -tearoff 0]
   $mf add cascade -label "Help" -menu $m -underline -1

   # Version 

   # Invoke Help window
   $m add command -label "About PasteUp" -command "help_about"
   $m add command -label "Online documents..." -command "help_browser"

   # Menu bar ----------
   $c conf -menu $mf

   # -------------------------------------------------------
   # Toolbar -----------------------------------------------
   set tf [frame $c.toolbar]

   # Open resource button
   button $tf.open -image open_icon -command "load_resource" -bd 1
   bind $tf.open <Enter> {balloon "Load parameters" %X %Y}

   # Save resource as button
   button $tf.saveas -image save_icon -command "save_resource_as" -bd 1
   bind $tf.saveas <Enter> {balloon "Save parameters as" %X %Y}

   # Print button
   button $tf.print -image printer_icon -command "print_pasteup" -bd 1
   bind $tf.print <Enter> {balloon "Print... (Meta-p)" %X %Y}

   # Pick window button
   button $tf.pick -image pick_icon -command "pick_window" -bd 1
   bind $tf.pick <Enter> {balloon "Pick... (Ctrl-p)" %X %Y}

   # Option button
   button $tf.option -image layout_icon -command "layout_window" -bd 1
   bind $tf.option <Enter>\
       {balloon "Layout and Format... (Ctrl-o)" %X %Y}

   # Analysis button
   button $tf.analysis -image analysis_icon -command "analysis_window" -bd 1
   bind $tf.analysis <Enter> {balloon "Analysis... (Ctrl-x)" %X %Y}

   # Database button
   button $tf.db -image db_icon -command "data_set_window" -bd 1
   bind $tf.db <Enter> {balloon "DataBase window... (Ctrl-Shift-d)" %X %Y}

   # Preferences button
   button $tf.pref -image config_icon -command "customize_window" -bd 1
   bind $tf.pref <Enter> {balloon "Preferences window... (Meta-d)" %X %Y}

   # Quit button
   button $tf.quit -image exit_icon -command "quit_application" -bd 1 
   bind $tf.quit <Enter> {balloon "Quit (Ctrl-q)" %X %Y}

   # Draw button
   button $tf.draw -image draw_icon -command "draw_pasteup"  -bd 1
   bind $tf.draw <Enter> {balloon "Draw (Ctrl-l)" %X %Y}

   pack $tf.open $tf.saveas $tf.print $tf.pick $tf.option $tf.analysis $tf.db \
       $tf.pref $tf.quit -side left -in $tf

   pack $tf.draw -side right -in $tf

   # leave bindings for toolbar buttons
   foreach b {open saveas print pick option analysis db quit draw} {
      bind $tf.$b <Leave> {cancel_balloon}
   }

   # -------------------------------------------------------
   # Data parameters ---------------------------------------

   # title frame --------------------
   make_parameter_title_frame $c "Data" data

   frame $c.frame(data)
   console_data_frame 1 1 1 1 1

   # ------------------------------------------------------
   # Display ranges ---------------------------------------
   set df $c.frame(range)
   frame $df
   
   # title frame --------------------
   make_parameter_title_frame $c "Display Range" range

   # Distance range --------------------
   label $df.range_dist(title) -text "Range(X-Axis)     from"
   label $df.range_dist(label1) -text "Km   to "
   label $df.range_dist(label2) -text "Km,     Tic interval"\
       -width 16 -anchor w
   label $df.range_dist(label3) -text "Km."
   entry $df.range_dist(left) -width 7 -textvariable paste(left) -just right
   entry $df.range_dist(right) -width 7 -textvariable paste(right) -jus right
   entry $df.range_dist(xtic) -width 7 -textvariable gbl(xtic_int) -jus right
   button $df.range_dist(auto) -text "auto" -command {auto_offset_range ""}
   set args ""
   foreach {i} {title left label1 right label2 xtic label3 auto} {
      lappend args $df.range_dist($i)
   }
   eval {grid} $args -sticky nes
   bind $df.range_dist(left) <Return> {draw_pasteup}
   bind $df.range_dist(right) <Return> {draw_pasteup}
   bind $df.range_dist(xtic) <Return> {draw_axis $wk(w,current)}

   # Time range --------------------
   label $df.range_time(title) -text "Range(Time)     from"
   label $df.range_time(label1) -text "Sec   to "
   label $df.range_time(label2) -text "Sec,     Tic interval"\
       -width 16 -anchor w
   label $df.range_time(label3) -text "Sec."
   entry $df.range_time(top) -width 7 -textvariable paste(top) -just right
   entry $df.range_time(bot) -width 7 -textvariable paste(bottom) -jus right
   entry $df.range_time(tic) -width 7 -textvariable gbl(t_tic_int) -jus right
   set args ""
   foreach {i} {title top label1 bot label2 tic label3} {
      lappend args $df.range_time($i)
   }
   eval {grid} $args -sticky nes
   bind $df.range_time(top) <Return> {draw_pasteup}
   bind $df.range_time(bot) <Return> {draw_pasteup}
   bind $df.range_time(tic) <Return> {draw_axis $wk(w,current)}
   
   # -------------------------------------------------
   # Plot paramters ----------------------------------
   set pf [frame $c.frame(plot)]

   # title frame --------------------
   make_parameter_title_frame $c "Plot Parameters" plot

   # Plot parameters --------------------
   set pf1 [labelframe $pf.plot -text "Styles" -pady 2]

   # Plot style, format and canvas type
   label $pf1.label(disp_style) -text "style" -anchor e
   set m [tk_optionMenu $pf1.mb(disp_style) gbl(mode_label) ""]
   $m delete 0 end
   foreach {i j k} {\
			"Wiggle" "Wiggle" $wk(DRAW_MODE,WIGGLE_MODE)\
			"VA" "VA" $wk(DRAW_MODE,VA_MODE)\
			"VA(R&B)" "VA" $wk(DRAW_MODE,VA_REDBLUE_MODE)\
			"VA(w/o wiggle)" "VA" $wk(DRAW_MODE,VA_WITHOUT_WIGGLE)\
			"VA(w/o wig RB)" "VA" $wk(DRAW_MODE,VA_WO_WIGGLE_RB)\
			"VD" "VD" $wk(DRAW_MODE,VD)\
			"Wiggle on VD" "Wiggle+VD" $wk(DRAW_MODE,WIGGLE+VD)\
			"VA on VD" "VA+VD" $wk(DRAW_MODE,VA+VD)\
		     } {
      $m add radiobutton -value $i -var gbl(mode_label) -command update_mode
      eval {set img} dsicon($k)
      $m entryconfigure [$m index last] -image $img -compound left -label $i
   }
   label $pf1.label(vdcolor) -text "VD color" -anchor e
   set m [tk_optionMenu $pf1.mb(vdcolor) gbl(vdcolor) ""]
   $m delete 0 end
   $m add checkbutton -var gbl(vdcolor,onlyplus) -label "Apply only plus side"
   $m add separator
   foreach {i j} "\
		      Gray $wk(VD_COLOR,Gray)\
		      French $wk(VD_COLOR,BlueRed1)\
		      Blue-Red $wk(VD_COLOR,BlueRed2)\
                      Romanian $wk(VD_COLOR,BlueRed3)\
                      Temperature $wk(VD_COLOR,BlueWhite)\
		      Black-Red $wk(VD_COLOR,BlackRed)\
                      German $wk(VD_COLOR,BlackYellow)\
                      Ocean $wk(VD_COLOR,BlackWhite)\
		      Rainbow $wk(VD_COLOR,Rainbow)\
		      TwoTone $wk(VD_COLOR,TwoTone)\
		   " {
      $m add radiobutton -value $j -var gbl(vdcolor) -command update_vdcolor
      eval {set img} dsicon($j)
      $m entryconfigure [$m index last] -image $img -compound left -label $i
      if { $gbl(vdcolor) == $j } {
	 update_vdcolor
      }
   }

   label $pf1.label(mode) -text "mode" -anchor e
   pasteup_optionMenu_com $pf1.mb(mode) paste(image,mode,label) \
       update_image_mode \
       Vector Image Interpolate
   $pf1.mb(mode) conf -width 15
   button $pf1.conf(mode) -text "conf..." \
       -command "interpolate_configure_window"
   
   label $pf1.label(canvas) -text "canvas" -anchor e
   pasteup_optionMenu_com $pf1.mb(canvas) paste(canvas,label) \
       update_canvas_mode \
       Normal Graph

   grid $pf1.label(disp_style) $pf1.mb(disp_style) x -sticky news
   grid $pf1.label(vdcolor) $pf1.mb(vdcolor) x -sticky news
   grid $pf1.label(mode) $pf1.mb(mode) $pf1.conf(mode) -sticky news
   grid $pf1.label(canvas) $pf1.mb(canvas) x -sticky news
   update_displaystyle_menu

   # Plot parameters --------------------
   set pf2 [labelframe $pf.miscplot -text "Misc parameters" -pady 2]
   set wk(w,console,misc) $pf2

   # Reduction Velocity
   make_entry_right $pf2 red_vel "Reduction Velocity" paste(reduction)\
       "Km/sec."
   bind $pf2.entry(red_vel) <Return> {draw_pasteup}

   # Line width
   make_entry_right $pf2 line_width "Line width" gbl(line_width) ""
   bind $pf2.entry(line_width) <Return> {draw_pasteup}

   # Cutoff (Clipping) amplitude
   make_entry_right $pf2 amp_saturate "Clip amplitude"\
       paste(amp,cutoff) "pixel"
   bind $pf2.entry(amp_saturate) <Return> {draw_pasteup}

   # VA threshold
   make_entry_right $pf2 va_threshold "VA threshold"\
       paste(amp,threshold) "pixel"
   bind $pf2.entry(va_threshold) <Return> {draw_pasteup}

   # Thinning out --------------------
   set pf3 [labelframe $pf.decim -text "Thinning out (Decimation)" -pady 2]
   set wk(w,console,decim) $pf3

   # Plot stype and format
   make_spinbox $pf3 every "plot every" paste(wave_density)\
       1 50 1 "traces"
   bind $pf3.spinbox(every) <Return> {draw_pasteup}

   # Thinning out by distance
   make_entry_right $pf3 spacing "trace spacing"\
       paste(wave_distance) "km"
   bind $pf3.entry(spacing) <Return> {draw_pasteup}

   grid $pf1 $pf2 -sticky news -padx 3
   grid ^ $pf3 -sticky news -padx 3 -ipady 3

   # ---------------------------------------------------------
   # Filter paramter -----------------------------------------
   set ampfilt [frame $c.frame(filter)]

   # title frame --------------------
   make_parameter_title_frame $c "Amplitude and Filtering" filter

   # Amplitude parameters --------------------
   set af [labelframe $ampfilt.amp -text "Amplitude configurations" -pady 2]
   set wk(w,console,amp) $af

   # Amplitude correction
   checkbutton $af.check -var gbl(check,amp) -command update_amp_correct
   label $af.amp_label -text "Correction"
   set mw [tk_optionMenu $af.amp_mode paste(amp,correct_label)\
	       Geometric Relative AGC]
   $mw entryconf 0 -command "update_amp_correct"
   $mw entryconf 1 -command "update_amp_correct"
   $mw entryconf 2 -command "update_amp_correct"
   button $af.geo_conf -text "conf..." -command "geometric_configure_window"
   grid $af.check $af.amp_label $af.amp_mode - $af.geo_conf -sticky news
   grid conf $af.amp_label -sticky nws

   # Scale factor
   label $af.l1(scale) -text "Scale factor" -anchor e
   entry $af.e(scale) -width 12 -textvar paste(amp,amplify) -justify right
   label $af.l2(scale) -text "times" -anchor w
   button $af.b(scale) -text "auto" -command "auto_scale_factor"
   grid x $af.l1(scale) $af.e(scale) $af.l2(scale) $af.b(scale) -sticky news
   bind $af.e(scale) <Return> {draw_pasteup}

   # AGC
   make_entry_right_ch $af agc "AGC period" paste(amp,agc) "sec"
   bind $af.entry(agc) <Return> {draw_pasteup}

   # wiggle/vd ratio
   make_entry_right_ch $af ratio "Amp ratio" paste(amp,ratio) "(Wiggle/VD)"
   bind $af.entry(ratio) <Return> {draw_pasteup}

   # Filtering parameters --------------------
   set ff [labelframe $ampfilt.filter -text "Filtering" -pady 2]

   # SU filter
   make_checkbutton_com $ff "SU filter" sufilter gbl(check,sufilter)\
       "conf..." SUfilter::window
   enabling_su_filter_frame

   # Time domain filter
   set f [frame $ff.time(check)]
   checkbutton $f.check -var gbl(check,filter)\
       -command "widgets_update gbl(check,filter) $ff.time(button)"
   eval pasteup_optionMenu_com $f.menu filt(type_label)\
       update_filter $wk(filter,list)
   $f.menu conf -width 12 -anchor w
   bind $f.menu <Button-4> {filter_change prev}
   bind $f.menu <Button-5> {filter_change next}
   grid $f.check $f.menu -sticky nsw
   grid conf $f.check -sticky nse
   button $ff.time(button) -text "conf..." -command "filter_window"
   grid $ff.time(check) $ff.time(button) -sticky news

   # Bias removal
   make_checkbutton $ff "Bias removal" bias filt(offset)
   
   # Stack
   make_checkbutton_com $ff "Stack" stack gbl(check,stack)\
       "conf..." stack_parameter

   # update checkbutton status
   widgets_update gbl(check,filter) $ff.time(button)

   grid $af $ff -sticky news -padx 3

   # -----------------------------------------------
   # packing ---------------------------------------
   pack $tf -side top -fill x
   update_console_window
}

# --------------------------------------------------------------------
# make_parameter_title_frame procedure
#        Make title frames in the console window
# --------------------------------------------------------------------
# [arguments]
# w     : parent widget
# str   : title name for display
# name  : title name (internal name)
# --------------------------------------------------------------------
proc make_parameter_title_frame {w str name} {
   # global variable ------------------------------
   global gbl wk custom
   
   # main ------------------------------
   frame $w.title($name)
   label $w.title($name).l -text $str -relief flat -width $wk(title_width)\
       -font $custom(ConsoleTitleFont)\
       -bg $custom(ConsoleTitleBackground)\
       -fg $custom(ConsoleTitleForeground)\
       -bd 1 -relief groove
   button $w.title($name).b -image close_icon\
       -relief flat -bg $custom(ConsoleTitleBackground) -fg black -bd 0\
       -activeback $custom(ConsoleTitleBackground) -width 20\
       -command "toggle_console $name"
   pack $w.title($name).l -side left -fill both -expand yes
   pack $w.title($name).b -side right -fill y
}
# --------------------------------------------------------------------
# toggle_console 
#        toggle show/show-off frames in the console window
# --------------------------------------------------------------------
# [argument]
# name  : frame name
# --------------------------------------------------------------------
proc toggle_console {name} {
    # global variable ------------------------------
    global gbl wk
    
    # main ------------------------------
    if { $gbl(console,$name) == 1} {
	set gbl(console,$name) 0
	pack forget $wk(w,console).frame($name)
 	$wk(w,console).title($name).b conf -image down_arrow
    } else {
	set gbl(console,$name) 1
 	$wk(w,console).title($name).b conf -image close_icon
	update_console_window
    }
}
# --------------------------------------------------------------------
# console_data_frame
#        Create data frame
# --------------------------------------------------------------------
# wf     :  Show Wave Form ?       (1:on, 0:off)
# rh     :  Show Record Header ?   (1:on, 0:off)
# xx     :  Show x-axis component? (1:on, 0:off)
# ch     :  Show Channel number?   (1:on, 0:off)
# ll     :  Show OBS position?     (1:on, 0:off)
# --------------------------------------------------------------------
proc console_data_frame {wf rh xx ch ll} {
   # global variable ------------------------------
   global wk gbl

   # main ------------------------------
   set f $wk(w,console).frame(data)

   # kill children
   foreach cw [winfo children $f] {
      destroy $cw
   }

   # Data Type (SU, SEGY, ...) --------------------
   label $f.label(datatype) -text "Data Type" -anchor e
   eval pasteup_optionMenu_com $f.mb(datatype) gbl(data,type,label)\
       update_data_type $wk(datatype,list)
   set wk(data,byte,widget)\
       [tk_optionMenu $f.mb(databyte) wk(data,byte,label) "-"]
   # auto button
   button $f.auto -text "Auto" -command "auto_type_info \"\""

   grid $f.label(datatype) $f.mb(datatype) $f.mb(databyte) $f.auto\
       -sticky news
   
   # Filename entry and "ref" buttons --------------------
   if {$wf} {
      filename_hscroll_frame $f wave "Waveform file" file_name(wave) 55 "load"\
	  wave
      bind $f.entry(wave) <Return> {draw_pasteup}
      bind $f.entry(wave) <Button-3> {change_wavefile_associate +1}
      bind $f.entry(wave) <Control-Button-3> {change_wavefile_associate -1}
      bind $f.entry(wave) <Button-4> {change_data_set -1}
      bind $f.entry(wave) <Button-5> {change_data_set +1}
      trace_dsmode
   }

   if {$rh} {
      filename_hscroll_frame $f rh "Record Header file" file_name(rh)\
	  40 "load" rh
      bind $f.entry(rh) <Return> {draw_pasteup}
   }
   # x-axis component --------------------
   if {$xx} {
      label $f.label(xaxis) -text "X-axis" -anchor e
      eval pasteup_optionMenu_com $f.mb(xaxis) gbl(data,xaxis_label)\
	  update_xaxis offset sx sy gx gy fldr tracf tracl CDP
      checkbutton $f.check(xaxis) -text "reverse" -anchor w \
	  -variable gbl(data,xaxis_direction) -onvalue 1 -offvalue 0
      grid $f.label(xaxis) $f.mb(xaxis) $f.check(xaxis) \
	  -sticky news
   }

   # OBS position --------------------
   if {$ch} {
      make_optionMenu_button $f channel "Channel" gbl(obs,channel) 1 2 3 4 5 6
   }
   if {$ll} {
      label $f.label(lon) -text "OBS   Longitude:" -anchor e
      entry $f.obs(lon) -width 12 -textvar gbl(obs,lon) -justify right
      label $f.unit(lon) -text "degrees" -anchor w
      label $f.label(lat) -text "Latitude:" -anchor e
      entry $f.obs(lat) -width 12 -textvar gbl(obs,lat) -justify right
      label $f.unit(lat) -text "degrees" -anchor w
      grid $f.label(lon) $f.obs(lon) $f.unit(lon)\
	  $f.label(lat) $f.obs(lat) $f.unit(lat) -sticky news
      make_many_radio $f "Distance axis" dist_axis \
	  gbl(obs,direc) "East positive" 1  "North positive" 2
   }
}
