# --------------------------------------------------------------------
# apply theme
# --------------------------------------------------------------------
proc customize_switch_theme {args} {
   # global variables
   global custom wk DATADIR PREF

   # main --------------------------------------------------
   if { [llength $args] > 0 } {
      set theme $args
   } else {
      set theme $PREF(theme)
   }
   if { $theme == "custom" } {return}
   # theme file
   set fn [format "%s/theme/%s.tcl" $DATADIR $theme]
   if { ! [file readable $fn] } {
      tk_messageBox -message "Theme file \"$PREF(theme)\" is not found."\
		   -type ok -icon error
      return
   }
   # read default
   if { $theme != "default" } {
      source [format "%s/theme/default.tcl" $DATADIR]
   }
   # read theme
   source $fn

   # warning
   if { [llength $args] == 0 } {
      tk_messageBox -type ok -icon info\
	  -message "Theme will be applied after restarting Pasteup."
   }
}

# --------------------------------------------------------------------
# sub-procedure for sorting arrays by the string length 
# --------------------------------------------------------------------
proc customize_compare {a b} {
   if { [string length $a] < [string length $b] } {
      return -1
   } else {
      return 1
   }
}

# --------------------------------------------------------------------
# apply customize
# --------------------------------------------------------------------
proc customize {} {
   # global variables
   global custom

   # main --------------------------------------------------
   set keys [lsort -command customize_compare [array names custom \\**] ]
   option clear
   # add all defined.
   foreach key $keys {
      if {$custom($key) != ""} {
	 option add $key $custom($key)
      }
   }
   # additional options
   option add *Entry*selectForeground  $custom(*selectForeground)
   option add *disabledBackground      $custom(*Background)
   option add *readonlyBackground      $custom(*Background)
   option add *Tabset.Tab.selectBackground    $custom(*Background)
   option add *Tabset.Tab.selectForeground    $custom(*Foreground)
   option add *Button.activeForeground $custom(*activeForeground)
   option add *Spinbox.Background      $custom(*Entry.Background)
   option add *Listbox.Background      $custom(*Entry.Background)
   option add *Listbox*selectForeground $custom(*Foreground)
   # PATHS
   update_PATH
}

# --------------------------------------------------------------------
# Color chooser
# --------------------------------------------------------------------
proc customize_color_chooser {w colname i} {
   # global variables
   global custom PREF

   # main --------------------------------------------------
   # choose color
   set new [tk_chooseColor -initialcolor $custom($colname) -title $colname]
   if {$new != ""} {
      set PREF(theme) "custom"
      set custom($colname) $new
   }
   # update customize window
   $w.sample($i) conf -bg $custom($colname)
}

proc customize_font_chooser_determine {ff w} {
   global wk
   set wk(newfont) "[customize_make_selected_fontname $ff]"
   destroy $w
}

proc customize_make_selected_fontname {ff} {
   # get font name
   set family [$ff.family get [$ff.family cursel]]
   set style [$ff.style get [$ff.style cursel]]
   set size [$ff.size get [$ff.size cursel]]
   return [format "{%s} %s %s" $family $size $style]
}

proc customize_update_font_sample {ff} {
   # update
   $ff.sample conf -font [customize_make_selected_fontname $ff]
}

proc customize_font_chooser_window {initfont} {
   # global variables
   global custom wk

   # main --------------------------------------------------
   set w .font_chooser
   toplevel $w
   
   # title frame --------------------
   set tf $w.title_frame
   frame $tf

   # font chooser --------------------
   set ff $w.font_frame
   frame $ff
   
   # legend
   label $ff.lfamily -text "Family" -relief ridge -bd 2
   label $ff.lstyle -text "Style" -relief ridge -bd 2
   label $ff.lsize -text "Size" -relief ridge -bd 2
   grid $ff.lfamily - $ff.lstyle $ff.lsize - -sticky news
   # body
   listbox $ff.family -height 6 -yscrollcom "$ff.fscroll set"\
       -export false -width 20
   scrollbar $ff.fscroll -width 10 -command "$ff.family yview"
   listbox $ff.style -height 6 -width 8 -export false
   listbox $ff.size -height 6 -width 3 -yscrollcom "$ff.sscroll set" -export 0
   scrollbar $ff.sscroll -width 10 -command "$ff.size yview"

   grid $ff.family $ff.fscroll $ff.style $ff.size $ff.sscroll -sticky news
   # sample
   label $ff.sample -text "ABCDEFabcdef"
   grid $ff.sample - - - - -sticky news
   
   # bindings
   foreach widget {family style size} {
      bind $ff.$widget <<ListboxSelect>> "
	 update_listbox_selection $ff.$widget
	 customize_update_font_sample $ff
      "
   }
   # make lists
   set families [lsort -ascii [font families]]
   foreach fn $families {
      $ff.family insert end $fn
   }
   set styles {roman bold italic}
   foreach style $styles {
      $ff.style insert end $style
   }
   set sizes {}
   for {set i 5} {$i < 33} {incr i} {
      lappend sizes $i
   }
   foreach size $sizes {
      $ff.size insert end $size
   }

   # init values
   set initfont [string tolower $initfont]
   if {$initfont != "" && [llength $initfont] == 3} {
      $ff.family selection set [lsearch $families [lindex $initfont 0]]
      $ff.style selection set [lsearch $styles [lindex $initfont end]]
      $ff.size selection set [lsearch $sizes [lindex $initfont 1]]
   } else {
      $ff.family selection set 0
      $ff.style selection set 0
      $ff.size selection set 5
   }

   # see selection
   foreach widget {family style size} { 
      set selindex [$ff.$widget cursel]
      if { $selindex != "" } {
	 $ff.$widget see $selindex
      }
   }
   customize_update_font_sample $ff

   # buttons --------------------
   set bf $w.buttonframe
   frame $bf
   set wk(newfont) ""
   button $bf.ok -text "OK" -width 8 -command\
       "customize_font_chooser_determine $ff $w"
   button $bf.cancel -text "cancel" -width 8 -command "destroy $w"
   pack $bf.ok $bf.cancel -side left

   # packing --------------------
   pack $tf $ff $bf -side top -fill both -expand 1
   grab $w
   tkwait window $w
   return $wk(newfont)
}
# --------------------------------------------------------------------
# Font chooser
# --------------------------------------------------------------------
proc customize_font_chooser {w fontname i} {
   # global variables
   global custom

   # main --------------------------------------------------
   # select
   set new [customize_font_chooser_window $custom($fontname)]
   if {$new != ""} {
      set custom($fontname) $new
   }
   # update customize window
   if { $custom($fontname) != "" } {
      $w.sample($i) conf -font $custom($fontname)
   }
}

# --------------------------------------------------------------------
# customize_window ---------------------------------------------------
#    invoke preferences window
# --------------------------------------------------------------------
proc customize_window {} {
   # global variables
   global custom wk PREF env DATADIR

   # main --------------------------------------------------
   set cw .customize_window
   set geometry ""
   if { [winfo exist $cw] } {
      set geometry [wm geometry $cw]
      destroy $cw
   }
   toplevel $cw
   if {$geometry != ""} {
      wm geometry $cw\
	  [format "+%s" [join [lrange [split $geometry "+x"] 2 3] "+"]]
   }
   wm title $cw "PasteUp: Preferences Window"
   
   # title frame --------------------
   set tf [title_frame $cw.title_frame "Pasteup: Preferences"]

   # tabset ------------------------------
   set ts [tabset $cw.tabset -pagewidth 520 -relief flat -samewidth false]
   $ts conf -borderwidth 1 -highlightthickness 0 -side top
   $ts insert end "plot" -text "plot"
   $ts insert end "dataset" -text "data set"
   $ts insert end "external" -text "paths of ext."
   $ts insert end "lookfeel" -text "look & feel"
   $ts insert end "env" -text "environment"

   # tabset balloon help
   foreach {tab mes} {"plot" "parameters for reading data and plotting"\
			  dataset "configurations for automatic data-set change"
			  external "PATH setting for external programs"\
			  lookfeel "customize look & feel"\
			  env "system environment"
		       } {
      $ts bind $tab <Enter> "balloon \"$mes\" %X %Y"
      $ts bind $tab <Leave> {cancel_balloon}
   }

   set keys [lsort -command customize_compare\
		 [lsort -dict [array names custom] ] ]

   # --------------------------------------------------
   # plot tab -----------------------------------------
   # --------------------------------------------------

   set pf [frame $ts.plot]
   $ts tab conf "plot" -window $pf -fill both

   set def [labelframe $pf.default -pady 5 -text "Default drawing style"]
   make_menu_com $def va "VA and Wiggle" gbl(option,default_style_va) ""\
       Vector 0 Image 1
   make_menu_com $def vd "VD" gbl(option,default_style_vd) ""\
       Vector 0 Image 1 Interpolate 2

   set fmt [labelframe $pf.fmt -pady 5 -text "SU/SEG-Y header"]
   make_menu_com $fmt sht "Shot number" gbl(option,su,shotnumber) ""\
       tracl 0 tracr 1 fldr 2 tracf 3
   make_menu_com $fmt site "Site(OBS) number" gbl(option,su,sitenumber) ""\
       fldr 2 ep 4

   set cache [labelframe $pf.cache -pady 5 -text "Data cache"]
   make_spinbox $cache len "extra read length" gbl(option,cache_length)\
       0 5 1 "sec." 
   define_balloon $cache.spinbox(len)\
       "If you set cache length as t,\nPasteUp reads data from t1-t to t2+t\nfor plottin data from t1 to t2."

   set misc [labelframe $pf.misc -pady 5 -text "Miscellaneous parameters"]
   make_menu_com $misc scale "Criteria for auto-scaling factor"\
       gbl(option,scaling_factor) "" "1%" 1 "2%" 2 "5%" 5 "10%" 10 "20%" 20\
       "40%" 40 "60%" 60 "80%" 80 "100%" 100

   pack $def $fmt $misc $cache -side top -fill x -padx 10

   # --------------------------------------------------
   # data set tab -------------------------------------
   # --------------------------------------------------

   set df [frame $ts.ds]
   $ts tab conf "dataset" -window $df -fill both
   
   # naming strategy
   set name [labelframe $df.naming -text "Directory structure for waveform and pick files"]
   make_radio_vertical $name name gbl(option,dir_structure)\
       "Two directories (a waveform directory and a pick directory)\nAll waveform files are in the waveform directory and\nall pick files are in the pick directory." two\
       "Site directories (each site has an own directory)\nwaveform files and pick files of each site is located in their directory." many

   # channel number
   set channel [labelframe $df.ch -text "Channel number"]
   make_checkbutton $channel\
       "Waveform and pick filenames contain channel number." ch\
       gbl(option,dataset_channel)
   
   # redraw
   set after [labelframe $df.after -text "After changing data set"]
   make_checkbutton $after\
       "Draw automatically after changing data set" redraw\
       gbl(option,redraw_afterchange)
   make_checkbutton $after\
       "Read header info after changing data set" read\
       gbl(option,readinfo_afterchange)

   pack $name $channel $after -side top -fill x -ipady 10 -pady 5
   
   # --------------------------------------------------
   # external tab -------------------------------------
   # --------------------------------------------------

   set cf [frame $ts.ext]
   $ts tab conf "external" -window $cf -fill both

   set tmmray [labelframe $cf.dir \
		 -text "Installation directories of mapping and ray-tracing"]
   filename_hscroll_frame $tmmray tmm "TravelTime Mapping" \
       PREF(com,tmmpath) 40 dir dir
   filename_hscroll_frame $tmmray hamen "Ray-Tracing" \
       PREF(com,hamenpath) 40 dir dir

   set com [labelframe $cf.com -text "Misc. commands"]
   filename_hscroll_frame $com view "viewgrd" PREF(com,viewgrd) 45 load ""
   filename_hscroll_frame $com conv "convert" PREF(com,convert) 45 load ""
   filename_hscroll_frame $com 2cdf "model2cdf" PREF(com,model2cdf) 45 load ""

   set su [labelframe $cf.su \
		 -text "PATH for external commands, if necessary (like SU)"]
   filename_hscroll_frame $su su1 "PATH1" \
       PREF(com,supath1) 45 dir dir
   filename_hscroll_frame $su su2 "PATH2" \
       PREF(com,supath2) 45 dir dir

   pack $tmmray $com $su -side top -expand 1 -fill x -padx 10 -ipady 5

   # --------------------------------------------------
   # look and feel tab --------------------------------
   # --------------------------------------------------

   set lf [frame $ts.lookfeel]
   $ts tab conf "lookfeel" -window $lf -fill both
   
   set lfts [tabset $lf.tabset -relief flat -side left -rotate 90]
   $lfts conf -borderwidth 1 -highlightthickness 0 -side left -rotate 90
   $lfts insert end "theme" -text "theme"
   $lfts insert end "misc" -text "misc."
   $lfts insert end "fnkeys" -text "Fn-keys"
   
   foreach {tab mes} {theme "Colors, fonts, widgets properties" \
			  misc "Miscellaneous settings." \
 			  fnkeys "Function-key binding" \
		       } {
      $lfts bind $tab <Enter> "balloon \"$mes\" %X %Y"
      $lfts bind $tab <Leave> {cancel_balloon}
   }

   # ----------------------------------------
   # color/font/widget tab 

   set tabframe [labelframe $lfts.theme -text "Colors, fonts, widgets" -pady 10]
   $lfts tab conf "theme" -window $tabframe -fill both

   # themes --------------------
   set cf [labelframe $tabframe.colors -text "Colors"]
   if { $PREF(theme) == "custom" } {
      set themelist {custom default}
   } else {
      set themelist {default}
   }
   foreach theme [glob -dir $DATADIR/theme *.tcl] {
      set name [file rootname [file tail $theme]]
      if [regexp "^\[A-Z\]" $name] {
	 lappend themelist $name
      }
   }
   label $cf.label(theme) -text "Color theme" -anchor e
   eval pasteup_optionMenu_com $cf.mb(theme) PREF(theme) \
       customize_switch_theme $themelist
   button $cf.b(theme) -text "edit" -command "customize_color_edit"
   grid $cf.label(theme) $cf.mb(theme) $cf.b(theme) - - -sticky news 
	  
   # font --------------------
   set ff [labelframe $tabframe.font -text "Fonts in the PasteUp window"]
   set fonts [lsearch -all -inline -reg $keys "^\\*\[A-z.\]*\[fF\]ont"]
   set fonts [concat $fonts [lsearch -all -inline -reg $keys\
				 "^\[A-z.\]*\[fF\]ont"]]
   set i 0
   foreach fontname $fonts {
      label $ff.label($i) -text $fontname -anchor w
      if {$custom($fontname) != ""} {
	 label $ff.sample($i) -text "ABCDEFabcdef" -font $custom($fontname)
      } else {
	 label $ff.sample($i) -text "ABCDEFabcdef"
      }
      button $ff.button($i) -text "set"\
	  -command "customize_font_chooser $ff $fontname $i"
      grid $ff.label($i) $ff.sample($i) $ff.button($i) -sticky news
      incr i
   }
   button $ff.reset -text "reset font" -command\
       "customize_reset_fonts; customize; customize_window"
   grid x $ff.reset x -sticky news

   # widgets properties --------------------
   set wf [labelframe $tabframe.widgets -text "widget design"]
   set items [lsearch -all -inline -reg $keys "^\\*\[A-z.\]*Width"]
   set i 0
   foreach item $items {
      label $wf.label($i) -text $item
      entry $wf.entry($i) -width 8 -justify right -textvar custom($item)
      grid $wf.label($i) $wf.entry($i) -sticky news
      incr i
   }

   pack $cf $ff $wf -side top -expand 1 -fill x -padx 10
   pack $lfts -side top -fill both -expand 1
   
   # ----------------------------------------
   # bind tab

   set mbf [frame $lfts.bind]
   $lfts tab conf "fnkeys" -window $mbf -fill both

   set funclist {}
   foreach {name func} $wk(customize,bind,funclist) {
      lappend funclist $name
   }
   for {set i 1} {$i <= 12} {incr i} {
      set key [format "F%d" $i]
      label $mbf.label($i) -text $key
      eval {tk_optionMenu $mbf.menu($i) custom(function,$key)} $funclist
      grid $mbf.label($i) $mbf.menu($i) -sticky news
   }

   # ----------------------------------------
   # misc tab

   set mf [frame $lfts.misc]
   $lfts tab conf "misc" -window $mf -fill x -anchor n
   
   # balloon help
   make_menu_com $mf balloon "Mouse position balloon" gbl(option,showposition)\
       bind_show_position \
       "Show Always" 3  "Stay long (on demand)" 2 "On demand" 1  "Off" 0

   # image format
   make_menu_com $mf img "Image format" paste(image,format) "" \
       "PNG" 1 "JPEG" -1

   # colors
   make_optionMenu_button $mf cols "Number of colors" paste(ncolors) \
       256 64 16 4 3

   pack $lfts -side top -fill both -expand 1

   # --------------------------------------------------
   # environment tab ----------------------------------
   # --------------------------------------------------

   set ef [frame $ts.env]
   $ts tab conf "env" -window $ef -fill both

   set html [labelframe $ef.html -text "HTML Browser for viewing documents"\
		 -pady 5]
   eval make_optionMenu_button $html browser "Browser" gbl(help,browser) \
       $wk(help,browser_list)

   set print [labelframe $ef.print -text "Printing System" -pady 5]
   make_optionMenu_button $print print "System" PREF(print,system) \
       CUPS LPD

   set vb [labelframe $ef.verbose -text "Verbosity of the backend programs"]
   make_menu_com $vb vl "level" gbl(option,verbose) ""\
       Silent 0 Slightly 1 Appropriately 2 "Most Verbose" 3

   set ev [labelframe $ef.env -text\
	       "Environment variables for external programs"]
   make_spinbox_com $ev omp "OMP_NUM_THREADS" PREF(env,OMP_NUM_THREADS) \
       1 16 1 "" "update_env_variable 0"
   filename_hscroll_frame $ev cwproot "CWPROOT" PREF(env,CWPROOT) 35 dir dir
   

   pack $html $print $vb $ev -side top -fill x -padx 10 -ipady 5

   # Button frame ----------------------------------------
   set bf [button_frame $cw $cw.button_frame "apply" \
	       "customize; customize_function_key; customize_window"\
	       "reset" customize_reset_all]

   # Packing ----------
   pack $tf $ts $bf -side top -fill x -expand 1

   # binding
   basic_bindings $cw
}

# --------------------------------------------------------------------
# color edit window
# --------------------------------------------------------------------
proc customize_color_edit {} {
   # global variables
   global custom

   # main --------------------------------------------------
   set cw .customize_color_edit
   set geometry ""
   if { [winfo exist $cw] } {
      set geometry [wm geometry $cw]
      destroy $cw
   }
   toplevel $cw
   wm title $cw "PasteUp: Preferences : Color Edit"
   
   # title frame --------------------
   set tf $cw.title_frame
   label $tf -text "PasteUp: Preferences : Color Edit" -relief groove -bd 2\
       -font $custom(WindowTitleFont)

   # parameter frame --------------------
   set cf [labelframe $cw.color_frame -text "Custom colors" -padx 30 -pady 10]

   # list colors
   set keys [lsort -command customize_compare\
                 [lsort -dict [array names custom] ] ]
   set colors [lsearch -all -inline -reg $keys\
		   "\\*\[A-z.\]*Background|\\*\[A-z.\]*Foreground|\\*\[A-z.\]*Color"]
   set colors [concat $colors [lsearch -all -inline -reg $keys\
				   "^\[A-z\]+Background|^\[A-z\]+Foreground"]]
   set i 0
   foreach colname $colors {
      label $cf.label($i) -text $colname -anchor w
      label $cf.sample($i) -width 10 -bg $custom($colname)
      button $cf.button($i) -text "set" -command\
	  "customize_color_chooser $cf $colname $i"
      grid $cf.label($i) $cf.sample($i) $cf.button($i) -sticky news
      incr i
   }

   # button frame --------------------
   set bf [button_frame $cw $cw.button_frame]

   pack $tf -side top -fill x -expand 1
   pack $cf $bf -side top -fill x -expand 1 -padx 5
   grab $cw
}

# --------------------------------------------------------------------
# Save theme parameters to a file
# --------------------------------------------------------------------
proc customize_save_theme {fn} {
   # gloabl --------------------
   global wk custom
   
   # main --------------------
   if { [file isdir $wk(resource_dir)] == 0} {
      file mkdir $wk(resource_dir)
   }

   # make all lists
   set list ""
   foreach member [array name custom] {
      if { [string match function* $member] == 0 } {
	 lappend list [format "%s(%s)" custom $member]
      }
   }

   # Output to the file ------------------------------
   set fp [open $fn w]
   set t [clock format [clock second] -format "%Y/%m/%d %H:%M:%S"]
   puts $fp "\# This is a theme file for pasteup."
   puts $fp "\#         Generated at $t"
   foreach i $list {
      eval set val [list [set $i]]
      puts $fp [format "set %s \"%s\"" $i $val]
   }
   close $fp
}

# --------------------------------------------------------------------
# Save preferences
# --------------------------------------------------------------------
proc customize_save_preference {fn} {
   # gloabl --------------------
   global wk PREF
   
   # main --------------------
   if { [file isdir $wk(resource_dir)] == 0} {
      file mkdir $wk(resource_dir)
   }

   # make all lists
   set list ""
   foreach member [array name PREF] {
      lappend list [format "PREF(%s)" $member]
   }

   # Output to the file ------------------------------
   set fp [open $fn w]
   set t [clock format [clock second] -format "%Y/%m/%d %H:%M:%S"]
   puts $fp "\# This is a theme file for pasteup."
   puts $fp "\#         Generated at $t"
   foreach i $list {
      eval set val [list [set $i]]
      puts $fp [format "set %s \"%s\"" $i $val]
   }
   close $fp
}

# --------------------------------------------------------------------
# Customize function key (set bind)
# --------------------------------------------------------------------
proc customize_bind_resolve_funcname {funcname} {
   # gloabl --------------------
   global wk custom
   
   # main --------------------
   if { $funcname == "none" } { return "" }
   foreach {name func} $wk(customize,bind,funclist) {
      if { $name == $funcname } {
	 return $func
      }
   }
   return ""
}
# --------------------------------------------------------------------
# Customize function key (set bind)
# --------------------------------------------------------------------
proc customize_function_key {} {
   # gloabl --------------------
   global wk custom
   
   # main --------------------
   for {set i 1} {$i <= 12} {incr i} {
      set key [format "F%d" $i]
      set funcname [customize_bind_resolve_funcname $custom(function,$key)]
      bindfe $wk(w,canvas,c) KeyPress $key $funcname
      bindfe $wk(w,canvas,g) KeyPress $key $funcname
      bindfe $wk(w,console) KeyPress $key $funcname
   }
}

# --------------------------------------------------------------------
# Save binding info to a file
# --------------------------------------------------------------------
proc customize_save_bind {fn} {
   # gloabl --------------------
   global wk custom
   
   # main --------------------
   if { [file isdir $wk(resource_dir)] == 0} {
      file mkdir $wk(resource_dir)
   }

   # make all lists
   set list ""
   foreach member [array name custom] {
      if { [string match function* $member] } {
	 lappend list [format "%s(%s)" custom $member]
      }
   }

   # Output to the file ------------------------------
   set fp [open $fn w]
   set t [clock format [clock second] -format "%Y/%m/%d %H:%M:%S"]
   puts $fp "\# This is a binding file for pasteup."
   puts $fp "\#         Generated at $t"
   foreach i $list {
      eval set val [list [set $i]]
      puts $fp [format "set %s \"%s\"" $i $val]
   }
   close $fp
}

# --------------------------------------------------------------------
# Reset all the customize parameters to the default
# --------------------------------------------------------------------
proc customize_reset_all {} {
   # gloabl --------------------
   global DATADIR PREF
   
   # main --------------------
   set ans [tk_messageBox -default no -type yesno -title "Reset parameters?"\
		-icon question -message "Reset all the preferences paraemters except for Fn-binding?"]
   if { $ans == "no" } { return }
   
   # reset PREF to the "pasteup.tcl"
   set fn "$DATADIR/pasteup"
   set fp [open $fn]
   while { [gets $fp line] >= 0} {
      if [ string match "set PREF*" $line] {
	 eval $line
      }
   }
   # reset theme
   customize_switch_theme
   # reset font
   customize_reset_fonts
}

# --------------------------------------------------------------------
# Reset fonts
# --------------------------------------------------------------------
proc customize_reset_fonts {} {
   # gloabl --------------------
   global custom wk

   # main --------------------
   set custom(*font)                       ""
   set custom(*Dialog.msg.font)            "Times 14 roman"
   set custom(ConsoleTitleFont)            "$wk(font,title) 14 bold"
   set custom(WindowTitleFont)             "$wk(font,title) 16 bold"
}