# --------------------------------------------------------------------
# Invoke data set window
# --------------------------------------------------------------------
proc data_set_window {} {
   # Global variable
   global gbl wk
   
   # Main --------------------
   set dw .ds_window
   if [raise_window $dw] { return }
   toplevel $dw
   wm title $dw "PasteUp: Data set DataBase"
   
   # menu frame --------------------
   set mf [menu $dw.menu_frame -tearoff 0]
   
   # File menu
   set m [menu $mf.file -tearoff 0]
   $mf add cascade -label "File" -menu $m -underline 0
   
   $m add command -label "load DataBase" -command "data_set_load_db"
   $m add command -label "save DataBase As" -command "data_set_save_db"
   $m add separator
   $m add command -label "DataBase wizard" -command "data_set_wizard"
   $m add command -label "Clear DataBase" -command "data_set_clear_db"
   $m add separator
   $m add command -label "dismiss window" -command "destroy $dw"

   # Sort menu
   set m [menu $mf.sort -tearoff 0]
   $mf add cascade -label "Sort" -menu $m -underline 0
   
   set m2 [menu $m.site -tearoff 0]
   $m add cascade -label "Sort sites" -menu $m2
   $m2 add command -label "by name" -command "data_set_sort_site name"
   $m2 add command -label "by x-position" -command "data_set_sort_site x"
   set m2 [menu $m.wavefile -tearoff 0]
   $m add cascade -label "Sort waveform files" -menu $m2
   $m2 add command -label "by name" -command "data_set_sort_item wave name"
   $m2 add command -label "by date" -command "data_set_sort_item wave date"
   set m2 [menu $m.pickfile -tearoff 0]
   $m add cascade -label "Sort pickform files" -menu $m2
   $m2 add command -label "by name" -command "data_set_sort_item pick name"
   $m2 add command -label "by date" -command "data_set_sort_item pick date"

   # Option menu
   set m [menu $mf.option -tearoff 0]
   $mf add cascade -label "Option" -menu $m -underline 0

   $m add checkbutton -label "Use DataBase" -var gbl(ds,mode)\
       -command "update_data_set_mode"
   $m add checkbutton -label "Use Synthetics" -var gbl(ds,opt,syn)\
       -command "update_data_set_synthetic"
   $m add checkbutton -label "Show directory name"\
       -variable gbl(ds,opt,dirname) -command "data_set_update_window"
   $m add checkbutton -label "Show extension of filenames"\
       -variable gbl(ds,opt,extension) -command "data_set_update_window"
   $m add checkbutton -label "Use relative path"\
       -variable gbl(ds,opt,relative) -command "data_set_path_type"
   
   $dw conf -menu $mf

   # create new DB if necessary
   if { [llength $gbl(ds,db)] < 1} { 
      data_set_new_db
   }
   
   # site list frame --------------------
   set sf [labelframe $dw.sitelist -text "Site list"]
   set wk(w,ds,site) [listbox $sf.lb -width 15 -height 25\
			  -xscrollcommand "$sf.sh set"\
			  -yscrollcommand "$sf.sv set"\
			  -selectback darkseagreen1 -selectfo black\
			  -font normal_fixed]
   scrollbar $sf.sh -width 10 -orient horizontal -command "$sf.lb xview"
   scrollbar $sf.sv -width 10 -orient vertical -command "$sf.lb yview"
   grid $sf.lb $sf.sv -sticky news
   grid $sf.sh x -sticky ew
   
   # site frame --------------------
   set pf [labelframe $dw.site_frame -text "Data set for selected site"]

   # site name
   set wk(w,ds,sitename) [make_entry $pf name "Site name" "" ""]
   bind $wk(w,ds,sitename) <KeyRelease> "data_set_update_sitename"

   # waveform/pick/synthetic lists
   foreach {i j k} {"wave" "Waveform" 6\
		   "pick" "Pick" 4\
		   "syn" "Synthetic" 4} {
      set wf [labelframe $pf.frame($i) -text "$j files"]
      set wk(w,ds,$i) [listbox $wf.lb -width 60 -height $k\
			   -xscrollcommand "$wf.sh set"\
			   -yscrollcommand "$wf.sv set"\
			   -selectback lightblue1 -selectfo black\
			   -font normal_fixed]
      scrollbar $wf.sh -width 10 -orient horizontal -command "$wf.lb xview"
      scrollbar $wf.sv -width 10 -orient vertical -command "$wf.lb yview"
      button $wf.add -text "Add" -width 3 -command "data_set_add_item $i"
      button $wf.del -text "Del" -width 3 -command "data_set_del_item $i"
      grid $wf.lb $wf.sv $wf.add -sticky news
      grid ^ ^ $wf.del -sticky news
      grid conf $wf.add $wf.del -sticky ew
      grid $wf.sh x x -sticky news
      grid $wf - - -sticky news -ipadx 10 -ipady 5 -pady 5
   }

   # button frame ------------------------------
   set bf [button_frame $dw $dw.bf "Add Site" "data_set_add_site"\
	       "Del Site" "data_set_del_site"]
   grid $sf $pf -sticky news -padx 5
   grid $bf - -sticky news

   # binding ------------------------------
   basic_bindings $dw
   bind $wk(w,ds,site) <<ListboxSelect>> "data_set_select_site"
   bind $wk(w,ds,wave) <<ListboxSelect>> "data_set_select_item wave"
   bind $wk(w,ds,pick) <<ListboxSelect>> "data_set_select_item pick"
   bind $wk(w,ds,syn) <<ListboxSelect>> "data_set_select_item syn"

   # update
   update_data_set_synthetic
   data_set_update_window
   data_set_window_enable_disable
}

# --------------------------------------------------------------------
# Enabled/Disabled listbox in the dataset
# --------------------------------------------------------------------
proc data_set_window_enable_disable {} {
   # Global variable
   global gbl wk
   
   # Main --------------------
   # listbox + entry
   set wlist "site wave pick syn sitename"
   if { $gbl(ds,mode) == 1 } {
      set state "normal"
   } else {
      set state "disabled"
   }
   foreach w $wlist {
      if [winfo exist $wk(w,ds,$w)] {
	 $wk(w,ds,$w) conf -state $state
      }
   }
   # buttons
   set tw [winfo toplevel $wk(w,ds,site)]
   set wlist "$tw.bf.1 $tw.bf.2"
   foreach w $wlist {
      if [winfo exist $w] {
	 $w conf -state $state
      }
   }
   # menus
   set mlist "$tw.menu_frame.sort"
   foreach m $mlist {
      set n [expr [$m index end]]
      for {set i 0} {$i <= $n} {incr i} {
	 catch "$m entryconf $i -state $state"
      }
   }
}

# --------------------------------------------------------------------
# Create new database of data set
# --------------------------------------------------------------------
proc data_set_new_db {} {
   # Global variable
   global gbl
   
   # Main --------------------
   set gbl(ds,db) {  {Site1 {} {} {} }  }
   set gbl(ds,cur,site) 0
   set gbl(ds,cur,wave) -1
   set gbl(ds,cur,pick) -1
   set gbl(ds,cur,syn) -1
}

# --------------------------------------------------------------------
# Update data set database window
# --------------------------------------------------------------------
proc data_set_update_window {} {
   # Global variable
   global gbl wk
   
   # Main --------------------
   if { ![info exist wk(w,ds,site)] || ![winfo exist $wk(w,ds,site)] } {
      return
   }
   # site
   data_set_site_listbox
   # site name
   data_set_sitename_entry
   # waveform
   data_set_item_listbox wave
   # pick
   data_set_item_listbox pick
   # synthetic
   data_set_item_listbox syn
}

# --------------------------------------------------------------------
# Update listbox of the site list
# --------------------------------------------------------------------
proc data_set_site_listbox {} {
   # Global variable
   global gbl wk custom
   
   # Main --------------------
   # check existence of the listbox
   if ![winfo exist $wk(w,ds,site)] { return }
   # clear
   $wk(w,ds,site) delete 0 end
   # insert all
   foreach ds $gbl(ds,db) {
      $wk(w,ds,site) insert end [lindex $ds 0]
      $wk(w,ds,site) itemconf end -back $custom(*Entry.Background)
   }
   # highlight selected site
   if { [llength $gbl(ds,db)] > 0 && $gbl(ds,cur,site) >= 0} {
      $wk(w,ds,site) itemconf $gbl(ds,cur,site) -back darkseagreen1
      $wk(w,ds,site) see $gbl(ds,cur,site)
   }
}

# --------------------------------------------------------------------
# Update entry widget of site name
# --------------------------------------------------------------------
proc data_set_sitename_entry {} {
   # Global variable
   global gbl wk
   
   # Main --------------------
   # check existence of the entry and site
   if ![winfo exist $wk(w,ds,site)] { return }
   if { $gbl(ds,cur,site) < 0 } { return }
   # clear the previous
   $wk(w,ds,sitename) delete 0 end
   # insert
   set d [lindex $gbl(ds,db) $gbl(ds,cur,site)]
   $wk(w,ds,sitename) insert 0 [lindex $d 0]
}

# --------------------------------------------------------------------
# Update listbox of the waveform/pick
# --------------------------------------------------------------------
# type : wave or pick
# --------------------------------------------------------------------
proc data_set_item_listbox {type} {
   # Global variable
   global gbl wk custom
   
   # Main --------------------
   # check existence of the listbox and site
   if ![winfo exist $wk(w,ds,$type)] { return }
   if { $gbl(ds,cur,site) < 0 } { return }
   set ind [data_set_data_index $type]
   # clear
   $wk(w,ds,$type) delete 0 end
   # insert all ----------
   set files [lindex [lindex $gbl(ds,db) $gbl(ds,cur,site)] $ind]
   if { [llength $files] < 1 } { return }
   # set filenames as requested
   if { $gbl(ds,opt,dirname) == 0 } {
      set tmp {}
      foreach fn $files {
	 lappend tmp [lindex [file split $fn] end]
      }
      set files $tmp
   }
   if { $gbl(ds,opt,extension) == 0 } {
      set tmp {}
      foreach fn $files {
	 lappend tmp [file rootname $fn]
      }
      set files $tmp
   }
   foreach fn $files {
      $wk(w,ds,$type) insert end $fn
      $wk(w,ds,$type) itemconf end -back $custom(*Entry.Background)
   }
   # highlight selected file
   if {$gbl(ds,cur,$type) >= 0} {
      $wk(w,ds,$type) itemconf $gbl(ds,cur,$type) -back lightblue1
      $wk(w,ds,$type) see $gbl(ds,cur,$type)
   }
}

# --------------------------------------------------------------------
# Automatic generation of site name for a new data set
# --------------------------------------------------------------------
# n :  Add n-th data
# --------------------------------------------------------------------
proc data_set_auto_sitename {n} {
   # Global variable
   global gbl wk
   
   # Main --------------------
   set sitename "Site$n"
   foreach ds $gbl(ds,db) {
      set sn [lindex $ds 0]
      if { $sn == $sitename } {
	 return [data_set_auto_sitename [expr $n + 1]]
      }
   }
   return $sitename
}

# --------------------------------------------------------------------
# Processes when selecting a site
# --------------------------------------------------------------------
proc data_set_select_site {} {
   # Global variable
   global gbl wk custom mark_modification
   
   # Main --------------------
   # get the current selection for the site
   set n [$wk(w,ds,site) curselection]
   if { $n == "" } { return }
   set gbl(ds,cur,site) $n

   # waveform and pick files
   data_set_update_files
}

# --------------------------------------------------------------------
# Add a new site to data set
# --------------------------------------------------------------------
proc data_set_add_site {} {
   # Global variable
   global gbl wk
   
   # Main --------------------
   set n [llength $gbl(ds,db)]
   if { $n < 1 } {
      data_set_new_db
   } else {
      set site {}
      lappend site [data_set_auto_sitename [expr $n + 1]] {} {}
      lappend gbl(ds,db) $site
   }
   # update
   set gbl(ds,cur,site) $n
   data_set_update_files
}

# --------------------------------------------------------------------
# Delete a site from data set
# --------------------------------------------------------------------
proc data_set_del_site {} {
   # Global variable
   global gbl wk
   
   # Main --------------------
   # get the current selection
   if { $gbl(ds,cur,site) < 0 } { return }

   # confirmation
   set ans [tk_messageBox -type yesno -icon question -message\
		"Delete a selected site?"]
   if { $ans == "no" } { return }

   # delete from the data base
   set gbl(ds,db) [lreplace $gbl(ds,db) $gbl(ds,cur,site) $gbl(ds,cur,site)]
   set n [expr [llength $gbl(ds,db)] -1]
   if { $n < $gbl(ds,cur,site) } {
      set gbl(ds,cur,site) $n
   }

   # updates
   data_set_update_files
}
# --------------------------------------------------------------------
# return the data index
# --------------------------------------------------------------------
# type : one of "wave", "pick" and "syn"
# --------------------------------------------------------------------
proc data_set_data_index {type} {
   switch $type {
      "wave" { return 1 }
      "pick" { return 2 }
      "syn"  { return 3 }
   }
   return 3
}

# --------------------------------------------------------------------
# Add an item (waveform/pick/synthetic file) to a data set
# --------------------------------------------------------------------
proc data_set_add_item {type} {
   # Global variable
   global gbl wk
   
   # Main --------------------
   if {$gbl(ds,cur,site) < 0 } { return }
   # get corresponding data set
   set ind [data_set_data_index $type]
   set sitedata [lindex $gbl(ds,db) $gbl(ds,cur,site)]
   set files [lindex $sitedata $ind]
   set wk(tmp) [lindex $files end]
   if { $wk(tmp) == "" } {
      set wk(tmp) ""
   }
   # get file name
   file_selection wk(tmp) load $type
   if { [lsearch -exact $files $wk(tmp)] >= 0} {
      tk_messageBox -icon warning -type ok -message \
	  "$wk(tmp) is already in the list."
      return
   }
   # add to data base
   $wk(w,ds,$type) insert end $wk(tmp)
   lappend files $wk(tmp)
   lset sitedata $ind $files
   lset gbl(ds,db) $gbl(ds,cur,site) $sitedata
   # update
   set gbl(ds,cur,$type) [expr [llength $files] - 1]
   data_set_update_files
}

# --------------------------------------------------------------------
# Delete an item (waveform file or pick file) from a data set
# --------------------------------------------------------------------
proc data_set_del_item {type} {
   # Global variable
   global gbl wk
   
   # Main --------------------
   set ind [data_set_data_index $type]
   # get corresponding data set
   if {$gbl(ds,cur,site) < 0 } { return }
   if {$gbl(ds,cur,$type) < 0 } { return }
   set sitedata [lindex $gbl(ds,db) $gbl(ds,cur,site)]
   set files [lindex $sitedata $ind]
   # delete from data set
   if { $gbl(ds,cur,$type) >= [llength $files] } { return }
   set files [lreplace $files $gbl(ds,cur,$type) $gbl(ds,cur,$type)]
   lset sitedata $ind $files
   lset gbl(ds,db) $gbl(ds,cur,site) $sitedata
   # update listbox
   data_set_update_files
}

# --------------------------------------------------------------------
# Select a waveform or a pick file in the listbox
# --------------------------------------------------------------------
proc data_set_select_item {type} {
   # Global variable
   global gbl wk custom file_name mark_modification
   
   # Main --------------------
   # check existence of the widgets and site
   if {$gbl(ds,cur,site) < 0 } { return }

   # wave or pick
   set ind [data_set_data_index $type]

   # get corresponding data
   set n [$wk(w,ds,$type) curselection]
   if { $n == "" } { return }
   set gbl(ds,cur,$type) $n
   data_set_update_files
}

# --------------------------------------------------------------------
# update filenames (both of waveform and pick)
# --------------------------------------------------------------------
proc data_set_update_files {} {
   # Global variable
   global gbl wk file_name mark_modification pm macray paste

   # Main --------------------
   set sitedata [lindex $gbl(ds,db) $gbl(ds,cur,site)]
   if { $sitedata == "" } { return }

   # update cur selection
   if { [set nfiles [llength [lindex $sitedata 1]]] <= $gbl(ds,cur,wave) } {
      set gbl(ds,cur,wave) [expr $nfiles -1]
   }
   if { [set nfiles [llength [lindex $sitedata 2]]] <= $gbl(ds,cur,pick) } {
      set gbl(ds,cur,pick) [expr $nfiles -1]
   }
   if { [set nfiles [llength [lindex $sitedata 3]]] <= $gbl(ds,cur,syn) } {
      set gbl(ds,cur,syn) [expr $nfiles -1]
   }
   if { $gbl(ds,mode) == 0 } { 
      data_set_update_window
      return 
   }

   # waveform
   set wavefiles [lindex $sitedata 1]
   set nfiles [llength $wavefiles]
   if { $nfiles <= $gbl(ds,cur,wave) } {
      set gbl(ds,cur,wave) [expr $nfiles -1]
   }
   if { $nfiles > 0 && $gbl(ds,cur,wave) < 0} {
      set gbl(ds,cur,wave) 0
   }
   if { $gbl(ds,cur,wave) >= 0 } {
      set file_name(wave) [lindex $wavefiles $gbl(ds,cur,wave)]
   }

   # pick
   set pickfiles [lindex $sitedata 2]
   set nfiles [llength $pickfiles]
   if { $nfiles <= $gbl(ds,cur,pick) } {
      set gbl(ds,cur,pick) [expr $nfiles -1]
   }
   if { $nfiles > 0 && $gbl(ds,cur,pick) < 0} {
      set gbl(ds,cur,pick) 0
   }
   if { $gbl(ds,cur,pick) >= 0 } {
      set oldfn $file_name(pick)
      set fn [lindex $pickfiles $gbl(ds,cur,pick)]
      if { $oldfn != $fn } {
	 if { $mark_modification == 1 } {
	    set ans [tk_messageBox -type yesno -default yes\
			 -icon question -message\
			 "Save picks before change pick file? (if you select \"no\", discard your pick modification and pick file will be changed)"]
	    if {$ans == "yes"} {
	       save_pick
	    }
	 }
	 read_pick $fn
      }
   } else {
      if { $mark_modification == 1 } {
	 set ans [tk_messageBox -type yesno -default yes\
		      -icon question -message\
		      "Save picks? (if you select \"no\", discard your pick modification)"]
	 if {$ans == "yes"} {
	       save_pick
	 }
      }
      set pm {}
      delete_pick_marks
      set file_name(pick) ""
      set mark_modification 0
   }

   # synthetic
   set synfiles [lindex $sitedata 3]
   set nfiles [llength $synfiles]
   if { $nfiles <= $gbl(ds,cur,syn) } {
      set gbl(ds,cur,syn) [expr $nfiles -1]
   }
   if { $nfiles > 0 && $gbl(ds,cur,pick) < 0} {
      set gbl(ds,cur,syn) 0
   }
   if { $gbl(ds,opt,syn) && $nfiles > 0 } {
      set macray(filename) [lindex $synfiles $gbl(ds,cur,syn)]
   }
   #
   # under construction : to draw synthetic, if exist
   # 

   # other update
   data_set_update_window
   update_wm_title
   set paste(label,title) [make_title]
}

# --------------------------------------------------------------------
# update site name
# --------------------------------------------------------------------
proc data_set_update_sitename {} {
   # Global variable
   global gbl wk
   
   # Main --------------------
   if { $gbl(ds,cur,site) < 0 } { return }
   set name [$wk(w,ds,sitename) get]
   if { $name == "" } { return }
   set sitedata [lindex $gbl(ds,db) $gbl(ds,cur,site)]
   lset sitedata 0 $name

   # delete from the data base
   set gbl(ds,db) [lreplace $gbl(ds,db) $gbl(ds,cur,site) $gbl(ds,cur,site)\
		      $sitedata]
   data_set_site_listbox
}

# --------------------------------------------------------------------
# Change data set using database
# --------------------------------------------------------------------
# dir : -1(prev) or +1(next)
# --------------------------------------------------------------------
proc change_data_set_db {dir} {
   # Global variable
   global gbl wk file_name

   # set site ------------------------------
   incr gbl(ds,cur,site) $dir
   set ndata [llength $gbl(ds,db)]
   if { $gbl(ds,cur,site) >= $ndata } {
      set gbl(ds,cur,site) [expr $ndata -1]
   } elseif { $gbl(ds,cur,site) <= 0 } {
      set gbl(ds,cur,site) 0
   }
   data_set_update_files
}

# --------------------------------------------------------------------
# Change data set (waveform file and pick file)
# --------------------------------------------------------------------
# dir : +1/-1 (next/prev) or +10/-10(next/prev, silent mode)
# --------------------------------------------------------------------
proc change_data_set {dir} {
   # Global variables --------------------
   global gbl file_name wk paste
   
   # main --------------------------------------------------
   # verbose mode or not?
   if { [expr abs($dir)] > 1 } { 
      set dir [expr $dir/10]
      set verbose 0
   } else {
      set verbose 1
   }
   # This function is valid for only SU, SEGY format
   if {$gbl(data,type) != 2 && $gbl(data,type) < 5 } {
      if { $verbose } {
	 print_message "can't change data set for the current data type.\n"
      }
      return
   }

   # change data set ------------------------------
   if { $gbl(ds,mode) == 0 } {
      # automatic mode ----------
      if { $gbl(option,dir_structure) == "two" } {
	 # two directories (waveform and pick)
	 set new_pfn [change_data_set_auto_two $dir $verbose]
      } else {
	 # many directories for all the OBS
	 set new_pfn [change_data_set_auto_multiple $dir $verbose]
      }
      # plot picks
      if { $verbose } {
	 if [file exist $new_pfn] {
	    catch { after cancel $wk(after,plotpick) }
	    set wk(after,plotpick) [after idle "read_pick $new_pfn" ]
	 }
      } else {
	 # silent mode
	 return $new_pfn
      }
   } else {
      # using database ----------
      change_data_set_db $dir
   }
   
   # update title and report ------------------------------
   set paste(label,title) [make_title]

   update_wm_title
   print_message "new data: wave=\"$file_name(wave)\" and pick=\"$file_name(pick)\"\n"
   
   # draw record section ------------------------------
   if {$verbose && $gbl(option,redraw_afterchange)} {
      draw_pasteup
   } else {
      draw_axis $wk(w,current)
      draw_labels $wk(w,current)
   }
}

# --------------------------------------------------------------------
# Change data set automatically for "two" directory structure
# --------------------------------------------------------------------
# dir : +1/-1 (i.e. next/prev)
# --------------------------------------------------------------------
proc change_data_set_auto_two {dir verbose} {
   # Global variables --------------------
   global gbl file_name wk

   # ----------------------------------------------------------------
   # main -----------------------------------------------------------

   # old condition ----------
   set pf $file_name(pick)
   set old [filename_common_parts $file_name(wave) $pf]

   # pick file ----------
   set pf [change_pick_file $dir 0]

   # waveform file ----------
   set file_name(wave) [change_data_file $file_name(wave) $dir]

   # new condition ----------
   set new [filename_common_parts $file_name(wave) $pf]
   
   # check consistency ----------
   if {$new != $old && $pf != ""} {
      if { $verbose } {
	 tk_messageBox -type ok -icon warning -message\
	     "Combination of waveform file and pick file may be invalid?"
      }
      return ""
   } else {
      return $pf
   }
}

# --------------------------------------------------------------------
# Change data set automatically for "multiple" directory structure
# --------------------------------------------------------------------
# dir : +1/-1 (i.e. next/prev)
# --------------------------------------------------------------------
proc change_data_set_auto_multiple {dir verbose} {
   # Global variables --------------------
   global gbl file_name wk

   # ----------------------------------------------------------------
   # main -----------------------------------------------------------

   # determine new directory ------------------------------
   # fullpath name
   set current_wavefn [file normal $file_name(wave)]
   # current
   set current_dirname [file dirname $current_wavefn]
   set current_obsname [file tail $current_dirname]
   # parent dir list
   set parentdir [file dirname $current_dirname]
   set parentlist [lsort -ascii [glob -path $parentdir /*]]
   # new dir name
   set new_dirname [lindex $parentlist\
			[newindex $parentlist $current_dirname $dir]]
   set new_obsname [file tail $new_dirname]

   # determine new filenames ------------------------------
   # wavefile
   set new_wavefn [string map "$current_obsname $new_obsname" $current_wavefn]
   if ![file exist $new_wavefn] {
      if {$verbose} {
	 tk_messageBox -type ok -icon error\
	     -message "Cannot find next/prev waveform file."
      }
      return ""
   }
   set file_name(wave) $new_wavefn
   # pickfile
   set current_pfn [file normal $file_name(pick)]
   # try I : almost the same name
   set new_pfn [string map "$current_obsname $new_obsname" $current_pfn]
   if ![file exist $new_pfn] { 
      # try II : search pick file
      set ext [file extension $current_pfn]
      set ch ""
      set root [file rootname [file tail $current_pfn]]
      if { $gbl(option,dataset_channel) == 1 &&
	   ( [string match \*-\[1-4\] $root] || 
	     [string match \*_\[1-4\] $root] ) } {
	 set ch [string replace $root 0 end-2]
      }
      set pflist [lsort -ascii [glob -path $new_dirname /*$ch$ext]]
      if {[llength $pflist] > 0} {
	 set new_pfn [lindex $pflist 0]
      }
      if ![file exist $new_pfn] { 
	 if {$verbose} {
	    tk_messageBox -type ok -icon error\
		-message "Cannot find next/prev pick file."
	 }
	 return ""
      }
   }
   
   set file_name(pick) $new_pfn
   return $new_pfn
}

# --------------------------------------------------------------------
# Change a waveform file to the associated file.
# --------------------------------------------------------------------
# dir : +1 or -1 (next or previous)
# --------------------------------------------------------------------
proc change_wavefile_associate {dir} {
   # Global variables --------------------
   global gbl wk file_name paste

   # main -----------------------------------------------------------
   set wavefiles [lindex [lindex $gbl(ds,db) $gbl(ds,cur,site)] 1]
   set nfiles [llength $wavefiles]
   if { $gbl(ds,mode) == 1 } {
      # data base ------------------------------
      incr gbl(ds,cur,wave) $dir
      if { $gbl(ds,cur,wave) < 0} {
	 set gbl(ds,cur,wave) [expr $nfiles -1]
      } elseif { $gbl(ds,cur,wave) >= $nfiles } {
	 set gbl(ds,cur,wave) 0
      }
      data_set_update_files
   } else {
      # automatic mode ------------------------------
      # determine pick filename ----------
      # determine common parts of current filenames
      set pf $file_name(pick)
      # current wave filename
      set wf $file_name(wave)
      set common_parts [filename_common_parts $wf $pf]
      if {$common_parts == ""} {
	 # can't associate filenames due to the uncorrelated filenames
	 return
      }
      set update_title [expr { [make_title] == $paste(label,title) } ? 1 : 0 ]
      
      # get associates file list ----------
      # get all the filenames with the same suffix
      set fn [file normal $wf]
      set flist [make_filelist $fn]
      # filtering
      set associates {}
      foreach f $flist {
	 if { $common_parts == [filename_common_parts $f $pf] } {
	    lappend associates $f
	 }
      }
      # if noassociates file (can't occur)
      if { [llength $associates] < 1 } { return }
      
      # new filename ----------
      set ind [expr [lsearch $associates $fn] + $dir]
      if { $ind >= [llength $associates] } {
	 set ind 0
      } elseif { $ind < 0 } {
	 set ind end
      }
      set file_name(wave) [lindex $associates $ind]
      # update title ----------
      if { $update_title } {
	 set paste(label,title) [make_title]
      }
      update_wm_title
      print_message "new waveform file = \"$file_name(wave)\".\n"
   }
}

# --------------------------------------------------------------------
# save data base (query output filename)
# --------------------------------------------------------------------
proc data_set_save_db {} {
   global gbl
   if { $gbl(ds,mode) == 0 } {
      tk_messageBox -message "Database is currently disabled.  You cannot save any database until you turn on the database." -icon warning
      return
   }
   set types {
      {"db files"	{.db}	}
      {"All files"	*}
   }
   set fn [tk_getSaveFile -filetypes $types]
   if { $fn == "" } { return ""}
   data_set_write_db $fn
   return $fn
}

# --------------------------------------------------------------------
# write data base
# --------------------------------------------------------------------
proc data_set_write_db {fn} {
   # Global variables --------------------
   global gbl wk

   # Main ------------------------------
   # open
   set fp [open $fn "w"]

   # comments 
   set t [clock format [clock second] -format "%Y/%m/%d %H:%M:%S"]
   puts $fp "# This file is a DataBase for Pasteup $wk(VERSION)"
   puts $fp "# generated at $t"

   # body
   foreach sitedata $gbl(ds,db) {
      set name [lindex $sitedata 0]
      set wavefiles [lindex $sitedata 1]
      set pickfiles [lindex $sitedata 2]
      set synfiles [lindex $sitedata 3]
      puts $fp "# site $name"
      puts $fp $name
      foreach fn $wavefiles { puts $fp "w $fn" }
      foreach fn $pickfiles { puts $fp "p $fn" }
      foreach fn $synfiles { puts $fp "s $fn" }
      puts $fp ""
   }
   close $fp
}

# --------------------------------------------------------------------
# load data base (query output filename)
# --------------------------------------------------------------------
proc data_set_load_db {} {
   set types {
      {"db files"	{.db}	}
      {"All files"	*}
   }
   set fn [tk_getOpenFile -filetypes $types -title "DataBase"]
   if { $fn == "" } {
      return
   }
   data_set_read_db $fn
}

# --------------------------------------------------------------------
# read data base
# --------------------------------------------------------------------
proc data_set_read_db {fn} {
   # Global variables --------------------
   global gbl wk

   # Main ------------------------------
   # local defun
   proc append_site {sitename wavefiles pickfiles synfiles} {
      global gbl
      if { [llength $wavefiles] > 0 || [llength $pickfiles] > 0 } {
	 set sitedata {}
	 if { $sitename == "" } {
	    if { [llength $pickfiles] > 0} {
	       set fn [lindex $pickfiles 0]
	    } else {
	       set fn [lindex $wavefiles 0]
	    }
	    set sitename [file rootname [lindex [file split $fn] end]]
	 }
	 lappend sitedata $sitename $wavefiles $pickfiles $synfiles
	 lappend gbl(ds,db) $sitedata
      }
   }

   # open
   set fp [open $fn]

   # initialize
   set gbl(ds,db) {}
   set sitename ""
   set wavefiles {}
   set pickfiles {}
   set synfiles {}
   # read
   while {1} {
      set line [get_line $fp]
      if { [eof $fp] } { 
	 append_site $sitename $wavefiles $pickfiles $synfiles
	 break
      }
      if { $line == "" } {
	 # next site
	 append_site $sitename $wavefiles $pickfiles $synfiles
	 set sitename ""
	 set wavefiles {}
	 set pickfiles {}
	 set synfiles {}
      } else {
	 # data
	 if { [llength $line] > 1 } {
	    # have an identifier
	    set type [string tolower [string index [lindex $line 0] 0]]
	    set fn [lindex $line 1]
	 } else {
	    # auto identify
	    set fn $line
	    set type "unknown"
	    if [file exist $fn] { 
	       set ftype [identify_file $fn]
	       if {$ftype == "PICK"} {
		  set type "p"
	       } elseif { [regexp "SU|SEGY" $ftype] } {
		  set type "w"
	       }
	    }
	 }
	 if [file exist $fn] { 
	    switch $type {
	       "w" { # waveform file
		  lappend wavefiles $fn
	       }
	       "p" { # pick file
		  lappend pickfiles $fn
	       }
	       "s" { # synthetic file
		  lappend synfiles $fn
	       }
	    }
	 } else {
	    set sitename $line
	 }
      }
   }
   data_set_update_files
}
# --------------------------------------------------------------------
# register pick file to the data base
# --------------------------------------------------------------------
proc data_set_register_pick {newfn} {
   # Global variables --------------------
   global gbl wk

   # Main ------------------------------
   if { $gbl(ds,cur,site) < 0 } {return}
   set sitedata [lindex $gbl(ds,db) $gbl(ds,cur,site)]
   set files [lindex $sitedata 2]
   set newfn [absolute_path $newfn]
   foreach fn $files {
      set fn [absolute_path $fn]
      if { $fn == $newfn} { return }
   }
   if { $gbl(ds,opt,relative) } {
      set newfn [relative_path $newfn]
   }
   lappend files $newfn
   lset sitedata 2 $files
   lset gbl(ds,db) $gbl(ds,cur,site) $sitedata
   set gbl(ds,cur,pick) [expr [llength $files] -1]
   data_set_update_window
}
# --------------------------------------------------------------------
# Sort site
# --------------------------------------------------------------------
# mode : sort mode, one of "name", "x"
# --------------------------------------------------------------------
proc data_set_sort_site {mode} {
   # Global variables --------------------
   global gbl

   # Main ------------------------------
   switch $mode {
      name {
	 set sorted [lsort -dictionary -index 0 $gbl(ds,db)]
	 set gbl(ds,db) [lsort -dictionary -index 0 $gbl(ds,db)]
      }
      x {
	 # sort by x-position 
	 set tmp {}
	 foreach sitedata $gbl(ds,db) {
	    set pfn [lindex [lindex $sitedata 2] 0]
	    if { [file readable $pfn] } {
	       # read x from pick file
	       set x [pick_file_read_xpos $pfn]
	    } else {
	       set x 0.0
	    }
	    lappend tmp [list $x $sitedata]
	 }
	 set sorted [lsort -real -index 0 $tmp]
	 set gbl(ds,db) {}
	 foreach sitedata $sorted {
	    lappend gbl(ds,db) [lindex $sitedata 1]
	 }
      }
   }
   data_set_update_files
}
# --------------------------------------------------------------------
# Sort files
# --------------------------------------------------------------------
# type : "wave", "pick" or "syn"
# mode : sort mode, one of "name", "date"
# --------------------------------------------------------------------
proc data_set_sort_item {type mode} {
   # Global variables --------------------
   global gbl

   # Main ------------------------------
   # wave or pick
   set ind [data_set_data_index $type]

   for {set i 0} {$i < [llength $gbl(ds,db)]} {incr i} {
      set sitedata [lindex $gbl(ds,db) $i]
      set files [lindex $sitedata $ind]
      switch $mode {
	 name {
	    set sorted [lsort -dictionary $files]
	 }
	 date {
	    set sorted [sort_by_date $files]
	    
	 }
      }
      lset sitedata $ind $sorted
      lset gbl(ds,db) $i $sitedata
   }
   data_set_update_files
}
# --------------------------------------------------------------------
# Convert all the file paths, ralative path <-> absolute path
# --------------------------------------------------------------------
proc data_set_path_type {} {
   # Global variables --------------------
   global gbl wk

   # Main --------------------
   set newdb {}
   foreach sitedata $gbl(ds,db) {
      for {set i 1} { $i <= 3 } {incr i} {
	 set files [lindex $sitedata $i]
	 set newfiles {}
	 foreach fn $files {
	    if { $gbl(ds,opt,relative) } {
	       lappend newfiles [relative_path $fn]
	    } else {
	       lappend newfiles [absolute_path $fn]
	    }
	 }
	 lset sitedata $i $newfiles
      }
      lappend newdb $sitedata
   }
   set gbl(ds,db) $newdb
   data_set_update_window
}
# --------------------------------------------------------------------
# pick file check for data base
# --------------------------------------------------------------------
proc data_set_pick_check {fn} {
   # Global variables --------------------
   global gbl wk

   # Main --------------------
   if {$gbl(ds,cur,site) < 0} {return}
   set sitedata [lindex $gbl(ds,db) $gbl(ds,cur,site)]
   set pickfiles [lindex $sitedata 2]
   if { $gbl(ds,opt,relative) } {
      set fn [relative_path $fn]
   } else {
      set fn [absolute_path $fn]
   }
   if { [lsearch $pickfiles $fn] < 0 } {
      set ans [tk_messageBox -type yesno -icon question\
		   -message "\"$fn\" is not in the data base of the current site.  Do you want to insert this pick file into the data base?"]
      if { $ans == "no"} { return }
      lappend pickfiles $fn
      lset sitedata 2 $pickfiles
      lset gbl(ds,db) $gbl(ds,cur,site) $sitedata
      set gbl(ds,cur,pick) [expr [llength $pickfiles] -1]
      data_set_update_window
   }
}
# --------------------------------------------------------------------
# Create data set databe automatically
# --------------------------------------------------------------------
proc data_set_create_db_auto {} {
   # Global variables --------------------
   global gbl wk

   # Main --------------------
   # ok?
   if { $wk(invoke,autoDB) != 1 } {
      if { [llength $gbl(ds,db)] > 0 } {
	 set n 0
	 foreach sitedata $gbl(ds,db) {
	    set wfiles [lindex $sitedata 1]
	    set n [expr $n + [llength $wfiles]]
	 }
	 if {$n > 0} {
	    set ans [tk_messageBox -type yesno -icon question\
			 -message "Database is not empty.  Do you want to save the current database before creating a new one automatically?  If you select \"no\", the current database will be discarded."]
	    if { $ans == "yes" } {
	       set fn [data_set_save_db]
	       if {$fn == ""} {
		  tk_messageBox -type ok -icon info\
		      -message "Both of saving database and creating new database are canceled"
		  return
	       }
	    }
	 }
      }
   }
   data_set_create_db_combination
}

# --------------------------------------------------------------------
# Create database from a directory of pick or waveform
# --------------------------------------------------------------------
proc data_set_create_db_directory {dn mode} {
   # Global variables --------------------
   global gbl file_name

   # Main --------------------
   # clear database
   set gbl(ds,mode) 0
   set gbl(ds,db) {}
   # make new database
   if { $mode == "pick" } {
      set files [glob -directory $dn -types f -nocomplain *.pick *.macray]
   } else {
      set files [glob -directory $dn -types f -nocomplain *.su]
   }
   foreach fn $files {
      set sitename [file rootname [file tail $fn]]
      if { $mode == "pick"} {
	 set pickfiles $fn
	 set wavefiles {}
      } else {
	 set pickfiles {}
	 set wavefiles $fn
      }
      set synfiles {}
      lappend gbl(ds,db) [list $sitename $wavefiles $pickfiles $synfiles]
   }
   # fin
   if { [llength $gbl(ds,db)] < 1 } {
      tk_messageBox -type ok -icon info\
	  -message "Empty Database."
   }
   data_set_path_type
   data_set_update_files
   set gbl(ds,mode) 1
}

# --------------------------------------------------------------------
# Add files to database from a directory of pick or waveform
# --------------------------------------------------------------------
proc data_set_add_db_directory {dn mode} {
   # Global variables --------------------
   global gbl file_name

   # Main --------------------
   # make new database
   if { $mode == "pick" || $mode == "syn" } {
      set files [glob -directory $dn -types f -nocomplain *.pick *.macray]
   } else {
      set files [glob -directory $dn -types f -nocomplain *.su]
   }
   foreach fn $files {
      set root [file tail [file rootname $fn]]
      for {set i 0} {$i < [llength $gbl(ds,db)]} {incr i} {
	 set sitedata [lindex $gbl(ds,db) $i]
	 set sn [lindex $sitedata 0]
	 if { [string first $sn $root] >= 0 } {
	    set ind [data_set_data_index $mode]
	    set fs [lindex $sitedata $ind]
	    if { [lsearch -exact $fs $fn] < 0} {
	       lappend fs $fn
	    }
	    lset sitedata $ind $fs
	    lset gbl(ds,db) $i $sitedata
	    break
	 }
      }
   }
   data_set_path_type
   data_set_update_files
   set gbl(ds,mode) 1
}

# --------------------------------------------------------------------
# Create database by combination of current pick and waveform file
# --------------------------------------------------------------------
proc data_set_create_db_combination {} {
   # Global variables --------------------
   global gbl file_name

   # Main --------------------
   # clear database
   set gbl(ds,mode) 0
   set gbl(ds,db) {}
   # make new database
   set init_wave $file_name(wave)
   set init_pick $file_name(pick)
   foreach dir "-10 +10" {
      while {1} {
	 if { $file_name(wave) != "" && [file exist $file_name(wave)]\
		  && $file_name(pick) != "" && [file exist $file_name(pick)] } {
	    set wavefiles {}
	    set pickfiles {}
	    set synfiles {}
	    # waveform files
	    set fn1 [absolute_path $file_name(wave)]
	    lappend wavefiles $fn1
	    while {1} {
	       change_wavefile_associate +1
	       if {$file_name(wave) == $fn1} { break }
	       lappend wavefiles $file_name(wave)
	    }
	    # pick file
	    lappend pickfiles [absolute_path $file_name(pick)]
	    set name [lindex [file split [file rootname $file_name(pick)]] end]
	    # append
	    lappend gbl(ds,db) [list $name $wavefiles $pickfiles $synfiles]
	 }
	 set prev_wave $file_name(wave)
	 set status [change_data_set $dir]
	 if { $status == "" } { break }
	 if { $prev_wave == $file_name(wave) } { break }
      }
      # change direction
      set file_name(wave) $init_wave
      set file_name(pick) $init_pick
      if { $dir == -10 } {
	 set status [change_data_set +10]
	 if { $status == "" } { break }
      }
   }
   # fin
   if { [llength $gbl(ds,db)] < 1 } {
      tk_messageBox -type ok -icon info\
	  -message "Empty Database."
   }
   data_set_path_type
   data_set_update_files
   data_set_sort_site name
   set gbl(ds,mode) 1
}

# --------------------------------------------------------------------
# Clear current database
# --------------------------------------------------------------------
proc data_set_clear_db {} {
   # Global variables --------------------
   global gbl

   # Main --------------------
   # ok?
   if { [llength $gbl(ds,db)] > 0 } {
      set n 0
      foreach sitedata $gbl(ds,db) {
	 set wfiles [lindex $sitedata 1]
	 set n [expr $n + [llength $wfiles]]
      }
      if {$n > 0} {
	 set ans [tk_messageBox -type yesno -icon question -default no\
		      -message "Database is not empty.  Do you really want to discard your current database?"]
	 if { $ans == "no" } {
	    tk_messageBox -type ok -icon info\
		-message "Canceled"
	    return
	 }
      }
   }
   data_set_new_db
   data_set_update_files
}

# --------------------------------------------------------------------
# Data set DataBase Wizard window
# --------------------------------------------------------------------
proc data_set_wizard {} {
   # Global variables --------------------
   global gbl

   # Main --------------------
   set w .ds_wizard
   toplevel $w
   wm title $w "PasteUp: Data set DataBase Wizard"

   # window parts ------------------------------
   # title frame
   set tf [title_frame $w.title_frame "Pasteup: Data set DataBase Wizard"]
  
   # parameter frame
   set pf [frame $w.para_frame -height 200 -width 450 -relief groove -bd 2]

   # button frame
   set bf [frame $w.button_frame -height 30]
   
   # packing
   pack $tf $pf $bf -side top -fill both -padx 5

   # page definitions ------------------------------

   proc clear_page {pf bf} {
      foreach f "$pf $bf" {
	 foreach w [winfo children $f] {
	    destroy $w
	 }
      }
   }

   proc make_buttons {w args} {
      set n 0
      set buttons ""
      foreach {i j} $args {
	 button $w.b($n) -text $i -command $j -width 12
	 lappend buttons $w.b($n)
	 incr n
      }
      eval pack $buttons -side right -padx 5 -pady 2
   }

   # page 1 ----------
   proc page01 {pf bf} {
      global wk
      proc page01_next {pf bf} {
	 global wk
	 if { $wk(ds,wizard) } {
	    set stat [data_set_save_db]
	    if { $stat == ""} {
	       page01 $pf $bf
	       return
	    }
	 }
	 page02 $pf $bf
      }
      clear_page $pf $bf
      label $pf.title -text "Save current DataBase?"
      set wk(ds,wizard) 1
      radiobutton $pf.yes -text "Yes" -var wk(ds,wizard) -val 1 -padx 20
      radiobutton $pf.no -text "No " -var wk(ds,wizard) -val 0 -padx 20
      place $pf.title -anchor w -relx 0.3 -rely 0.4
      place $pf.yes -anchor w -relx 0.33 -rely 0.5
      place $pf.no -anchor w -relx 0.33 -rely 0.6
      set w [winfo parent $bf]
      make_buttons $bf Cancel "destroy $w" Next "page01_next $pf $bf" 
   }

   proc page02 {pf bf} {
      global wk
      proc page02_next {pf bf} {
	 global wk
	 if { $wk(ds,wizard) == 1 } {
	    # create new from scratch
	    page03 $pf $bf
	 } else {
	    # add data
	    page05 $pf $bf
	 }
      }
      clear_page $pf $bf
      set wk(ds,wizard) 2
      radiobutton $pf.new -text "Create new DataBase from scratch"\
	  -var wk(ds,wizard) -val 1 -padx 20
      radiobutton $pf.add -text "Add file(s) to current DataBase"\
	  -var wk(ds,wizard) -val 2 -padx 20
      place $pf.new -anchor w -relx 0.15 -rely 0.4
      place $pf.add -anchor w -relx 0.15 -rely 0.6
      set w [winfo parent $bf]
      make_buttons $bf Cancel "destroy $w" Next "page02_next $pf $bf" 
   }

   proc page03 {pf bf} {
      global wk
      proc page03_next {pf bf} {
	 global wk
	 switch $wk(ds,wizard) {
	    1 {
	       page04 $pf $bf
	    }
	    2 {
	       set dn [tk_chooseDirectory -title "Pick directory"\
			   -mustexist 0 -parent [winfo parent $bf]]
	       if { $dn == "" } { return ""}
	       data_set_create_db_directory $dn pick
	       page05 $pf $bf
	    }
	    3 {
	       set dn [tk_chooseDirectory -title "Waveform directory"\
			   -mustexist 0 -parent [winfo parent $bf]]
	       if { $dn == "" } { return ""}
	       data_set_create_db_directory $dn wave
	       page05 $pf $bf
	    }
	 }
      }
      clear_page $pf $bf
      label $pf.title -text "Create new DataBase by"
      set wk(ds,wizard) 1
      radiobutton $pf.m1 -var wk(ds,wizard) -val 1 -padx 20\
	  -text "Current combination of waveform and pick files"
      radiobutton $pf.m2 -text "a directory of pick files"\
	  -var wk(ds,wizard) -val 2 -padx 20
      radiobutton $pf.m3 -text "a directory of waveform files"\
	  -var wk(ds,wizard) -val 3 -padx 20
      place $pf.title -anchor w -relx 0.15 -rely 0.35
      place $pf.m1 -anchor w -relx 0.18 -rely 0.45
      place $pf.m2 -anchor w -relx 0.18 -rely 0.55
      place $pf.m3 -anchor w -relx 0.18 -rely 0.65
      set w [winfo parent $bf]
      make_buttons $bf Cancel "destroy $w"  Next "page03_next $pf $bf" 
   }

   proc page04 {pf bf} {
      global wk
      proc page04_next {pf bf} {
	 global gbl
	 set gbl(ds,mode) 1
	 data_set_window_enable_disable
	 data_set_create_db_combination
	 page05 $pf $bf
      }
      clear_page $pf $bf
      label $pf.title -text "Configuration for automatic creation"
      checkbutton $pf.m1 -var gbl(option,dir_structure) -padx 20\
	  -text "I have a waveform directory and a pickfile directory"\
	  -onvalue "two" -offvalue "many"
      checkbutton $pf.m2 -text "File names include channel number"\
	  -var gbl(option,dataset_channel) -padx 20
      place $pf.title -anchor w -relx 0.15 -rely 0.4
      place $pf.m1 -anchor w -relx 0.1 -rely 0.5
      place $pf.m2 -anchor w -relx 0.1 -rely 0.6
      set w [winfo parent $bf]
      make_buttons $bf Cancel "destroy $w"  Next "page04_next $pf $bf" 
   }
   
   proc page05 {pf bf} {
      global wk gbl
      proc page05_apply {pf bf} {
	 global wk gbl
	 set gbl(ds,mode) 1
	 data_set_window_enable_disable
	 switch $wk(ds,wizard) {
	    1 {
	       # waveform files
	       set dn [tk_chooseDirectory -title "Wave directory"\
			   -mustexist 0 -parent [winfo parent $bf]]
	       if { $dn == "" } { return ""}
	       data_set_add_db_directory $dn wave
	    }
	    2 {
	       # pick files
	       set dn [tk_chooseDirectory -title "Pick directory"\
			   -mustexist 0 -parent [winfo parent $bf]]
	       if { $dn == "" } { return ""}
	       data_set_add_db_directory $dn pick
	    }
	    3 {
	       # synthetic files
	       set dn [tk_chooseDirectory -title "Synthetic directory"\
			   -mustexist 0 -parent [winfo parent $bf]]
	       if { $dn == "" } { return ""}
	       data_set_add_db_directory $dn syn
	    }
	 }
      }
      clear_page $pf $bf
      label $pf.title -text "Add files to DataBase from a directory"
      set wk(ds,wizard) 2
      radiobutton $pf.wave -text "waveform files" -var wk(ds,wizard)\
	  -val 1 -padx 20
      radiobutton $pf.pick -text "pick files" -var wk(ds,wizard)\
	  -val 2 -padx 20
      radiobutton $pf.syn -text "synthetic files" -var wk(ds,wizard)\
	  -val 3 -padx 20
      place $pf.title -anchor w -relx 0.2 -rely 0.35
      place $pf.wave -anchor w -relx 0.23 -rely 0.45
      place $pf.pick -anchor w -relx 0.23 -rely 0.55
      place $pf.syn -anchor w -relx 0.23 -rely 0.65
      set w [winfo parent $bf]
      make_buttons $bf Exit "destroy $w" Next "page05_apply $pf $bf"
   }

   # ------------------------------

   # invoke page 1
   page01 $pf $bf

   grab $w
}