#include <stdio.h>
#include <math.h>
#include "pasteup.h"

/********************************************************************
distcal: Function. 
  Return the distance from the origin to the given point.
--------------------------------------------------------------------
Created by Kimihiro MOCHIZUKI.

Modified by FUJIE Gou.
      Modifications:
           1. Added comments.
	   2. Change PI to M_PI(System difinition of pi)
	   3. Code Indention...
	   4. Change K&R C code into ANSI-C code.
--------------------------------------------------------------------
Arguments:
   lat     : latitude
   lon     : longitude
   obslat  : latitude of origin.
   obslon  : longitude of origin.
   sgn     : sign of axis direction. NORTH or EAST
********************************************************************/
double distcal(double lat, double lon, 
	       double obslat, double obslon, int sgn)
{
    /***** Local Variables *****/
    double fact, dist, conv, mid, delta;
    double AE, BE, CE, AS, BS, CS;
  
    /***** Subroutine Main: *****/
    conv = M_PI/180.;
  
    /***** Check direction *****/
    fact = -1;
    switch(sgn){
	case NORTH:
	    if(lat>=obslat) fact = 1.;
	    break;
	case EAST:
	    if(lon>=obslon) fact = 1;
	    break;
    }
    lat *= conv;	lon *= conv;	obslat *= conv;		obslon *= conv;
    lat    -= 11.55/60.*conv*sin(2.*lat);
    obslat -= 11.55/60.*conv*sin(2.*obslat);
  
    AE = cos(lat)*cos(lon);	BE = cos(lat)*sin(lon);	CE = sin(lat);
    AS = cos(obslat)*cos(obslon);	BS = cos(obslat)*sin(obslon);
    CS = sin(obslat);
  
    mid = sqrt(pow((AE-AS),2.) + pow((BE-BS),2.) + pow((CE-CS),2.));
    delta = 4. * asin(mid/4.);
    dist = 6369.*delta * fact;
  
    return(dist);
}
