proc draw_pasteup {} {
   # global --------------------
   global gbl wk paste file_name filt macray TMPDIR fp

   # main --------------------
   # Parameter check
   if {[check_parameter] == 1} {
      return
   }

   # draw preparation --------------------------------------------------
   # set filter type appropriately
   set filt_type [stripzeros [format "%d%d" [expr $filt(n) -1] $filt(type)]]
   if {$filt(offset)} {
      set filt_type [expr $filt_type * -1]
   }

   # x-axis manipulation
   set xaxis $gbl(data,xaxis)
   if { $gbl(data,type) == 3 } {
      set xaxis $gbl(obs,direc)
   }

   # hide ballon help
   cancel_balloon

   # clear canvas --------------------------------------------------
   set wk(vline,data) {}
   delete_all $wk(w,current)

   # update canvas size and draw frame
   update_canvas_size

   # condition check -------------------------------------------
   set wk(condition) [check_same_draw_condition]

   # sufilter --------------------------------------------------
   set orgtype $gbl(data,type)
   set orgbyte $gbl(data,byte)
   if { $wk(condition) < 0 } {
      set orgname [SUfilter::apply]
   } else {
      set orgname 0
   }
   if { $orgname == -1 } {
      # invalid sufilter
      set gbl(data,type) $orgtype
      set gbl(data,byte) $orgbyte
      return
   }

   # Draw!! -----------------------------------------------------
   set ww [wait_window "now drawing..."]
   # Start time
   print_message [format "%s style   Start Drawing(%s) ---> "\
		      $gbl(mode_label) [clock format [clock sec] -format %T] ]
   update
   # draw pasteup
   if {$gbl(mode) > 100} {
      # Wiggle+VD/VA+VD ----------
      # background : vd
      exec_draw_pasteup $wk(w,current)\
	  $gbl(vdcolor) $paste(image,mode) $paste(amp,amplify)
      # foreground : wiggle/va
      exec_draw_pasteup $wk(w,current) [expr $gbl(mode)/100] 0\
	  [expr $paste(amp,ratio) * $paste(amp,amplify)]
   } elseif { $gbl(mode) == $wk(DRAW_MODE,VD) } {
      # VD ----------
      exec_draw_pasteup $wk(w,current)\
	  $gbl(vdcolor) $paste(image,mode) $paste(amp,amplify)
   } else {
      # Wiggle/VA ----------
      exec_draw_pasteup $wk(w,current)\
	  $gbl(mode) $paste(image,mode) $paste(amp,amplify)
   }

   # recover original file/mode (if sufilter)
   set tmpfn $file_name(wave)
   if {$orgname != 0} {
      set file_name(wave) $orgname
      set gbl(data,type) $orgtype
      set gbl(data,byte) $orgbyte
   }

   # draw direct wave arrivals and auxiliary lines
   draw_auxiliary

   # draw tau-p related
   draw_taup

   # draw synthetic
   if { $macray(mode) == "on" } {
      Synthetic::plot
   }

   # draw pickmarks
   plot_pick_marks 0

   # update trace window
   display_trace 0

   # show time of termination
   print_message [format "End (%s)\n"\
		      [clock format [clock seconds] -format %T] ]
   destroy $ww
   update_wm_title

   # preserve drawing conditions
   preserve_draw_condition
}

# -----------------------------------------------------------------
# draw pasteup
# -----------------------------------------------------------------
proc exec_draw_pasteup {w draw_mode image_mode amp} {
   # global --------------------------------------------------
   global paste wk

   # main --------------------------------------------------
   switch -- $image_mode {
      0 {
	 getwave $draw_mode $image_mode $amp
	 return
      }
      1 {
	 image create photo pasteupimage -data\
	     [getwave $draw_mode $image_mode $amp]
      }
      2 {
	 image create photo pasteupimage -data\
	     [getinterp $draw_mode $paste(image,format) $amp]
      }
   }
   put_pasteupimage $w
}
