#include <stdio.h>
#include <math.h>

#define	CANCEL 11
#define	NO     22

#define CAUSAL 1
#define UNCAUSAL 2

#define NOFILTER 100
#define LOWPASS  0  /** Chebyshev, low pass **/
#define HIGHPASS 1  /** Chebyshev, high pass **/
#define BANDPASS 2  /** Chebyshev, band pass **/
#define BANDSTOP 3  /** Chebyshev, band stop **/
#define BUTBPASS 4  /** Butterworth, band pass **/
#define BUTLPASS 5  /** Butterworth, low pass **/
#define BUTHPASS 6  /** Butterworth, high pass **/
#define OFFSETFILTER -1

/********************************************************************
 Remove Offset from the time siries array "a",
 "a" has "count" components.
 Offset value is defined by simply mean amplitude.
********************************************************************/
void remove_offset(int count, float *a)
{
    /***** Local Variables *****/
    register int i;
    register float mean;
    
    /********** Main **********/
    mean = 0.0;
    for(i=0;i<count;i++) mean += a[i];
    mean /= (float)count;
    for(i=0;i<count;i++) a[i] -= mean;
}

/*
  --------------------------------------------------------------------
 */

void filter(float *a, int count, double sampling, int type, int causal_type,
	    double f1, double f2, double f3, double f4)
{
    /***** Local Variables *****/
    int npoleshi, npoleslo;
    float f3dblo, f3dbhi;
    double astoplo, apasslo, apasshi, astophi;

    /***** Local Function *****/
    char locut, hicut;
    void bfdesign (float fpass, float apass, float fstop, float astop,
		   int *npoles, float *f3db);
    void bfhighpass (int npoles, float f3db, int n, float p[], float q[]);
    void bflowpass (int npoles, float f3db, int n, float p[], float q[]);
    void do_filter(int count, int filt_m, double filt_gn, 
		   double h[], int *canc, float a[], 
		   double sampling, int causal_type);
    void tchlop(int *filt_m,double fp,double fs,double *filt_gn,double h[],
		int *canc,double ap,double as);
    void tchpas(int *filt_m,double fl,double fh,double fs,double *filt_gn,
		double h[],int *canc,double ap,double as);
    void tchhip(int *filt_m,double fp,double fs,double *filt_gn,double h[],
		int *canc,double ap,double as);
    void tchstp(int *filt_m,double fl,double fh,double fs,double *filt_gn,
		double h[],int *canc,double ap,double as);

    /******************** Main ********************/
    /***** bias(offset) filter ?? *****/
    if(type<0){
	remove_offset(count, a);
	type = -type;
    }
    if(type == NOFILTER || type == 0) return;  /** no filter **/

    /***** branch : Chebyshev or Butterworth *****/
    if(type <= BANDSTOP) {
	/********** Chebyshev **********/
	double ap, as, fp, fs, fl, fh;
	int filt_m, canc;
	double filt_gn, h[100];

	ap = 0.1;
	as = 5.;
	/** determine filter coefficients **/
	switch (type) {
	    case BANDPASS:
		fl = f2/sampling;
		fh = f3/sampling;
		fs = f4/sampling;
		tchpas(&filt_m,fl,fh,fs,&filt_gn,h,&canc,ap,as);
		break;
	    case NOFILTER:
		return;
		break;
	    case LOWPASS:
		fp = f1/sampling;
		fs = f2/sampling;
		tchlop(&filt_m,fp,fs,&filt_gn,h,&canc,ap,as);
		break;
	    case HIGHPASS:
		fp = f1/sampling;
		fs = f2/sampling;
		tchhip(&filt_m,fp,fs,&filt_gn,h,&canc,ap,as);
		break;
	    case BANDSTOP:
		fl = f2/sampling;
		fh = f3/sampling;
		fs = f4/sampling;
		tchstp(&filt_m,fl,fh,fs,&filt_gn,h,&canc,ap,as);
		break;
	    default:
		break;
	}
	/** Apply filter **/
	do_filter(count, filt_m, filt_gn, h, &canc, a, sampling, causal_type);
    } else {
	/********** Butterworth **********/
	/***** Amplitude at specified frequencies *****/
	astoplo = .05;
	apasslo = .95;
	apasshi = .95;
	astophi = .05;
	/* Adapt user frequencies if zerophase selected */
	if (causal_type == UNCAUSAL) {
	    astoplo = sqrt(astoplo);
	    apasslo = sqrt(apasslo);
	    astophi = sqrt(astophi);
	    apasshi = sqrt(apasshi);
	}

	/***** design a filter *****/
	locut = hicut = 0;
	switch (type) {
	    case BUTLPASS:  /** high cut , f1 < f2 **/
		f1 /= sampling;
		f2 /= sampling;
		bfdesign(f1, apasshi, f2, astophi, &npoleshi, &f3dbhi);
		hicut = 1;
		break;
	    case BUTBPASS: /** low and high cut, f1 < f2 < f3 < f4 **/
		f1 /= sampling;
		f2 /= sampling;
		f3 /= sampling;
		f4 /= sampling;
		bfdesign(f2, apasslo, f1, astoplo, &npoleslo, &f3dblo);
		bfdesign(f3, apasshi, f4, astophi, &npoleshi, &f3dbhi);
		locut = hicut = 1;
		break;
	    case BUTHPASS: /** low cut , f1 < f2 **/
		f1 /= sampling;
		f2 /= sampling;
		bfdesign(f2, apasslo, f1, astoplo, &npoleslo, &f3dblo);
		locut = 1;
		break;
	    default:
		break;
	}

	/***** Filtering *****/
	/* low-cut (high pass) filter */
	if (locut) {
	    bfhighpass(npoleslo, f3dblo, count, a, a);
	    if (causal_type == UNCAUSAL) {
		register int i;
		for (i=0; i<count/2; ++i) { /* reverse trace in place */
		    register float tmp = a[i];
		    a[i] = a[count-1 - i];
		    a[count-1 - i] = tmp;
		}
		bfhighpass(npoleslo, f3dblo, count, a, a);
		for (i=0; i<count/2; ++i) { /* flip trace back */
		    register float tmp = a[i];
		    a[i] = a[count-1 - i];
		    a[count-1 - i] = tmp;
		}
	    }
	}
	
	/* high-cut (low pass) filter */
	if (hicut) {
	    bflowpass(npoleshi, f3dbhi, count, a, a);
	    if (causal_type == UNCAUSAL) {
		register int i;
		for (i=0; i<count/2; ++i) { /* reverse trace */
		    register float tmp = a[i];
		    a[i] = a[count-1 - i];
		    a[count-1 - i] = tmp;
		}
		bflowpass(npoleshi, f3dbhi, count, a, a);
		for (i=0; i<count/2; ++i) { /* flip trace back */
		    register float tmp = a[i];
		    a[i] = a[count-1 - i];
		    a[count-1 - i] = tmp;
		}
	    }
	}
    }
}


