# --------------------------------------------------------------------
# Filter configuration window
# --------------------------------------------------------------------
proc filter_window {} {
   # Global variables ----------
   global filt gbl wk custom
   
   # Main ------------------------------
   # Make window --------------------
   set w .filter
   set pf $w.parameter_frame
   if [raise_window $w] {
      destroy $pf
   } else {
      # invoke new window ----------
      toplevel $w
      wm title $w "PasteUp:Filter Config"
      # binding 
      basic_bindings $w
      # title frame
      set tf [title_frame $w.title_frame "Pasteup: Filter Configuration"]
      pack $tf -side top -fill x
      # button frame
      set bf [button_frame $w $w.button_frame]
      pack $bf -side bottom -fill x
   }
   
   # parameter frame ------------------------------
   labelframe $pf -text "Filter parameters" -pady 20 -padx 15
   
   # Filter type --------------------
   eval {make_optionMenu_com $pf filter_type "FilterType" filt(type_label)\
	     "update_filter; filter_window"} $wk(filter,list)
   bind $pf.button(filter_type) <Button-4> {filter_change prev}
   bind $pf.button(filter_type) <Button-5> {filter_change next}
   make_many_radio $pf "Causality" causality filt(causal)\
       Causal 1 Uncausal 2
   
   # Filter iteration number --------------------
   make_entry $pf filter_n "Number of times" filt(n) ""
   
   # Filter coefficients --------------------
   switch -- $filt(type_label) {
      "Low Pass " {
	 make_entry $pf passband "PassBand" filt(f1) "Hz"
	 make_entry $pf stopband "StopBand" filt(f2) "Hz"
      }
      "High Pass" {
	 make_entry $pf stopband "StopBand" filt(f1) "Hz"
	 make_entry $pf passband "PassBand" filt(f2) "Hz"
      }
      "Band Pass" {
	 make_entry $pf passband1 "PassBand Low Freq." filt(f2) "Hz"
	 make_entry $pf passband2 "PassBand High Freq." filt(f3) "Hz"
	 make_entry $pf stopband2 "StopBand High Freq." filt(f4) "Hz"
      }
      "Band Stop" {
	 make_entry $pf passband1 "Low Freq. Cut-Off" filt(f2) "Hz"
	 make_entry $pf passband2 "High Freq. Cut-Off" filt(f3) "Hz"
	 make_entry $pf stopband2 "Stop Band  Freq." filt(f4) "Hz"
      }
      "(B)Band Pass" {
	 make_entry $pf stopband1 "StopBand Low Freq." filt(f1) "Hz"
	 make_entry $pf passband1 "PassBand Low Freq." filt(f2) "Hz"
	 make_entry $pf passband2 "PassBand High Freq." filt(f3) "Hz"
	 make_entry $pf stopband2 "StopBand High Freq." filt(f4) "Hz"
      }
      "(B)Low Pass " {
	 make_entry $pf passband "PassBand" filt(f1) "Hz"
	 make_entry $pf stopband "StopBand" filt(f2) "Hz"
      }
      "(B)High Pass" {
	 make_entry $pf stopband "StopBand" filt(f1) "Hz"
	 make_entry $pf passband "PassBand" filt(f2) "Hz"
      }
      default -
      "No Filter" {
      }
   }
   
   # Packing --------------------
   pack $pf -side top -fill x
}
# --------------------------------------------------------------------
# Explanation of filter types
# --------------------------------------------------------------------
proc filter_explanation {} {
   # global ----------
   global filt gbl

   # main ------------------------------
   if { $gbl(check,filter) == 0 } { return "none" }
   switch $filt(type) {
      0 { # Chebyshev LowPass
	 return [format "%s (%4.2fHz)" $filt(type_label) $filt(f1)]
      }
      1 { # Chebyshev HighPass
	 return [format "%s (%4.2fHz)" $filt(type_label) $filt(f1)]
      }
      2 { # Chebyshev BandPass
	 return [format "%s (%4.2fHz - %4.2fHz)"\
		     $filt(type_label) $filt(f2) $filt(f3)]
      }
      3 { # Chebyshev BandStop
	 return [format "%s (%4.2fHz - %4.2fHz)"\
		     $filt(type_label) $filt(f2) $filt(f3)]
      }
      4 { # Butterworth BandPass
	 return  [format "%s (%4.2fHz - %4.2fHz)"\
		      $filt(type_label) $filt(f2) $filt(f3)]
      }
      5 { # Butterworth LowPass
	 return [format "%s (%4.2fHz)" $filt(type_label) $filt(f1)]
      }
      6 { # Butterworth HighPass
	 return [format "%s (%4.2fHz)" $filt(type_label) $filt(f2)]
      }
      default {
	 return $filt(type_label)
      }
   }
}

# --------------------------------------------------------------------
# Change time-domain filter (next or prev)
# --------------------------------------------------------------------
proc filter_change {direc} {
   # global ----------
   global filt wk
   
   # main --------------------------------------------------
   set ind [lsearch $wk(filter,list) $filt(type_label)]
   if {$direc == "next"} {
      incr ind
      if {$ind >= [llength $wk(filter,list)]} {
	 incr ind -1
      }
   } else {
      if {$ind > 0} {
	 incr ind -1
      }
   }
   set filt(type_label) [lindex $wk(filter,list) $ind]
   update_filter
   if [winfo exists .filter] {
      filter_window
   }
}