# --------------------------------------------------------------------
#
#
#                        GraphCanvas widget module
#
#
# --------------------------------------------------------------------

namespace eval GraphCanvas {
   namespace export *

   # -----------------------------------------------------------------
   # use graph widget for drawing
   # -----------------------------------------------------------------
   proc setup_canvas {} {
      # global --------------------------------------------------
      global wk paste
      
      # main --------------------------------------------------
      clear_slaves $wk(w,canvas)
      grid $wk(w,canvas,g) -sticky news
      grid rowconfig    $wk(w,canvas) 0 -weight 1 -minsize 0
      grid columnconfig $wk(w,canvas) 0 -weight 1 -minsize 0
   }
   # -----------------------------------------------------------------
   # Make GraphCanvas widget ; declare pick elements
   # -----------------------------------------------------------------
   proc make_canvas_widget {} {
      # global --------------------------------------------------
      global wk gbl paste
      
      # main --------------------------------------------------
      # GraphCanvas widget
      set g $wk(w,canvas,g)
      graph $g
      $g conf -plotpadx 0 -plotpady 0 -plotborderwidth 1 -borderwidth 0\
	  -relief flat -plotrelief flat
      $g axis conf x2 -hide false
      $g axis conf y2 -hide false
      $g legend conf -hide true
      Blt_Crosshairs $g
      Blt_ZoomStack $g
      # pick elements
      for {set i 0} {$i < 20} {incr i} {
	 set col [lindex $wk(color,list) $i]
	 $g elem create pick$i -xdata vx$i -ydata vt$i -yhigh vh$i -ylow vl$i\
	     -linewidth 0 -pixels 10 -scale 0 -symbol $paste(mark,style)\
	     -color gray -outline $col -fill $col -pixels $paste(mark,size)
      }
   }
   
   # -----------------------------------------------------------------
   # draw axis including ticks and tick labels
   # -----------------------------------------------------------------
   proc draw_axis {g} {
      # global --------------------------------------------------
      global gbl paste wk zoomInfo

      # main --------------------------------------------------

      # zoom stack related ------------------------------
      if { [llength $zoomInfo($g,stack)] > 0 } {
	 # now using zoom stack
	 set s [lindex $zoomInfo($g,stack) end]
	 scan [list [lindex $s 5] [lindex $s 7] [lindex $s 13] [lindex $s 15]]\
	     "%g%g%g%g" xmin xmax ymin ymax
	 if { [expr abs($xmin - $paste(left)) + abs($xmax - $paste(right)) + \
		   abs($ymin-$paste(top)) + abs($ymax-$paste(bottom))] == 0. \
		  && [$g axis cget y -descending] == $gbl(t_axis) } {
	    # no changes on zoom and y-axis direction
	    # update zoom stack if x_axis is "offset+obs_x"
	    if { $gbl(x_axis,top) != 1 } {
	       set old $zoomInfo($g,stack)
	       set zoomInfo($g,stack) {}
	       foreach coms $old {
		  set c [split $coms "\n"]
		  set c1 [lindex $c 0]
		  set c2 [lindex $c 1]
		  set c3 [lindex $c 2]
		  set c4 [lindex $c 3]
		  scan [list [lindex $c1 5] [lindex $c1 7]] "%g%g" xmin xmax
		  set c3 [lreplace [lreplace $c3 5 5 [expr $xmin + $gbl(obs,x)]]\
			      7 7 [expr $xmax + $gbl(obs,x)]]
		  lappend zoomInfo($g,stack) "$c1\n$c2\n$c3\n$c4"
	       }
	       $g axis conf x2 -min [expr $gbl(obs,x) + [$g axis cget x -min]] \
		   -max [expr $gbl(obs,x) + [$g axis cget x -max]]
	    }
	    return
	 }
      }
      blt::InitStack $g

      # set frame ------------------------------
      $g axis conf x -min $paste(left) -max $paste(right)
      $g axis conf y -min $paste(top) -max $paste(bottom)
      $g axis conf y2 -min $paste(top) -max $paste(bottom)
      if { $gbl(x_axis,top) != 1 } {
	 $g axis conf x2 -min [expr $paste(left) + $gbl(obs,x)]\
	     -max [expr $paste(right) + $gbl(obs,x)]
      } else {
	 $g axis conf x2 -min $paste(left) -max $paste(right)
      }

      # ticks
      set xsub [expr $gbl(x_sub_tic) + 1]
      $g axis conf x -subdivisions $xsub -stepsize $gbl(xtic_int)
      $g axis conf x2 -subdivisions $xsub -stepsize $gbl(xtic_int)
      set tsub [expr $gbl(t_sub_tic) + 1]
      $g axis conf y -subdivisions $tsub -stepsize $gbl(t_tic_int)
      $g axis conf y2 -subdivisions $tsub -stepsize $gbl(t_tic_int)

      # axis direction
      $g axis conf x -descending $gbl(data,xaxis_direction)
      $g axis conf x2 -descending $gbl(data,xaxis_direction)
      $g axis conf y -descending $gbl(t_axis)
      $g axis conf y2 -descending $gbl(t_axis)
     
   }

   # -----------------------------------------------------------------
   # draw labels
   # -----------------------------------------------------------------
   proc draw_labels {g} {
      # global --------------------------------------------------
      global gbl paste wk zoomInfo

      # main --------------------------------------------------
      # fonts
      $g conf -font $paste(fn,title)

      $g axis conf x -tickfont $paste(fn,tick)
      $g axis conf y -tickfont $paste(fn,tick)
      $g axis conf x2 -tickfont $paste(fn,tick)
      $g axis conf y2 -tickfont $paste(fn,tick)

      $g axis conf x -titlefont $paste(fn,tick)
      $g axis conf x2 -titlefont $paste(fn,tick)
      $g axis conf y -titlefont $paste(fn,tick)
      $g axis conf y2 -titlefont $paste(fn,tick)

      # axis labels
      foreach {i a} {top x2 bottom x left y right y2} {
	 if {$paste(string,[format "%stick" $i])} {
	    $g axis conf $a -command ""
	 } else {
	    $g axis conf $a -command GraphCanvas::no_label
	 }
      }

      # title
      $g conf -title $paste(label,title)
      # y-axis labels
      if {$paste(string,unit) == "tx"} {
	 if {$paste(reduction) != 0.0} {
	    $g axis conf y -title [format "t - x/%g (sec)" $paste(reduction)]
	 } else {
	    $g axis conf y -title "t (sec)"
	 }
      } elseif { $paste(string,unit) == "xz" } {
	 $g axis conf y -title [format "Depth (km)" $paste(reduction)]
      } else {
	 $g axis conf y -title ""
      }
      # x-axis labels
      if { $paste(check,toplabel) } {
	 $g axis conf x2 -title $paste(label,topaxis)
      } else {
	 $g axis conf x2 -title ""
      }
      # x1
      if { $paste(check,bottomlabel) } {
	 $g axis conf x -title $paste(label,bottomaxis)
      } else {
	 $g axis conf x -title ""
      }
   }

   # -----------------------------------------------------------------
   # No label for tick mark
   # -----------------------------------------------------------------
   proc no_label {a b} {
      return ""
   }
   # -----------------------------------------------------------------
   # Put a pasteup image
   # -----------------------------------------------------------------
   proc put_pasteupimage {g} {
      global paste
      $g marker create image -image pasteupimage -name wave -under 1\
	  -coord "$paste(left) $paste(top) $paste(right) $paste(bottom)"
   }
   # -----------------------------------------------------------------
   # clear graph canvas
   # -----------------------------------------------------------------
   proc delete_all {g} {
      # main --------------------------------------------------
      eval $g element delete [$g element names syn* multiples]
      eval $g marker delete [$g marker names]
   }
   # -----------------------------------------------------------------
   # in GraphCanvas, no need to delete 'cause element is automatically updated
   # -----------------------------------------------------------------
   proc delete_pick_marks {} {
      pick_label_erase
      return
   }

   # -----------------------------------------------------------------
   # Plot synthetics
   # -----------------------------------------------------------------
   proc plot_synthetic {w} {
      global macray
      proc exec_plot {w id x y col sym ss lw ls} {
	 $w element create syn$id -xdata $x -ydata $y -pixels $ss -color $col\
	     -fill $col -linewidth $lw -symbol $sym -dash $ls
      }
      # main --------------------------------------------------
      # determine symbol
      if { $macray(style) == "Line" } {
	 set sym ""
      } else {
	 switch -- $macray(symbol_style) {
	    oval {
	       set sym "circle"
	    }
	    cross {
	       set sym "cross"
	    }
	 }
      }
      # detrmine line width
      set lw [expr { $macray(style) == "Symbol" } ? 0 : $macray(width) ]
      # determine line style
      switch -- $macray(line_style) {
	 dashed {
	    set ls "10 10"
	 }
	 dotted {
	    set ls "3 3"
	 }
	 default {
	    set ls ""
	 }
      }
      set id 0
      set xc {}
      set tc {}
      set cmode 10000
      set prevx 0
      set ss [expr $macray(symbol_size) * 2.0]
      for {set i 0} {$i < $macray(n)} {incr i} {
	 if { $cmode < 1 || $cmode != $macray(m,$i)\
		  || [expr $macray(x,$i) * $prevx] < 0.0 } {
	    if { [llength $xc] > 0 } {
	       exec_plot $w $id $xc $tc [Synthetic::color $cmode] $sym $ss\
		   $lw $ls
	       incr id
	       set xc {}
	       set tc {}
	    }
	    set cmode $macray(m,$i)
	 }
	 if [coord inout $macray(x,$i) $macray(t,$i)] {
	    lappend xc $macray(x,$i)
	    lappend tc [coord tt2rt $macray(x,$i) $macray(t,$i)]
	 }
	 set prevx $macray(x,$i)
      }
      exec_plot $w $id $xc $tc [Synthetic::color $cmode] $sym $ss $lw $ls
   }
   # -----------------------------------------------------------------
   # Delete synthetics
   # -----------------------------------------------------------------
   proc delete_synthetic {g} {
      set tags [$g element names syn*]
      if { $tags != ""} {
	 eval $g element delete $tags
      }
   }
   # -----------------------------------------------------------------
   # Delete closest pick mark
   # -----------------------------------------------------------------
   proc delete_closest_pick {w mx my} {
      set x [$w axis invtransform x $mx]
      set rt [$w axis invtransform y $my]
      delete_single_pick $x $rt
   }

   # --------------------------------------------------------------------
   # start to a single pick
   # --------------------------------------------------------------------
   proc pick_start {w mx my} {
      # global --------------------
      global pick paste gbl
      
      # main --------------------
      # searching closest trace
      scan [$w invtransform $mx $my] "%g%g" x pick(start_t)
      scan [getrh nearest $x] "%d%g%d%d"\
	  pick(near_no) pick(dist) pick(obs) pick(depth)
      if {$pick(near_no) < 0} { return }
      set pick(status) 1
      set pick(width) [expr ( $paste(right) - $paste(left) ) * $gbl(mark_size)\
			   / ( $paste(graph_width) * 1.0) ]
      
      # temporary binding
      bind $w <B2-Motion> "pick_motion $w %x %y"
      bind $w <ButtonRelease-2> "pick_end $w %x %y"
   }

   # --------------------------------------------------------------------
   # while picking... (determining accuracy)
   # --------------------------------------------------------------------
   proc pick_motion {w mx my} {
      # global --------------------
      global pick 
      
      # main --------------------
      # check
      if { !$pick(status) } {
	 cancel_button2 $w
	 return;
      }
      # draw temp mark
      scan [$w invtransform $mx $my] "%g%g" x y
      $w marker create polygon -outline gray -fill "" -linew 1 -name temp_mark\
	  -coord "[expr $pick(dist)-$pick(width)] $pick(start_t)\
	  [expr $pick(dist) - $pick(width)] $y\
	  [expr $pick(dist) + $pick(width)] $y\
	  [expr $pick(dist) + $pick(width)] $pick(start_t)"
   }
   
   # --------------------------------------------------------------------
   # end pick (confirmation)
   # --------------------------------------------------------------------
   proc pick_end {w mx my} {
      # global --------------------
      global pick mark_modification pm undopm paste gbl
      
      # main --------------------
      set pick(status) 0
      cancel_button2 $w

      # delete temp mark
      catch { $w marker delete temp_mark }
      
      # pick info
      scan [$w invtransform $mx $my] "%g%g" x y
      set start_t [coord rt2tt $pick(dist) $pick(start_t)]
      set end_t [coord rt2tt $pick(dist) $y]
      set pt [expr ($start_t + $end_t)/2.0]
      set acc [expr abs($start_t - $end_t) / 2.0]
      set acc [expr {$acc < $pick(default_accuracy)} ? $pick(default_accuracy)\
		   : $acc]

      # register it
      undo_stack
      switch $paste(correct,shot_mode) {
	 0 {
	    # none
	 }
	 2 {
	    # shots move to sea floor
	    set pt [expr $pt + $pick(depth) / ($gbl(water_v) * 1000.0)]
	 }
	 1 {
	    # flat sea floor correction
	    set pt [expr $pt + ($gbl(obs,depth) - $pick(depth))\
			* (1.0/($gbl(sediment_v)*1000.0)\
			       - 1.0/($gbl(water_v)*1000.0)) ]
	 }
      }
      lappend pm "$pick(near_no) $pick(dist) $pt $acc $pick(current_type)"
      set mark_modification 1

      # plot
      plot_single_pick $pick(dist) $pt $acc $pick(current_type)

      # report pick info
      print_message [format "Shot #%4d (OBS %2d) : Location(%fkm, %fsec), Accuracy %fsec, Pick Type%d\n"\
			 $pick(near_no) $pick(obs) $pick(dist) $pt $acc\
			 $pick(current_type)]
   }

   # -----------------------------------------------------------------
   # Line pick start
   # -----------------------------------------------------------------
   proc line_pick_start {w mx my} {
      # global --------------------
      global pick
      
      # main --------------------
      if { $pick(line,status) } { return }
      set pick(line,status) 1
      scan [$w invtrans $mx $my] "%g%g" pick(line,start_x) pick(line,start_y)
      bind $w <B2-Motion> "line_pick_move $w %x %y %X %Y"
      bind $w <ButtonRelease-2> "line_pick_end $w %x %y %X %Y"
   }
   # -----------------------------------------------------------------
   # Line pick move
   # -----------------------------------------------------------------
   proc line_pick_move {w mx my RX RY} {
      # global --------------------
      global pick
      
      # main --------------------
      if { !$pick(line,status) } { 
	 return 
	 cancel_button2 $w
      }
      scan [$w invtransform $mx $my] "%g%g" x y
      $w marker create line\
	  -coord "$pick(line,start_x) $pick(line,start_y) $x $y"\
	  -name line_pick_tmp -outline green -linewidth 1
      show_position $w $mx $my $RX $RY
   }

   # -----------------------------------------------------------------
   # Line pick move
   # -----------------------------------------------------------------
   proc line_pick_end {w mx my RX RY} {
      # global --------------------
      global wk paste pick
      
      # main --------------------
      # erase a temporal line
      if { $pick(line,status) == 0 } { return }
      cancel_button2 $w
      cancel_line_pick $w
      
      # Convert canvas coord into graph coord
      scan [$w invtransform $mx $my] "%g%g" x1 t1
      set x0 $pick(line,start_x)
      set t0 $pick(line,start_y)

      # exec line pick 
      undo_stack
      exec_line_pick $x0 $t0 $x1 $t1 $RX $RY
      set pick(line,status) 0
   }

   # -----------------------------------------------------------------
   # Line pick cancel
   # -----------------------------------------------------------------
   proc cancel_line_pick {w} {
      # global --------------------
      global pick
      
      # main --------------------
      set pick(line,status) 0
      catch {$w marker delete line_pick_tmp}
      cancel_button2 $w
   }

   # -----------------------------------------------------------------
   # Show cursor position in the balloon window
   # -----------------------------------------------------------------
   proc show_position {w mx my RX RY} {
      # global --------------------
      global gbl wk
      
      # Main --------------------
      set x [format "%.3f" [$w axis invtransform x $mx]]
      set t [format "%.3f" [$w axis invtransform y $my]]
      if [coord inout2 $x $t] { return }
      set x2 [expr ( $gbl(x_axis,top) == 1 ) ? $x : [expr $x + $gbl(obs,x)] ]
      show_position_balloon [format "x=%.3f (%.3f), t=%.3f" $x $x2 $t] $RX $RY
   }

   # -----------------------------------------------------------------
   # Show position balloon with shot number
   # -----------------------------------------------------------------
   proc show_position_shot {w mx my RX RY} {
      set gx [$w axis invtransform x $mx]
      set near_rh [getrh nearest $gx]
      set rt [$w axis invtransform y $my]
      if [coord inout2 $gx $rt] { return }
      set tt [coord rt2tt $gx $rt]
      set sn [lindex $near_rh 0]
      set obsn [lindex $near_rh 2]
      report_position_shot $gx $tt $rt $sn $obsn $RX $RY
   }

   # -----------------------------------------------------------------
   # Show pick mark info
   # -----------------------------------------------------------------
   proc show_pick_info {w mx my PX PY} {
      set gx [$w axis invtransform x $mx]
      set rt [$w axis invtransform y $my]
      report_pick_info $gx $my $rt $PX $PY
   }
   # -----------------------------------------------------------------
   # Select region for pick marks
   # -----------------------------------------------------------------
   proc change_pick_start {w mx my} {
      # global --------------------
      global change_rect
      
      # main --------------------
      set change_rect(status) 1
      set change_rect(x1) $mx
      set change_rect(y1) $my
      
      # temporal binding
      bind $w <B3-Motion> {
	 change_pick_motion %W %x %y
      }
      bind $w <ButtonRelease-3> {
	 change_pick_end %W %x %y %X %Y
      }
   }
   # -----------------------------------------------------------------
   # while selecting a region for pick property
   # -----------------------------------------------------------------
   proc change_pick_motion {w mx my} {
      # global --------------------
      global change_rect
      
      # main --------------------
      update
      if { ! $change_rect(status) } {
	 bind $w <B3-Motion> ""
	 return
      }
      # show a temporal region
      set x1 [$w axis invtrans x $change_rect(x1)]
      set y1 [$w axis invtrans y $change_rect(y1)]
      set x2 [$w axis invtrans x $mx]
      set y2 [$w axis invtrans y $my]
      $w marker create polygon -name selection\
	  -coord "$x1 $y1 $x1 $y2 $x2 $y2 $x2 $y1"\
	  -outline red -linewidth 2 -fill ""
   }
   
   # -----------------------------------------------------------------
   # end selecting a region for pick property
   # -----------------------------------------------------------------
   proc change_pick_end {w mx my RX RY} {
      # global --------------------
      global change_rect
      
      # main --------------------
      set x1 [$w axis invtrans x $change_rect(x1)]
      set y1 [$w axis invtrans y $change_rect(y1)]
      set x2 [$w axis invtrans x $mx]
      set y2 [$w axis invtrans y $my]
      
      # show the region
      $w marker create polygon -name selection\
	  -coord "$x1 $y1 $x1 $y2 $x2 $y2 $x2 $y1"\
	  -outline green -linewidth 2 -fill {}
      
      # invoke pick property window
      change_pick_property $w $change_rect(x1) $mx $change_rect(y1) $my $RX $RY
      
      # erase the region
     catch { tkwait window .change_pick_property}
      $w marker delete selection
   }
   # -----------------------------------------------------------------
   # Plot all the multiple marks
   # -----------------------------------------------------------------
   proc plot_all_multiple {dt} {
      # global --------------------
      global wk pm
      
      # main --------------------
      set x {}
      set t {}
      clear_multiple
      foreach d $pm { 
	 scan $d "%d%s%s%*s%*d" sn px pt
	 lappend x $px
	 lappend t [pick tt2gt $sn $px [expr $pt + $dt]]
      }
      $wk(w,canvas,g) element create multiples -pixel 1.5 -linewidth 0\
	  -xdata $x -ydata $t -symbol circle -scalesym 1\
	  -outline $wk(color,multiple) -color $wk(color,multiple)
   }
   # -----------------------------------------------------------------
   # Erase multiple marks
   # -----------------------------------------------------------------
   proc clear_multiple {} {
      # global --------------------
      global wk
      
      # main --------------------
      if [$wk(w,canvas,g) element exists multiples] {
	 $wk(w,canvas,g) element delete multiples
      }
   }

   # -----------------------------------------------------------------
   # Measure apprarent velocity (start to draw a line)
   # -----------------------------------------------------------------
   proc vel_line_start {w mx my} {
      # global --------------------
      global wk
      
      # main --------------------
      set wk(vline,x0) [$w axis invtransform x $mx]
      set wk(vline,t0) [$w axis invtransform y $my]
      bind $w <B2-Motion> "GraphCanvas::vel_line_move $w %x %y"
      bind $w <ButtonRelease-2> "GraphCanvas::vel_line_end $w %x %y %X %Y"
   }
   # -----------------------------------------------------------------
   # Measure apprarent velocity (move)
   # -----------------------------------------------------------------
   proc vel_line_move {w mx my} {
      # global --------------------
      global wk
      
      # main --------------------
      set x [$w axis invtransform x $mx]
      set t [$w axis invtransform y $my]
      $w marker create line -name tmp_vel_line\
	  -coord "$wk(vline,x0) $wk(vline,t0) $x $t" -outline green
   }
   # -----------------------------------------------------------------
   # Measure apprarent velocity (finish)
   # -----------------------------------------------------------------
   proc vel_line_end {w mx my RX RY} {
      # global --------------------
      global wk
      
      # main --------------------
      if { [$w marker names tmp_vel_line] != "" } {
	 $w marker delete tmp_vel_line
      }
      set x [$w axis invtransform x $mx]
      set t [$w axis invtransform y $my]
      set n [llength [$w marker names vel_line*]]
      $w marker create line -name vel_line$n\
	  -coord "$wk(vline,x0) $wk(vline,t0) $x $t" -outline green
      vel_line_result $wk(vline,x0) [coord rt2tt $wk(vline,x0) $wk(vline,t0)]\
	  $x [coord rt2tt $x $t] $RX $RY
   }
   # -----------------------------------------------------------------
   # Clear velocity line
   # -----------------------------------------------------------------
   proc draw_all_vline {} {
      # global --------------------
      global wk
      
      # main --------------------
      set w $wk(w,canvas,g)
      set tags [$w marker names vel_line*]
      if { $tags != "" } {
	 eval $w marker delete $tags
      }
      set i 0
      foreach d $wk(vline,data) {
	 scan $d "%s %s %s %s %s %s" x1 x2 t1 t2 vel t
	 set t1 [coord tt2rt $x1 $t1]
	 set t2 [coord tt2rt $x2 $t2]
	 $w marker create line -name vel_line$i\
	     -coord "$x1 $t1 $x2 $t2" -outline green
	 incr i
      }
   }
   # -----------------------------------------------------------------
   # Make Print command
   # -----------------------------------------------------------------
   proc make_print_command {outfn} { 
      global paste ps_page_long ps_page_short print fontmap wk
      update_paper_size
      set w $wk(w,canvas,g)
      set landscape [expr {$print(orientation) == "Landscape"} ? {1} : {0}]
      if { $print(normalize) == "h" } {
	 # normalized by x
	 set pagex [format "%sc" $ps_page_short]
	 set pagey [format "%sc" $ps_page_long]
      } else {
	 # normalized by t
	 set pagex [format "%sc" $ps_page_short]
	 set pagey [format "%sc" $ps_page_long]
      }
      $w postscript conf -fontmap fontmap -colormode $print(mode)\
	  -center 1 -decorations 0 -maxpect 1 -landscape $landscape\
	  -width $paste(graph_width) -height $paste(graph_height)\
	  -paperwidth $pagex -paperheight $pagey
      return "$w postscript output $outfn"
   }
   # -----------------------------------------------------------------
   # Hide (show off) pick markers
   # -----------------------------------------------------------------
   proc hide_picks {g} {
      eval $g elem conf [$g elem names pick*] -hide 1
   }
   # -----------------------------------------------------------------
   # Show pick markers
   # -----------------------------------------------------------------
   proc show_picks {g} {
      eval $g elem conf [$g elem names pick*] -hide 0
   }
   # -----------------------------------------------------------------
   # dummy procedure for null click
   # -----------------------------------------------------------------
   proc dummy {} {
      return
   }
   # -----------------------------------------------------------------
   # Clear zero level
   # -----------------------------------------------------------------
   proc clear_zero_level {g} {
      set tags [$g marker names zero*]
      if { $tags != "" } {
	 eval $g marker delete $tags
      }
   }
   # -----------------------------------------------------------------
   # Plot zero level
   # -----------------------------------------------------------------
   proc plot_zero_level {g xlist color} {
      global paste
      foreach x $xlist {
	 $g marker create line -coord {$x $paste(top) $x $paste(bottom)}\
	     -linewidth 1 -name [format "zero%s" $x] -outl $color
      }
   }
   # -----------------------------------------------------------------
   # Clear water wave
   # -----------------------------------------------------------------
   proc clear_water_wave {g} {
      set tags [$g marker names waterwave*]
      if { $tags != "" } {
	 eval $g marker delete $tags
      }
   }
   # -----------------------------------------------------------------
   # Plot water wave
   # -----------------------------------------------------------------
   proc plot_water_wave {g coord color} {
      if { [ llength $coord ] < 3 } { return }
      eval $g marker create line -coord {$coord} -linewidth 2\
	  -name [format "waterwave%s" [lindex $coord 0]] -outl $color
   }
   # -----------------------------------------------------------------
   # Magnify single trace
   # -----------------------------------------------------------------
   proc magnify_trace {w mx my} {
      # global --------------------
      global wk
      
      # main --------------------
      # coordinates
      set gx [$w axis invtransform x $mx]
      set near_rh [getrh nearest $gx]
      set rt [$w axis invtransform y $my]
      set wk(trace,current,dist) [lindex $near_rh 1]
      
      # valid ranve?
      if [coord inout2 $gx $rt] {
	 print_message "Warning: Out of record section.\n"
      }
      # invoke trace window
      trace_window
      
      # compute coordinates
      trace_coordinate
      
      # show the trace
      display_trace 0
   }
   # -----------------------------------------------------------------
   # clear strings
   # -----------------------------------------------------------------
   proc clear_string {g} {
      eval $g marker delete [$g marker names str*]
   }
   # -----------------------------------------------------------------
   # Plot a string
   # -----------------------------------------------------------------
   proc plot_string {g x t str anc fn col shadow shadow_col} {
      set t [coord tt2rt $x $t]
      if { $shadow >0 } {
	 $g marker create text -name strshadow$x\_$t -text $str -anchor $anc\
	     -coord {$x $t} -font $fn -xoffset $shadow -yoffset $shadow\
	     -outline $shadow_col -background ""
      }
      $g marker create text -name str$x\_$t -text $str -anchor $anc\
	  -coord {$x $t} -font $fn -outline $col -background ""
   }
   # -----------------------------------------------------------------
   # draw hyperbola
   # -----------------------------------------------------------------
   proc draw_hyperbola {} {
      # global variable
      global paste wk
      
      # main ------------------------------
      # Clear
      eval $wk(w,canvas,g) marker delete\
	  [$wk(w,canvas,g) marker names hyperbola]
      # Compute hyperbola coordiantes
      set xy {}
      set PLOT_NO 100
      set v2 [expr $paste(stack,vel)*$paste(stack,vel)]
      set t2 [expr $paste(stack,time)*$paste(stack,time)]
      for {set i 0} {$i <= $PLOT_NO} {incr i} {
	 set gx [expr $paste(left) + $i*($paste(right) - $paste(left))\
		     /double($PLOT_NO)]
	 set rt [coord tt2rt $gx [expr sqrt($t2 + $gx*$gx/$v2)]]
	 set xy [concat $xy $gx $rt]
      }
      # Draw it!
      eval {$wk(w,canvas,g) marker create line} -coord {$xy} -linewidth 2\
	  -name hyperbola -outl green
   }
   # -----------------------------------------------------------------
   # draw sub plot
   # -----------------------------------------------------------------
   proc draw_sub_plot {g} {
      # sub plot is not capable in graph canvas
      return
   }   
   # --------------------------------------------------------------------
   # Double click button 1
   # --------------------------------------------------------------------
   proc double_click1 {g mx my} {
      global paste
      set mx [$g axis invtransform x $mx]
      set my [$g axis invtransform y $my]
      if { $mx < $paste(left) } {
	 shiftsection -1 0
      }
      if { $mx > $paste(right) } {
	 shiftsection 1 0
      }
      if { $my < $paste(top) } {
	 shiftsection 0 -1
      }
      if { $my > $paste(bottom) } {
	 shiftsection 0 1
      }
   }
}
