# --------------------------------------------------------------------
# Invoke header-words window
# --------------------------------------------------------------------
proc headerwords_window {} {
   # Global --------------------------------------------------
   global gbl wk file_name custom

   # Main --------------------------------------------------
   if { [check_su_mode] == 0 } {return}
   if {$gbl(data,type) != 2  && $gbl(data,type) < 5} {
      tk_messageBox -icon error --type ok\
	  -message "Header words window requires SU/SEGY data."
      return
   }

   set w .hw
   set ff $w.file_frame
   set mf $w.main_frame
   set geometry ""
   if { [winfo exist $w] } {
      wm deiconify $w
      update
      set geometry [wm geometry $w]
      set ww [wait_window "Regenerating..."]
      foreach cw [winfo children $mf] {
	 destroy $cw
      }
   } else {
      set ww [wait_window "Header window..."]

      # --------------------------------------------------
      # invoke window ------------------------------------
      toplevel $w
      wm title $w "PasteUp: Header words window"
      wm resizable $w 1 0
      if {$geometry != ""} {
	 wm geometry $w $geometry
      }
      basic_bindings $w
      
      # menu/tool bar ------------------------------------
      set tb [frame $w.toolbar -relief raised -bd 2]
   
      # File menu ----------
      set fm $tb.file
      set m $fm.menu
      menubutton $fm -text "File" -menu $m -relief flat
      menu $m -tearoff 0
      $m add command -label "Update SU/SEG-Y file"\
	  -command "headerwords_update_wavefile"
      $m add command -label "Save header words in ASCII format"\
	  -command "headerwords_save_ascii"
      $m add separator
      $m add command -label "Dismiss" -command "headerwords_window_quit $w" 
      
      # Header words menu ----------
      set hwm $tb.headerwords
      set m $hwm.menu
      menubutton $hwm -text "HeaderWords" -menu $m -relief flat
      menu $m -tearoff 0
      $m add command -label "Major header words list"\
	  -command "headerwords_selection major"
      $m add command -label "All header words list"\
	  -command "headerwords_selection all"
      
      # Option menu ----------
      set om $tb.optionmenu
      set m $om.menu
      menubutton $om -text "Option" -menu $m -relief flat
      menu $m -tearoff 0
      # Load module-list
      $m add check -label "Show date" -variable gbl(hw,toggle_date)\
	  -command "headerwords_restart"
      $m add check -label "Show SHOT position" -variable gbl(hw,toggle_sxsy)\
	  -command "headerwords_restart"
      $m add check -label "Show REC. position" -variable gbl(hw,toggle_gxgy)\
	  -command "headerwords_restart"
      
      # Help menu ----------
      set hm $tb.help
      set m $hm.menu
      menubutton $hm -text "Help" -menu $m -relief flat
      menu $m -tearoff 0
      $m add command -label "Header words list"\
	  -command "SUfilter::show_headerwords"
      pack $fm $hwm $om -side left
      pack $hm -side right

      # filename frame -----------------------------------
      labelframe $ff -text "Data file" -relief ridge
      label $ff.name -text $file_name(wave) -anchor w -relief flat
      pack $ff.name -side left -expand yes -fill x

      # button frame -------------------------------------
      set bf [button_frame "" $w.button_frame dismiss\
		  "headerwords_window_quit $w"]
      wm protocol $w WM_DELETE_WINDOW "headerwords_window_quit $w" 

      # header word frame
      labelframe $mf -relief ridge -text "Header words"

      # packing 
      grid $tb -sticky news
      grid $ff -sticky news
      grid $mf -sticky news
      grid $bf -sticky news
   }

   # --------------------------------------------------
   # header words -------------------------------------
   set TEXT_HEIGHT 33
   set LISTBOX_HEIGHT 24

   # scrollbars and main window(text widget) ----------
   set hw $mf.headerwords
   set hs [scrollbar $mf.hscroll -orient horiz -command "$hw xview" -width 10]
   text $hw -xscrollcommand "$hs set" -height $TEXT_HEIGHT

   # listbox frame
   set embed [frame $hw.embedded]
   set vs\
       [scrollbar $mf.vscroll -command "headerwords_vscroll $embed" -width 10]
   set n 0
   foreach key $gbl(hw,keylist) {
      listbox $embed.$key -borderwidth 0 -height $LISTBOX_HEIGHT -width 11\
 	  -yscrollcommand "headerwords_lbscroll $vs" -selectmode extended
      button $embed.legend($key) -text $key -relief raised -bd 1 -pady 0\
	  -command "headerwords_edit_all_trace $key"
      grid $embed.legend($key) -row 0 -column $n -sticky news
      grid $embed.$key -row 1 -column $n -sticky news
      incr n
   }
   $hw window create end -window $embed -stretch yes
   $hw conf -state disabled

   # update listbox 
   set wk(hw,mainlistbox) $embed
   headerwords_update_listbox $embed $gbl(hw,keylist)

   # reference window (tracl) ----------
   set ref $mf.ref
   text $ref -height $TEXT_HEIGHT -width 6
   set embed [frame $ref.embedded]
   listbox $embed.tracl -borderwidth 0 -height $LISTBOX_HEIGHT -width 6\
       -yscrollcommand "headerwords_lbscroll $vs" -selectmode extended
   button $embed.legend -text tracl -relief raised -bd 1 -pady 0 -padx 1
   grid $embed.legend -sticky news
   grid $embed.tracl -sticky news
   $ref window create end -window $embed -stretch yes
   $ref conf -state disable

   # update listbox
   set wk(hw,reflistbox) $embed
   headerwords_update_listbox $embed "tracl"

   # date&position window ----------
   set dpf $mf.date_position
   text $dpf -height $TEXT_HEIGHT
   set embed [frame $dpf.embedded]
   set n 0
   
   set wk(hw,datepos_showlist) {}
   if {$gbl(hw,toggle_date)} { lappend wk(hw,datepos_showlist) date }
   if {$gbl(hw,toggle_sxsy)} { lappend wk(hw,datepos_showlist) shot_x shot_y }
   if {$gbl(hw,toggle_gxgy)} { lappend wk(hw,datepos_showlist) rec_x rec_y }

   foreach key $wk(hw,datepos_showlist) {
      listbox $embed.$key -borderwidth 0 -height $LISTBOX_HEIGHT -width 17\
 	  -yscrollcommand "headerwords_lbscroll $vs"
      button $embed.legend($key) -text $key -relief raise -bd 1 -pady 0\
	  -state disabled -disabledfore $custom(*Foreground)
      grid $embed.legend($key) -row 0 -column $n -sticky news
      grid $embed.$key -row 1 -column $n -sticky news
      incr n
   }
   $dpf window create end -window $embed -stretch yes
   $dpf conf -width [expr $n * 17]
   $dpf conf -state disable

   # update listbox
   set wk(hw,dplistbox) $embed
   headerwords_update_dateposition $embed

   # placement ----------
   grid $vs -row 0 -column 0 -sticky news
   grid $ref -row 0 -column 1 -sticky news
   grid $hw -row 0 -column 3 -sticky news
   grid $hs -row 1 -column 3 -sticky news
   if { $gbl(hw,toggle_date) } {
      grid $dpf -row 0 -column 2 -sticky news
   }

   grid rowconfig    $mf 0 -weight 1 -minsize 0
   grid columnconfig $mf 3 -weight 1 -minsize 0

   # binding --------------------
   headerwords_listbox_binding

   # --------------------------------------------------
   # initializing -------------------------------------
   set wk(hw,prevselect) 0
   set wk(hw,edititem,flag) {}
   set wk(hw,n_trace) [$wk(hw,reflistbox).tracl index end]

   destroy $ww
}

# --------------------------------------------------------------------
# Update SU/SEGY waveform data file
# --------------------------------------------------------------------
proc headerwords_update_wavefile {} {
   # Global --------------------------------------------------
   global wk file_name TMPDIR

   # main --------------------------------------------------
   
   # check flag ------------------------------
   set n [llength $wk(hw,edititem,flag)]
   if { $n < 1} {  return }
   
   # confirmation ------------------------------
   set ans [tk_messageBox -type yesno -icon question\
		-message "Data file is going to be overwritten. Are you sure?"]
   if { $ans != "yes" } { 
      print_message "Header words update is canceled.\n"
      return 
   }
   
   # read listbox ------------------------------
   foreach key $wk(hw,edititem,flag) {
      if { $key == "tracl" } {
	 set w $wk(hw,reflistbox)
      } else {
	 set w $wk(hw,mainlistbox)
      }
      set dat($key) [$w.$key get 0 end]
   }

   # make ASCII file ------------------------------
   set tmpfn(ascii) [format "%s/pasteup.hw.ascii.%s" $TMPDIR [pid]]
   set fp [open $tmpfn(ascii) "w"]
   for {set i 0} {$i < $wk(hw,n_trace) } {incr i} {
      set line {}
      foreach key $wk(hw,edititem,flag) {
	 set item [lindex $dat($key) $i]
	 lappend line $item
      }
      puts $fp $line
   }
   close $fp

   # convert into bindary file ------------------------------
   set tmpfn(binary) [format "%s/pasteup.hw.binary.%s" $TMPDIR [pid]]
   set com [format "a2b n1=%s < %s > %s" $n $tmpfn(ascii) $tmpfn(binary)]
   set stat [exec_external_command $com "A2B"]
   file delete -force -- $tmpfn(ascii)
   if { $stat == 1 } { 
      file delete -force -- $tmpfn(binary)
      return 
   }

   # Apply ------------------------------
   set tmpfn(wave) [format "%s/pasteup.hw.wavedata.%s" $TMPDIR [pid]]
   set com [make_su_command 1\
		[format "| sushw key=%s infile=%s"\
		     [join $wk(hw,edititem,flag) ","] $tmpfn(binary)]\
	       $file_name(wave) $tmpfn(wave) "" ""]
   set stat [exec_external_command $com "SUSHW"]
   file delete -force -- $tmpfn(binary)
   if { $stat == 1 } { 
      file delete -force $tmpfn(wave)
      return 
   }
   file rename -force $tmpfn(wave) $file_name(wave)

   # fin
   print_message "\"$file_name(wave)\" is updated.\n"
   set wk(hw,edititem,flag) {}
}

# --------------------------------------------------------------------
# Update contents of header words listbox
# --------------------------------------------------------------------
proc headerwords_update_listbox {w keylist} {
   # Global --------------------------------------------------
   global file_name TMPDIR

   # main --------------------------------------------------
   # check keylist
   if { [llength $keylist] < 1 } { return }

   # read header words
   set tmp [format "%s/pasteup.headerwords.%s" $TMPDIR [pid]]
   set com [make_su_command 0\
		[format "| sugethw key=%s output=geom" [join $keylist ","]]\
		$file_name(wave) $tmp "" ""]
   set stat [exec_external_command $com "SUGETHW"]
   # clear old data
   foreach key $keylist {
      $w.$key delete 0 end
   }
   # update listbox
   set fp [open $tmp]
   while { [gets $fp line] >= 0 } {
      set n 0
      foreach key $keylist {
	 $w.$key insert end [lindex $line $n]
	 incr n
      }
   }
   close $fp
   file delete -force $tmp
}

# --------------------------------------------------------------------
# Update contents of date and position listbox
# --------------------------------------------------------------------
proc headerwords_update_dateposition {w} {
   # Global --------------------------------------------------
   global wk file_name TMPDIR

   # main --------------------------------------------------
   set keylist {scalco counit sx sy gx gy year day hour minute sec}

   # --------------------------------------------------
   # read header words --------------------------------
   set tmp [format "%s/pasteup.headerwords.%s" $TMPDIR [pid]]
   set com [make_su_command 0\
		[format "| sugethw key=%s output=geom" [join $keylist ","]]\
		$file_name(wave) $tmp "" ""]
   set stat [exec_external_command $com "SUGETHW"]

   # --------------------------------------------------
   # clear old data -----------------------------------
   foreach key $wk(hw,datepos_showlist) {
      $w.$key delete 0 end
   }

   # --------------------------------------------------
   # update listbox -----------------------------------
   set fp [open $tmp]
   while { [gets $fp line] >= 0 } {
      # compute position --------------------
      # scale for coordinates
      eval {scan $line "%d%d%d%d%d%d%d%d%d%d%d"} $keylist
      if { $scalco < 0 } {
	 set sc [expr -1.0 / $scalco]
      } else {
	 set sc $scalco
      }
      # coordinate unit
      set counit [lindex $line 1]
      switch $counit {
	 1 -
	 3 {
	    # length (meter or feet) || Decimal degrees
	    set shot_x [expr $sx * $sc ]
	    set shot_y [expr $sy * $sc ]
	    set rec_x [expr $gx * $sc ]
	    set rec_y [expr $gy * $sc ]
	 }
	 2 {
	    # second of arc
	    set shot_x [expr $sx * $sc / 3600.0]
	    set shot_y [expr $sy * $sc / 3600.0]
	    set rec_x [expr $gx * $sc / 3600.0]
	    set rec_y [expr $gy * $sc / 3600.0]
	 }
	 4 {
	    # Degree + minutes + second (DMS)
	    # not implemented yet
	 }
	 default {
	    set shot_x $sx
	    set shot_y $sy
	    set rec_x $gx
	    set rec_x $gy
	    # invalid, unknown
	    # output as is
	 }
      }

      # format date --------------------
      if { $year < 200 } {
	 # convert year: e.g. 99 -> 1999, 104 -> 2004
	 set year [expr $year + 1900]
	 if { $year < 1970 } {
	    set year 1970
	 }
      }
      set day [expr $day - 1]
      set utime [clock scan "$day day" -base\
		     [clock scan "1/1/$year $hour:$minute:$sec"]]
      set date [clock format $utime -format "%Y/%m/%d %T"]

      foreach key $wk(hw,datepos_showlist) {
	 $w.$key insert end [set $key]
      }
   }
   close $fp
   file delete -force $tmp
}

# --------------------------------------------------------------------
# Vertical scrollbar command for headerwords window
# --------------------------------------------------------------------
proc headerwords_vscroll {w args} {
   # Global --------------------------------------------------
   global gbl

   # main --------------------------------------------------
   foreach key $gbl(hw,keylist) {
      eval $w.$key yview $args
   }
}

# --------------------------------------------------------------------
# Listbox scrool commands for headerwords window
# --------------------------------------------------------------------
proc headerwords_lbscroll {vs args} {
   # Global --------------------------------------------------
   global gbl wk

   # main --------------------------------------------------
   # set vertical scrollbar
   eval {$vs set} $args

   # header word listbox
   foreach key $gbl(hw,keylist) {
      eval $wk(hw,mainlistbox).$key yview moveto [lindex $args 0]
   }
   # date/position listbox
   foreach key $wk(hw,datepos_showlist) {
      eval $wk(hw,dplistbox).$key yview moveto [lindex $args 0]
   }
   # reference listbox (tracl)
   eval $wk(hw,reflistbox).tracl yview moveto [lindex $args 0]
}

# --------------------------------------------------------------------
# set bindings for all the listbox
# --------------------------------------------------------------------
proc headerwords_listbox_binding {} {
   # Global --------------------------------------------------
   global gbl wk

   # main --------------------------------------------------
   # header word listbox
   foreach key $gbl(hw,keylist) {
      set w $wk(hw,mainlistbox).$key 
      bind $w <Double-Button-1> "headerwords_listbox_select_current $w"
      bind $w <Button-3> "headerwords_listbox_popup_menu $w %X %Y"
   }
   # date/position listbox
   foreach key $wk(hw,datepos_showlist) {
      set w $wk(hw,dplistbox).$key 
      bind $w <Double-Button-1> "headerwords_listbox_select_current $w"
   }
   # reference listbox (tracl)
   set w $wk(hw,reflistbox).tracl 
   bind $w <Double-Button-1> "headerwords_listbox_select_current $w"
   bind $w <Button-3> "headerwords_listbox_popup_menu $w %X %Y"
}

# --------------------------------------------------------------------
# Popup Button-3 menu
# --------------------------------------------------------------------
proc headerwords_listbox_popup_menu {w x y} {
   # check selection ------------------------------
   set ind [$w curselection]
   set n [llength $ind ] 
   if { $n < 1 } { return }

   # popup menu ------------------------------
   set m .headerwords_listbox_popup
   if [ winfo exist $m ] {
      destroy $m
   }
   menu $m -tearoff 0
   if { $n > 1 } {
      # multiple selection
      $m add command -label "Edit these items"\
	  -command "headerwords_listbox_edit_plural_items $w \"$ind\" $x $y"
   } else {
      # single selection
      $m add command -label "Edit this item"\
	  -command "headerwords_listbox_edit_item $w $ind $x $y"
   }
   tk_popup $m $x $y
}

# --------------------------------------------------------------------
# Highlight a single trace in the header words window
# --------------------------------------------------------------------
# arguments
# ind   : index
# col   : highlight color
# --------------------------------------------------------------------
proc headerwords_highlight_single_trace {ind col} {
   # Global --------------------------------------------------
   global gbl wk
   
   # main --------------------------------------------------
   # header word listbox
   foreach key $gbl(hw,keylist) {
      $wk(hw,mainlistbox).$key itemconf $ind -background $col
   }
   # date/position listbox
   foreach key $wk(hw,datepos_showlist) {
      $wk(hw,dplistbox).$key itemconf $ind -background $col
   }
   # reference listbox (tracl)
   $wk(hw,reflistbox).tracl itemconf $ind -background $col
}

# --------------------------------------------------------------------
# Select a trace in the listbox
# --------------------------------------------------------------------
proc headerwords_listbox_select_current {w} {
   # Global --------------------------------------------------
   global wk custom

   # main --------------------------------------------------
   set ind [lindex [$w curselection] 0]
   headerwords_highlight_single_trace $wk(hw,prevselect) ""
   headerwords_highlight_single_trace $ind $custom(*selectColor)
   set wk(hw,prevselect) $ind
}

# --------------------------------------------------------------------
# Edit an item in the listbox $pw, index $ind
# --------------------------------------------------------------------
proc headerwords_listbox_edit_item {pw ind x y} {
   # Global --------------------------------------------------
   global wk file_name

   # main --------------------------------------------------
   set w .headerwords_listbox_edit_item
   if { [winfo exist $w] } {
      wm deiconify $w
      destroy $w
   }
   toplevel $w
   wm title $w "PasteUp: Edit an item"
   wm geometry $w +$x+$y

   # --------------------------------------------------
   # Parameter frame ----------------------------------
   set pf [frame $w.parameter_frame]

   # filename
   label $pf.lfn -text "Data file:" -anchor e
   label $pf.fn -text $file_name(wave)  -anchor w
   grid $pf.lfn $pf.fn - -sticky news

   # tracl
   set tracl [$wk(hw,reflistbox).tracl get $ind]
   label $pf.ltracl -text "tracl:"  -anchor e
   label $pf.tracl -text $tracl  -anchor w
   grid $pf.ltracl $pf.tracl -sticky news

   # header words to set ------------------------------
   set key [lindex [split $pw "."] end]
   make_entry_right $pf value "$key:" wk(hw,edititem,a) ""

   # initial value
   set wk(hw,edititem,a) [$pw get $ind]
   set wk(hw,edititem,org) $wk(hw,edititem,a)

   # --------------------------------------------------
   # Button frame -------------------------------------
   set bf [frame $w.button_frame]
   button $bf.ok -text "set" -width 6\
       -command "headerwords_listbox_edit_item_update $pw $ind $key;destroy $w"
   button $bf.cancel -text "cancel" -command "destroy $w"
   grid $bf.ok $bf.cancel -sticky news

   # --------------------------------------------------
   # packing ------------------------------------------
   pack $pf $bf -side top -fill x -expand 1
   
   grab $w
}

# --------------------------------------------------------------------
# Edit plural items in the listbox $pw, indices $ind
# --------------------------------------------------------------------
proc headerwords_listbox_edit_plural_items {pw ind x y} {
   # Global --------------------------------------------------
   global wk file_name custom

   # main --------------------------------------------------
   set w .headerwords_listbox_edit_plural_items
   if { [winfo exist $w] } {
      wm deiconify $w
      destroy $w
   }
   toplevel $w
   wm title $w "PasteUp: Edit items"
   wm geometry $w +$x+$y

   # selection
   headerwords_highlight_single_trace $wk(hw,prevselect) ""
   foreach i $ind {
      $pw itemconf $i -background $custom(*selectColor)
   }
   
   # --------------------------------------------------
   # Parameter frame ----------------------------------
   set pf [frame $w.parameter_frame]

   # filename
   label $pf.lfn -text "Data file:" -anchor e
   label $pf.fn -text $file_name(wave)  -anchor w
   grid $pf.lfn $pf.fn - -sticky news

   # tracl
   set min [$wk(hw,reflistbox).tracl get [lindex $ind 0]]
   set max [$wk(hw,reflistbox).tracl get [lindex $ind end]]
   label $pf.ltracl -text "tracl:"  -anchor e
   label $pf.tracl -anchor w\
       -text [format "%s - %s   (%s-traces)" $min $max [expr $max - $min + 1]]
   grid $pf.ltracl $pf.tracl -sticky news

   # header words to set ------------------------------
   set key [lindex [split $pw "."] end]
   make_entry_right $pf value "$key:" wk(hw,edititem,a) ""

   # initial value
   set wk(hw,edititem,a) [$pw get [lindex $ind 0]]
   set wk(hw,edititem,org) $wk(hw,edititem,a)

   # --------------------------------------------------
   # Button frame -------------------------------------
   set bf $w.button_frame
   frame $bf
   button $bf.ok -text "set" -width 6 -command\
       "headerwords_listbox_edit_items_set $pw \"$ind\" $key;destroy $w"
   button $bf.cancel -text "cancel" -command\
       "foreach i \"$ind\" {$pw itemconf \$i -background \"\" }; destroy $w"
   grid $bf.ok $bf.cancel -sticky news

   # --------------------------------------------------
   # packing ------------------------------------------
   pack $pf $bf -side top -fill x -expand 1
   
   grab $w
}

# --------------------------------------------------------------------
# Edit an item in the listbox $pw, index $ind, header name is $key
# --------------------------------------------------------------------
proc headerwords_listbox_edit_item_update {pw ind key} {
   # Global --------------------------------------------------
   global wk
   
   # main --------------------------------------------------
   # check previous
   if { $wk(hw,edititem,org) == $wk(hw,edititem,a)\
	    || [llength $wk(hw,edititem,a)] < 1 } {
      # not modified or blank entery
      return
   }
   # delete old value
   $pw delete $ind
   # insert new
   $pw insert $ind $wk(hw,edititem,a)
   # add edit flag
   if { [ lsearch $wk(hw,edititem,flag) $key] < 0} { 
      lappend wk(hw,edititem,flag) $key
   }
}

# --------------------------------------------------------------------
# Set plural items in the listbox $pw, indeices $ind, header name is $key
# --------------------------------------------------------------------
proc headerwords_listbox_edit_items_set {pw ind key} {
   # Global --------------------------------------------------
   global wk
   
   # main --------------------------------------------------
   # check value
   if { [llength $wk(hw,edititem,a)] < 1 } {
      # too short
      return
   }
   foreach i $ind {
      # delete old value
      $pw delete $i
      # insert new
      $pw insert $i $wk(hw,edititem,a)
   }
   # add edit flag
   if { [ lsearch $wk(hw,edititem,flag) $key] < 0} { 
      lappend wk(hw,edititem,flag) $key
   }
   # selection
   foreach i $ind {
      $pw itemconf $i -background ""
   }
   $pw selection set [lindex $ind 0] [lindex $ind end]
}

# --------------------------------------------------------------------
# Edit all the trace of $key
# --------------------------------------------------------------------
proc headerwords_edit_all_trace {key} {
   # Global --------------------------------------------------
   global wk file_name custom

   # main --------------------------------------------------
   set w .headerwords_listbox_edit_all_trace
   if { [winfo exist $w] } {
      wm deiconify $w
      destroy $w
   }
   toplevel $w
   wm title $w "PasteUp: Edit header values"
   wm geometry $w

   # --------------------------------------------------
   # highlight listbox --------------------------------
   if { $key == "tracl" } {
      set lb $wk(hw,reflistbox).$key
   } else {
      set lb $wk(hw,mainlistbox).$key
   }
   headerwords_highlight_single_trace $wk(hw,prevselect) ""
   $lb conf -background $custom(*selectColor)

   # --------------------------------------------------
   # title frame --------------------------------------
   set tf [frame $w.title_frame]
   # title
   label $tf.title -text "Pasteup: Edit header word ; $key"\
       -relief groove -bd 2 -font $custom(WindowTitleFont)
   # filename
   label $tf.fn -text "File name: $file_name(wave)" -bd 2 -anchor w -pady 10
   pack $tf.title $tf.fn -side top -fill x -expand yes

   # --------------------------------------------------
   # parameter frame ----------------------------------
   set pf [frame $w.param_frame]
   
   # use "sushw a=, b=, c=, d=..." --------------------
   set af [labelframe $pf.abcd_frame -text "Apply a simple equation"]

   entry $af.abcd(a) -width 5
   entry $af.abcd(b) -width 5
   entry $af.abcd(c) -width 5
   entry $af.abcd(d) -width 5
   label $af.abcd(left)\
       -text "$key\(n) = a + b*(i%j) + c*(i/j);    where i=n+d" -anchor w
   label $af.abcd(la) -text "     a="
   label $af.abcd(lb) -text " b="
   label $af.abcd(lc) -text " c="
   label $af.abcd(ld) -text " d="
   label $af.abcd(lj) -text " j="
   entry $af.abcd(j) -width 5
   button $af.abcd(exec) -text "apply"\
       -command "headerwords_edit_all_trace_abcd $lb $af $key"
   grid $af.abcd(left) - - - - - - - - - - -sticky nws
   grid $af.abcd(la) $af.abcd(a) $af.abcd(lb) $af.abcd(b) $af.abcd(lc)\
       $af.abcd(c) $af.abcd(ld) $af.abcd(d) $af.abcd(lj) $af.abcd(j)\
       $af.abcd(exec) -sticky news

   # initililize
   $af.abcd(a) insert 0 0
   $af.abcd(b) insert 0 0
   $af.abcd(c) insert 0 0
   $af.abcd(d) insert 0 0
   $af.abcd(j) insert 0 $wk(hw,n_trace)

   # read from an ASCII file --------------------
   set rf [labelframe $pf.readASCII_frame -text "Read from an ASCII data file"]
   filename_hscroll_frame $rf datafn "ASCII data file :" wk(hw,ascii_data,fn) \
       40 load {.dat}
   make_spinbox $rf column "Column number" wk(hw,ascii_data,col) 1 50 1 ""
   button $rf.exec -text "read file" \
       -command "headerwords_all_trace_read_file $lb $af $key"
   grid x x x x $rf.exec - -sticky ns

   # packing --------------------
   pack $af $rf -side top -expand yes -fill x -pady 10 -padx 10

   # --------------------------------------------------
   # Button frame -------------------------------------
   set bf [button_frame "" $w.button_frame dismiss \
	      "headerwords_quit_alltrace_edit $w $lb"]

   # --------------------------------------------------
   # packing ------------------------------------------
   pack $tf $pf $bf -side top -fill x -expand 1
   
   grab $w
}

# --------------------------------------------------------------------
# Update contents of header words listbox
# --------------------------------------------------------------------
# arguments:
# mode : "all" or "major"
# --------------------------------------------------------------------
proc headerwords_selection {mode} {
   # Global --------------------------------------------------
   global gbl wk custom

   # main --------------------------------------------------
   
   # --------------------------------------------------
   # confirmation -------------------------------------
   if { $wk(hw,edititem,flag) != {} } {
      set ans [tk_messageBox -icon question -type okcancel\
		   -message "You've modified some header values.  If you select \"ok\", you'll lost all the modifications, and re-read all the header values from the data file.  Are you sure to continue?"]
      if { $ans != "ok" } {
	 return
      }

   }
   
   # --------------------------------------------------
   # new window ---------------------------------------
   set w .hwselect
   if [raise_window $w] { return }
   toplevel $w
   wm title $w "PasteUp: Header words selection"
   wm resizable $w 1 0

   # binding ------------------------------
   basic_bindings $w

   # --------------------------------------------------
   # title frame --------------------------------------
   set tf $w.title_frame
   label $tf -text "Pasteup: Select header words ($mode)" -relief flat\
       -font $custom(WindowTitleFont)

   # --------------------------------------------------
   # parameter frame ----------------------------------
   if { $mode == "major" } {
      # major header words
      set keylist {fldr ep offset scalco scalel LF\
		       sx sy sdepth selev swdep LF\
		       gx gy gelev gwdep LF\
		       tstat laga lagb delrt gstat LF\
		       ns dt LF\
		       year day hour minute sec LF}
   } else {
      # all the header words
      set keylist {tracr fldr tracf ep cdp cdpt trid LF\
		       nvs nhs duse offset LF\
		       gelev selev sdepth gdel sdel swdep gwdep LF\
		       scalel scalco sx sy gx gy LF\
		       counit wevel swevel sut gut sstat gstat LF\
		       tstat laga lagb delrt LF\
		       muts mute ns dt gain igc igi corr LF\
		       sfs sfe slen styp stas stae tatyp LF\
		       afilf afils nofilf nofils LF\
		       lcf hcf lcs hcs LF\
		       year day hour minute sec timebas LF\
		       trwf grnors grnofr grnlof gaps otrav LF}
   }

   set pf [labelframe $w.parameter_frame -text "header words list"]
   set wlist {}
   foreach key $keylist {
      if { $key == "LF" && [llength $wlist] > 0} {
	 eval grid $wlist -sticky news
	 set wlist {}
	 continue
      }
      if { [lsearch $gbl(hw,keylist) $key] >= 0 } {
	 set wk(hw,check,$key) 1
      } else {
	 set wk(hw,check,$key) 0
      }
      checkbutton $pf.b($key) -var wk(hw,check,$key)
      label $pf.l($key) -text "$key" -width 9 -anchor w
      lappend wlist $pf.b($key) $pf.l($key)
   }

   # --------------------------------------------------
   # button frame -------------------------------------
   set bf $w.button_frame
   frame $bf -relief flat
   button $bf.dismiss -text "dismiss" -width 15 -bd 2\
       -bg $custom(DismissButtonBackground)\
       -command "headerwords_regenerate_window $w \"$keylist\"" 
   grid $bf.dismiss -sticky ns

   # --------------------------------------------------
   # packing ------------------------------------------
   pack $tf $pf $bf -side top -fill x -expand yes

   grab $w
}

# --------------------------------------------------------------------
# Update contents of header words listbox
# --------------------------------------------------------------------
# arguments:
#  w       : widget name of header words selection window
#  keylist : header key list in the $w.
# --------------------------------------------------------------------
proc headerwords_regenerate_window {w keylist} {
   # Global --------------------------------------------------
   global gbl wk

   # main --------------------------------------------------

   # --------------------------------------------------
   # update keys --------------------------------------
   set gbl(hw,keylist) {}
   foreach key $keylist {
      if { $key == "LF" } { continue }
      if { $wk(hw,check,$key) == 1} {
	 lappend gbl(hw,keylist) $key
      }
   }

   # --------------------------------------------------
   # destroy selection window -------------------------
   destroy $w
   headerwords_window
}

# --------------------------------------------------------------------
# Update contents of header words listbox
# --------------------------------------------------------------------
# arguments:
#  w       : window name of all_trace_edit window
#  lb      : listbox widget name under edit.
# --------------------------------------------------------------------
proc headerwords_quit_alltrace_edit {w lb} {
   # Global --------------------------------------------------
   global custom

   # main --------------------------------------------------
   # unhighlight listbox
   $lb conf -background $custom(*Entry.Background)

   # destroy window
   destroy $w
}

# --------------------------------------------------------------------
# Edit all the traces in the listbox $lb by like "sushw a= b= c=..."
#  lb     : listbox widget
#  w      : widget name of entries
#  key    : header name
# --------------------------------------------------------------------
proc headerwords_edit_all_trace_abcd {lb w key} {
   # Global --------------------------------------------------
   global wk

   # main --------------------------------------------------

   # get a, b, c, d and j from entry widget
   foreach val {a b c d j} {
      set tmp [$w.abcd($val) get]
      set $val [regsub -all \[\\s\] $tmp ""]
   }
   if { [string length $a] < 1} { set a 0 }
   if { [string length $b] < 1} { set b 0 }
   if { [string length $c] < 1} { set c 0 }
   if { [string length $d] < 1} { set d 0 }
   if { [string length $j] < 1} { set j $wk(hw,n_trace) }

   # clear old data
   $lb delete 0 end

   # update
   for {set n 0} {$n < $wk(hw,n_trace)} {incr n} {
      set i [expr $n + $d]
      $lb insert end [expr $a + $b * ($i % $j) + $c * ($i / $j)]
   }

   # add edit flag
   if { [ lsearch $wk(hw,edititem,flag) $key] < 0} { 
      lappend wk(hw,edititem,flag) $key
   }
}

# --------------------------------------------------------------------
# Edit all the trace in the listbox $lb by reading from file
#  lb     : listbox widget
#  w      : widget name of entries
#  key    : header name
# --------------------------------------------------------------------
proc headerwords_all_trace_read_file {lb w key} {
   # Global --------------------------------------------------
   global wk

   # main --------------------------------------------------

   # check file existence
   if { ![ file readable $wk(hw,ascii_data,fn) ] } {
      tk_messageBox -icon error -type ok\
	  -message "File \"$wk(hw,ascii_data,fn)\" cannot readable."
      return
   }
   
   # read data from the file
   set fp [open $wk(hw,ascii_data,fn)]
   set col [expr $wk(hw,ascii_data,col) - 1]
   set data {}
   set n 0
   while { [gets $fp line] >= 0 } {
      incr n
      if { [llength $line] <= $col } {
	 tk_messageBox -icon error -type ok\
	     -message "Number of columns is short.  (line $n)"
	 return
      }
      lappend data [lindex $line $col]
      if { $n >= $wk(hw,n_trace) } { break }
   }
   catch {close $fp}
   # adjust number of data
   while { [llength $data] < $wk(hw,n_trace) } {
      set data [concat $data $data]
   }

   # clear old data
   $lb delete 0 end

   # update
   for {set n 0} {$n < $wk(hw,n_trace)} {incr n} {
      $lb insert end [lindex $data $n]
   }

   # add edit flag
   if { [ lsearch $wk(hw,edititem,flag) $key] < 0} { 
      lappend wk(hw,edititem,flag) $key
   }
}

# --------------------------------------------------------------------
# Restart headerwords window after confirmation
# --------------------------------------------------------------------
proc headerwords_restart {} {
   # Global --------------------------------------------------
   global gbl wk

   # main ------------------------------
   if { $wk(hw,edititem,flag) != {} } { 
      set ans [tk_messageBox -icon question -type okcancel\
		   -message "You've modified some header values.  If you select \"ok\", you'll lost all the modifications.  Are you sure to continue?"]
      if { $ans != "ok" } {
	 return
      }
   }
   headerwords_window
}

# --------------------------------------------------------------------
# Quit headerwords window with confirmation
# --------------------------------------------------------------------
proc headerwords_window_quit {w} {
   # Global --------------------------------------------------
   global wk

   # main ------------------------------
   if { $wk(hw,edititem,flag) != {} } { 
      set ans [tk_messageBox -icon question -type okcancel\
		   -message "You've modified some header values.  If you select \"ok\", you'll lost all the modifications.  Are you sure to quit?"]
      if { $ans != "ok" } {
	 return
      }
   }
   destroy $w
}

# --------------------------------------------------------------------
# Save visible header woreds in an ASCII file
# --------------------------------------------------------------------
proc headerwords_save_ascii {} {
   # Global --------------------------------------------------
   global gbl wk

   # main ------------------------------
   # determine filename
   file_selection wk(hw,ascii_data,outfn) save txt
   if { $wk(hw,ascii_data,outfn) == "" } { return }

   # write header words --------------------
   set fp [open $wk(hw,ascii_data,outfn) w]
   # comments
   set str "\#"
   foreach key [concat "tracl" $wk(hw,datepos_showlist) $gbl(hw,keylist)] {
      set fmt [format "%%s%%%ds" [headerwords_key_width $key]]
      set str [format $fmt $str $key]
   }
   puts $fp $str

   # write data body --------------------
   set n [$wk(hw,reflistbox).tracl size]
   for {set i 0} {$i < $n} {incr i} {
      set str " "
      foreach {w keylist} "$wk(hw,reflistbox) {tracl}\
			  $wk(hw,dplistbox) \"$wk(hw,datepos_showlist)\"\
			  $wk(hw,mainlistbox) \"$gbl(hw,keylist)\"" {
	 foreach key $keylist {
	    set fmt [format "%%s%%%ds" [headerwords_key_width $key]]
	    set str [format $fmt $str [$w.$key get $i]]
	 }
      }
      puts $fp $str
   }
   close $fp
   print_message [format \
		      "Header info was written in an ASCII file \"%s\""\
		      $wk(hw,ascii_data,outfn)]
}

# --------------------------------------------------------------------
# Proper width (char length) of the key
# --------------------------------------------------------------------
proc headerwords_key_width {key} {
   switch -- $key {
      date {
	 return 21
      }
      shot_x -
      shot_y {
	 return 15
      }
      laga -
      lagb -
      tstat -
      delrt -
      scalco -
      counit {
	 # 2byte intger
	 return 6
      }
      default {
	 # 4byte integer
	 return 12
      }
   }
}