# --------------------------------------------------------------------
# Initialize Pasteup -------------------------------------------------
# --------------------------------------------------------------------

namespace eval Init {
   namespace export initialize

   proc initialize {} {
      # global variables
      global zoomMod change_rect wk
      global vx0
      
      # -----------------------------------------------------------------
      # basic configuration for tk and BLT
      # -----------------------------------------------------------------
      tk useinputmethods 0
      tk appname PasteUp
      set zoomMod "Control-"
      
      # check the X-server dimension
      set com [open "| xdpyinfo | grep dimension" ]
      set dim [split [lindex [gets $com] 1] x]
      set wk(xdimension,width) [lindex $dim 0]
      set wk(xdimension,height) [lindex $dim 1]
      close $com
      set wk(SU,mode) [check_su_environment] ; # check shell status

      # -----------------------------------------------------------------
      # check whether stable/bleeding_edge version
      # -----------------------------------------------------------------
#      check_stable
      
      # -----------------------------------------------------------------
      # Window title font
      # -----------------------------------------------------------------
      declare_title_font
      
      # -----------------------------------------------------------------
      # Declare global CONSTANTS, such as widget paths
      # -----------------------------------------------------------------
      declare_global_constants
      
      # -----------------------------------------------------------------
      # Declare global VARIABLES
      # -----------------------------------------------------------------
      declare_global_variables
      
      # -----------------------------------------------------------------
      # Configurations for balloon help
      # -----------------------------------------------------------------
      initialize_balloon
      
      # -----------------------------------------------------------------
      # Colors/Icons
      # -----------------------------------------------------------------
      declare_VD_colors
      declare_icons
      
      # -----------------------------------------------------------------
      # Declare pick related
      # -----------------------------------------------------------------
      initialize_pick
      
      # -----------------------------------------------------------------
      # custom variables for Pasteup decoration
      # -----------------------------------------------------------------
      customize_switch_theme default
      customize_reset_fonts
      
      # -----------------------------------------------------------------
      # process command line argumernts
      # -----------------------------------------------------------------
      check_argument 
      
      # -----------------------------------------------------------------
      # Font (dependent on dpi of X server)
      # -----------------------------------------------------------------
      declare_fonts
      
      # -----------------------------------------------------------------
      # memory allocation
      # -----------------------------------------------------------------
      malloc
      
      # -----------------------------------------------------------------
      # LOGO
      # -----------------------------------------------------------------
      invoke_logo
      
      # -----------------------------------------------------------------
      # Evaluate parameter file, if necessary
      # -----------------------------------------------------------------
      eval_parameter_file

      # -----------------------------------------------------------------
      # recover user custom theme
      # -----------------------------------------------------------------
      eval_custom
      set wk(SU,mode) [check_su_environment] ; # check after reading preferences

      # -----------------------------------------------------------------
      # Seismic Unix related
      # -----------------------------------------------------------------
      define_su_modules
   }

   # --------------------------------------------------------------------
   # Evaluate parameter file --------------------------------------------
   # --------------------------------------------------------------------
   proc eval_parameter_file {} {
      global wk file_name gbl paste pick print filt macray

      switch $wk(invoke,mode) {
	 "noparam" {
	    # no parameter file
	 }
	 "default" {
	    # default parameter file
	    if [file readable $wk(default_resource)] {
	       source $wk(default_resource)
	    }
	 }
	 "default+wavefile" {
	    # default parameter with specified waveform file and type
	    set tmpfn $file_name(wave)
	    if [file readable $wk(default_resource)] {
	       source $wk(default_resource)
	    }
	    set file_name(wave) $tmpfn
	    auto_type_info $tmpfn
	    set gbl(data,prev_type) -1
	 }
	 "specified" {
	    # specified parameter file
	    if [file readable $wk(resource_file)] {
	       source $wk(resource_file)
	       set gbl(data,prev_type) $gbl(data,type)
	    } else {
	       puts stderr "Parameter file $wk(resource_file) isn't readable."
	       exit
	    }
	 }
	 "specified_both" {
	    # both waveform and parameter file
	    set tmpfn $file_name(wave)
	    set tmplabel $gbl(data,type,label)
	    set tmpbyte $gbl(data,byte)
	    if [file readable $wk(resource_file)] {
	       source $wk(resource_file)
	       set gbl(data,prev_type) $gbl(data,type)
	    } else {
	       puts stderr "Parameter file $wk(resource_file) isn't readable."
	       exit
	    }
	    set file_name(wave) $tmpfn
	    set gbl(data,type,label) $tmplabel
	    set gbl(data,byte) $tmpbyte
	 }
      }
      resource_compatibility

      # no specified parameter file
      if {[lsearch "noparam default default+wavefile" $wk(invoke,mode)] >= 0} {
	 set file_name(postscript) "./pasteup.ps"
	 set file_name(synthetic) ""
	 set paste(label,title) [make_title]
	 set gbl(obs,depth) 0.0
	 set gbl(obs,x) 0.0
	 if ![file exists file_name(pick)] {
	    set file_name(pick) ""
	 }
      }

      # update, if necessary
      set pick(range) [expr ($pick(current_type) - 1)/ $pick(n_color)]

      # No specified data file
      if {[lsearch "default+wavefile specified_both" $wk(invoke,mode)] < 0} {
	 if ![file exists $file_name(wave)] {
	    switch $gbl(data,type) {
	       1 {
		  set file_name(wave) ./ch1.orig
		  set file_name(rh) ./rh.dat
	       }
	       2 {
		  set file_name(wave) [su_search_datafile "."]
	       }
	       3 {
		  set file_name(wave) [thk_search_datafile "."]
	       }
	       4 {
		  set file_name(wave) [chiba_search_datafile "."]
		  set file_name(rh) [chiba_search_rhfile "."]
	       }
	       5 -
	       6 -
	       7 {
		  set file_name(wave) [segy_search_datafile "."]
	       }
	    }
	 }
      }
   }

   # --------------------------------------------------------------------
   # custom info
   # --------------------------------------------------------------------
   proc eval_custom {} {
      global wk DATADIR PREF custom
      if { $wk(invoke,mode) != "vanilla" && [file readable $wk(pref_file)]} {
	 source $wk(pref_file)
      }
      update_env_variable 1
      update_env_variable 0
      
      if { $wk(invoke,mode) != "vanilla" && [file readable $wk(theme_file)]} {
	 source $wk(theme_file)
      }
      if { $wk(invoke,mode) != "vanilla" && [file readable $wk(binding_file)]} {
	 source $wk(binding_file)
      } else {
	 # default bindings
	 source $DATADIR/theme/bind.tcl
      }
   }

   # --------------------------------------------------------------------
   # Font configurations ------------------------------------------------
   # --------------------------------------------------------------------
   proc declare_fonts {} {
      global wk fontmap
      
      set fl [string tolower [font families]]
      set wk(font,families) {times helvetica utopia courier}
      set wk(font,sizes)    {10 12 14 16 18 20 24 28 32 36 40 44 48 56 64 72}
      
      set i 0
      if { $wk(invoke,Xdpi) != "" } {
	 set c [expr $wk(invoke,Xdpi) / 72.0]
      } else {
	 set c [tk scaling]
      }
      foreach f $wk(font,families) {
	 if { [lsearch -exact $fl $f] < 0} { continue }
	 foreach s $wk(font,sizes) {
	    font create font$i -family $f -size $s
	    set fontmap(font$i) [format "%s %d" $f [expr int($s*$c)]]
	    incr i
	 }
      }
      if {$i < 1} {
	 puts stderr "Cannot define any font..."
	 exit
      }
      
      # fixed font
      font create small_fixed -family fixed -size [expr int(5.5*$c)]
      font create normal_fixed -family fixed -size [expr int(8*$c)]

      # for Help window
      font create large_font -family Times -size 16
      
   }

   # --------------------------------------------------------------------
   # Title fonts --------------------------------------------------------
   # --------------------------------------------------------------------
   proc declare_title_font {} {
      global wk

      set fl [string tolower [font families]]
      set wk(font,title) ""
      foreach f {"arial" "utopia" "times" "helvetica" "courier"} {
	 if { [lsearch -exact $fl $f] >= 0} {
	    set wk(font,title) $f
	    break
	 }
      }
      if {$wk(font,title) == ""} {
	 puts stderr "Cannot find valid font for title frame."
	 exit
      }
      switch $wk(font,title) {
	 arial -
	 helvetica -
	 utopia {
	    set wk(title_width) 48
	 }
	 times {
	    set wk(title_width) 52
	 }
	 default {
	    set wk(title_width) 48
	 }
      }
   }
   
   # --------------------------------------------------------------------
   # Declaration of global constants ------------------------------------
   # --------------------------------------------------------------------
   proc declare_global_constants {} {
      global wk DATADIR HOMEDIR

      # Widget paths ------------------------------
      set wk(w,canvas) .cw.canvas_frame
      set wk(w,console) .console
      set wk(w,canvas,c) $wk(w,canvas).c
      set wk(w,canvas,g) $wk(w,canvas).g
      set wk(w,current) $wk(w,canvas,c)
      set wk(w,analysis) .analysis_window
      set wk(w,sufilter) .sufilter_window
      set wk(w,trace) .trace_window

      # Command/Library paths ------------------------------
      set wk(IMAGEDIR)          $DATADIR/image

      # resource(parameter file) related
      set wk(resource_dir) "$HOMEDIR/.pasteup"
      set wk(default_resource) "$wk(resource_dir)/parameters"
      set wk(theme_file) "$wk(resource_dir)/theme"
      set wk(binding_file) "$wk(resource_dir)/bindings"
      set wk(pref_file) "$wk(resource_dir)/preferences"
      set wk(sudb_dir) "$wk(resource_dir)/sudb"

      # color related
      set wk(color,list) {\
			      "#ffff00" "#ff0000" "#ff00ff" "#00ffff" "#0000ff"\
			      "#ff8000" "#00ff80" "#8000ff"\
			      "#ff0080" "#80e000" "#0080ff"\
			      "#ff8080" "#80ff80" "#8080ff"\
			      "#c00000" "#00c000" "#3070b0"\
			      "#c0c000" "#c000c0" "#00c0c0"}
      set wk(color,water) "#ff00ff"
      set wk(color,auxline) "#00ffff"
      set wk(color,multiple) "\#00ff00"

      # related to SU
      set wk(SU,SEGY2SU) segy2su
      set wk(SU,SU2SEGY) segywrite

      # for Drawing mode selection
      set wk(DRAW_MODE,WIGGLE_MODE)       1
      set wk(DRAW_MODE,VA_MODE)           2
      set wk(DRAW_MODE,VA_REDBLUE_MODE)   3
      set wk(DRAW_MODE,VA_WITHOUT_WIGGLE) 4
      set wk(DRAW_MODE,VA_WO_WIGGLE_RB)   5
      set wk(DRAW_MODE,VD)                10
      set wk(DRAW_MODE,WIGGLE+VD)         110
      set wk(DRAW_MODE,VA+VD)             210

      set wk(VD_COLOR,TwoTone)    11
      set wk(VD_COLOR,Gray)       12
      set wk(VD_COLOR,BlueRed1)   13
      set wk(VD_COLOR,BlueRed2)   14
      set wk(VD_COLOR,BlueRed3)   15
      set wk(VD_COLOR,BlueWhite)  16
      set wk(VD_COLOR,BlackRed)   17
      set wk(VD_COLOR,BlackYellow) 18
      set wk(VD_COLOR,BlackWhite) 19
      set wk(VD_COLOR,Rainbow)    20

      # for data_type_label
      set wk(datatype_label,1) "KASA-lab"
      set wk(datatype_label,2) "SU"
      set wk(datatype_label,3) "THK"
      set wk(datatype_label,4) "CHIBA"
      # set wk(datatype_label,5) "SEGY(IASPEI)"
      set wk(datatype_label,6) "SEGY(1Ch)"
      set wk(data,bytelist) {}
      set wk(data,byte,label) ""

      set wk(datatype,list) { "KASA-lab" "SU" "THK" "CHIBA" "SEGY(1Ch)"}

      # for default_accuracy 
      set wk(accuracy,default_list) "0.020 0.050 0.100 0.150 0.200 0.300"
      
      # function key binding related
      set wk(customize,bind,funclist) {
	 none none\
	     "(Re)Draw" draw_pasteup\
	     "Flip distance range and draw" flip_distance_range\
	     "Save Parameters" save_resource\
	     "Recover Parameters" "exec_load_resource $wk(default_resource)"\
	     "Pick Undo" undo_pop\
	     "Pick Redo" redo_pop\
	     "Clear all pick marks" forget_picks\
	     "Toggle message frame" toggle_message_window\
	     "Set distance range automatically" {auto_offset_range ""}\
	     "Set scale factor automatically" {auto_scale_factor}\
	     "Select next data set" {change_data_set +1}\
	     "Select previous data set" {change_data_set -1}\
	     "Plot Multiples" multiple_plot_all\
	     "Erase Multiple marks" clear_multiple\
	     "Plot pick mark type label" pick_label_plot\
	     "Erase pick mark type label" pick_label_erase\
	     "Flip Multiple direction" multiple_flip_direction\
	     "Exec RayTracing" raytrace_exec\
	     "Exec Mapping" mapping_pick_image\
	     "Delete Synthetic files" raytrace_delete_this_site\
	     "Make Traveltime Fields for mapping" mapping_make_tfield\
	     "Set current pick type as mapping pick type" mapping_set_current_type\
	  }
      
      # filter related
      set wk(filter,list) {"Low Pass " "High Pass" "Band Pass" "Band Stop"\
			       "(B)Band Pass" "(B)Low Pass " "(B)High Pass"}

      # helper (html browsers)
      set wk(help,browser_list) {}
      foreach b $wk(help,browsers) {
	 lappend wk(help,browser_list) $b
      }
      if { [llength $wk(help,txt_browsers)] > 0 &&\
	       [llength $wk(help,terminals)] > 0 } {
	 set terminal [lindex $wk(help,terminals) 0]
	 foreach b $wk(help,txt_browsers) {
	    lappend wk(help,browser_list) [format "%s -e %s" $terminal $b]
	 }
      }
   }

   # --------------------------------------------------------------------
   # Declaration of global variables ------------------------------------
   # --------------------------------------------------------------------
   #
   #    wk    : working variables (temporary)
   #    change_rect : change rectangle related (temoporary)
   #    paste : draw related (saved in the parameter file)
   #    pick  : pick related (saved in the parameter file)
   #    taup  : taup-analysis related (saved in the parameter file)
   #    filt  : filter related (saved in the parameter file)
   #    macray: synthetic traveltime related (saved in the parameter file)
   #    print : print related (saved in the parameter file)
   #    gbl   : the other (saved in the parameter file)
   #
   # --------------------------------------------------------------------
   proc declare_global_variables {} {
      global wk paste pick taup filt macray gbl change_rect print 
      global file_name PREF TMPDIR
      
      # file names
      set file_name(wave) ./ch1.su
      set file_name(rh) ./rh.dat
      set file_name(pick) ""
      set file_name(synthetic) ""
      set file_name(postscript) ./pasteup.ps

      # parameters will be saved in prameter file
      set gbl(data,prev_type) 0
      set gbl(data,type,label) "SU"
      set gbl(data,type) 2          ; # SU
      set gbl(data,byte) 14         ; # 4byte float
      set gbl(data,xaxis) 0         ; # offset(0), tracl(1), cdp(2)
      set gbl(data,xaxis_label) "offset"
      set gbl(data,xaxis_direction) 0;# normal(0), reverse(1)
      set gbl(ds,mode) 0            ; # automatic(0), data base(1)
      set gbl(ds,opt,dirname) 1     ; # show directory name(1) or not(0)
      set gbl(ds,opt,extension) 1   ; # show filename extension
      set gbl(ds,opt,relative) 0    ; # use relative path(1) or absolute path(0)
      set gbl(ds,opt,syn) 0         ; # use synthetics in database
      set gbl(ds,db) {}             ; # database of data set
      set gbl(ds,cur,site) -1       ; # current selected site of data set
      set gbl(ds,cur,wave) -1
      set gbl(ds,cur,pick) -1
      set gbl(ds,cur,syn) -1
      set gbl(mode) 1
      set gbl(mode_label) Wiggle
      set gbl(vdcolor) 12
      set gbl(vdcolor,onlyplus) 0
      set gbl(x_sub_tic) 1
      set gbl(t_sub_tic) 1
      set gbl(xtic_int) 5
      set gbl(t_tic_int) 0.5
      set gbl(t_axis) 0
      set gbl(x_axis,top) 1
      set gbl(x_axis,bottom) 1
      set gbl(line_width) 0.3
      set gbl(mark_size) 3
      set gbl(bind) "Analysis Mode"
      set gbl(water_v) 1.50
      set gbl(sediment_v) 1.50
      set gbl(console,data) 1
      set gbl(console,range) 1
      set gbl(console,plot) 1
      set gbl(console,amp) 0
      set gbl(console,filter) 0
      set gbl(obs,channel) 1
      set gbl(obs,lon) 143.0
      set gbl(obs,lat) 39.0
      set gbl(obs,direc) 1
      set gbl(obs,depth) 0.0
      set gbl(taup,water_wave) 1
      set gbl(pos,dw) 0.01
      set gbl(pos,dh) 1.0
      set gbl(pos,dd) 0.1
      set gbl(pos,dt) 0.0
      set gbl(pos,min) -3.0
      set gbl(pos,max) 3.0
      set gbl(pos,maxorder) 80
      set gbl(pos,limit) 0.001
      set gbl(pos,direc) 1
      set gbl(corpick,applyfilter) 0
      set gbl(corpick,srcwin) 0.5
      set gbl(corpick,redvel) 0.0
      set gbl(corpick,winbeg) 0.0
      set gbl(corpick,wtrsw) 1
      set gbl(corpick,winsz) 0.2
      set gbl(mapping,tfdir) ""
      set gbl(mapping,xorg) 0.0
      set gbl(mapping,renum) 0
      set gbl(mapping,km) 0.1
      set gbl(mapping,byte) 2
      set gbl(mapping,tbase) 0.001
      set gbl(mapping,wmode) 4
      set gbl(mapping,wsign) "deeper"
      set gbl(mapping,alpha) 0.2
      set gbl(mapping,wpick) 1.0
      set gbl(mapping,normalize) 0
      set gbl(mapping,angle) 30
      set gbl(mapping,smute) 0.0
      set gbl(mapping,types) 1,2,3
      set gbl(mapping,bin) 0.2
      set gbl(mapping,dx) 0.2
      set gbl(mapping,threshold) 0
      set gbl(mapping,show_position) 0
      set gbl(mapping,view,overlay) 0
      set gbl(mapping,view,plotxy) 0
      set gbl(mapping,view,xydata) ""
      set gbl(mapping,view,xycolor) "yellow"
      set gbl(mapping,view,palette) gray
      set gbl(mapping,view,inv) 1
      set gbl(vmodel,format) 1
      set gbl(vmodel,ray86) ""
      set gbl(vmodel,gm) ""
      set gbl(vmodel,mboundary) 0
      set gbl(vmodel,netcdf) ""
      set gbl(vmodel,cptfile) ""
      set gbl(vmodel,contour) 0.0
      set gbl(vmodel,aspect) 3
      set gbl(vmodel,density) 100
      set gbl(vmodel,show_position) 0
      set gbl(vmodel,width) 480
      set gbl(vmodel,view,plotxy) 0
      set gbl(raytrace,raycolor) "black"
      set gbl(hamen,accuracy) 5.0e-3
      set gbl(hamen,raycode) 0
      set gbl(hamen,filename) "./synthetic.macray"
      set gbl(hamen,rout) 0
      set gbl(hamen,rayfilename) "./raypath.dat"
      set gbl(hamen,twopo) 0
      set gbl(hamen,twopo,type) 1
      set gbl(hamen,raypath) 0
      set gbl(su,autoscale) 1
      set gbl(su,scaleperc) 20
      set gbl(su,autorange) 0
      set gbl(su,autowindow) 0
      set gbl(su,rangedx) 10
      set gbl(su,mod) {}
      set gbl(su,endian) 0
      set gbl(su,mod,DB) {}
      set gbl(option,cache_length) 2
      set gbl(option,maxundo) 5
      set gbl(option,dir_structure) "two"
      set gbl(option,redraw_afterchange) 0
      set gbl(option,readinfo_afterchange) 0
      set gbl(option,dataset_channel) 0
      set gbl(option,showposition) 2
      set gbl(option,default_style_va) 0
      set gbl(option,default_style_vd) 2
      set gbl(option,scaling_factor) 20
      set gbl(option,verbose) 1
      set gbl(option,su,shotnumber) 0
      set gbl(option,su,sitenumber) 2
      set gbl(option,raytrace,preserve) 0
      set gbl(hw,keylist) {tracl offset sx sy dt}
      set gbl(hw,toggle_date) 0
      set gbl(hw,toggle_sxsy) 0
      set gbl(hw,toggle_gxgy) 0
      set gbl(help,browser) [lindex $wk(help,browser_list) 0]
      set gbl(check,sufilter) 0
      set gbl(check,stack)  0
      set gbl(check,filter) 0
      set gbl(check,amp) 0
      set gbl(check,showpick) 1

      set paste(default,graph_width)  890
      set paste(default,graph_height) 600
      set paste(default,message_height) 4
      set paste(default,txratio) 10.0
      set paste(default,trace_width) 400
      set paste(default,trace_height) 550
      set paste(fontsize,title) Medium
      set paste(fontsize,tic) Medium
      set paste(left) -10
      set paste(right) 10
      set paste(top) 0
      set paste(bottom) 5
      set paste(image,mode) 1
      set paste(image,mode,label) "Image"
      set paste(image,format) -1    ; # default is jpeg
      set paste(interpolate,threshold) 0.5
      set paste(reduction) 0.0
      set paste(wave_density) 10
      set paste(wave_distance) 0.0
      set paste(correct,shot_mode) 0
      set paste(frame_design) Normal
      set paste(string,reduction) 1
      set paste(string,filter) 1
      set paste(string,toptick) 1
      set paste(string,bottomtick) 0
      set paste(string,lefttick) 1
      set paste(string,righttick) 0
      set paste(string,filename) 1
      set paste(string,unit) "tx"
      set paste(amp,amplify) 1
      set paste(amp,threshold) 0
      set paste(amp,class) 100
      set paste(amp,correct) 3
      set paste(amp,correct_label) Geometric
      set paste(amp,geometric,standard) 100
      set paste(amp,geometric,power) 1
      set paste(amp,cutoff) 0
      set paste(amp,agc) 0.0
      set paste(amp,ratio) 8
      set paste(stack,mode) slant
      set paste(stack,range) 0.2
      set paste(stack,vel)  5.0
      set paste(stack,time) 5.0
      set paste(sub,axis) 1
      set paste(sub,check) 0
      set paste(sub,color) black
      set paste(sub,data) "swdep"
      set paste(sub,dy) 2000
      set paste(sub,height) 80
      set paste(sub,max) 6000
      set paste(sub,min) 0
      set paste(sub,position) top
      set paste(ratio) 8.0
      set paste(acc,raiselower) lower
      set paste(geometry) "990x820"
      set paste(amp2acc,mode) 0
      set paste(amp2acc,tafter) 0.4
      set paste(amp2acc,tbefore) 0.4
      set paste(amp2acc,sn)  "    8,    4,    2,  1.5,  1.2,    1"
      set paste(amp2acc,acc) "0.020,0.050,0.075,0.100,0.150,0.200"
      set paste(ncolors) 256
      set paste(plabel,separator) 5
      set paste(plabel,groupsize) 5
      set paste(plabel,shadow) 1
      set paste(plabel,shadow_color) \#202020
      set paste(plabel,anchor) n
      set paste(plabel,font) font5
      set paste(canvas,label) "Normal"
      set paste(canvas,mode) 1
      set paste(graph,dx) 0.4
      set paste(graph,dt) 0.02
      set paste(label,title) "none"
      set paste(label,autotitle) Dirname(fullpath)
      set paste(label,topaxis) "Offset(km)"
      set paste(label,bottomaxis) "Offset(km)"
      set paste(fn,title) font5
      set paste(fn,tick)  font3
      set paste(fn,axis)  font3
      set paste(fn,unit)  font1
      set paste(check,toplabel) 0
      set paste(check,bottomlabel) 0
      set paste(mark,uncertainty) 0
      set paste(mark,size) 5
      set paste(mark,style) splus

      # Canvas size
      set paste(graph_width) $paste(default,graph_width)
      set paste(graph_height) $paste(default,graph_height)
      set paste(canvas_width) $paste(default,graph_width)
      set paste(canvas_height) $paste(default,graph_height)
      set paste(tic_size) 7
      set paste(sub_tic_size) 4
      set paste(title_font_height) 0
      set paste(label_font_height) 0
      set paste(label_font_width) 0
      set paste(tick_font_height) 0
      set paste(tick_font_width) 0
      set paste(message_height) $paste(default,message_height)

      set paste(trace,usecache) 0
      set paste(trace,clip) 0
      set paste(trace,showpick) 1
      set paste(trace,check_aux) 0
      set paste(trace,aux) 0

      set print(file_or_printer) file
      set print(printer) lp
      set print(mode) color
      set print(size) A4
      set print(orientation) "Landscape"
      set print(normalize) h
      set print(format) "PostScript"
      set print(density) 200

      set filt(type_label) "Band Pass"
      set filt(type) 100
      set filt(offset) 0
      set filt(causal) 1
      set filt(f1) 2.0
      set filt(f2) 3.0
      set filt(f3) 12.0
      set filt(f4) 15.0
      set filt(n) 1

      set pick(current_type) 1
      set pick(default_accuracy) 0.02
      set pick(line,status) 0

      set taup(no) 0
      set taup(slow) {}
      set taup(tau) {}
      set taup(pick_size) 2
      set taup(color) green
      set taup(file) ""

      set type 1
      set pick(n_color) [llength $wk(color,list)]
      # set pick colors: 200 pick types
      foreach {i} $wk(color,list) {
	 for {set j 0} {$j < 11} {incr j} {
	    set index [expr $type + $j*20]
	    set pick(color,$index) $i
	 }
	 incr type
      }

      set pick(range,list) {}
      for {set i 1} {$i < 200} {incr i $pick(n_color)} {
	 lappend pick(range,list) [format "( %d - %d )" $i\
				       [expr $i + $pick(n_color) -1] ]
      }
      set pick(range) [expr ($pick(current_type) - 1)/ $pick(n_color)]

      set macray(n) 0
      set macray(color) green
      set macray(mode) "off"
      set macray(button_message) "Visible"
      set macray(data_file) ""
      set macray(style) "Symbol"
      set macray(width) 2
      set macray(symbol_size) 2
      set macray(filename) ""
      set macray(color_mode) 0
      set macray(line_style) "solid"
      set macray(symbol_style) "oval"

      set change_rect(status) 0

      # invoke mode (parameter file related)
      #          noparam           : use default value but read theme and pref.
      #          default           : read default parameter file
      #          default+specified : read default parameter file and 
      #                                                    specified wave file
      #          specified         : read specified parameter file
      #          specified_both    : use specified parameter file and wave file
      #          vanilla           : don't use any parameter files
      set wk(invoke,mode) default

      # 4-both wave and parameters
      set wk(resource_file) ""

      # plot status, etc
      set wk(vline,data) {}
      set wk(vline,listbox) "dummy"
      set wk(plot,water_wave) 0
      set wk(plot,auxline) 0
      set wk(pick,blink) 1500
      set wk(pick,format) 0
      set wk(pick,format_name) Pasteup
      set wk(total_shot) 0
      set wk(prev,wave) ""
      set wk(prev,tmin) 0
      set wk(prev,tmax) -1
      set wk(condition) -100    ; # re-read full data

      set wk(multiple,direc) 1
      set wk(multiple,status) off

      # pick edit
      set wk(pickedit,from) 1
      set wk(pickedit,from,range) 0
      set wk(pickedit,into) 2
      set wk(pickedit,into,range) 0
      set wk(pickedit,del) 1
      set wk(pickedit,del,range) 0
      set wk(pickedit,undoremain) 0
      set wk(pickedit,applyto) 0
      set wk(pickedit,apply,type) 1
      set wk(pickedit,apply,range) 0

      # debug related
      set wk(debug) 0

      # zoom related
      set wk(zoom,status) "off"
      set wk(zoom,count) 0
      set wk(zoom,left,0) 0
      set wk(zoom,right,0) 0

      # trace window related
      set wk(trace,width) $paste(default,trace_width)
      set wk(trace,height) $paste(default,trace_height)
      set wk(trace,top_margin) 20
      set wk(trace,bot_margin) 10
      set wk(trace,left_margin) 40
      set wk(trace,right_margin) 10
      set wk(trace,tick_length) 3
      set wk(trace,psfile) "./trace.ps"
      exec_update_trace_window_size

      # convert data related
      set wk(conv,output_fn) ""

      # raytracing related
      set wk(raytrace,dir) "./.pasteup_synthetics"

      # velocity model related
      set wk(vmodel,widget) .vmodel
      set wk(vmodel,filetime) "-1"
      set wk(vmodel,height_coef) 1

      # Command-line Option related
      set wk(invoke,saveparafn) ""
      set wk(invoke,logo_duration) 1500
      set wk(invoke,autoDB) 0
      set wk(invoke,autoPS) 0
      set wk(invoke,autodraw) 0
      set wk(invoke,autogain) 0
      set wk(invoke,autoquit) 0
      set wk(invoke,autorange) 0
      set wk(invoke,autorsize) 0
      set wk(invoke,agc) ""
      set wk(invoke,ampc) ""
      set wk(invoke,analysis) 0
      set wk(invoke,causal) 0
      set wk(invoke,clip) ""
      set wk(invoke,density) ""
      set wk(invoke,dpi) ""
      set wk(invoke,drawwater) 0
      set wk(invoke,filter) ""
      set wk(invoke,freq) ""
      set wk(invoke,havepick) 0
      set wk(invoke,havesynthe) 0
      set wk(invoke,maxsample) 4000
      set wk(invoke,maxtrace)  5000
      set wk(invoke,plabel) 0
      set wk(invoke,rvel) ""
      set wk(invoke,style) ""
      set wk(invoke,style2) ""
      set wk(invoke,su) ""
      set wk(invoke,sufilter) ""
      set wk(invoke,taup) 0
      set wk(invoke,title) ""
      set wk(invoke,thinout) ""
      set wk(invoke,tmin) ""
      set wk(invoke,tmax) ""
      set wk(invoke,ttic) ""
      set wk(invoke,tmp) ""
      set wk(invoke,txratio) ""
      set wk(invoke,usedb) -1
      set wk(invoke,Xdpi) ""
      set wk(invoke,xaxis) ""
      set wk(invoke,xmax) ""
      set wk(invoke,xmin) ""
      set wk(invoke,xtic) ""

      # edit header related
      set wk(hw,ascii_data,fn) ""
      set wk(hw,ascii_data,col) 1
      set wk(hw,ascii_data,outfn) ""

      # temoporary filenames
      set wk(fn,pick) [format "%s/pasteup.%s.pick" $TMPDIR [pid]]
   }

   # --------------------------------------------------------------------
   # Initialize balloon help --------------------------------------------
   # --------------------------------------------------------------------
   proc initialize_balloon {} {
      global wk

      set wk(balloon,widget) .tips
      toplevel $wk(balloon,widget) -background lightyellow -borderwidth 1\
	  -relief solid
      label $wk(balloon,widget).mes -background lightyellow -text "Tips"
      pack $wk(balloon,widget).mes
      wm overrideredirect $wk(balloon,widget) 1
      wm withdraw $wk(balloon,widget)
      
      # position balloon
      set wk(balloon,pos) .pos
      toplevel $wk(balloon,pos) -background lightyellow -borderwidth 1\
	  -relief solid
      label $wk(balloon,pos).mes -background lightyellow -text "Position"\
	  -justify left
      pack $wk(balloon,pos).mes
      wm overrideredirect $wk(balloon,pos) 1
      wm withdraw $wk(balloon,pos)
   }

   # --------------------------------------------------------------------
   # initialize pick ----------------------------------------------------
   # --------------------------------------------------------------------
   proc initialize_pick {} {
      global pm undopm redopm mark_modification

      set mark_modification 0
      set pm {}           ; # list of pick marks
      set undopm {}       ; # list of pick undo info (previous pick data)
      set redopm {}       ; # list of pick redo info
   }

   # --------------------------------------------------------------------
   # Opening logo -------------------------------------------------------
   # --------------------------------------------------------------------
   proc invoke_logo {} {
      global wk
      if { $wk(invoke,logo_duration) > 0 } {
	 opening_logo
      } else {
	 wm withdraw .
      }
   }
}