#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <sys/time.h>

#include "pasteup.h"

/********************************************************************
Read record header file of KASAHARA-lab format.
                                    for new format defined in 2001.
--------------------------------------------------------------------
Return the number of traces
********************************************************************/
int read_rh(FILE *fp)
{
    /***** Variables *****/
    int i, j;
    char tmp[17][20], dum[150];
    struct tm tt;

    struct timeval now;
    
    /***** Clear tmp *****/
    for(i=0;i<17;i++){
	for(j=0;j<20;j++){
	    tmp[i][j] = 0;
	}
    }
    
    i=0;
    gettimeofday(&now, NULL);
    tt = *localtime(&now.tv_sec);
    while(1){ 
	if (fgets(dum,120,fp) == NULL){
	    break;
	}
	if (dum[0] == '#') continue;
	sscanf(dum,"%4c%5c%3c%2c%2c%2c%6c%9c%9c%6c%8c%5c%3c%5c%4c%6c%5c%*c",
	       tmp[0],tmp[1],tmp[2],tmp[3],tmp[4],tmp[5],tmp[6],tmp[7],
	       tmp[8],tmp[9],tmp[10],tmp[11],tmp[12],tmp[13],tmp[14],
	       tmp[15],tmp[16]);
	gbl_rh[i].hw[RECORD_HEADER_TRACL]
	    = gbl_rh[i].hw[RECORD_HEADER_TRACR]
	    = gbl_rh[i].hw[RECORD_HEADER_FLDR]
	    = gbl_rh[i].hw[RECORD_HEADER_TRACF]
	    = atoi(tmp[1]);
	gbl_rh[i].hw[RECORD_HEADER_EP]         = atoi(tmp[0]);

	tt.tm_mon  = atoi(tmp[2]) -1;
	tt.tm_mday = atoi(tmp[3]);
	tt.tm_hour = atoi(tmp[4]);
	tt.tm_min  = atoi(tmp[5]);
	tt.tm_sec  = (int)atof(tmp[6]);

	gbl_rh[i].st   = mktime(&tt);
	gbl_rh[i].msec = (atof(tmp[6]) - (int)atof(tmp[6])) * 1000.0;
	gbl_rh[i].dsec = - atof(tmp[7]);

	gbl_rh[i].x = gbl_rh[i].offset = atof(tmp[8]) - atof(tmp[9]);
	gbl_rh[i].sampling    = atof(tmp[10]);
	gbl_rh[i].start_point = atoi(tmp[11])*1024;
	gbl_rh[i].end_point   = gbl_rh[i].start_point 
	    + (atoi(tmp[12])-1)*1024 + atoi(tmp[13]);
	gbl_rh[i].dsec        -= atof(tmp[15]); /* dsexp */
	gbl_rh[i].swdep     = atoi(tmp[16]);
	
	gbl_rh[i].counit = gbl_rh[i].scalco = 0;
	gbl_rh[i].sx = gbl_rh[i].sy = gbl_rh[i].gx = gbl_rh[i].gy = 0;
	i++;

	/***** require re-allocation ? *****/
	if(i>= maxtrace) {
	    maxtrace += 1000;
	    if(realloc_global_variable() == 1) {
		fprintf(stderr, "ERROR: Memory reallocation failed.\n");
		exit(1);
	    }
	}
    }
    return(i);
}


/*
  --------------------------------------------------------------------
*/

/********************************************************************
Read trace body in the KASAHARA-lab format
--------------------------------------------------------------------
[arguments]
a           : amplitude array (for output)
t           : time of the first sample
fp          : file pointer for read
bfmt        : data binary format (2:2byte int, 3:3byte int, 14:float)
data_point1 : trace start point
data_point2 : trace end point
start_time  : cut out time (start)
end_time    : cut out time (end)
amp         : amplification
sampling    : sampling rate (Hz)
dt_clock    : clock error (sec)
********************************************************************/
int read_kasalab_data(float *a, float *t, FILE *fp, int bfmt,
		      int data_point1, int data_point2,
		      float start_time, float end_time,
		      float sampling, float dt_clock)
{
    /********** Local Variables **********/
    int i, i_start, i_end, count, address, data_byte;
    char buf[72000]; /*** enough for 4minute, 3byte, 100Hz ***/
    
    /********** Local Function **********/
    float mkval(char *buf, int num, int data_type);
    
    /********** Main **********/
    data_byte = (bfmt == FMT_4BYTE_FLOAT) ? 4 : bfmt;

    /***** determine reading ddress *****/
    /** start point **/
    if ( start_time + dt_clock >= 0.0 ) {
	i_start = (start_time + dt_clock )*sampling;
    } else {
	i_start = 0;
    }
    /** end point **/
    i_end = ceil((end_time + dt_clock)*sampling);
    if (i_end > data_point2 - data_point1)
	i_end = data_point2 - data_point1;
    count = i_end - i_start;
    if (count < 1) return(0);
    address = (data_point1 + i_start) * data_byte;
    
    /***** read data *****/
    fseek(fp, address, SEEK_SET);
    fread(buf, sizeof(char), count*data_byte, fp);
    *t = (float)i_start/sampling - dt_clock;
    for(i=0;i<count;i++){
	a[i] = mkval(buf, i, bfmt);
    }
    return(count);
}

/********************************************************************
 Return the year of observation
********************************************************************/
int get_kasalab_year(char *data_file)
{
    /** No year info in KASA-lab format w/o calib.dat **/
    return(-1);
}
