# --------------------------------------------------------------------
# layout_window ------------------------------------------------------
#    invoke layout and format window
# --------------------------------------------------------------------
proc layout_window {} {
   # Global variable
   global gbl wk paste custom
   
   # Main --------------------
   set lw .layout_window
   if [raise_window $lw] { return }
   toplevel $lw
   wm title $lw "PasteUP: Layout and Format"
   
   # binding ------------------------------
   basic_bindings $lw

   # title frame --------------------
   set tf [title_frame $lw.title_frame "Pasteup: Layout and Format"]
   
   # tabset ------------------------------
   set ts $lw.tabset
   tabset $ts -pagewidth 450 -relief flat
   $ts conf -borderwidth 1 -highlightthickness 0 -side top
   $ts insert end "text" -text "Labels"
   $ts insert end "axis" -text "Axis"
   $ts insert end "sub" -text "Sub Plot"
   $ts insert end "font" -text "Font"
   $ts insert end "fsize" -text "Canvas"

   # tabset balloon help
   foreach {tab mes}\
       {text "Configure figure title/axis labels"\
	    axis "Configure axis data/direction"\
	    sub "Configure sub plot"\
	    font "Configure Fonts of record section"\
	    fsize "Confingure size of record section"
	 } {
	    $ts bind $tab <Enter> "balloon \"$mes\" %X %Y"
	    $ts bind $tab <Leave> {cancel_balloon}
	 }

   # ------------------------------------------------------------
   # Labels tab -------------------------------------------------

   set pf $ts.labels
   frame $pf
   $ts tab conf "text" -window $pf -fill both

   # Title ----------
   set ft [labelframe $pf.title -text "Figure title" -padx 10 -pady 5]
   make_long_entry $ft figure_title "Title" paste(label,title)

   # Auto-title ----------
   set at [labelframe $pf.autotitle -text "Auto title" -pady 5]
   make_optionMenu_com $at auto "" paste(label,autotitle) update_figure_title\
       none Filename Dirname Dirname(fullpath)

   # horizontal axis label ----------
   set ha [labelframe $pf.haxis -text "Horizontal axis label" -padx 10 -pady 5]
   make_check_entry $ha "Top" top paste(check,toplabel) paste(label,topaxis)
   make_check_entry $ha "Bottom" bot paste(check,bottomlabel)\
       paste(label,bottomaxis)

   # Tick labels ----------
   set tl [labelframe $pf.tick -text "Tick label"]
   foreach {i} {top bottom left right} {
      make_checkbutton $tl [format "%s" [string totitle $i]]\
	  $i paste(string,[format "%stick" $i])
   }

   # Several informations ----------
   set si [labelframe $pf.info -text "Several information"]
   foreach {i j k} {\
			"Filename" filename paste(string,filename)\
			"Filter information" finfo paste(string,filter)\
			"Reduction Velocity" redvel paste(string,reduction)\
		     } {
      make_checkbutton $si $i $j $k
   }

   # unit ----------
   set uf [labelframe $pf.unit -text "Unit"]
   make_optionMenu_button $uf unit "" paste(string,unit) none tx xz

   # packing ----------
   grid $ft - -sticky news -pady 3 -padx 3
   grid $ha - -sticky news -pady 3 -padx 3
   grid $tl $si -sticky news -pady 0 -padx 3
   grid $at $uf -sticky news -pady 0 -padx 3

   # ------------------------------------------------------------
   # Figure Axis Tab --------------------------------------------
   set pf $ts.axis
   frame $pf
   $ts tab conf "axis" -window $pf -fill both

   # x-axis selection ----------
   foreach side {top bottom} {
      set xaxis($side) [labelframe $pf.xaxis($side) -text "$side axis"]
      make_radio_vertical $xaxis($side) $side gbl(x_axis,$side)\
 	  $gbl(data,xaxis_label) 1\
 	  [format "%s+OBS_x" $gbl(data,xaxis_label)] 2
   }

   # direction of time-axis ----------
   set da $pf.direc_axis
   labelframe $da -text "Direction of Vertical-axis"
   make_radio_vertical $da axis gbl(t_axis) downward 1 upward 0

   # tic interval/spacing ----------
   set tk $pf.ticks
   labelframe $tk -text "Major tick interval"
   make_entry_right $tk dist "Horizontal" gbl(xtic_int) km
   make_entry_right $tk time "Vertical" gbl(t_tic_int) sec
   
   # sub tic marks ----------
   set st [labelframe $pf.subticks -text "Number of subticks"]
   make_spinbox $st x_subtic "Horizontal" gbl(x_sub_tic) 0 9 1 ""
   make_spinbox $st t_subtic "Vertical" gbl(t_sub_tic) 0 9 1  ""

   # Tick labels ----------
   set tl [labelframe $pf.tick -text "Tick label"]
   foreach {i} {top bottom left right} {
      make_checkbutton $tl [format "%s label" [string totitle $i]]\
	  $i paste(string,[format "%stick" $i])
   }

   # placing
   grid $xaxis(top) $xaxis(bottom) -sticky news -pady 5 -padx 5
   grid $tk $st -sticky news -pady 5 -padx 5
   grid $da $tl -sticky news -pady 5 -padx 5

   # ------------------------------------------------------------
   # Sub Plot Tab --------------------------------------------
   set pf [labelframe $ts.sub]
   $ts tab conf "sub" -window $pf -fill both
   
   # radiobutton for "sub plot"
   checkbutton $ts.check_subplot -text "Sub Plot" -var paste(sub,check)\
       -command "labelframe_checkbutton_status $pf paste(sub,check)"
   $pf conf -labelw $ts.check_subplot

   # plot position ----------
   set pp [labelframe $pf.plot_position -text "Plot Position"]
   make_radio_vertical $pp position paste(sub,position) top top bottom bottom
   
   # plot data ----------
   set pd [labelframe $pf.plot_data -text "Data"]
   make_optionMenu_button $pd selection "Header name of SU format"\
       paste(sub,data) "swdep" "gwdep" "tracl" "tracr" "tracf" "fldr"

   # plot color ----------
   set pc [labelframe $pf.plot_color -text "Color"]
   make_color_button $pc color "" "" $paste(sub,color) paste(sub,color)\
       "setColor $pc $pc.button(color) paste(sub,color)"

   # vertical axis ----------
   set ax [labelframe $pf.axis -text "Vertical Axis"]
   make_entry_right $ax min "Min" paste(sub,min) ""
   make_entry_right $ax max "Max" paste(sub,max) ""
   make_entry_right $ax dy "Tick spacing" paste(sub,dy) ""
   make_entry_right $ax height "Height" paste(sub,height) "pixel"
   make_many_radio $ax "Direction" axis paste(sub,axis) downward 1 upward 0

   labelframe_checkbutton_status $pf paste(sub,check)
   grid $pd - -sticky news -ipady 20 -ipadx 15 -padx 5
   grid $pp $ax -sticky news -ipady 10 -ipadx 15 -padx 5
   grid $pc ^ -sticky news -ipady 10 -ipadx 15 -padx 5

   # ------------------------------------------------------------
   # Figure font tab --------------------------------------------
   set pf $ts.font
   frame $pf
   $ts tab conf "font" -window $pf -fill both

   # font ----------
   set fn [labelframe $pf.font -text "Font" -pady 20]
   PSFontSelect::make_widget $fn title "Figure title" paste(fn,title)
   PSFontSelect::make_widget $fn tick "Tick" paste(fn,tick)
   PSFontSelect::make_widget $fn axis "Axis label" paste(fn,axis)
   PSFontSelect::make_widget $fn unit "Unit" paste(fn,unit)

   pack $fn -side top -fill x -expand 1

   # ------------------------------------------------------------
   # Figure size tab --------------------------------------------
   set pf $ts.fsize
   frame $pf
   $ts tab conf "fsize" -window $pf -fill both

   # normal mode
   set fs $pf.figure_size
   labelframe $fs -text "\"Normal\" mode" -pady 3

   make_entry_com $fs "Width of graph area" cw paste(graph_width)\
       "Default" "set paste(graph_width) $paste(default,graph_width)"
   make_entry_com $fs "Height of graph area" ch paste(graph_height)\
       "Default" "set paste(graph_height) $paste(default,graph_height)"
   make_entry_com $fs "Vertical/Horizontal ratio" txratio\
       paste(default,txratio) "Apply" "resize_canvas_size_by_ratio"

   # graph canvas mode
   labelframe $pf.graph -text "\"Graph\" mode" -pady 3
   make_entry_right $pf.graph x "Resolution of x" paste(graph,dx) "km"
   make_entry_right $pf.graph t "Resolution of t" paste(graph,dt) "sec"

   # update button
   pack $fs $pf.graph -side top -fill x -expand 1

   # Button frame ----------------------------------------
   set bf [button_frame $lw $lw.button_frame "update" "update_canvas_size"]

   # Packing ----------
   pack $tf $ts $bf -side top -fill x -expand 1
}

proc draw_options {} {
   global wk
   # Main --------------------
   draw_axis $wk(w,current)
   draw_labels $wk(w,current)
   draw_auxiliary
}

proc hscroll_canvas {w dir} {
   # main --------------------
   $w xview scroll $dir unit
}

proc vscroll_canvas {w dir} {
   # main --------------------
   $w yview scroll $dir unit
}

proc resize_canvas_size_by_ratio {} {
   # global ------------------------------
   global paste

   # main ------------------------------
   set dt [expr abs( $paste(bottom) - $paste(top) )]
   set dx [expr abs( $paste(right) - $paste(left) )]
   set paste(graph_width) [format "%.0f"\
			       [expr $paste(graph_height) * $dx \
				    /($dt * $paste(default,txratio) )]]
   print_message [format "Figure size is updated (x=%s, t=%s)\n"\
		      $paste(graph_width) $paste(graph_height)]
}