#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

#include "pasteup.h"
#include "su.h"

/********************************************************************
Convert data into SU
********************************************************************/

void write_su_header(FILE *fp, int i, int ns)
{
    /********** Variables **********/
    int dt;
    struct su_header tr;
    struct tm tt;
    
    /**********************************************/
    /******************** Main ********************/
    /**********************************************/
    
    /** initilize tr **/
    memset( (void *) &tr, (int) '\0', sizeof(tr));

    /** mandatory info **/
    tr.tracl  = gbl_rh[i].hw[RECORD_HEADER_TRACL];
    tr.fldr   = gbl_rh[i].hw[RECORD_HEADER_FLDR];
    tr.ep     = gbl_rh[i].hw[RECORD_HEADER_EP];
    tr.offset = (int)(gbl_rh[i].x * 1000.0);
    tr.swdep  = gbl_rh[i].swdep;
    tr.delrt  = -gbl_rh[i].dsec*1.0e3;
    tr.ns     = ns;
    tr.dt     = (int)(1.0e6/gbl_rh[i].sampling);

    /** shot time **/
    tt = *localtime(&gbl_rh[i].st);
    tr.year   = tt.tm_year + 1900;
    tr.day    = tt.tm_yday +1;
    tr.hour   = tt.tm_hour;
    tr.minute = tt.tm_min;
    tr.sec    = tt.tm_sec;

    /** time shift **/
    dt = - (int)((gbl_rh[i].dsec)*1000.0);
    if ( dt > 32767) {
	tr.delrt = 32767;
	dt = dt - 32767;
	if (dt > 32767) {
	    tr.tstat = 32767;
	    tr.lagb  = dt - 32767;
	} else {
	    tr.tstat = dt;
	}
    } else {
	tr.delrt = dt;
    }

    /** additional info ; filter-info is not implemented **/
    tr.tracr  = tr.tracl;
    tr.trid   = 1;
    tr.duse   = 1;
    tr.gain   = 1;
    tr.corr   = 1;
    tr.timbas = 1;
    /***** write header to fp *****/
    fwrite(&tr, sizeof(struct su_header), 1, fp);
}

int pasteup_convertSUProc(
			ClientData clientData, /* not used                */
			Tcl_Interp *interp,    /* current tcl interpreter */
			int objc,              /* number of arguments     */
			Tcl_Obj* CONST objv[]  /* arguments               */
			)
{
    /********** Variables *********/
    /***** Variables for tcl interaction *****/
    char *su_filename;       /** output data filename                  **/
    char *data_filename;     /** waveform filename                     **/
    int data_format;         /** waveform data format (SU, SEG-Y...)   **/
    int bfmt;                /** data binary format (2:2byte int, etc) **/
    int channel;             /** Channel number                        **/

    /**** etc. ****/
    FILE *fp, *fp_out;
    
    int i, j, n, ns;
    float *t, *a;

    /**********************************************/
    /******************** Main ********************/
    /**********************************************/
    
    data_filename = TCL_GETSTRING(file_name, "wave");
    su_filename = TCL_GETSTRING(wk, "conv,output_fn");
    TCL_GET(Int, gbl, "data,type", data_format);
    TCL_GET(Int, gbl, "data,byte", bfmt);
    TCL_GET(Int, gbl, "obs,channel", channel);

    /*** memory allocation : SU can handle 65536 at most samples ***/
    if(NULL == (t = (float *)malloc(65536 * sizeof(float))) ||
       NULL == (a = (float *)malloc(65536 * sizeof(float))) ){
	fprintf(stderr, "Error: Memory allocation failed.\n");
	return TCL_ERROR;
    }

    /*** read header ***/
    total_shot = make_cache_data(interp, 0, 0);

    /*** recover clock correction ***/
    for(i=0;i<total_shot;i++){
	gbl_rh[i].dsec -= gbl_rh[i].shift;
    }

    /*** maximum trace length ***/
    ns = 0;
    for(i=0;i<total_shot;i++){
	if ( data_format == KASA_FORMAT) {
	    n = gbl_rh[i].end_point - gbl_rh[i].start_point;
	} else {
	    n = gbl_rh[i].end_point;
	}
	if(n > ns) ns=n;
    }

    /*** read and draw trace by trace ***/
    fp = open_file(data_filename, "waveform");
    fp_out = open_write_file(su_filename, "su-waveform");

    for(i=0;i<total_shot;i++){
	/* read */
	n = read_data(a, t, fp, data_format, bfmt, channel,
		      gbl_rh[i].start_point, gbl_rh[i].end_point,
		      -1.0e5, 1.0e5, gbl_rh[i].sampling, gbl_rh[i].dsec);
	if (n != ns) {
	    fprintf(stderr,
		    "WARNING: %d sample are required but %d samples are read (tracl=%d)\n",
		    ns, n, gbl_rh[i].hw[RECORD_HEADER_TRACL]);
	    for(j=n;j<ns;j++) a[j] = 0.0;
	}
	/* write */
	write_su_header(fp_out, i, ns);
	fwrite(a, sizeof(float), ns, fp_out);
    }
    fclose(fp_out);
    fclose(fp);
    free(t);
    free(a);
    return TCL_OK;
}

