#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "tcl.h"
#include "pasteup.h"

int pasteup_coordProc_inout2(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    double gx;      /** offset distance in km               **/
    double gt;      /** true time                           **/
    double top ;    /** t-min (in sec)                      **/
    double bottom;  /** t-max (in sec)                      **/
    double left;    /** x-min (in km)                       **/
    double right;   /** x-max (in km)                       **/

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  coord inout2 gx gt\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 4) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Get graph-x, true-time from command line argument *****/
    if( TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &gx)
	|| TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &gt) ) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Read global variables *****/
    TCL_GET(Double, paste, "top", top);
    TCL_GET(Double, paste, "bottom", bottom);
    TCL_GET(Double, paste, "left", left);
    TCL_GET(Double, paste, "right", right);

    /***** Check *****/
    if( (top-gt)*(gt-bottom) < 0.0 || (right-gx)*(gx-left) < 0.0 ) {
	/** outside **/
	Tcl_SetObjResult(interp, Tcl_NewIntObj(1));
	return TCL_OK;
    }
    /** inside **/
    Tcl_SetObjResult(interp, Tcl_NewIntObj(0));
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

int pasteup_coordProc_inout(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    double gx;      /** offset distance in km               **/
    double tt;      /** true time                           **/
    double top ;    /** t-min (in sec)                      **/
    double bottom;  /** t-max (in sec)                      **/
    double left;    /** x-min (in km)                       **/
    double right;   /** x-max (in km)                       **/
    double rv;      /** reduction velocity (km/sec)         **/

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  coord inout gx tt\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 4) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Get graph-x, true-time from command line argument *****/
    if( TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &gx)
	|| TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &tt) ) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Read global variables *****/
    TCL_GET(Double, paste, "top", top);
    TCL_GET(Double, paste, "bottom", bottom);
    TCL_GET(Double, paste, "left", left);
    TCL_GET(Double, paste, "right", right);
    TCL_GET(Double, paste, "reduction", rv);

    /***** Apply reduction *****/
    if (rv != 0.0) {
	tt = tt - fabs(gx)/rv;
    }

    /***** Check *****/
    if( (top-tt)*(tt-bottom) < 0.0 || (right-gx)*(gx-left) < 0.0 ) {
	/** outside **/
	Tcl_SetObjResult(interp, Tcl_NewIntObj(0));
	return TCL_OK;
    }
    /** inside **/
    Tcl_SetObjResult(interp, Tcl_NewIntObj(1));
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

/********************************************************************
pasteup_sub_tt2cy :  Convert TrueTime into CanvasCoordinate
   t    :   true tiem in second
 offset :   offset distance in km
********************************************************************/
double pasteup_sub_tt2cy(Tcl_Interp *interp, double t, double offset)
{
    /***** Variables *****/
    int t_axis;     /** flag of time-axis direction         **/
    double toffset; /** top offset                          **/
    double top ;    /** t-min (in sec)                      **/
    double bottom;  /** t-max (in sec)                      **/
    double height;  /** graph height                        **/
    double rv;      /** reduction velocity in km/sec        **/
    double rt;

    /***** Read global variables *****/
    TCL_GET(Double, paste, "top_margin", toffset);
    TCL_GET(Double, paste, "top", top);
    TCL_GET(Double, paste, "bottom", bottom);
    TCL_GET(Double, paste, "graph_height", height);
    TCL_GET(Double, paste, "reduction", rv);
    TCL_GET(Int, gbl, "t_axis", t_axis);

    /***** Convert cy into graph coordinate t without reduction *****/

    /** reduction **/
    if(rv <= 0.0) {
	rt = t;
    } else {
	rt = t - fabs(offset / rv);
    }
    /** compute **/
    if(t_axis > 0) {
	return(toffset + (rt - top)*height/(bottom - top));
    } else {
	return(height + toffset - (rt - top)*height/(bottom - top));
    }
}

/*
  --------------------------------------------------------------------
*/

int pasteup_coordProc_tt2cy(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    double x;       /** offset distance in km               **/
    double t;       /** true time                           **/

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  coord tt2cy x t\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 4) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Get canvas-y and offset-distance from command line argument *****/
    if( TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &t)
	|| TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &x) ) {
	show_usage();
	return TCL_ERROR;
    }

    Tcl_SetObjResult(interp,Tcl_NewDoubleObj(pasteup_sub_tt2cy(interp, t, x)));
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

int pasteup_coordProc_tt2rt(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    double x;       /** offset distance in km               **/
    double t;       /** true time                           **/
    double rv;      /** reduction velocity in km/sec        **/

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  coord tt2rt x t\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 4) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Get canvas-y and offset distance from command line argument *****/
    if( TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &t)
	|| TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &x)) {
	show_usage();
	return TCL_ERROR;
    }

    TCL_GET(Double, paste, "reduction", rv);
    if(rv > 0.0) {
	t = t - fabs(x)/rv;
    }

    Tcl_SetObjResult(interp,Tcl_NewDoubleObj(t));
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

int pasteup_coordProc_rt2tt(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    double x;       /** offset distance in km               **/
    double t;       /** true time                           **/
    double rv;      /** reduction velocity in km/sec        **/

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  coord rt2tt x t\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 4) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Read global variables *****/
    TCL_GET(Double, paste, "reduction", rv);

    /***** Get reduced time and offset from command line argument *****/
    if( TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &t)
	|| TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &x) ) {
	show_usage();
	return TCL_ERROR;
    }

    if(rv > 0.0){
	t = t + fabs(x)/rv;
    }

    Tcl_SetObjResult(interp,Tcl_NewDoubleObj(t));
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

/********************************************************************
pasteup_sub_gx2cx :  Convert Graph-x into CanvasCoordinate-x
   gx  : graph x
********************************************************************/
double pasteup_sub_gx2cx(Tcl_Interp *interp, double gx)
{
    /***** Variables *****/
    double loffset; /** left offset                         **/
    double left;    /** x-min (in km)                       **/
    double right;   /** x-max (in km)                       **/
    double width;   /** graph width                         **/
    int xaxis_direc;

    /******************** Main ********************/

    /***** Read global variables *****/
    TCL_GET(Double, paste, "left_margin", loffset);
    TCL_GET(Double, paste, "left", left);
    TCL_GET(Double, paste, "right", right);
    TCL_GET(Double, paste, "graph_width", width);
    TCL_GET(Int, gbl, "data,xaxis_direction", xaxis_direc);

    /***** Convert gx into canvas coordinate *****/
    if(xaxis_direc == 0) {
	return(loffset + (gx-left)*width/(right-left));
    } else {
	return(loffset + (right - gx)*width/(right-left));
    }
}

/*
  --------------------------------------------------------------------
*/

int pasteup_coordProc_gx2cx(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    double gx;      /** graph x in km                       **/

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  coord gx2cx gx\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 3) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Get canvas-y from command line argument *****/
    if( TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &gx) ) {
	show_usage();
	return TCL_ERROR;
    }

    Tcl_SetObjResult(interp, Tcl_NewDoubleObj(pasteup_sub_gx2cx(interp, gx)));
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

double pasteup_sub_cy2tt(Tcl_Interp *interp, double x, double cy)
{
    /***** Variables *****/
    int t_axis;     /** flag of time-axis direction         **/
    double toffset; /** top offset                          **/
    double top ;    /** t-min (in sec)                      **/
    double bottom;  /** t-max (in sec)                      **/
    double height;  /** graph height                        **/
    double rv;      /** reduction velocity in km/sec        **/
    double t;       /** result                              **/
    
    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  coord cy2tt x cy\n",
	     (char *)NULL);
    };

    /***** Read global variables *****/
    TCL_GET(Double, paste, "top_margin", toffset);
    TCL_GET(Double, paste, "top", top);
    TCL_GET(Double, paste, "bottom", bottom);
    TCL_GET(Double, paste, "graph_height", height);
    TCL_GET(Double, paste, "reduction", rv);
    TCL_GET(Int, gbl, "t_axis", t_axis);

    /***** Convert cy into graph coordinate t without reduction *****/
    if(t_axis > 0) {
	t = top + (cy - toffset)*(bottom - top) / height;
    } else {
	t = top + (height - cy + toffset)*(bottom - top) / height;
    }
    if(rv == 0.0){
	return(t);
    } else {
	return(t + fabs(x)/rv);
    }
}

/*
  --------------------------------------------------------------------
*/

int pasteup_coordProc_cy2tt(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    double x;       /** offset distance in km               **/
    double cy;      /** canvas y ; set cy [$p canvasy $my]  **/
    
    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  coord cy2tt x cy\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 4) {
	show_usage();
	return TCL_ERROR;
    }
    /***** Read graph-x *****/
    if( TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &x) ) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Get canvas-y from command line argument *****/
    if( TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &cy) ) {
	show_usage();
	return TCL_ERROR;
    }
    Tcl_SetObjResult(interp,
		     Tcl_NewDoubleObj(pasteup_sub_cy2tt(interp, x, cy)));
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

int pasteup_coordProc_cy2gt(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    double cy;      /** canvas y ; set cy [$p canvasy $my]  **/
    int t_axis;     /** flag of time-axis direction         **/
    double toffset; /** top offset                          **/
    double top ;    /** t-min (in sec)                      **/
    double bottom;  /** t-max (in sec)                      **/
    double height;  /** graph height                        **/

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  coord cy2gt cy\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 3) {
	show_usage();
	return TCL_ERROR;
    }
    /***** Get canvas-y from command line argument *****/
    if( TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &cy) ) {
	show_usage();
	return TCL_ERROR;
    }
    /***** Read global variables *****/
    TCL_GET(Double, paste, "top_margin", toffset);
    TCL_GET(Double, paste, "top", top);
    TCL_GET(Double, paste, "bottom", bottom);
    TCL_GET(Double, paste, "graph_height", height);
    TCL_GET(Int, gbl, "t_axis", t_axis);

    /***** Convert cy into graph coordinate gt (with reduction) *****/
    if(t_axis > 0) {
	Tcl_SetObjResult
	    (interp,
	     Tcl_NewDoubleObj(top + (cy - toffset)*(bottom - top) / height));
    } else {
	Tcl_SetObjResult
	    (interp,
	     Tcl_NewDoubleObj(top + (height-cy+toffset)*(bottom-top)/height));
    }
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

int pasteup_coordProc_cx2gx(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    double cx;      /** canvas x ; set cx [$p canvasx $mx]  **/
    double loffset; /** left offset                         **/
    double left;    /** x-min (in km)                       **/
    double right;   /** x-max (in km)                       **/
    double width;   /** graph width                         **/
    int xaxis_direc;

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  coord cx2gx cx\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 3) {
	show_usage();
	return TCL_ERROR;
    }
    /***** Get canvas-x from command line argument *****/
    if( TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &cx) ) {
	show_usage();
	return TCL_ERROR;
    }
    /***** Read global variables *****/
    TCL_GET(Double, paste, "left_margin", loffset);
    TCL_GET(Double, paste, "left", left);
    TCL_GET(Double, paste, "right", right);
    TCL_GET(Double, paste, "graph_width", width);
    TCL_GET(Int, gbl, "data,xaxis_direction", xaxis_direc);

    /***** Convert cx into graph coordinate gx *****/
    if(xaxis_direc==0) {
	/* normal */
	Tcl_SetObjResult(interp,
			 Tcl_NewDoubleObj(left+(cx-loffset)*(right-left)/width));
    } else {
	/* reverse */
	Tcl_SetObjResult(interp,
			 Tcl_NewDoubleObj(right-(cx-loffset)*(right-left)/width));
    }
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

int pasteup_coordProc(
		   ClientData clientData, /* not used                */
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    char *opt;

    /******************** Main ********************/

    void usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n coord option ...\n",
	     "   option is one of followings\n",
	     "       cx2gx: convert canvas_x into graph_x\n",
	     "       cy2gt: convert canvas_y into graph_t(reduced)\n",
	     "       cy2tt: convert canvas_y into true_t(non-reduced)\n",
	     "       gx2cx: convert graph_x into canvas_x\n",
	     "       rt2tt: convert reduced-t into true-t\n",
	     "       tt2rt: convert trut_t into reduction time\n",
	     "       tt2cy: convert trut_t into canvas_y\n",
	     "       inout: check whether a given point is in or out.\n",
	     "       inout2: check whether a given point is in or out(reduced).",
	     "",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc < 3){
	usage();
	return TCL_ERROR;
    }
    /***** Get Option *****/
    opt = Tcl_GetStringFromObj(objv[1], NULL);

    /*************** Operation ***************/
    if (strcmp("cx2gx", opt) == 0) {
	return(pasteup_coordProc_cx2gx(interp, objc, objv));
    } else if (strcmp("cy2gt", opt) == 0) {
	return(pasteup_coordProc_cy2gt(interp, objc, objv));
    } else if (strcmp("cy2tt", opt) == 0) {
	return(pasteup_coordProc_cy2tt(interp, objc, objv));
    } else if (strcmp("gx2cx", opt) == 0) {
	return(pasteup_coordProc_gx2cx(interp, objc, objv));
    } else if (strcmp("rt2tt", opt) == 0) {
	return(pasteup_coordProc_rt2tt(interp, objc, objv));
    } else if (strcmp("tt2rt", opt) == 0) {
	return(pasteup_coordProc_tt2rt(interp, objc, objv));
    } else if (strcmp("tt2cy", opt) == 0) {
	return(pasteup_coordProc_tt2cy(interp, objc, objv));
    } else if (strcmp("inout", opt) == 0) {
	return(pasteup_coordProc_inout(interp, objc, objv));
    } else if (strcmp("inout2", opt) == 0) {
	return(pasteup_coordProc_inout2(interp, objc, objv));
    } else {
	usage();
	return TCL_ERROR;
    }
    return TCL_OK;
}
