#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <tcl.h>

#include "pasteup.h"

/********************************************************************

linepick command 

********************************************************************/

int pasteup_linepickProc(
			 ClientData clientData, /* not used                */
			 Tcl_Interp *interp,    /* current tcl interpreter */
			 int objc,              /* number of arguments     */
			 Tcl_Obj* CONST objv[]  /* arguments               */
			 )
{
    /********** Variables *********/
    Tcl_Obj* pmlist;     /** list of pick marks                       **/
    int type;            /** pick type                                **/
    double acc;          /** accuracy                                 **/
    int hw_shot;
    
    double x1, x2;           /*** left and right end ***/
    double t1, t2;           /*** top and bottom end ***/
    
    /**** etc ****/
    int i;
    double x, dx, t;
    Tcl_Obj* ans;

    /******************** Main ********************/
    
    void usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n linepick gx1 rt1 gx2 rt2\n",
	     "    gx?       : graph x-coordinate\n",
	     "    rt?       : graph t-coordinate (reduced time)\n",
	     "",
	     (char *)NULL);
    }

    /********** Check arguments **********/
    if(objc != 5 ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[1], &x1) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &t1) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &x2) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[4], &t2) ) {
	usage();
	return TCL_ERROR;
    }

    /********** Initialize **********/
    pmlist = Tcl_NewListObj(0, NULL);

    /***** calc dx: if dx less than 1meter, no picks *****/
    dx = x2 - x1;
    if(total_shot < 1 || fabs(dx) < 0.001){
	return TCL_OK;
    }

    /*************** Read global variables ***************/
    TCL_GET(Double, pick, "default_accuracy", acc);
    TCL_GET(Int, pick, "current_type", type);
    TCL_GET(Int, gbl, "option,su,shotnumber", hw_shot);

    /********** Exec pick along a line **********/
    for(i=0;i<total_shot;i++){
	/***** read record header info *****/
	x = gbl_rh[i].x;
	if( (x - x1)*(x2 - x) >= 0.0) {
	    t = (t2*(x-x1) + t1*(x2-x)) / dx + gbl_rh[i].shift;
	    ans = Tcl_NewListObj(0, NULL);
	    TCL_APPEND(ans, Int, gbl_rh[i].hw[hw_shot]);
	    TCL_APPEND(ans, Double, x);
	    TCL_APPEND(ans, Double, t);
	    TCL_APPEND(ans, Double, acc);
	    TCL_APPEND(ans, Int, type);
	    Tcl_ListObjAppendElement(interp, pmlist, ans);
		
	}
    }
    Tcl_SetObjResult(interp, pmlist);
    return TCL_OK;
}
