#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "tcl.h"
#include "tk.h"
#include "blt.h"
#include "pasteup.h"

#define MARK_SPLUS 0
#define MARK_PLUS 1
#define MARK_CROSS 2
#define MARK_DIAMOND 3
#define MARK_CIRCLE 4
#define MARK_SQUARE 5

extern double pasteup_sub_cy2tt(Tcl_Interp *interp, double x, double cy);

/********************************************************************
Plot a pick mark on the normal Canvas
   w    : width of the mark (size)
   x    : x-position (in Canvas coordiante)
   t    : reduced time in second
   type : mark type (color)
********************************************************************/
static int plot2normal(Tcl_Interp *interp, char *canvas, int w,
		       int type, double toffset, double height,
		       int t_axis, double tmin, double tmax,
		       double x, double t, int symbol_type)
{
    char com[256];  /** Buffer for Tcl/Tk command           **/

    /** convert graph-coordinate into canvas-coordinate **/
    t = (t_axis > 0) ? (toffset + (t - tmin)*height/(tmax - tmin)) :
	(height + toffset - (t - tmin)*height/(tmax - tmin));
    
    /** draw pick mark **/
    switch(symbol_type) {
	case MARK_SPLUS:
	    sprintf(com,
		    "eval %s create line %g %g %g %g -fill \"$pick(color,%d)\" -tags pk -width 2",
		    canvas, x - w, t, x + w, t, type);
	    break;
	case MARK_PLUS:
	    sprintf(com,
		    "eval %s create line %g %g %g %g -fill \"$pick(color,%d)\" -tags pk -width 1;"
		    "eval %s create line %g %g %g %g -fill \"$pick(color,%d)\" -tags pk -width 1",
		    canvas, x - w, t, x + w, t, type,
		    canvas, x, t+w, x, t-w, type);
	    break;
	case MARK_CROSS:
	    w = w / 1.41421356; /* w/sqrt(2) */
	    sprintf(com,
		    "eval %s create line %g %g %g %g -fill \"$pick(color,%d)\" -tags pk -width 1;"
		    "eval %s create line %g %g %g %g -fill \"$pick(color,%d)\" -tags pk -width 1",
		    canvas, x-w, t-w, x+w, t+w, type,
		    canvas, x-w, t+w, x+w, t-w, type);
	    break;
	case MARK_DIAMOND:
	    sprintf(com,
		    "eval %s create polygon %g %g %g %g %g %g %g %g -fill \"$pick(color,%d)\" -tags pk -width 1;",
		    canvas, x-w, t, x, t+w, x+w, t, x, t-w, type);
	    break;
	case MARK_CIRCLE:
	    sprintf(com,
		    "eval %s create oval %g %g %g %g -fill \"$pick(color,%d)\" -tags pk -width 1 -outline {\"\"}",
		    canvas, x-w, t-w, x+w, t+w, type);
	    break;
	case MARK_SQUARE:
	    sprintf(com,
		    "eval %s create polygon %g %g %g %g %g %g %g %g -fill \"$pick(color,%d)\" -tags pk -width 1;",
		    canvas, x-w, t-w, x-w, t+w, x+w, t+w, x+w, t-w, type);
	    break;
	default:
	    sprintf(com, " ");
	    break;
    }
    if(TCL_ERROR == Tcl_Eval(interp, com)) { return TCL_ERROR; }
    return TCL_OK;
}

/********************************************************************
Plot a pick mark accuracy on the normal Canvas
********************************************************************/
static int plotacc2normal(Tcl_Interp *interp, char *canvas, int mark_size,
			  double toffset, double height,
			  int t_axis, double tmin, double tmax,
			  double canvas_x, double t, double acc)
{
    double t1, t2;
    char com[256];  /** Buffer for Tcl/Tk command           **/

    /** convert into canvas coordintate **/
    if( t_axis > 0) {
	t1 = (toffset + (t - acc - tmin)*height/(tmax - tmin));
	t2 = (toffset + (t + acc - tmin)*height/(tmax - tmin));
    } else {
	t1 = (height + toffset - (t-acc-tmin)*height/(tmax - tmin));
	t2 = (height + toffset - (t+acc-tmin)*height/(tmax - tmin));
    }
    /** plot accuracy **/
    sprintf(com,
	    "%s create rectangle %g %g %g %g -fill gray -tags acc -width 0",
	    canvas, canvas_x - mark_size, t1, canvas_x + mark_size, t2); 
    if(TCL_ERROR == Tcl_Eval(interp, com)) { return TCL_ERROR; }
    return TCL_OK;
}

static int symboltype(Tcl_Interp *interp)
{
    char *stylename;
    stylename = TCL_GETSTRING(paste, "mark,style");
    if(strcmp(stylename, "splus") == 0) return(MARK_SPLUS);
    if(strcmp(stylename, "plus") == 0) return(MARK_PLUS);
    if(strcmp(stylename, "cross") == 0) return(MARK_CROSS);
    if(strcmp(stylename, "diamond") == 0) return(MARK_DIAMOND);
    if(strcmp(stylename, "circle") == 0) return(MARK_CIRCLE);
    if(strcmp(stylename, "square") == 0) return(MARK_SQUARE);
    return(MARK_SPLUS);
}

int pasteup_pickProc_sort(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    int i, j;

    Tcl_Obj *pmlist;       /** list of pick marks             **/
    Tcl_Obj *pick1,*pick2; /** pick data body                 **/
    int n_pick;            /** number of all the pick marks   **/
    double x1,x2;          /** x-position of pick             **/
    int type1,type2;       /** pick type                      **/

    Tcl_Obj *wk[1];
    
    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  pick sort list\n",
	     "     list   : list of pick marks (not a listname)\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 3) {
	show_usage();
	return TCL_ERROR;
    }

    /***** get the list *****/
    pmlist = objv[2];
    Tcl_ListObjLength(interp, pmlist, &n_pick);
    if (Tcl_IsShared(pmlist)) {
	pmlist = Tcl_DuplicateObj(pmlist);
    }

    /***** Bubble Sort by type *****/
    for(j=n_pick-1; j>0; j--) {
	for(i=0; i<j; i++) {
	    /** read i-th **/
	    Tcl_ListObjIndex(interp, pmlist, i, &pick1);
	    Tcl_ListObjIndex(interp, pick1, 4, wk);
	    Tcl_GetIntFromObj(interp, wk[0], &type1);
	    
	    /** read (i+1)-th **/
	    Tcl_ListObjIndex(interp, pmlist, i+1, &pick2);
	    Tcl_ListObjIndex(interp, pick2, 4, wk);
	    Tcl_GetIntFromObj(interp, wk[0], &type2);
	    
	    if( type1 > type2) {
		/** swap **/
		Tcl_ListObjReplace(interp, pmlist, i, 1, 1, &pick2);
		Tcl_ListObjReplace(interp, pmlist, i+1, 1, 1, &pick1);
	    }
	}
    }

    /***** Bubble Sort by offset and type *****/
    for(j=n_pick-1; j>0; j--) {
	for(i=0; i<j; i++) {
	    /** read i-th **/
	    Tcl_ListObjIndex(interp, pmlist, i, &pick1);
	    Tcl_ListObjIndex(interp, pick1, 1, wk);
	    Tcl_GetDoubleFromObj(interp, wk[0], &x1);
	    Tcl_ListObjIndex(interp, pick1, 4, wk);
	    Tcl_GetIntFromObj(interp, wk[0], &type1);
	    
	    /** read (i+1)-th **/
	    Tcl_ListObjIndex(interp, pmlist, i+1, &pick2);
	    Tcl_ListObjIndex(interp, pick2, 1, wk);
	    Tcl_GetDoubleFromObj(interp, wk[0], &x2);
	    Tcl_ListObjIndex(interp, pick2, 4, wk);
	    Tcl_GetIntFromObj(interp, wk[0], &type2);
	    
	    if(x1 > x2 && type1 == type2) {
		/** swap **/
		Tcl_ListObjReplace(interp, pmlist, i, 1, 1, &pick2);
		Tcl_ListObjReplace(interp, pmlist, i+1, 1, 1, &pick1);
	    }
	}
    }

    /***** Set result *****/
    Tcl_SetObjResult(interp, pmlist);
    return TCL_OK;
}

static int correspond_trace(double x, double sn, int hw_shot)
{
    int i;
    for(i=0;i<total_shot;i++){
	if( sn == gbl_rh[i].hw[hw_shot] && /* same shot and 5m uncertainty */
	    fabs(x - gbl_rh[i].x) <= 0.005 ) {
	    return(i);
	}
    }
    return(-1); /* no corresponding trace */
}

static void read_single_pick(Tcl_Interp *interp, Tcl_Obj *pick,
			    int *sn, double *x, double *t, double *acc,
			    int *type)
{
    Tcl_Obj* dummy[1];
    /** get x, t and type of this pick **/
    Tcl_ListObjIndex(interp, pick, 0, dummy);
    Tcl_GetIntFromObj(interp, dummy[0], sn);
    Tcl_ListObjIndex(interp, pick, 1, dummy);
    Tcl_GetDoubleFromObj(interp, dummy[0], x);
    Tcl_ListObjIndex(interp, pick, 2, dummy);
    Tcl_GetDoubleFromObj(interp, dummy[0], t);
    Tcl_ListObjIndex(interp, pick, 3, dummy);
    Tcl_GetDoubleFromObj(interp, dummy[0], acc);
    Tcl_ListObjIndex(interp, pick, 4, dummy);
    Tcl_GetIntFromObj(interp, dummy[0], type);
}

static double shift_time(int sn, double gx, double t, double rv){
    if(sn<0) { /* no corresponding trace */
	if(rv>0.0) return(t - fabs(gx / rv));
    } else {   /* shift */
	return(t - gbl_rh[sn].shift);
    }
    return(t);
}

int pasteup_pickProc_plotall(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    int i;

    Tcl_Obj *pmlist;/** list of pick marks                  **/
    Tcl_Obj *pick;  /** pick data body                      **/
    int n_pick;     /** number of all the pick marks        **/
    int n_start;    /** plot only n-th and later picks      **/

    double gx;      /** offset distance in km               **/
    double t;       /** graph time (reduction time)         **/
    double acc;     /** accuracy                            **/
    double cx;      /** canvas x                            **/
    
    int type;       /** type                                **/
    char com[256];  /** Buffer for Tcl/Tk command           **/
    int t_axis;     /** flag of time-axis direction         **/
    
    double rv;      /** reduction velocity (km/sec)         **/
    double tmin;
    double tmax;
    double xmin;
    double xmax;
    int xaxis_direc;/** xaxis direction                     **/
    int mark_size;  /** horizontal mark size in pixel       **/
    int flag_acc;   /** flag for show/hide uncertainty      **/
    double toffset; /** top offset                          **/
    double height;  /** graph height                        **/
    double loffset; /** left offset                         **/
    double width;   /** graph width                         **/
    char *canvas;   /** canvas widget name                  **/
    int canvas_mode;/** canvas mode                         **/
    int hw_shot;
    int mark;       /** mark type                           **/

    int sn;          /** shot number **/

    char vname[5];
    Blt_Vector *vx[20], *vt[20], *vh[20], *vl[20];
    int n[20], max[20], nacc;
    double *px[20], *pt[20], *ph[20], *pl[20];

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  pick plotall listName n\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 4) {
	show_usage();
	return TCL_ERROR;
    }
    /***** Get pick mark list *****/
    pmlist = Tcl_GetVar2Ex(interp, Tcl_GetStringFromObj(objv[2], NULL),
			   NULL, TCL_GLOBAL_ONLY);
    Tcl_ListObjLength(interp, pmlist, &n_pick);
    /***** Get n *****/
    if( TCL_ERROR == Tcl_GetIntFromObj(interp, objv[3], &n_start) ) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Read global variables *****/
    TCL_GET(Int, paste, "canvas,mode", canvas_mode);
    TCL_GET(Int, paste, "mark,uncertainty", flag_acc);
    TCL_GET(Double, paste, "reduction", rv);
    TCL_GET(Int, gbl, "option,su,shotnumber", hw_shot);

    /*************** Plot all the picks inside area ***************/
    if(canvas_mode == 2) {
	if(n_start > 0) {
	    /***** plot onto the GraphCanvas, from n_start to end *****/
	    for(i=n_start;i<n_pick;i++){
		Tcl_ListObjIndex(interp, pmlist, i, &pick);
		read_single_pick(interp, pick, &sn, &gx, &t, &acc, &type);
		sn = correspond_trace(gx, sn, hw_shot);
		t = shift_time(sn, gx, t, rv);
		type = (type-1) % 20;
		sprintf(com, "vx%d append %g", type, gx);
		if(TCL_ERROR == Tcl_Eval(interp, com)) { return TCL_ERROR; }
		sprintf(com, "vt%d append %g", type, t);
		if(TCL_ERROR == Tcl_Eval(interp, com)) { return TCL_ERROR; }
		if (flag_acc == 1) {
		    sprintf(com, "vh%d append %g", type, t+acc);
		    if(TCL_ERROR == Tcl_Eval(interp, com)) { return TCL_ERROR; }
		    sprintf(com, "vl%d append %g", type, t-acc);
		    if(TCL_ERROR == Tcl_Eval(interp, com)) { return TCL_ERROR; }
		}
	    }
	} else {
	    /***** plot onto the GraphCanvas, all data *****/
	    for(i=0;i<20;i++){
		max[i] = 500;  /* initial value for array size */
		n[i] = 0;
		if(NULL == (px[i]=(double *)malloc(max[i] * sizeof(double))) ||
		   NULL == (pt[i]=(double *)malloc(max[i] * sizeof(double))) ||
		   NULL == (ph[i]=(double *)malloc(max[i] * sizeof(double))) ||
		   NULL == (pl[i]=(double *)malloc(max[i] * sizeof(double)))) {
		    fprintf(stderr,
			    "Error: Memory allocation failed in %s, line %d\n",
			    __FILE__, __LINE__);
		    return TCL_ERROR;
		}
	    }
	    for(i=n_start;i<n_pick;i++){
		Tcl_ListObjIndex(interp, pmlist, i, &pick);
		read_single_pick(interp, pick, &sn, &gx, &t, &acc, &type);
		sn = correspond_trace(gx, sn, hw_shot);
		t = shift_time(sn, gx, t, rv);
		type = (type - 1)%20;
		if( n[type]+1 == max[type] ) {
		    max[type] += 500;
		    if(NULL == (px[type]=(double *)realloc(px[type], max[type] * sizeof(double))) ||
		       NULL == (pt[type]=(double *)realloc(pt[type], max[type] * sizeof(double))) ||
		       NULL == (ph[type]=(double *)realloc(ph[type], max[type] * sizeof(double))) ||
		       NULL == (pl[type]=(double *)realloc(pl[type], max[type] * sizeof(double)))){
			fprintf(stderr,
				"Error: Memory allocation failed in %s, line %d\n",
				__FILE__, __LINE__);
			return TCL_ERROR;
		    }
		}
		px[type][ n[type] ] = gx;
		pt[type][ n[type] ] = t;
		ph[type][ n[type] ] = t + acc;
		pl[type][ n[type]++ ] = t - acc;
	    }
	    for(i=0;i<20;i++){
		sprintf(com, "vx%d length 0;"
			"vt%d length 0;"
			"vh%d length 0;"
			"vl%d length 0;", i, i, i, i);
		if(TCL_ERROR == Tcl_Eval(interp, com)) { return TCL_ERROR; }
		nacc = (flag_acc) ? n[i] : 0;
		sprintf(vname, "vx%d", i);
		if( (Blt_GetVector(interp,vname,&vx[i])!=TCL_OK) ||
		    (Blt_ResetVector(vx[i], px[i], n[i], n[i], TCL_VOLATILE)
		     != TCL_OK) ) return TCL_ERROR;
		sprintf(vname, "vt%d", i);
		if( (Blt_GetVector(interp,vname,&vt[i])!=TCL_OK) ||
		    (Blt_ResetVector(vt[i], pt[i], n[i], n[i], TCL_VOLATILE)
		     != TCL_OK) ) return TCL_ERROR;
		sprintf(vname, "vh%d", i);
		if( (Blt_GetVector(interp,vname,&vh[i])!=TCL_OK) ||
		    (Blt_ResetVector(vh[i], ph[i], nacc, nacc, TCL_VOLATILE)
		     != TCL_OK) ) return TCL_ERROR;
		sprintf(vname, "vl%d", i);
		if( (Blt_GetVector(interp,vname,&vl[i])!=TCL_OK) ||
		    (Blt_ResetVector(vl[i], pl[i], nacc, nacc, TCL_VOLATILE)
		     != TCL_OK) ) return TCL_ERROR;
		free(px[i]);
		free(pt[i]);
		free(ph[i]);
		free(pl[i]);
	    }
	}
    } else {
	/***** Plot onto the NormalCanvas *****/
	TCL_GET(Double, paste, "left", xmin);
	TCL_GET(Double, paste, "right", xmax);
	TCL_GET(Double, paste, "top", tmin);
	TCL_GET(Double, paste, "bottom", tmax);
	TCL_GET(Int, paste, "mark,size", mark_size);
	TCL_GET(Double, paste, "graph_height", height);
	TCL_GET(Double, paste, "graph_width", width);
	TCL_GET(Int, gbl, "t_axis", t_axis);
	TCL_GET(Double, paste, "left_margin", loffset);
	TCL_GET(Double, paste, "top_margin", toffset);
	TCL_GET(Int, gbl, "data,xaxis_direction", xaxis_direc);

	mark = symboltype(interp);
	canvas = TCL_GETSTRING(wk, "w,canvas,c");
	for(i=n_start;i<n_pick;i++){
	    Tcl_ListObjIndex(interp, pmlist, i, &pick);
	    read_single_pick(interp, pick, &sn, &gx, &t, &acc, &type);
	    sn = correspond_trace(gx, sn, hw_shot);
	    t = shift_time(sn, gx, t, rv);
	    if( (tmax-t)*(t-tmin) >= 0.0 && (xmax-gx)*(gx-xmin) >= 0.0 ) {
		if(xaxis_direc == 0) {
		    /* normal */
		    cx = loffset + (gx-xmin)*width/(xmax - xmin);
		} else {
		    /* reverse */
		    cx = loffset + (xmax - gx)*width/(xmax - xmin);
		}
		if(flag_acc == 1) {
		    plotacc2normal(interp, canvas, mark_size, toffset,
				   height, t_axis, tmin, tmax, cx, t, acc);
		}
		plot2normal(interp, canvas, mark_size, type, toffset,
			    height, t_axis, tmin, tmax, cx, t, mark);
	    }
	}
	sprintf(com, "%s raise pk", canvas);
	if(TCL_ERROR == Tcl_Eval(interp, com)) { return TCL_ERROR; }
    }
    return TCL_OK;
}

int pasteup_pickProc_plot1(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    double x;       /** offset distance in km               **/
    double t;       /** graph time (reduction time)         **/
    double acc;     /** accuracy for pick time (sec)        **/
    int type;       /** pick type                           **/
    int img_mode;   /** flag of image mode                  **/
    int mark_size;  /** horizontal mark size in pixel       **/
    int flag_acc;   /** flag for show/hide uncertainty      **/
    char *canvas;   /** canvas widget name                  **/
    int canvas_mode;/** canvas mode                         **/
    
    double rv;      /** reduction velocity (km/sec)         **/
    double toffset; /** top offset                          **/
    double height;  /** graph height                        **/
    double loffset; /** left offset                         **/
    double width;   /** graph width                         **/
    double tmin;
    double tmax;
    double xmin;
    double xmax;
    double cx;      /** canvas x                            **/
    int t_axis;     /** flag of time-axis direction         **/
    int xaxis_direc;/** xaxis direction                     **/

    char com[256];  /** Buffer for Tcl/Tk command           **/

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  pick plot x t acc type\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 6) {
	show_usage();
	return TCL_ERROR;
    }
    /***** Get pick info *****/
    if(
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &x)  ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &t)  ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[4], &acc)||
       TCL_ERROR == Tcl_GetIntFromObj(interp, objv[5], &type)) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Read global variables *****/
    TCL_GET(Double, paste, "reduction", rv);
    TCL_GET(Double, paste, "left", xmin);
    TCL_GET(Double, paste, "right", xmax);
    TCL_GET(Double, paste, "graph_width", width);
    TCL_GET(Int, paste,"mark,size", mark_size);
    TCL_GET(Int, paste, "mark,uncertainty", flag_acc);

    /** reduction **/
    if(rv>0.0) t = t - fabs(x / rv);

    /***** Draw a pick mark *****/
    TCL_GET(Int, paste, "canvas,mode", canvas_mode);
    if(canvas_mode == 2){
	/***** Plot onto the GraphCanvas *****/
	type = (type-1) % 20;
	sprintf(com, "vx%d append %g", type, x);
	if(TCL_ERROR == Tcl_Eval(interp, com)) { return TCL_ERROR; }
	sprintf(com, "vt%d append %g", type, t);
	if(TCL_ERROR == Tcl_Eval(interp, com)) { return TCL_ERROR; }
	sprintf(com, "vh%d append %g", type, t+acc);
	if(TCL_ERROR == Tcl_Eval(interp, com)) { return TCL_ERROR; }
	sprintf(com, "vl%d append %g", type, t-acc);
	if(TCL_ERROR == Tcl_Eval(interp, com)) { return TCL_ERROR; }
    } else {
	/***** Plot onto the NormalCanvas *****/
	TCL_GET(Int, paste, "image,mode", img_mode);
	TCL_GET(Double, paste, "top", tmin);
	TCL_GET(Double, paste, "bottom", tmax);
	TCL_GET(Int, gbl, "t_axis", t_axis);
	TCL_GET(Int, gbl, "data,xaxis_direction", xaxis_direc);
	TCL_GET(Double, paste, "left_margin", loffset);
	TCL_GET(Double, paste, "top_margin", toffset);
	TCL_GET(Double, paste, "graph_height", height);
	canvas = TCL_GETSTRING(wk, "w,canvas,c");
	
	/* conver graph_x into canvas_x */
	if(xaxis_direc == 0) {
	    /* normal */
	    cx = loffset + (x-xmin)*width/(xmax - xmin);
	} else {
	    /* reverse */
	    cx = loffset + (xmax - x)*width/(xmax - xmin);
	}
	
	/* draw uncertainty, if necessary */
	if(flag_acc == 1) {
	    plotacc2normal(interp, canvas, mark_size, toffset,
			   height, t_axis, tmin, tmax, cx, t, acc);
	}
	/** draw pick mark **/
	plot2normal(interp, canvas, mark_size, type, toffset,
		    height, t_axis, tmin, tmax, cx, t, symboltype(interp));
    }
    return TCL_OK;
}

int pasteup_pickProc_listup(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    int i;
    char *arg;

    Tcl_Obj *pmlist; /** list of pick marks                 **/
    Tcl_Obj *pick;   /** pick data body                     **/
    int n_pick;      /** number of all the pick marks       **/
    int hw_shot, sn;

    Tcl_Obj* dummy[1];
    double x, t;
    double x1, x2;
    double t1, t2;
    double rv;


    Tcl_Obj* list;
    
    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  pick listup list x1 x2 t1 t2 rv\n",
	     "     list   : listname of pick mark\n",
	     "     x1, x2 : offset range (km)\n",
	     "     t1, t2 : time range (sec)\n",
	     "     rv     : reduction velocity (km/sec)\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 8) {
	show_usage();
	return TCL_ERROR;
    }
    /***** Get pick mark list *****/
    arg = Tcl_GetStringFromObj(objv[2], NULL);
    pmlist = Tcl_GetVar2Ex(interp, arg, NULL, 0);

    if(
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &x1) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[4], &x2) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[5], &t1) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[6], &t2) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[7], &rv) ) {
	show_usage();
	return TCL_ERROR;
    }

    /***** For shot time correction *****/
    TCL_GET(Int, gbl, "option,su,shotnumber", hw_shot);

    /***** Initialize result list *****/
    list = Tcl_NewListObj(0, NULL);

    /***** Check all the picks whether in or out? *****/
    Tcl_ListObjLength(interp, pmlist, &n_pick);
    for(i=0;i<n_pick;i++){
	/** get corresponding pick data **/
	Tcl_ListObjIndex(interp, pmlist, i, &pick);
	/** get x and t of this pick **/
	Tcl_ListObjIndex(interp, pick, 0, dummy);
	Tcl_GetIntFromObj(interp, dummy[0], &sn);
	Tcl_ListObjIndex(interp, pick, 1, dummy);
	Tcl_GetDoubleFromObj(interp, dummy[0], &x);
	Tcl_ListObjIndex(interp, pick, 2, dummy);
	Tcl_GetDoubleFromObj(interp, dummy[0], &t);
	/** time correction **/
	sn = correspond_trace(x, sn, hw_shot);
	if(sn<0) { /* no corresponding trace */
	    if(rv>0.0) t = t - fabs(x / rv);
	} else {   /* shift */
	    t = t - gbl_rh[sn].shift;
	}

	/** in/out? **/
	if( (t2 - t)*(t - t1) >= 0.0 && (x2 - x)*(x - x1) >= 0.0 ) {
	    TCL_APPEND(list, Int, i);
	}	 
    }
    /***** Set result *****/
    Tcl_SetObjResult(interp, list);
    return TCL_OK;
}

int pasteup_pickProc_listtype(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    int i, j;
    char *arg;

    Tcl_Obj *pmlist; /** list of pick marks               **/
    Tcl_Obj *pick;   /** pick data body                   **/
    int n_pick;      /** number of all the pick marks     **/

    Tcl_Obj* dummy[1];

    Tcl_Obj* list;

    int type, n_type;
    int typelist[500];
    
    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  pick listtype list\n",
	     "     list   : listname of pick mark\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 3) {
	show_usage();
	return TCL_ERROR;
    }
    /***** Get pick mark list *****/
    arg = Tcl_GetStringFromObj(objv[2], NULL);
    pmlist = Tcl_GetVar2Ex(interp, arg, NULL, 0);

    /***** Make list type list *****/
    n_type = 0;
    Tcl_ListObjLength(interp, pmlist, &n_pick);
    for(i=0;i<n_pick;i++){
	/** get corresponding pick data **/
	Tcl_ListObjIndex(interp, pmlist, i, &pick);
	/** get type **/
	Tcl_ListObjIndex(interp, pick, 4, dummy);
	Tcl_GetIntFromObj(interp, dummy[0], &type);
	/** new one? **/
	for(j=0;j<n_type;j++) {
	    if(type == typelist[j])break;
	}
	if(j>=n_type){
	    typelist[n_type++] = type;
	    if(n_type>=500){
		fprintf(stderr, "too many pick type, unbelievable.....\n");
		exit(1);
	    }
	}
    }
    /***** Set result *****/
    list = Tcl_NewListObj(0, NULL);
    for(j=0;j<n_type;j++) {
	TCL_APPEND(list, Int, typelist[j]);
    }
    Tcl_SetObjResult(interp, list);
    return TCL_OK;
}

int pasteup_pickProc_nearest(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    int i;
    char *arg;

    Tcl_Obj *pmlist; /** list of pick marks               **/
    Tcl_Obj *pick;   /** pick data body                   **/
    int n_pick;      /** number of all the pick marks     **/
    double x;        /** x-position of pick               **/
    double t;        /** pick time                        **/

    Tcl_Obj* dummy[1];
    double gx;      /** graph x-coordinate                **/
    double gt;      /** graph t-coordinate                **/
    double tol;     /** horizontal distance tolerance (km)**/

    double height;  /** graph height                      **/
    double top ;    /** t-min (in sec)                    **/
    double bottom;  /** t-max (in sec)                    **/
    double rv;      /** reduction velocity                **/

    int hw_shot, sn;
    double dx, min_dt, dt_graph;
    int imin;
    
    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  pick nearest list gx rt tol\n",
	     "     list   : listname of pick mark\n",
	     "     x      : x-position (km)\n",
	     "     rt     : reduced time (sec)\n",
	     "     tol    : tolerance (km)\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 6) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Read global variables *****/
    TCL_GET(Double, paste, "top", top);
    TCL_GET(Double, paste, "bottom", bottom);
    TCL_GET(Double, paste, "graph_height", height);
    TCL_GET(Int, gbl, "option,su,shotnumber", hw_shot);
    TCL_GET(Double, paste, "reduction", rv);

    /***** Get pick mark list *****/
    arg = Tcl_GetStringFromObj(objv[2], NULL);
    pmlist = Tcl_GetVar2Ex(interp, arg, NULL, 1);

    if(
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &gx) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[4], &gt) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[5], &tol) ) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Initialize *****/
    imin = -1;
    min_dt = 1.0e10;

    /***** Check all the picks  *****/
    Tcl_ListObjLength(interp, pmlist, &n_pick);
    for(i=0;i<n_pick;i++){
	/** get corresponding pick data **/
	Tcl_ListObjIndex(interp, pmlist, i, &pick);
	/** get x of this pick **/
	Tcl_ListObjIndex(interp, pick, 1, dummy);
	Tcl_GetDoubleFromObj(interp, dummy[0], &x);
	dx = fabs(x - gx);
	if(dx < tol){
	    /** get t of this pick **/
	    Tcl_ListObjIndex(interp, pick, 0, dummy);
	    Tcl_GetIntFromObj(interp, dummy[0], &sn);
	    Tcl_ListObjIndex(interp, pick, 2, dummy);
	    Tcl_GetDoubleFromObj(interp, dummy[0], &t);
	    /** time correction **/
	    sn = correspond_trace(x, sn, hw_shot);
	    if(sn<0) { /* no corresponding trace */
		if(rv>0.0) t = t - fabs(x / rv);
	    } else {   /* shift */
		t = t - gbl_rh[sn].shift;
	    }
	    dt_graph = fabs(t - gt)*height/(bottom-top);
	    if( dt_graph < min_dt ) {
		min_dt = dt_graph;
		imin = i;
	    }
	}
    }
    /***** Set result *****/
    if(min_dt > 10) {
	Tcl_SetObjResult(interp, Tcl_NewIntObj(-1));	
    } else {
	Tcl_SetObjResult(interp, Tcl_NewIntObj(imin));	
    }
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

int pasteup_pickProc_convert(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    int i;

    Tcl_Obj *pmlist; /** list of pick marks               **/
    Tcl_Obj *pick;   /** pick data body                   **/
    Tcl_Obj *newpm;  /** result pick mark list            **/
    Tcl_Obj *newpick;/** a new pick datum for output      **/
    int n_pick;      /** number of all the pick marks     **/

    Tcl_Obj *wk[1];

    int type, type1, type2;

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  pick convert list type1 type2\n",
	     "     list   : pick mark list\n",
	     "     type1  : pick type (current)\n",
	     "     type2  : pick type (new)\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 5) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Get pick mark list *****/
    pmlist = objv[2];
    Tcl_ListObjLength(interp, pmlist, &n_pick);
    if (Tcl_IsShared(pmlist)) {
	pmlist = Tcl_DuplicateObj(pmlist);
    }

    /***** read arguments *****/
    if(
       TCL_ERROR == Tcl_GetIntFromObj(interp, objv[3], &type1) ||
       TCL_ERROR == Tcl_GetIntFromObj(interp, objv[4], &type2) ) {
	show_usage();
	return TCL_ERROR;
    }
    if (type1 == type2) {
	/* no conversion */
	return TCL_OK;
    }

    /***** Create new pick mark list (result) *****/
    newpm = Tcl_NewListObj(0, NULL);

    /***** Conversion *****/
    for(i=0;i<n_pick;i++){
	Tcl_ListObjIndex(interp, pmlist, i, &pick);
	Tcl_ListObjIndex(interp, pick, 4, wk);
	Tcl_GetIntFromObj(interp, wk[0], &type);
	if (type == type1) {
	    /*** convert ***/
	    newpick = Tcl_NewListObj(0, NULL);
	    Tcl_ListObjIndex(interp, pick, 0, wk);
	    Tcl_ListObjAppendElement(interp, newpick, wk[0]);
	    Tcl_ListObjIndex(interp, pick, 1, wk);
	    Tcl_ListObjAppendElement(interp, newpick, wk[0]);
	    Tcl_ListObjIndex(interp, pick, 2, wk);
	    Tcl_ListObjAppendElement(interp, newpick, wk[0]);
	    Tcl_ListObjIndex(interp, pick, 3, wk);
	    Tcl_ListObjAppendElement(interp, newpick, wk[0]);
	    wk[0] = Tcl_NewIntObj(type2);
	    Tcl_ListObjAppendElement(interp, newpick, wk[0]);

	    Tcl_ListObjAppendElement(interp, newpm, newpick);
	    
	} else {
	    /** output as is **/
	    Tcl_ListObjAppendElement(interp, newpm, pick);
	}
    }

    /***** Set result *****/
    Tcl_SetObjResult(interp, newpm);
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

int pasteup_pickProc_delete(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    int i;

    Tcl_Obj *pmlist; /** list of pick marks               **/
    Tcl_Obj *pick;   /** pick data body                   **/
    Tcl_Obj *newpm;  /** result pick mark list            **/
    int n_pick;      /** number of all the pick marks     **/

    Tcl_Obj *wk[1];

    int type, del_type;

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  pick delete list type\n",
	     "     list   : pick mark list\n",
	     "     type   : pick type to delete\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 4) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Get pick mark list *****/
    pmlist = objv[2];
    Tcl_ListObjLength(interp, pmlist, &n_pick);
    if (Tcl_IsShared(pmlist)) {
	pmlist = Tcl_DuplicateObj(pmlist);
    }

    /***** read arguments *****/
    if( TCL_ERROR == Tcl_GetIntFromObj(interp, objv[3], &del_type) ) {
       show_usage();
       return TCL_ERROR;
    }

    /***** Create new pick mark list (result) *****/
    newpm = Tcl_NewListObj(0, NULL);

    /***** Conversion *****/
    for(i=0;i<n_pick;i++){
	Tcl_ListObjIndex(interp, pmlist, i, &pick);
	Tcl_ListObjIndex(interp, pick, 4, wk);
	Tcl_GetIntFromObj(interp, wk[0], &type);
	if (type != del_type) {
	    /** output as is **/
	    Tcl_ListObjAppendElement(interp, newpm, pick);
	}
    }

    /***** Set result *****/
    Tcl_SetObjResult(interp, newpm);
    return TCL_OK;
}

int pasteup_pickProc_region(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    int i;
    char *arg;
    char *subopt;

    Tcl_Obj *pmlist; /** list of pick marks               **/
    Tcl_Obj *pick;   /** a pick datum (sn,x,t,acc,type)   **/
    Tcl_Obj *subpick;/** a member of a pick datum         **/ 
    Tcl_Obj *newpick;/** a new pick datum for output      **/
    Tcl_Obj *newpm;  /** result pick mark list            **/

    double x1, x2;   /** region                           **/
    double t1, t2; 
    double rv;       /** reduction velocity               **/
    double x, t;     /** pick data(x, t)                  **/
    double rt;       /** reduced time                     **/

    int hw_shot, sn;

    int n_pick;
    int type, applytype;

    /******************** Main ********************/

    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n pick region subopt listName Type x1 x2 t1 t2 rv\n",
	     "  subopt is one of accuracy, type, delete",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc < 10) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Get sub-Option *****/
    subopt = Tcl_GetStringFromObj(objv[2], NULL);

    /***** Get pick mark list *****/
    arg = Tcl_GetStringFromObj(objv[3], NULL);
    pmlist = Tcl_GetVar2Ex(interp, arg, NULL, 1);

    /***** Get region info ****/
    if(
       TCL_ERROR == Tcl_GetIntFromObj(interp, objv[4], &applytype) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[5], &x1) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[6], &x2) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[7], &t1) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[8], &t2) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[9], &rv) ) {
	show_usage();
	return TCL_ERROR;
    }

    /***** For shot time correction *****/
    TCL_GET(Int, gbl, "option,su,shotnumber", hw_shot);

    /***** Create new pick mark list (result) *****/
    newpm = Tcl_NewListObj(0, NULL);

    if (strcmp("type", subopt) == 0) {
	/*************** Set type in the region ***************/
	void show_sub_usage () {
	    Tcl_AppendResult
		(interp,
		 "wrong usage :\n",
		 "  pick region type listName Type x1 x2 t1 t2 rvn\n",
		 "     listName    : listname of all the pick marks\n",
		 "     Type        : apply to this type (0 means all)\n",
		 "     x1,x2,t1,t2 : region\n",
		 "     n           : new type number\n",
		 (char *)NULL);
	};
	/** check number of arguments **/
	if(objc < 11){
	    show_sub_usage();
	    return TCL_ERROR;
	}

	/***** Check all the picks whether in or out? *****/
	Tcl_ListObjLength(interp, pmlist, &n_pick);
	for(i=0;i<n_pick;i++){
	    /** get corresponding pick data **/
	    Tcl_ListObjIndex(interp, pmlist, i, &pick);
	    /** check type **/
	    if(applytype != 0) {
		Tcl_ListObjIndex(interp, pick, 4, &subpick);
		Tcl_GetIntFromObj(interp, subpick, &type);
		if(type != applytype){
		    /** different type; output as is **/
		    Tcl_ListObjAppendElement(interp, newpm, pick);
		    continue;
		}
	    }
	    /** get x, t of this pick **/
	    Tcl_ListObjIndex(interp, pick, 0, &subpick);
	    Tcl_GetIntFromObj(interp, subpick, &sn);
	    Tcl_ListObjIndex(interp, pick, 1, &subpick);
	    Tcl_GetDoubleFromObj(interp, subpick, &x);
	    Tcl_ListObjIndex(interp, pick, 2, &subpick);
	    Tcl_GetDoubleFromObj(interp, subpick, &t);

	    /** time correction **/
	    sn = correspond_trace(x, sn, hw_shot);
	    if(sn<0) { /* no corresponding trace */
		if(rv>0.0) rt = t - fabs(x / rv);
		else rt = t;
	    } else {   /* shift */
		rt = t - gbl_rh[sn].shift;
	    }

	    /** in/out? **/
	    if( (t2 - rt)*(rt - t1) >= 0.0 && (x2 - x)*(x - x1) >= 0.0 ) {
		/** in; change type **/
		newpick = Tcl_NewListObj(0, NULL);
		/* shot number */
		Tcl_ListObjIndex(interp, pick, 0, &subpick);
		Tcl_ListObjAppendElement(interp, newpick, subpick);
		/* offset ;x */
		TCL_APPEND(newpick, Double, x);
		/* pick imte */
		TCL_APPEND(newpick, Double, t);
		/* accuracy */
		Tcl_ListObjIndex(interp, pick, 3, &subpick);
		Tcl_ListObjAppendElement(interp, newpick, subpick);
		/* pick type */
		Tcl_ListObjAppendElement(interp, newpick, objv[10]);

		/* append */
		Tcl_ListObjAppendElement(interp, newpm, newpick);
	    } else {
		/** out; output as is **/
		Tcl_ListObjAppendElement(interp, newpm, pick);
	    }
	}
	Tcl_SetObjResult(interp, newpm);
    } else if (strcmp("accuracy", subopt) == 0) {
	/*************** change accuracy in the region ***************/
	void show_sub_usage () {
	    Tcl_AppendResult
		(interp,
		 "wrong usage :\n",
		 "  pick region accuracy listName Type x1 x2 t1 t2 acc\n",
		 "     listName    : listname of all the pick marks\n",
		 "     Type        : apply to this type (0 means all)\n",
		 "     x1,x2,t1,t2 : region\n",
		 "     acc         : new accuracy\n",
		 (char *)NULL);
	};
	/** check number of arguments **/
	if(objc < 10) {
	    show_sub_usage();
	    return TCL_ERROR;
	}
	/***** Check all the picks whether in or out? *****/
	Tcl_ListObjLength(interp, pmlist, &n_pick);
	for(i=0;i<n_pick;i++){
	    /** get corresponding pick data **/
	    Tcl_ListObjIndex(interp, pmlist, i, &pick);
	    /** check type **/
	    if(applytype != 0) {
		Tcl_ListObjIndex(interp, pick, 4, &subpick);
		Tcl_GetIntFromObj(interp, subpick, &type);
		if(type != applytype){
		    /** different type; output as is **/
		    Tcl_ListObjAppendElement(interp, newpm, pick);
		    continue;
		}
	    }
	    /** get x and t of this pick **/
	    Tcl_ListObjIndex(interp, pick, 0, &subpick);
	    Tcl_GetIntFromObj(interp, subpick, &sn);
	    Tcl_ListObjIndex(interp, pick, 1, &subpick);
	    Tcl_GetDoubleFromObj(interp, subpick, &x);
	    Tcl_ListObjIndex(interp, pick, 2, &subpick);
	    Tcl_GetDoubleFromObj(interp, subpick, &t);

	    /** time correction **/
	    sn = correspond_trace(x, sn, hw_shot);
	    if(sn<0) { /* no corresponding trace */
		if(rv>0.0) rt = t - fabs(x / rv);
		else rt = t;
	    } else {   /* shift */
		rt = t - gbl_rh[sn].shift;
	    }

	    /** in/out? **/
	    if( (t2 - rt)*(rt - t1) >= 0.0 && (x2 - x)*(x - x1) >= 0.0 ) {
		/** in; change type **/
		newpick = Tcl_NewListObj(0, NULL);
		/* shot number */
		Tcl_ListObjIndex(interp, pick, 0, &subpick);
		Tcl_ListObjAppendElement(interp, newpick, subpick);
		/* offset ;x */
		TCL_APPEND(newpick, Double, x);
		/* pick imte */
		TCL_APPEND(newpick, Double, t);
		/* accuracy */
		Tcl_ListObjAppendElement(interp, newpick, objv[10]);
		/* pick type */
		Tcl_ListObjIndex(interp, pick, 4, &subpick);
		Tcl_ListObjAppendElement(interp, newpick, subpick);

		/* append */
		Tcl_ListObjAppendElement(interp, newpm, newpick);
	    } else {
		/** out; output as is **/
		Tcl_ListObjAppendElement(interp, newpm, pick);
	    }
	}
	Tcl_SetObjResult(interp, newpm);
    } else if (strcmp("delete", subopt) == 0) {
	/*************** delete picks in the region ***************/

	/***** Check all the picks whether in or out? *****/
	Tcl_ListObjLength(interp, pmlist, &n_pick);
	for(i=0;i<n_pick;i++){
	    /** get corresponding pick data **/
	    Tcl_ListObjIndex(interp, pmlist, i, &pick);
	    /** check type **/
	    if(applytype != 0) {
		Tcl_ListObjIndex(interp, pick, 4, &subpick);
		Tcl_GetIntFromObj(interp, subpick, &type);
		if(type != applytype){
		    /** different type; output as is **/
		    Tcl_ListObjAppendElement(interp, newpm, pick);
		    continue;
		}
	    }
	    /** get x and t of this pick **/
	    Tcl_ListObjIndex(interp, pick, 0, &subpick);
	    Tcl_GetIntFromObj(interp, subpick, &sn);
	    Tcl_ListObjIndex(interp, pick, 1, &subpick);
	    Tcl_GetDoubleFromObj(interp, subpick, &x);
	    Tcl_ListObjIndex(interp, pick, 2, &subpick);
	    Tcl_GetDoubleFromObj(interp, subpick, &t);

	    /** time correction **/
	    sn = correspond_trace(x, sn, hw_shot);
	    if(sn<0) { /* no corresponding trace */
		if(rv>0.0) rt = t - fabs(x / rv);
		else rt = t;
	    } else {   /* shift */
		rt = t - gbl_rh[sn].shift;
	    }
	    
	    /** in/out? **/
	    if( (t2 - rt)*(rt - t1) < 0.0 || (x2 - x)*(x - x1) < 0.0 ) {
		/** out; output as is **/
		Tcl_ListObjAppendElement(interp, newpm, pick);
	    }
	}
	Tcl_SetObjResult(interp, newpm);
    } else {
	/** invalid sub-option for the region operation **/
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n pick region suboption varName1 list\n",
	     "  suboption is one of accuracy, type and delete",
	     (char *)NULL);
	return TCL_ERROR;
    }
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

int pasteup_pickProc_tt2cy(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    double x;       /** offset distance in km               **/
    double t;       /** true time                           **/

    int t_axis;     /** flag of time-axis direction         **/
    double toffset; /** top offset                          **/
    double top ;    /** t-min (in sec)                      **/
    double bottom;  /** t-max (in sec)                      **/
    double height;  /** graph height                        **/
    double rv;      /** reduction velocity in km/sec        **/
    int hw_shot, sn;


    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  pick tt2cy sn x t\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 5) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Read arguments *****/
    if(
       TCL_ERROR == Tcl_GetIntFromObj(interp, objv[2], &sn) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &x) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[4], &t) ) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Read global variables *****/
    TCL_GET(Double, paste, "top_margin", toffset);
    TCL_GET(Double, paste, "top", top);
    TCL_GET(Double, paste, "bottom", bottom);
    TCL_GET(Double, paste, "graph_height", height);
    TCL_GET(Double, paste, "reduction", rv);
    TCL_GET(Int, gbl, "t_axis", t_axis);
    TCL_GET(Int, gbl, "option,su,shotnumber", hw_shot);

    /***** Convert cy into graph coordinate t without reduction *****/
    sn = correspond_trace(x, sn, hw_shot);
    if(sn<0) { /* no corresponding trace */
	if(rv>0.0) t -= fabs(x / rv);
    } else {   /* shift */
	t -= gbl_rh[sn].shift;
    }

    /** compute **/
    if(t_axis > 0) {
	t = toffset + (t - top)*height/(bottom - top);
    } else {
        t = height + toffset - (t - top)*height/(bottom - top);
    }

    Tcl_SetObjResult(interp,Tcl_NewDoubleObj(t));
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

int pasteup_pickProc_tt2gt(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    double x;       /** offset distance in km               **/
    double t;       /** true time                           **/

    double rv;      /** reduction velocity in km/sec        **/
    int hw_shot, sn;


    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  pick tt2gt sn x t\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 5) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Read arguments *****/
    if(
       TCL_ERROR == Tcl_GetIntFromObj(interp, objv[2], &sn) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &x) ||
       TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[4], &t) ) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Read global variables *****/
    TCL_GET(Double, paste, "reduction", rv);
    TCL_GET(Int, gbl, "option,su,shotnumber", hw_shot);

    /** time correction **/
    sn = correspond_trace(x, sn, hw_shot);
    if(sn<0) { /* no corresponding trace */
	if(rv>0.0) t -= fabs(x / rv);
    } else {   /* shift */
	t -= gbl_rh[sn].shift;
    }

    Tcl_SetObjResult(interp,Tcl_NewDoubleObj(t));
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/
int pasteup_pickProc(
		   ClientData clientData, /* not used                */
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    char *opt;

    /******************** Main ********************/

    void usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n pick option (suboption) varName...",
	     "  option is one of followings\n",
	     "      region  : set types, accuracy in the region or delete\n",
	     "      nearest : search nearest pick mark\n",
	     "      plot    : plot a pick mark\n",
	     "      plotall : plot all the pick marks\n",
	     "      sort    : sort pick marks by offset and types\n",
	     "      convert : convert pick type\n",
	     "      delete  : delete one of pick types\n",
	     "      tt2cy   : convert true-time into canvas-y (eval shot time correction)\n",
	     "      tt2gt   : cnvert true-time into graph-t (reduced, shot time corrected)\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc < 3){
	usage();
	return TCL_ERROR;
    }
    /***** Get Option *****/
    opt = Tcl_GetStringFromObj(objv[1], NULL);

    /*************** Operation ***************/
    if (strcmp("region", opt) == 0) {
	return(pasteup_pickProc_region(interp, objc, objv));
    } else if (strcmp("listup", opt) == 0) {
	return(pasteup_pickProc_listup(interp, objc, objv));
    } else if (strcmp("listtype", opt) == 0) {
	return(pasteup_pickProc_listtype(interp, objc, objv));
    } else if (strcmp("nearest", opt) == 0) {
	return(pasteup_pickProc_nearest(interp, objc, objv));
    } else if (strcmp("plot", opt) == 0) {
	return(pasteup_pickProc_plot1(interp, objc, objv));
    } else if (strcmp("plotall", opt) == 0) {
	return(pasteup_pickProc_plotall(interp, objc, objv));
    } else if (strcmp("sort", opt) == 0) {
	return(pasteup_pickProc_sort(interp, objc, objv));
    } else if (strcmp("convert", opt) == 0) {
	return(pasteup_pickProc_convert(interp, objc, objv));
    } else if (strcmp("delete", opt) == 0) {
	return(pasteup_pickProc_delete(interp, objc, objv));
    } else if (strcmp("tt2cy", opt) == 0) {
	return(pasteup_pickProc_tt2cy(interp, objc, objv));
    } else if (strcmp("tt2gt", opt) == 0) {
	return(pasteup_pickProc_tt2gt(interp, objc, objv));
    } else {
	usage();
	return TCL_ERROR;
    }
    return TCL_OK;
}
