#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "tcl.h"
#include "pasteup.h"


int pasteup_raysProc_plot(
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    char *fn;       /** filename for raypaths               **/
    char *g;        /** graph widget                        **/
    char *color;    /** ray color                           **/
    FILE *fp;
    char line[129];
    char com[129];
    Tcl_Obj* coordlist;
    double x, z;
    int i;

    /******************** Main ********************/
    void show_usage () {
	Tcl_AppendResult
	    (interp,
	     "wrong usage :\n",
	     "  rays plot fileName graphWidget\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 4) {
	show_usage();
	return TCL_ERROR;
    }

    /***** Get command line arguments and color *****/
    fn = Tcl_GetStringFromObj(objv[2], NULL);
    g = Tcl_GetStringFromObj(objv[3], NULL);
    color = Tcl_GetStringFromObj
	(Tcl_GetVar2Ex(interp, "gbl", "raytrace,raycolor",
		       TCL_GLOBAL_ONLY), NULL);
    /***** Read coordinates *****/
    fp = open_file(fn, "raypath");
    coordlist = Tcl_NewListObj(0, NULL);
    Tcl_SetVar2Ex(interp, "coordlist", NULL, coordlist, 0);
    i = 0;
    while(fgets(line, 128, fp) != NULL) {
	if(line[0] == '#') continue; /* comment */
	if(line[0] == '>') {
	    /* plot */
	    sprintf(com, "%s marker create line -coord $coordlist -name r%d -out %s ; update",
		    g, i, color);
 	    Tcl_Eval(interp, com);
	    /* for next ray */
	    i++;
	    Tcl_SetListObj(coordlist, 0, NULL);
	} else {
	    /* data */
	    sscanf(line, "%lf%lf", &x, &z);
	    TCL_APPEND(coordlist, Double, x);
	    TCL_APPEND(coordlist, Double, z);
	}
    }
    fclose(fp);
    return TCL_OK;
}

/*
  --------------------------------------------------------------------
*/

int pasteup_raysProc(
		   ClientData clientData, /* not used                */
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    char *opt;

    /******************** Main ********************/

    void usage () {
	Tcl_AppendResult
	    (interp,
	     "usage :\n",
	     "   rays plot fileName graphWidget\n",
	     "",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc < 4){
	usage();
	return TCL_ERROR;
    }
    /***** Get Option *****/
    opt = Tcl_GetStringFromObj(objv[1], NULL);

    /*************** Operation ***************/
    if (strcmp("plot", opt) == 0) {
	return(pasteup_raysProc_plot(interp, objc, objv));
    } else {
	usage();
	return TCL_ERROR;
    }
    return TCL_OK;
}
