#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "tcl.h"
#include "pasteup.h"



/*
  --------------------------------------------------------------------
*/

int pasteup_subplotProc(
		   ClientData clientData, /* not used                */
		   Tcl_Interp *interp,    /* current tcl interpreter */
		   int objc,              /* number of arguments     */
		   Tcl_Obj* CONST objv[]  /* arguments               */
		   )
{
    /***** Variables *****/
    int fmt, direction;
    double x0, y0, x1, y1, min, max, height;
    int total_shot;
    double tmin, tmax, left_offset, left, right, graph_width;
    char *color;
    double *x, *d;
    double tmpx, tmpd;
    int i, j, len;
    double c;
    Tcl_Obj* coordlist;
    char com[256];

    /******************** Main ********************/

    void usage () {
	Tcl_AppendResult
	    (interp,
	     "usage :\n",
	     "   subplot format x0 y0 x1 y1 min max height axis\n",
	     "       format: data format (0:sdepth, 1:tracl,...)\n",
	     (char *)NULL);
    };

    /***** Check arguments *****/
    if(objc != 10){
	usage();
	return TCL_ERROR;
    }
    /***** get arguments *****/
    if( TCL_ERROR == Tcl_GetIntFromObj(interp, objv[1], &fmt)
	|| TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[2], &x0)
	|| TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[3], &y0)
	|| TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[4], &x1)
	|| TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[5], &y1)
	|| TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[6], &min)
	|| TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[7], &max)
	|| TCL_ERROR == Tcl_GetDoubleFromObj(interp, objv[8], &height)
	|| TCL_ERROR == Tcl_GetIntFromObj(interp, objv[9], &direction)
	) {
	usage();
	return TCL_ERROR;
    }

    /*************** Read global variables ***************/
    TCL_GET(Double, paste, "graph_width", graph_width);
    TCL_GET(Double, paste, "left", left);
    TCL_GET(Double, paste, "right", right);
    TCL_GET(Double, paste, "top", tmin);
    TCL_GET(Double, paste, "bottom", tmax);
    TCL_GET(Double, paste, "left_margin", left_offset);
    color = Tcl_GetStringFromObj(Tcl_GetVar2Ex(interp, "paste", "sub,color",
					       TCL_GLOBAL_ONLY), NULL);

    /************************************************************/
    /************************ Make cache ************************/
    /************************************************************/
    
    total_shot = make_cache_data(interp, tmin, tmax);

    /************************************************************/
    /*********************** Preparations ***********************/
    /************************************************************/

    /***** memory allocation *****/
    if( NULL == (x = (double *)malloc(total_shot * sizeof(double))) ||
	NULL == (d = (double *)malloc(total_shot * sizeof(double))) ) {
	fprintf(stderr, "Error: Memory allocation failed (%s)\n", __FILE__);
	return TCL_ERROR;
    }

    /***** read data *****/
    for(i=0;i<total_shot;i++){
	x[i] = gbl_rh[i].x;
	switch(fmt){
	    case 0: /* swdep : shot depth */
		d[i] = gbl_rh[i].swdep;
		break;
	    case 1: /* tracl : shot number */
		d[i] = (double)gbl_rh[i].hw[RECORD_HEADER_TRACL];
		break;
	    case 2: /* tracr */
		d[i] = (double)gbl_rh[i].hw[RECORD_HEADER_TRACR];
		break;
	    case 3: /* tracf */
		d[i] = (double)gbl_rh[i].hw[RECORD_HEADER_TRACF];
		break;
	    case 4: /* fldr */
		d[i] = (double)gbl_rh[i].hw[RECORD_HEADER_FLDR];
		break;
	    case 5: /* ep */
		d[i] = (double)gbl_rh[i].hw[RECORD_HEADER_EP];
		break;
	    default:
		d[i] = min;
		break;
	}
    }
    /***** sort *****/
    for(i=total_shot-1;i>0;i--){
	for(j=0;j<i;j++) {
	    if(x[j] > x[j+1]) {
		tmpx = x[j];
		tmpd = d[j];
		x[j] = x[j+1];
		d[j] = d[j+1];
		x[j+1] = tmpx;
		d[j+1] = tmpd;
	    }
	}
    }
    /***** coordinate conversion and make coorlist *****/
    coordlist = Tcl_NewListObj(0, NULL);
    Tcl_SetVar2Ex(interp, "coordlist", NULL, coordlist, 0);
    c = height / (max - min);
    if(direction>0){ /* downward */
	for(i=0;i<total_shot;i++){
	    if( (x[i] - left)*(right - x[i]) < 0.0 ||
		(d[i] - min)*(max - d[i]) < 0.0) continue;
	    TCL_APPEND(coordlist, Double, 
		       (x[i] - left)/(right - left)*graph_width + left_offset);
	    TCL_APPEND(coordlist, Double, (d[i] - min) * c + y1);
	}
    } else {
	for(i=0;i<total_shot;i++){
	    if( (x[i] - left)*(right - x[i]) < 0.0 ||
		(d[i] - min)*(max - d[i]) < 0.0) continue;
	    TCL_APPEND(coordlist, Double, 
		       (x[i] - left)/(right - left)*graph_width + left_offset);
	    TCL_APPEND(coordlist, Double, y0 - (d[i] - min)*c);
	}
    }
    /***** plot *****/
    Tcl_ListObjLength(interp, coordlist, &len);
    if ( len >= 4 ) {
	sprintf(com, "%s create line $coordlist -width 1 -tags frame -fill %s",
		TCL_GETSTRING(wk, "w,canvas,c"), color);
	return Tcl_Eval(interp, com);
    }
    return TCL_OK;
}
