#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "pasteup.h"

#define MAX_REDBLUE 255
#define MAX_TABLE_SIZE 70

static int interpolate_code(int amp, int n, double *a, int *R, int *G, int *B)
{
    int i;
    int r, g, b;
    double d;
    if(amp < a[0]) return(0x010000*R[0] + 0x000100*G[0] + B[0]);
    for(i=0;i<n-1;i++){
	if( (amp - a[i])*(a[i+1] - amp) >= 0.0) {
	    if( (d = a[i+1] - a[i]) == 0.0 ) {
		return(0x010000*R[i] + 0x000100*G[i] + B[i]);
	    }
	    r = (R[i]*(a[i+1] - amp) + R[i+1]*(amp - a[i])) / d;
	    g = (G[i]*(a[i+1] - amp) + G[i+1]*(amp - a[i])) / d;
	    b = (B[i]*(a[i+1] - amp) + B[i+1]*(amp - a[i])) / d;
	    return(0x010000*r + 0x000100*g + b);
	}
    }
    return(0x010000*R[n-1] + 0x000100*G[n-1] + B[n-1]);
}

static int interpolate_color(gdImagePtr im, int amp,
				  int n, double *a, int *R, int *G, int *B)
{
    int i;
    int r, g, b;
    double d;
    if(amp < a[0]) return(gdImageColorResolve(im, R[0], G[0], B[0]));
    for(i=0;i<n-1;i++){
	if( (amp - a[i])*(a[i+1] - amp) >= 0.0) {
	    if( (d = a[i+1] - a[i]) == 0.0 ) {
		return(gdImageColorResolve(im, R[i], G[i], B[i]));
	    }
	    r = (R[i]*(a[i+1] - amp) + R[i+1]*(amp - a[i])) / d;
	    g = (G[i]*(a[i+1] - amp) + G[i+1]*(amp - a[i])) / d;
	    b = (B[i]*(a[i+1] - amp) + B[i+1]*(amp - a[i])) / d;
	    return(gdImageColorResolve(im, r, g, b));
	}
    }
    return(gdImageColorResolve(im, R[n-1], G[n-1], B[n-1]));
}

/********************************************************************
Make color tables
********************************************************************/
void declare_colors(Tcl_Interp *interp,
		    gdImagePtr im, int *color, int mode, int ncol)
{
    /********** Variables ***********/
    int i, n, onlyplus;
    char str[100];
    double a[MAX_TABLE_SIZE];
    int R[MAX_TABLE_SIZE], G[MAX_TABLE_SIZE], B[MAX_TABLE_SIZE];
    Tcl_Obj *cpt;
    Tcl_Obj *dummy[1];

    /********** Main ***********/
    /***** Black and White *****/
    color[RED]   = gdImageColorResolve(im, 255, 0, 0);
    color[BLUE]  = gdImageColorResolve(im, 0, 0, 255);
    color[WHITE] = gdImageColorResolve(im, 255, 255, 255);
    color[BLACK] = gdImageColorResolve(im, 0, 0, 0);

    if(mode<10)return; /* no VD colors */

    /***** read color table *****/
    sprintf(str, "vd,%d", mode);
    cpt = Tcl_GetVar2Ex(interp, "wk", str, TCL_GLOBAL_ONLY);
    Tcl_ListObjLength(interp, cpt, &n);
    n = n/4;
    for(i=0;i<n;i++){
	Tcl_ListObjIndex(interp, cpt, i*4, dummy);
	Tcl_GetDoubleFromObj(interp, dummy[0], &a[i]);
	Tcl_ListObjIndex(interp, cpt, i*4+1, dummy);
	Tcl_GetIntFromObj(interp, dummy[0], &R[i]);
	Tcl_ListObjIndex(interp, cpt, i*4+2, dummy);
	Tcl_GetIntFromObj(interp, dummy[0], &G[i]);
	Tcl_ListObjIndex(interp, cpt, i*4+3, dummy);
	Tcl_GetIntFromObj(interp, dummy[0], &B[i]);
    }
    /***** Is the color table use only plus side, or both side ? *****/
    TCL_GET(Int, gbl, "vdcolor,onlyplus", onlyplus);
    if(onlyplus == 1 && mode != 11)
	for(i=0;i<n;i++) a[i] = (a[i] + 100.0) / 2.0;

    /***** generate colors *****/
    for(i=0;i<=512;i++){
	color[i] = interpolate_color(im, (int)(i - 256)*100.0/256.0,
				     n, a, R, G, B);
    }
}

/********************************************************************
Make color code tables
********************************************************************/
void declare_color_codes(Tcl_Interp *interp, int *color, int mode, int ncol)
{
    /********** Variables ***********/
    int i, n, onlyplus;
    char str[100];
    double a[MAX_TABLE_SIZE];
    int R[MAX_TABLE_SIZE], G[MAX_TABLE_SIZE], B[MAX_TABLE_SIZE];
    Tcl_Obj *cpt;
    Tcl_Obj *dummy[1];

    /********** Main ***********/
    
    if(mode<10)return; /* unless VD */

    /***** read color table *****/
    sprintf(str, "vd,%d", mode);
    cpt = Tcl_GetVar2Ex(interp, "wk", str, TCL_GLOBAL_ONLY);
    Tcl_ListObjLength(interp, cpt, &n);
    n = n/4;
    for(i=0;i<n;i++){
	Tcl_ListObjIndex(interp, cpt, i*4, dummy);
	Tcl_GetDoubleFromObj(interp, dummy[0], &a[i]);
	Tcl_ListObjIndex(interp, cpt, i*4+1, dummy);
	Tcl_GetIntFromObj(interp, dummy[0], &R[i]);
	Tcl_ListObjIndex(interp, cpt, i*4+2, dummy);
	Tcl_GetIntFromObj(interp, dummy[0], &G[i]);
	Tcl_ListObjIndex(interp, cpt, i*4+3, dummy);
	Tcl_GetIntFromObj(interp, dummy[0], &B[i]);
    }

    /***** Is the color table use only plus side, or both side ? *****/
    TCL_GET(Int, gbl, "vdcolor,onlyplus", onlyplus);
    if(onlyplus == 1 && mode != 11)
	for(i=0;i<n;i++) a[i] = (a[i] + 100.0) / 2.0;

    /***** generate color codes *****/
    for(i=0;i<=512;i++){
	color[i] = interpolate_code((int)(i - 256)*100.0/256.0, n, a, R, G, B);
    }
}

/********************************************************************
return index of color table for VD style
********************************************************************/
int amp2index(float amp, int step)
{
    /********** Variables ***********/
    int index;
     
    /********** Main ***********/
    index = (int)amp/step;
    if(index<=0){
	index = (index < -256) ? 0 : 256 + index;
    } else {
	index = (index > 256) ? 512 : 256 + index;
    }
    return(index);
}
