#include <stdio.h>
#include <string.h>
#include <math.h>
#include "pasteup.h"

/********************************************************************
Make one trace array
--------------------------------------------------------------------
Input:
 i           : trace sequetial number
 fp          : file pointer for the waveform
 data_format : KASA_FORMAT or SU_FORMAT and so on.
 bfmt        : data binary format 
 ch          : channel number
 dat         : Record header 
 start_time  : start time
 end_time    : end time
 rd_vel      : Reduction Velocity in Km/sec.
 dt_clock    : clock correction
 filter_type : filter type (no(0), lowpass(1), highpass(2), bandpass(3)
               bandstop(4))
 causal_type : filter causality. causal(1), uncausal(2)
 f1,f2,f3,f4 : filter coef. 
 stack       : 0: no stack,  1:hyperbola,  2:slant
 stack_range : Stacking range in km.
 stack_vel   : Stacking velocity in km/sec.
 stack_time  : Stacking time, in sec.

Output:
 a           : an array for amplitude
 t           : time of the first sample
********************************************************************/
int make_one_trace(int i, FILE *fp, int data_format, int bfmt, int ch,
		   float start_time, float end_time, double rd_vel,
		   int filter_type, int causal_type, 
		   double f1, double f2, double f3, double f4,
		   int stack, double stack_range,
		   double stack_vel, double stack_time,
		   float *a, float *t)
{
    /********** Local Variables *********/
    int n;

    /******************** Main *******************/
    if (stack == 0) {
	/***** No Stacking *****/
	n = make_data(a, t, fp, data_format, bfmt, ch,
		      gbl_rh[i].start_point, gbl_rh[i].end_point,
		      start_time, end_time, gbl_rh[i].sampling, gbl_rh[i].dsec,
		      filter_type, causal_type, f1, f2, f3, f4);
    } else {
	/** Do Stacking **/
	n = make_stacked(i, data_format, bfmt, ch,
			 a, t, start_time, end_time, rd_vel,
			 stack, stack_range, stack_vel, stack_time,
			 filter_type, causal_type, f1, f2, f3, f4, fp);
    }
    return(n);
}

int form_trace(float *a_org, float t_org, int n_org,
	       float st, float et, float sampling,
	       double amp, int amp_correct,  double agc, double kika_coef,
	       float *a, float *t)
{
    /***** Local Variables *****/
    int i, n;
    float tmp;

    /******************** Main *******************/
    for(i=0;i<n_org;i++) a[i] = a_org[i];

    /***** amplitude correction *****/
    amplitude_correct(a, n_org, sampling, amp_correct, agc, amp, kika_coef);

    /***** Cut off ouside of the range *****/
    n = 0;
    for(i=0;i<n_org;i++){
	tmp = t_org + i/sampling;
	if( (tmp - st)*(et - tmp) >= 0.0 ) {
            t[n] = tmp;
            a[n++] = a[i];
        }
    }
    return(n);
}
