# --------------------------------------------------------------------
# List up all the pick types
# --------------------------------------------------------------------
proc mapping_listup_picktypes {} {
   # global --------------------------------------------------
   global gbl pm

   # main --------------------------------------------------
   set gbl(mapping,types) [join [lsort -integer [pick listtype pm] ] ","]
}

# --------------------------------------------------------------------
# Set current type
# --------------------------------------------------------------------
proc mapping_set_current_type {} {
   # global --------------------------------------------------
   global gbl pick

   # main --------------------------------------------------
   set gbl(mapping,types) $pick(current_type)
}

# --------------------------------------------------------------------
# Make config file for tmm
# --------------------------------------------------------------------
proc mapping_make_tmmcfg {fn pickfn} {
   # global --------------------------------------------------
   global gbl

   # main --------------------------------------------------
   set fp [open $fn "w"]
   # model ----------
   puts $fp "\# Model configuration."
   # netCDF
   if { [file readable $gbl(vmodel,netcdf)] == 0 } {
      tk_messageBox -type ok -icon error\
	  -message "netCDF file is not readable."
      return
   }
   puts $fp [format "model=%s" $gbl(vmodel,netcdf)]

   # pick ----------
   puts $fp "\# Pick data configuration."
   puts $fp [format "pick=%s" $pickfn]
   puts $fp [format "every=%s" 1]
   puts $fp [format "thinout=%f" 0.0]
   puts $fp [format "minoffset=%s" 0.0]
   puts $fp [format "maxoffset=%s" 1000.0]
   if {$gbl(mapping,types) != ""} {
      puts $fp [format "type=%s" $gbl(mapping,types)]
   }

   # traveltime fields ----------
   puts $fp "\# Traveltime fields configuration."
   puts $fp [format "tfdir=%s" $gbl(mapping,tfdir)]
   puts $fp [format "dx=%s" $gbl(mapping,dx)]
   puts $fp [format "byte=%s" $gbl(mapping,byte)]
   puts $fp [format "tbase=%s" $gbl(mapping,tbase)]
   puts $fp [format "zmax=%s" -1]

   # mapping condition ----------
   puts $fp "\# Mapping condition."
   if { $gbl(mapping,wsign) == "deeper" } {
      puts $fp [format "mode=-%s" $gbl(mapping,wmode)]
   } else {
      puts $fp [format "mode=%s" $gbl(mapping,wmode)]
   }
   puts $fp [format "alpha=%s" $gbl(mapping,alpha)]
   puts $fp [format "wpick=%s" $gbl(mapping,wpick)]
   puts $fp [format "normalize=%s" $gbl(mapping,normalize)]
   puts $fp [format "aperture=%s" $gbl(mapping,angle)]
   puts $fp [format "smute=%s" $gbl(mapping,smute)]
  
   # verboseness
   puts $fp [format "verbose=%s" $gbl(option,verbose)]
   close $fp
}

# --------------------------------------------------------------------
# Make traveltime field
# --------------------------------------------------------------------
proc mapping_make_tfield {} {
   # global --------------------------------------------------
   global gbl TMPDIR PREF pm

   # main --------------------------------------------------
   if {$gbl(vmodel,format) == 0 } {
      tk_messageBox -type ok -icon error\
	  -message "Velocity structure must be in the netCDF format."
      return
   }

   # make filenames
   set pickfn [format "%s/pasteup.pick.%s" $TMPDIR [pid]]
   set cfgfn [format "%s/pasteup.tmm.cfg.%s" $TMPDIR [pid]]

   # check ------------------------------
   if { [file exist $gbl(mapping,tfdir)] == 0 && $gbl(mapping,tfdir) != "" } {
      file mkdir $gbl(mapping,tfdir)
   }
   if { [file isdirectory $gbl(mapping,tfdir)] == 0} {
      tk_messageBox -type ok -icon error\
	  -message "Traveltime field dir name is invalid."
      return
   }

   # make config file ------------------------------
   mapping_make_tmmcfg $cfgfn $pickfn

   # make integrated-pick file ------------------------------
   mapping_make_pickfile $pickfn

   # exec ------------------------------
   set com [format "%s/grd2tfield %s" $PREF(com,tmmpath) $cfgfn]
   set ww [wait_window "Making t-fields"]
   exec_external_command_fifo $com "Making T-fields"
   destroy $ww


   # clean up ------------------------------
   file delete $pickfn
   file delete $cfgfn
}

# --------------------------------------------------------------------
# Merge grd files
# --------------------------------------------------------------------
proc mapping_merge_grd {fn} {
   # global --------------------------------------------------
   global gbl

   # main --------------------------------------------------
   set flist [glob -nocomplain image_???.grd]
   
   # delete output file
   file delete -force $fn

   # check number of files
   set n [llength $flist]
   if { $n < 1 } { return }
   if { $n == 1 } { 
      file copy $flist $fn
      return
   }
   
   # make command
   set com [format "grdmath %s %s ADD " [lindex $flist 0] [lindex $flist 1]]
   if { $n > 2 } {
      set flist [lrange $flist 2 end]
      foreach f $flist {
	 set com [format "%s %s ADD " $com $f]
      }
   }
   set com [format "%s = %s" $com $fn]
   
   # exec
   exec_external_command $com "Merging"
}

# --------------------------------------------------------------------
# Make traveltime field and view result
# --------------------------------------------------------------------
proc mapping_pick_image {} {
   # global --------------------------------------------------
   global gbl TMPDIR PREF wk

   # main --------------------------------------------------
   if {$gbl(vmodel,format) == 0 } {
      tk_messageBox -type ok -icon error\
	  -message "Velocity structure must be in the netCDF format."
      return
   }

   # make filenames
   set pickfn [format "%s/pasteup.%s.pick" $TMPDIR [pid]]
   set cfgfn [format "%s/pasteup.tmm.%s.cfg" $TMPDIR [pid]]
   set mrgfn [format "%s/pasteup.merge.%s.grd" $TMPDIR [pid]]

   # check ------------------------------
   if { [file isdirectory $gbl(mapping,tfdir)] == 0 } {
      tk_messageBox -type ok -icon error\
	  -message "Traveltime field dir name is invalid."
      return
   }

   # make config file ------------------------------
   mapping_make_tmmcfg $cfgfn $pickfn

   # make integrated-pick file ------------------------------
   mapping_make_pickfile $pickfn

   # exec pick image ------------------------------
   set ww [wait_window "Mapping..."]
   set com [format "%s/pick_image %s" $PREF(com,tmmpath) $cfgfn]
   exec_external_command_fifo $com "Mapping"

   # merge grd
   mapping_merge_grd $mrgfn

   # view results ------------------------------
   if { [file exists $mrgfn] } {
      mapping_viewimage $mrgfn
   } else {
      tk_messageBox -type ok -icon error -message "No mapping result."
   }
   destroy $ww
   
   # fin ------------------------------
   set flist [glob -nocomplain image_???.grd]
   if { $flist != "" } {
      eval {file delete -force} $flist
   }
   file delete -force $mrgfn $cfgfn $pickfn
   focus -force $wk(w,canvas)
}

# --------------------------------------------------------------------
# setup mapping window (subcontractor of mapping_viewimage)
# --------------------------------------------------------------------
proc setup_mapping_window {w} {
   # global --------------------------------------------------
   global gbl TMPDIR wk
   
   # main --------------------------------------------------
   set g $w.graph

   # top level widget ------------------------------
   if [raise_window $w] { return }
   toplevel $w
   wm title $w "PasteUp: Mapping window"
   wm resizable $w 0 0

   # graph widget ------------------------------
   graph $g
   $g yaxis conf -descending 1
   $g y2axis conf -descending 1
   $g crosshairs conf -color green
   $g conf -plotpadx 0 -plotpady 0 -plotborderwidth 0 -borderwidth 0\
       -topmargin 50 -bottommargin 25 -leftmargin 50 -rightmargin 50
   Blt_Crosshairs $g
   Blt_ZoomStack $g
   
   # buttons ------------------------------
   set bf [button_frame $w $w.button_frame\
	       property "mapping_property_window $g"\
	       "save PS" "VModel::save_postscript $g mapping"]

   # packing ------------------------------
   pack $g -side top -fill both -expand yes
   pack $bf -side bottom -fill x
   VModel::show_position $g mapping
}

# --------------------------------------------------------------------
# make_view_image 
# --------------------------------------------------------------------
# fn   : netCDF filename (result of mapping)
# --------------------------------------------------------------------
proc mapping_viewimage {fn} {
   # global ------------------------------
   global gbl TMPDIR wk file_name
   
   # main ------------------------------
   # invoke window
   set w .mapping
   set g $w.graph
   if [raise_window $w] { 
      # nothing
   } else {
      # new window
      setup_mapping_window $w 
   }
   # delete previous ----------
   eval $g marker delete [$g marker name]

   # Read netCDF info and make mapping image ----------
   # read netCDF file ----------
   set f [open "| grdinfo -C $fn"]
   scan [gets $f] "%*s%f%f%f%f%f%f" xmin xmax zmin zmax amin amax
   close $f

   # aspect ratio
   set wk(mapping,height_coef) [expr $gbl(vmodel,aspect) * ($zmax - $zmin)\
				    / ($xmax - $xmin)]
   mapping_create_image $g $fn $xmin $xmax $zmin $zmax $amin $amax

   # set up graph area ----------
   set gz [expr $gbl(vmodel,width) * $wk(mapping,height_coef) + 75]
   set gx [expr $gbl(vmodel,width) + 100]
   $g conf -height $gz -width $gx
   
   # graph area
   set xstep [raytrace_comp_tick_spacing [expr $xmax - $xmin]]
   set zstep [raytrace_comp_tick_spacing [expr $zmax - $zmin]]
   
   $g axis conf x -min $xmin -max $xmax -step $xstep
   $g axis conf y -min $zmin -max $zmax -step $zstep
   $g axis conf x2 -min $xmin -max $xmax -step $xstep -hide false
   $g axis conf y2 -min $zmin -max $zmax -step $zstep -hide false

   # plot mapping image ----------
   $g marker create image -image mapping -name mapping\
       -coord "$xmin $zmin $xmax $zmax"

   # boundary ----------
   if { $gbl(mapping,view,plotxy) } { 
      mapping_plot_model_boundary $g
   }

   # change title ----------
   $g conf -title [format "%s : pick=%s"\
		       [file rootname [file tail $file_name(pick)]]\
		       $gbl(mapping,types)]

   # update window size&position ----------
   set pos [string trimleft [wm geometry $w] 1234567890x]
   wm geometry $w [format "%dx%d%s" $gx [expr int($gz + 40)] $pos]

   # binding
   basic_bindings $w
}

# --------------------------------------------------------------------
# mapping_create_image -----------------------------------------------
#           Create mapping image
# --------------------------------------------------------------------
proc mapping_create_image {g fn xmin xmax zmin zmax amin amax} {
   # global --------------------------------------------------
   global gbl TMPDIR wk PREF

   # main --------------------------------------------------

   # temporary files ----------
   set cptfn [format "%s/pasteup.mapping.%s.cpt" $TMPDIR [pid]]
   set imgfn  [format "%s/pasteup.mapping.%s.png" $TMPDIR [pid]]
   set psfn  [format "%s/pasteup.mapping.%s.ps" $TMPDIR [pid]]
   set velfn [format "%s/pasteup.vmodel.%s.png" $TMPDIR [pid]]
   set overlayfn [format "%s/pasteup.overlay.%s.png" $TMPDIR [pid]]

   set com {}
   lappend com "gmtset PAPER_MEDIA a4+"

   # color palette ----------
   set threshold [expr ($amax - $amin) * $gbl(mapping,threshold) / 100.0]
   set i [expr {$gbl(mapping,view,inv)} ? {"-I"} : {""} ]
   if { [expr $amax - $amin] <= 0.0 } {
      set amax [expr $amin + 1.0]
   }
   lappend com [format "grd2cpt %s -C%s -L%f/%f %s -Z > %s"\
		    $fn $gbl(mapping,view,palette) $threshold $amax $i $cptfn]

   # ------------------------------------------------------------
   # make image -------------------------------------------------

   # aspect
   set w 20
   set h [expr - $w * $wk(mapping,height_coef)]
   # make mapping image
   lappend com [format "grdimage %s -C%s -JX%g/%g -R%f/%f/%f/%f -P > %s"\
		    $fn $cptfn $w $h $xmin $xmax $zmin $zmax $psfn]

   if { $gbl(mapping,view,overlay) } {
      # overlay mode ----------
      set col [mapping_transparent_color]
      lappend com [format "%s -density %g -transparent %s %s %s"\
		       $PREF(com,convert) $gbl(vmodel,density)\
		       $col $psfn $overlayfn]

      # velocity mode ----------
      if { [lsearch [image names] vmodel] < 0 ||
	   [expr abs($gbl(vmodel,aspect) * ($zmax - $zmin) * [image width vmodel]\
			 / ($xmax - $xmin) - [image height vmodel])] > 1 } {
	 VModel::create_image
      }

      # composite vel-model and mapping image
      vmodel write $velfn -format png
      lappend com [format "composite %s %s %s" $overlayfn $velfn $imgfn]
   } else {
      # only mapping image ----------
      lappend com [format "%s -density %g %s %s"\
		       $PREF(com,convert) $gbl(vmodel,density) $psfn $imgfn]
   }   

   # exec external command
   exec_shell_script $com "Make mapping image"

   image create photo mapping -file $imgfn
   file delete -force $cptfn $psfn $imgfn $velfn $overlayfn
}

# --------------------------------------------------------------------
# make_integrated_pickfile -------------------------------------------
#           Make integrated pick file with binning
# --------------------------------------------------------------------
proc mapping_make_pickfile {fn} {
   # Global --------------------------------------------------
   global gbl pick file_name pm
   
   # Main --------------------------------------------------
   set fp [open $fn w]

   # Header ----------
   # Comments
   puts $fp "# Integrated pick file."
   puts $fp "#\n# This file was generated by 'pasteup' automatically\n#"
   # Number of receivers
   puts $fp [format "# No of receivers\n1"]
   
   # Pick data ----------
   puts $fp "# pick data body"
   # Position and name (xz, name) -----
   if { $file_name(pick) != "" } {
      set obsname [file rootname [lindex [split $file_name(pick) "/"] end]]
   } else {
      set obsname [file rootname [lindex [split $file_name(wave) "/"] end]]
   }
   puts $fp "# location (x, z) in Km."
   puts $fp [format "%f %f %s" $gbl(obs,x) $gbl(obs,depth) $obsname]
   puts $fp [format "%5d" [llength $pm]]
   if { $gbl(mapping,renum) } {
      # renumbering by binning
      set xpos [expr $gbl(obs,x) - $gbl(mapping,xorg)]
      foreach pdata $pm {
	 scan $pdata "%d%s%s%s%d" sn px pt acc type
	 set sn [expr int( ($xpos + $px) / $gbl(mapping,bin) )]
	 puts $fp [format "%8d%10.3f%10.3f%10.3f%5d" $sn $px $pt $acc $type]
      }
   } else {
      # output as is.
      foreach pdata $pm {
	 scan $pdata "%d%s%s%s%d" sn px pt acc type
	 puts $fp [format "%8d%10.3f%10.3f%10.3f%5d" $sn $px $pt $acc $type]
      }
   }
   close $fp
}

# --------------------------------------------------------------------
# Invoke mapping property window
# --------------------------------------------------------------------
proc mapping_property_window {g} {
   # Global --------------------------------------------------
   global gbl wk

   # main --------------------------------------------------
   set w .mapping_property
   
   if [raise_window $w] { return }
   toplevel $w
   wm title $w "Pasteup : Properties for the mapping image"
   
   # title frame --------------------
   set tf [title_frame $w.title_frame\
	       "Pasteup: Properties for the mapping image"]

   # parameter frame ------------------------------
   # mapping parameters ----------
   set pf $w.parameters
   frame $pf 

   # color palette
   labelframe $pf.cpt -text "Color palette for mapping" -pady 3 -padx 5
   make_optionMenu_button $pf.cpt color "Palette" gbl(mapping,view,palette)\
       gray cool hot
   make_menu_com $pf.cpt thresh "Threshold(%)" gbl(mapping,threshold)\
       "" 0% 0 10% 10 20% 20 30% 30 40% 40 50% 50 60% 60 70% 70 80% 80 90% 90
   make_checkbutton $pf.cpt "Reverse color" rev gbl(mapping,view,inv)

   # image dimension
   labelframe $pf.isize -text "Image dimension" -pady 3 -padx 5
   make_entry_right $pf.isize asp "Aspect ratio (height/width)"\
       gbl(vmodel,aspect) ""
   make_entry_right $pf.isize w "Width of the plot area"\
       gbl(vmodel,width) "pixel"
   make_entry_right $pf.isize density "Image resolution"\
       gbl(vmodel,density) "dpi"
   grid $pf.cpt $pf.isize -sticky news -padx 5

   # plot or not
   labelframe $pf.plot -text "Plot" -pady 3 -padx  5
   make_checkbutton $pf.plot "Velocity model" over gbl(mapping,view,overlay)
   make_checkbutton $pf.plot "Layer boundary" plotxy gbl(mapping,view,plotxy)
   set c [make_checkbutton $pf.plot "Cursor position"\
	      cursr gbl(mapping,show_position)]
   $c conf -command "VModel::show_position $g mapping"

   # velocity model parameters
   set vm [labelframe $pf.vmodel -text "Options for velocity model" -pady 10]
   make_entry_right $vm cint "Contour interval" gbl(vmodel,contour) "km/sec."
   make_color_button $vm xycolor "Color of layer boundary" ""\
       $gbl(mapping,view,xycolor) gbl(mapping,view,xycolor)\
       "setColor $vm $vm.button(xycolor) gbl(mapping,view,xycolor)"

   grid $pf.plot $vm -sticky news -padx 5

   # filenames
   labelframe $pf.fn -text "Files" -pady 3 -padx 5
   filename_hscroll_frame $pf.fn cptfile "CPT" gbl(vmodel,cptfile) 45 load cpt
   filename_hscroll_frame $pf.fn xydata "Layer boundary"\
       gbl(mapping,view,xydata) 45 load "dat"
   grid $pf.fn - -sticky news
   

   # button frame ------------------------------
   set bf [button_frame $w $w.button_frame apply mapping_pick_image]

   # packing ------------------------------
   pack $tf $pf $bf -side top -fill x -padx 10 -pady 3
   grab $w
}

# --------------------------------------------------------------------
# Plot model boundary for mapping image
# --------------------------------------------------------------------
proc mapping_plot_model_boundary {g} {
   # Global --------------------------------------------------
   global gbl

   # main --------------------------------------------------
   # the file exists?
   if { $gbl(mapping,view,xydata) == "" || \
	    ![file exist $gbl(mapping,view,xydata)] } { return }
   
   # plot 
   set fp [open $gbl(mapping,view,xydata)]
   set c ""
   while { [gets $fp line] >= 0 } {
      set fc [string index $line 0]
      if { $fc == "\#" } { continue }
      if { $fc == ">" } {
	 # delimitter
	 $g marker create line -coord $c -out $gbl(mapping,view,xycolor)
	 set c ""
	 update
      } else {
	 set c "$c $line"
      }
   }
   close $fp
}

# --------------------------------------------------------------------
# Determine transparent color 
# --------------------------------------------------------------------
proc mapping_transparent_color {} {
   # Global --------------------------------------------------
   global gbl

   # main --------------------------------------------------
   if { $gbl(mapping,view,inv) == 1 } {
      # inverted color palette
      return white
   } else {
      # normal color palette
      return black
   }
}