# --------------------------------------------------------------------
# file selection dialog
# --------------------------------------------------------------------
# fn   : filename variable (not filename itself), 
#        must be array member of "file_name", "macray", "wk" or "gbl"
# op   : operation among "save", "load" and "dir"
# type : file type such as "pick", "wave" and so on (see the following source)
# --------------------------------------------------------------------
proc file_selection {fn op type} {
   # upvar
   upvar $fn tmp_file
   global file_name macray wk gbl PREF
   
   # main
   switch -- [string tolower $type] {
      wave {
	 switch -- $gbl(data,type) {
	    1 {
	       # KASA-lab
	       set types {
		  {"orig files"          *.orig}
		  {"ch1 files"          ch1.*}
		  {"ch2 files"          ch2.*}
		  {"ch3 files"          ch3.*}
		  {"ch4 files"          ch4.*}
		  {"All files"		*}
	       }
	    }
	    2 {
	       # SU
	       set types {
		  {"SU  files"          *.su}
		  {"All files"		*}
		  {"dat files"            *.dat}
		  {"SEGY files"         {*.sgy *.segy}}
		  {"ch1 files"          ch1.*}
		  {"ch2 files"          ch2.*}
		  {"ch3 files"          ch3.*}
		  {"ch4 files"          ch4.*}
	       }
	    }
	    3 {
	       # THK
	       set types {
		  {".pst files"         *.pst}
		  {".thk files"         *.thk}
		  {"dat files"          *.dat}
		  {"pasteup files"      *.pasteup}
		  {"All files"		*}
	       }
	    }
	    4 {
	       # CHIBA
	       set types {
		  {"Urabe-Hirata files" *.ube}
		  {"dat files"          *.dat}
		  {"All files"		*}
	       }
	    }
	    5 -
	    6 -
	    7 {
	       # SEG-Y
	       set types {
		  {"SEGY files"         {*.segy *.sgy}}
		  {"All files"		*}
		  {"dat files"          *.dat}
		  {"ch1 files"          ch1.*}
		  {"ch2 files"          ch2.*}
		  {"ch3 files"          ch3.*}
		  {"ch4 files"          ch4.*}
	       }
	    }
	 }
      }
      rh {
	 switch -- $gbl(data,type) {
	    1 {
	       # KASA-lab
	       set types {
		  {"rh files"           {*.rh rh.*}}
		  {"dat files"          *.dat}
		  {"All files"		*}
	       }
	    }
	    4 {
	       # CHIBA
	       set types {
		  {"rh files"           {*.rh rh.*}}
		  {"All files"		*}
	       }
	    }
	 }
      }
      su {
	 set types {
	    {"SU  files"          *.su}
	    {"All files"	  *}
	    {"ch1 files"          ch1.*}
	    {"ch2 files"          ch2.*}
	    {"ch3 files"          ch3.*}
	    {"ch4 files"          ch4.*}
	 }
      }
      postscript -
      ps {
	 set types {
	    {"PS files"           *.ps}
	    {"Image files"        {*.png *.jpg}}
	    {"All files"          *}
	 }
      }
      png -
      jpeg -
      jpg {
	 set types {
	    {"Image files"        {*.png *.jpg}}
	    {"All files"          *}
	 }
      }
      pick {
	 set types {
	    {"Pick files"	   {*.pick *.macray}}
	    {"tt files"            *.tt}
	    {"dat files"           *.dat}
	    {"All files"           *}
	 }
      }
      ray86 {
	 set types {
	    {"Ray86 files"	   *.ray86}
	    {"All files"           *}
	 }
      }
      gm {
	 set types {
	    {"gm files"	           *.gm}
	    {"All files"           *}
	 }
      }
      netcdf {
	 set types {
	    {"netCDF files"        *.grd}
	    {"All files"           *}
	 }
      }
      cpt {
	 set types {
	    {"CPT files"           *.cpt}
	    {"All files"           *}
	 }
      }
      txt {
	 set types {
	    {"txt files"           *.txt}
	    {"All files"           *}
	 }
      }
      default {
	 set types {
	    {"All files"          *}
	    {"dat files"          *.dat}
	    {"SU  files"          *.su}
	    {"SEGY files"         {*.segy *.sgy}}
	    {"ch1 files"          ch1.*}
	    {"ch2 files"          ch2.*}
	    {"ch3 files"          ch3.*}
	    {"ch4 files"          ch4.*}
	    {"Pick files"	  *.pick}
	    {"MacRay files"	  *.macray}
	 }
      }
   }

   # determine default directory
   if { $tmp_file != "" } {
      set dir [file dirname $tmp_file]
   } else {
      set dir ./
   }

   set newfn ""
   switch $op {
      "load" {
	 set newfn [tk_getOpenFile -filetypes $types -initialdir $dir\
			-initialfile [file tail $tmp_file]]
      }
      "save" {
	 set newfn [tk_getSaveFile -filetypes $types -initialdir $dir\
			-initialfile [file tail $tmp_file]]
      }
      "dir" {
	 set newfn [tk_chooseDirectory -initialdir $dir]
      }
   }
   if { $newfn == "" } { return "cancel" }
   set $fn $newfn
   update_wm_title
}
# --------------------------------------------------------------------
# Make canvas widget frame
# --------------------------------------------------------------------
proc make_canvas_frame {} {
   # Global variables --------------------
   global paste gbl pick mf p wk custom

   # ----------------------------------------------------------------
   # main -----------------------------------------------------------

   set w .cw
   toplevel $w

   # [DESIGN]
   #   ---------------------------------------------------------
   #   |      	       	       	       	       	       	       |
   #   |						       |
   #   |	       					       |
   #   |	     Canvas frame : $w.canvas_frame    	       |
   #   |	     		       	  	   	       |
   #   |	     		       	  	   	       |
   #   |--------------------+----------------+-----------------|
   #   |  Binding Menu 	    | Check buttons  | 	       	       |
   #   |        $bind       |     $check     | 	 Help menu     |
   #   |-------------------------------------+ 	     $h        |
   #   |      Menus dep. on bind  $bmenu     |		       |
   #   ---------------------------------------------------------
   #           $bind  = $w.left.up.bind
   #           $check = $w.left.up.check
   #           $bmenu = $w.left.bmenu     = $wk(widget,binding)
   #           $h     = $w.help

   # wm settings
   wm title $w "PasteUp"
   wm minsize $w 700 500
   wm withdraw $w
   wm protocol $w WM_DELETE_WINDOW "quit_application"
   if { $wk(invoke,autoPS) } {
      wm geometry $w +0+0
   }

   # Binding frame ------------------------------
   set bf    [frame $w.bind]
   set left  [frame $bf.left]
   frame $left.up
   set bind  [frame $left.up.bind]
   set check [frame $left.up.check]
   set bmenu [frame $left.bmenu]
   # mouse
   set mw [make_optionMenu_value $bind menu "Mouse:" gbl(bind) binding\
	       "Pick Mode(Plot and delete traveltime picks.)" "Pick Mode"\
	       "Line Pick Mode(Pick traveltimes along a line)" "Line Pick Mode"\
	       "Analysis Mode(velocity, multiples)" "Analysis Mode"\
	       "View Mode(View record section)" "View Mode"\
	       "Amplitude Mode(Report Maximum amplitude around the click)" "Amplitude Mode"\
	       "Tau-P/Tau-Sum inversion Mode" "Tau-P Mode"]
   set b $bind.button(menu) 
   $b conf -width 20

   # wheel mouse
   bind $b <Button-4> "wheel_binding_menu $mw prev"
   bind $b <Button-5> "wheel_binding_menu $mw next"

   # check button(s)
   checkbutton $check.showpick -text "Show pick" -var gbl(check,showpick) \
       -command "plot_pick_marks 0"
   pack $check.showpick

   # pack in $left.up
   pack $bind -side left
   pack $check -side right

   # menus depending on the mouse bindings 
   set wk(widget,binding) $bmenu
   menus_depend_binding
   pack $left.up $bmenu -side top -anchor w -fill x
   pack $left -side left

   # Binding help region
   set h [frame $bf.help -relief ridge -bd 1]
   set up {}
   set mid {}
   set down {}
   for {set i 1} {$i <= 3} {incr i} {
      label $h.up_blabel$i -text   [format "   B%d:" $i]
      label $h.up_b$i -text "None" -relief groove -bd 2 -width 20\
	  -font small_fixed

      label $h.mid_blabel$i -text   [format " S-B%d:" $i]
      label $h.mid_b$i -text "None" -relief groove -bd 2 -width 20\
	  -font small_fixed

      label $h.down_blabel$i -text [format " C-B%d:" $i]
      label $h.down_b$i -text "None" -relief groove -bd 2 -width 20\
	  -font small_fixed

      lappend up $h.up_blabel$i $h.up_b$i
      lappend mid $h.mid_blabel$i $h.mid_b$i
      lappend down $h.down_blabel$i $h.down_b$i
   }
   eval grid $up -sticky news
   eval grid $mid -sticky news
   eval grid $down -sticky news

   pack $h -side right
   # set to global variable
   set gbl(widget,bind_info) $bf

   # Message frame ------------------------------
   set mf .cw.message_frame
   frame $mf
   scrollbar $mf.scrollbar -bd 2 -relief ridge -width 10\
       -command "$mf.message yview"
   text $mf.message -height $paste(message_height) -relief sunken -wrap word\
       -font 7x14 -width 80 -yscrollcommand "$mf.scrollbar set"
   pack $mf.message -side left -in $mf -fill x -expand 1
   pack $mf.scrollbar -side left -fill y
   
   # Pack --------------------
   pack $bf $mf -side bottom -fill x

   # Pasteup Canvas Frame ------------------------------
   frame $wk(w,canvas)
   pack $wk(w,canvas) -side top -fill both -expand yes
   
   # NormalCanvas
   NormalCanvas::make_canvas_widget
   # GraphCanvas
   GraphCanvas::make_canvas_widget

   # placement
   update_mode
   update_canvas_mode

   # geometry setting
   update
   wm geometry .cw $paste(geometry)
}

# --------------------------------------------------------------------
# Quit Pasteup
# --------------------------------------------------------------------
proc quit_application {} {
   # gloabl --------------------
   global file_name mark_modification pm wk
   
   # Main --------------------
   if {$mark_modification != 0 && [llength $pm] != 0} {
      set ans [tk_messageBox -message "Do you want to save Pick data?"\
		   -type yesno -default yes -icon question -parent .]
      if {$ans == yes} {
	 if { [save_pick] == "cancel" } {
	    return
	 }
      }
   }
   # Save parameters
   save_resource
   customize_save_theme $wk(theme_file)
   customize_save_bind $wk(binding_file)
   customize_save_preference $wk(pref_file)

   # Clear temporary file if exist
   file delete -force $wk(fn,pick)

   # Confirmation
   set ans [tk_messageBox -type yesno -default no -parent . -icon question\
		-message "Quit this program?"]
   if { $ans == yes } {
      destroy .
   }
}
# --------------------------------------------------------------------
# Show messages in the message box
# --------------------------------------------------------------------
proc print_message {message} {
    # Global --------------------
    global mf

    # Main --------------------
    $mf.message insert end $message
    $mf.message see end
}
# --------------------------------------------------------------------
# Read a text file and send it to STDOUT
# --------------------------------------------------------------------
# [argument]
# f   :  text file name
# --------------------------------------------------------------------
proc cat_file {f} {
    # check file existence
    if { ![file exists $f] || ![file readable $f] } {
	return
    }
    set fp [open $f "r"]
    while {1} {
	if { [gets $fp line] == -1 } {
	    break
	}
	puts stdout $line
    }
    close $fp
}

# --------------------------------------------------------------------
# wait window: 
#    return value is the window ID.
# --------------------------------------------------------------------
# [argument]
# m  :  message to show
# --------------------------------------------------------------------
proc wait_window {m} {
   # global variable ------------------------------
   global gbl wk custom
   
   # main ------------------------------
   # configuration for window
   set width 240
   set height 120
   
   # invoke window
   set w .wait_message_window
   catch {destroy $w}
   toplevel $w
   wm title $w "Wait for a while..."
   
   # position window
   wm geometry $w [relative_window_geometry .cw $width $height]
   
   # make message
   label $w.message -justify left -text $m -wraplength 12c\
       -font $custom(WindowTitleFont)
   pack $w.message -expand 1 -fill y
   update
   grab $w
   return $w
}

# --------------------------------------------------------------------
# Compute coordinates for a new window to be places at the center of canvas
# --------------------------------------------------------------------
# [argument]
# w               : reference window name
# width, height   : width and height of the new window
# --------------------------------------------------------------------
proc relative_window_geometry {w width height} {
    # global variable ------------------------------
    global gbl
    
    # main ------------------------------
    set ref_geometry [split [wm geometry $w] "x+"]
    scan $ref_geometry "%s%s%s%s" cw ch cx cy
    set x [expr $cx + ($cw - $width)/ 2]
    set y [expr $cy + ($ch - $height)/2]
    return [format "%dx%d+%d+%d" $width $height $x $y]
}

# --------------------------------------------------------------------
# Raise window if it already exist
# --------------------------------------------------------------------
# [argument]
#   w : window (toplevel widget)
# --------------------------------------------------------------------
proc raise_window {w} {
   if { [winfo exist $w] } {
#      winop warpto $w
      # metacity doesn't work well.
#      wm deiconify $w
#      winop raise $w
#      return 1
      # metacity, compiz work well.
      set geom [wm geometry $w]
      wm withdraw $w
      wm deiconify $w
      wm geometry $w $geom
      return 1
   }
   return 0
}

# --------------------------------------------------------------------
# messageBox : large messageBox substitution for tk_messageBox
# --------------------------------------------------------------------
# [arguments]
# m     : messages
# title : title for the window manager
# width : width in pixels.
# tfont : title font
# mfont : message font 
# icon  : icon bitmap name, "question" 
# type  : button type, "yesno"
# --------------------------------------------------------------------
proc messageBox {m title width tfont mfont icon type} {
   global wk

   # main ------------------------------

   # Invoke window
   set w .temporary_messageBox
   if [winfo exist $w] {
      destroy $w
   }
   toplevel $w
   wm title $w $title

   # Title frame
   set tf $w.title_frame
   label $tf -text $title -relief groove -bd 2 -font "$tfont 18 bold"

   # message frame
   set mf $w.message
   message $mf -width $width -text $m -font $mfont

   # button frame
   set bf $w.button
   frame $bf -relief groove -bd 1
   set wk(tmp) "yet"
   switch -- $type {
      "yesno" {
	 button $bf.yes -text "yes" -width 10\
	     -command "set wk(tmp) yes"
	 button $bf.no -text "no" -width 10\
	     -command "set wk(tmp) no"
	 pack $bf.yes $bf.no -side left -expand 1
      }
      "okcancel" {
	 button $bf.yes -text "OK" -width 10\
	     -command "set wk(tmp) ok"
	 button $bf.no -text "cancel" -width 10\
	     -command "set wk(tmp) cancel"
	 pack $bf.yes $bf.no -side left -expand 1
      }
   }

   # pack
   pack $tf $mf $bf -side top -fill x -expand 1

   # positioning and grab
   update
   grab $w
   tkwait variable wk(tmp)
   destroy $w
   return $wk(tmp)
}
# --------------------------------------------------------------------
# get_line -----------------------------------------------------------
#            Read one line from a line.  Neglect comments.
# --------------------------------------------------------------------
# fp : File pointer
# --------------------------------------------------------------------
proc get_line {fp} {
    # Global variables --------------------------------------------------
    global gbl

   # Main --------------------------------------------------
   set line "#"
   while {[string match {\#*} $line]} {
      if { [eof $fp] } {
         return ""
      }
      gets $fp line
   }
   return $line
}

# --------------------------------------------------------------------
# Search external command
# --------------------------------------------------------------------
proc search_external_command {com} {
   global wk env
   # main ------------------------------
   set com [format "%s %s 2> /dev/null" $wk(path,which) $com]
   set status [catch {eval exec $com}]
   if { $status == 0 } {
      # command found
      return 1
   }
   # NOT found
   return 0
}

# --------------------------------------------------------------------
# Strip Leading Zeros
# --------------------------------------------------------------------
proc stripzeros {value} {
   regsub ^0+(.+) $value \\1 retval
   return $retval
}

# --------------------------------------------------------------------
# Exec external command ; return the exec status (if invalid return 1)
# --------------------------------------------------------------------
# com   : external commands,  e.g. "segywrite tape=hoge.sgy < hoge.su"
# name  : explanation of the command, e.g. "SEGYWRITE"
# --------------------------------------------------------------------
proc exec_external_command {com name} {
   # global ------------------------------
   global wk TMPDIR

   # main ------------------------------
   set stderr_file [format "%s/pasteup.stderr.%s" $TMPDIR [pid]]
   if $wk(debug) {
      # debug mode
      set com [format "%s 2> %s" $com $stderr_file]
      puts stderr "\# $name command is "
      puts stderr $com
   } else {
      set com [format "%s 2> /dev/null" $com ]
   }

   # exec
   set status [catch {eval exec $com}]

   # check results
   if $wk(debug) {
      puts stderr "\# $name exec log is "
      cat_file $stderr_file
      file delete -force $stderr_file
   }
   if $status {
      # some error occurred
      tk_messageBox -icon error -type ok\
	  -message "Error occurred during $name."
      return 1
   }
   return 0
}

# --------------------------------------------------------------------
# Exec shell script
# --------------------------------------------------------------------
# s     : shell script
# exp   : explanation of the shell script, e.g. "make velocity model"
# --------------------------------------------------------------------
proc exec_shell_script {s exp} {
   # global ------------------------------
   global wk TMPDIR

   # main ------------------------------
   foreach line $s {
      exec_external_command $line $exp
   }
}

# --------------------------------------------------------------------
# Exec external command opening with FIFO
# --------------------------------------------------------------------
# com   : external commands,  e.g. "segywrite tape=hoge.sgy < hoge.su"
# name  : explanation of the command, e.g. "SEGYWRITE"
# --------------------------------------------------------------------
proc exec_external_command_fifo {com name} {
   global wk

   puts "\#\#\#\#\# $name : BEGIN"
   if { $wk(debug) } {
      puts "\# command is \"$com\""
   }
   set com [open "| $com" r]
   while { [gets $com line] >= 0 } {
      puts $line
   }
   catch {close $com}
   puts "\#\#\#\#\# $name : END"
}

# --------------------------------------------------------------------
# Make filelist : 
#     file list in the same directory and with the same extension
# --------------------------------------------------------------------
proc make_filelist {fn} {
   # Global variables --------------------
   global gbl file_name

   # main -----------------------------------------------------------
   set dir [file dirname $fn]
   set ext [file extension $fn]

   set ch ""
   set root [file rootname [file tail $fn]]
   if { $gbl(option,dataset_channel) == 1 &&
	( [string match \*-\[1-4\] $root] || 
	  [string match \*_\[1-4\] $root] ) } {
      set ch [string replace $root 0 end-2]
   }
   return [lsort -dictionary [glob $dir/*$ch$ext]]
}

# --------------------------------------------------------------------
# Determine New index
# --------------------------------------------------------------------
proc newindex {filelist cur direction} {
   set ind [lsearch $filelist $cur]
   incr ind $direction
   if { $ind >= [llength $filelist] } { incr ind -1 }
   if { $ind < 0 } { set ind 0 }
   return $ind
}

# --------------------------------------------------------------------
# Change a data file (with same extension)
# --------------------------------------------------------------------
proc change_data_file {fn direction} {
   # Global variables --------------------
   global gbl

   # main -----------------------------------------------------------
   if { $fn == "" || ![file exists $fn] } {  return "" }
   set fn [file normal $fn]
   set flist [make_filelist $fn]
   return [lindex $flist [newindex $flist $fn $direction]]
}

# --------------------------------------------------------------------
# Change pick file
# --------------------------------------------------------------------
# direction : -1/+1 ("prev" or "next")
# flag      : 1:draw pick marks after change, otherwise no-drawing
# --------------------------------------------------------------------
proc change_pick_file {direction flag} {
   # Global variables --------------------
   global gbl file_name wk mark_modification undopm pm macray

   # ----------------------------------------------------------------
   # main -----------------------------------------------------------

   # check pick mark modification
   if {$mark_modification == 1} {
      set ans [tk_messageBox -type yesnocancel -default cancel\
		   -icon question -message\
		   "Pick data was modified.  Save picks and change data set? (if you select \"no\", discard your pick modification and change data set)"]
      if {$ans == "cancel"} {
	 return ""
      }
      if {$ans == "yes"} {
	 save_pick
      }
   }

   # preparation
   set pm {}
   set mark_modification 0

   # synthetic file also has the same name?
   if { [file tail $file_name(pick)] == [file tail $macray(filename)] } {
      set synthetic 1
   } else {
      set synthetic 0
   }

   # change pick file itselft
   set file_name(pick) [change_data_file $file_name(pick) $direction]

   # change synthetic file if required
   if { $synthetic } {
      set newname [format "%s/%s" [file dirname $macray(filename)]\
		       [file tail $file_name(pick)]]
      if { [file readable $newname] } {
	 set macray(filename) $newname
      }
   }

   # read pick data file
   if { $flag == 1 && [file exist $file_name(pick)] } {
      read_pick $file_name(pick)
   }
   return $file_name(pick)
}

# --------------------------------------------------------------------
# execute external program
# --------------------------------------------------------------------
# pname : an external program name
# --------------------------------------------------------------------
proc exec_external_program {pname} {

   catch { exec $pname } result
   if {$result!=""} {
      puts $result
   }
}

# --------------------------------------------------------------------
# Search common part among two filenames.
# --------------------------------------------------------------------
proc filename_common_parts {fn1 fn2} {
   # ----------------------------------------------------------------
   # main -----------------------------------------------------------
   
   # split filenames by ".", "-" and "_".
   set parts1 [split [file rootname [file tail $fn1]] ".-_"]
   set parts2 [split [file rootname [file tail $fn2]] ".-_"]
   
   # search common parts
   set ans {}
   for {set i 0} { $i < [llength $parts1] } { incr i } {
      set p1 [lindex $parts1 $i]
      set id [lsearch -exact $parts2 $p1]
      if { $id >= 0 } {
	 # found
	 lappend ans "$i $id"
      }
   }
   return $ans
}
# --------------------------------------------------------------------
# flip distance range
# --------------------------------------------------------------------
proc flip_distance_range {} {
   # Global variables --------------------
   global gbl paste

   # main -----------------------------------------------------------
   set tmp $paste(left)
   set paste(left) [expr $paste(right) * -1]
   set paste(right) [expr $tmp * -1]
   draw_pasteup
}
# -----------------------------------------------------------------
# Clear slaves of the widget "w"
# -----------------------------------------------------------------
proc clear_slaves {w} {
   set slaves [grid slaves $w]
   if { $slaves != ""} {
      eval grid forget [grid slaves $w]
   }
}

# --------------------------------------------------------------------
# split by "blank" chars.
# --------------------------------------------------------------------
proc mysplit {str} {
   return [split [string trim [regsub -all \[\\s\]+ $str { }]]]
}

# --------------------------------------------------------------------
# change suffix of given filename
# --------------------------------------------------------------------
proc change_suffix {fn suffix} {
   return [format "%s%s" [file rootname $fn] $suffix]
}
# --------------------------------------------------------------------
# absolute_path ------------------------------------------------------
#            Return absolute path of a given file
# --------------------------------------------------------------------
# fn : filename
# --------------------------------------------------------------------
proc absolute_path {fn} {
   global env
   if { [llength $fn] < 1 } { return $fn }
   set plist [file split $fn]
   if { [llength $plist] == 1 } {
      set fn [format "./%s" $fn]
      set plist [file split $fn]
   }
   set top [lindex $plist 0]

   # absolute path
   if { $top == [file volumes] } {
      return $fn
   }
   # special paths
   if { [lsearch [list "." ".." "~"] $top] >= 0} {
      # current directory
      if { $top == "." } {
	 set plist [lreplace $plist 0 0 [pwd]]
      }
      # my home
      if { $top == "~" } {
	 set plist [lreplace $plist 0 0 $env(HOME)]
      }
      # count upward directories
      set up 0
      while { $top == ".." } {
	 incr up
	 set top [lindex $plist $up]
      }
      if { $up > 0 } {
	 set cwd [file split [pwd]]
	 set n [llength $cwd]
	 set parents [lrange $cwd 0 [expr $n - $up -1]]
	 set plist [eval list $parents [lrange $plist $up end]]
      }
   } else {
      # current direcotry without "./"
      set plist [concat [pwd] $plist]
   }
   return [eval file join $plist]
}
# --------------------------------------------------------------------
# relative_path ------------------------------------------------------
#            Return relative path of a given file
# --------------------------------------------------------------------
# fn : filename
# --------------------------------------------------------------------
proc relative_path {fn} {
   if { [llength $fn] < 1 } { return $fn }
   set afn [file split [absolute_path $fn]]
   set cwd [file split [pwd]]
   set n [llength $cwd]
   set commons 0
   for {set i 0} {$i < $n} {incr i} {
      set c [lindex $cwd $i]
      set a [lindex $afn $i]
      if { $a == $c } {
	 incr commons
      }
   }
   set afn [lreplace $afn 0 [expr $commons -1]]
   set parents [expr $n - $commons]
   if { $parents == 0 } {
      set relative "./"
   } else {
      set relative ""
      while { $parents > 0 } {
	 incr parents -1
	 set relative [format "../%s" $relative]
      }
   }
   return [format "%s%s" $relative [eval file join $afn]]
}
# --------------------------------------------------------------------
# sort by date of files
# --------------------------------------------------------------------
# flist : filename list
# --------------------------------------------------------------------
proc sort_by_date {flist} {
   set datelist {}
   foreach fn $flist {
      lappend datelist [list [file mtime $fn] $fn]
   }
   set sorted [lsort -index 0 -decreasing $datelist]
   set newlist {}
   foreach d $sorted {
      lappend newlist [lindex $d 1]
   }
   return $newlist
}
