# --------------------------------------------------------------------
# Show multiple of a point
# --------------------------------------------------------------------
proc multiple_point {pick_x pick_y} {
   # global --------------------
   global gbl wk

   # Main --------------------
   # get (x and non-reduced t)
   set x [coord cx2gx [$wk(w,canvas,c) canvasx $pick_x]]
   set t [coord cy2tt $x [$wk(w,canvas,c) canvasy $pick_y]]
   # calc delay 
   if {$wk(multiple,direc) < 0} {
      # prior
      set t [expr - $gbl(obs,depth) * 2.0 / $gbl(water_v) + $t]
   } else {
      # posterior 
      set t [expr $gbl(obs,depth) * 2.0 / $gbl(water_v) + $t]
   }
   multiple_plot $x $t
}

# --------------------------------------------------------------------
# Show multiple of a line
# --------------------------------------------------------------------
proc multiple_line_start {pick_x pick_y} {
   # global --------------------
   global wk

   # Main --------------------
   # get (x and non-reduced t)
   set wk(multiple,line_x) [coord cx2gx [$wk(w,canvas,c) canvasx $pick_x]]
   set wk(multiple,line_t)\
       [coord cy2tt $wk(multiple,line_x) [$wk(w,canvas,c) canvasy $pick_y]]
   set wk(multiple,status) on

   # temporary binding
   bind $wk(w,canvas,c) <B2-Motion> {
      multiple_line_motion %x %y
   }
   bind $wk(w,canvas,c) <ButtonRelease-2> {
      multiple_line_end %x %y
   }
}

# --------------------------------------------------------------------
# while searching multiple line...
# --------------------------------------------------------------------
proc multiple_line_motion {pick_x pick_y} {
   # global --------------------
   global wk

   # main --------------------
   if { $wk(multiple,status) != "on"} {
      bind $wk(w,canvas,c) <B2-Motion> ""
      bind $wk(w,canvas,c) <ButtonRelease-2> ""
      set wk(multiple,status) off
      return;
   }
   catch {$wk(w,canvas,c) delete temp_mark}
   set x [coord cx2gx [$wk(w,canvas,c) canvasx $pick_x]]
   set y [coord cy2tt $x [$wk(w,canvas,c) canvasy $pick_y]]

   # draw a temporary line
   set y0 [coord tt2cy $wk(multiple,line_x) $wk(multiple,line_t)]
   set x0 [coord gx2cx $wk(multiple,line_x)]
   $wk(w,canvas,c) create line $x0 $y0\
       [$wk(w,canvas,c) canvasx $pick_x] [$wk(w,canvas,c) canvasy $pick_y]\
       -width 1 -fill red -tags temp_mark -dash {2 4}
}

# --------------------------------------------------------------------
# determine a multiple line
# --------------------------------------------------------------------
proc multiple_line_end {pick_x pick_y} {
   # global --------------------
   global wk gbl

   # main --------------------
   bind $wk(w,canvas,c) <B2-Motion> ""
   bind $wk(w,canvas,c) <ButtonRelease-2> ""
   catch {$wk(w,canvas,c) delete temp_mark}
   if { $wk(multiple,status) != "on"} { return }
   set wk(multiple,status) off

   # multiple time
   if {$wk(multiple,direc) < 0} {
      # prior
      set dt [expr - $gbl(obs,depth) * 2.0 / $gbl(water_v)]
   } else {
      # posterior 
      set dt [expr $gbl(obs,depth) * 2.0 / $gbl(water_v)]
   }

   # draw a multiple line
   set y0 [coord tt2cy $wk(multiple,line_x) [expr $wk(multiple,line_t) + $dt]]
   set x0 [coord gx2cx $wk(multiple,line_x)]
   set x1 [coord cx2gx [$wk(w,canvas,c) canvasx $pick_x]]
   set y1 [coord tt2cy $x1\
	       [expr [coord cy2tt $x1 [$wk(w,canvas,c) canvasy $pick_y]] + $dt] ]
   set x1 [coord gx2cx $x1]
   $wk(w,canvas,c) create line $x0 $y0 $x1 $y1 -width 2\
       -fill $wk(color,multiple) -tags multiple
}

# --------------------------------------------------------------------
# Plot multiple mark (x and t is true coordinate (km, sec))
# --------------------------------------------------------------------
proc multiple_plot {x t} {
   # global --------------------
    global gbl wk

    # main --------------------
    # convert (x,t) into (cx,cy)
    set y [coord tt2cy $x $t]
    set x [coord gx2cx $x]

    $wk(w,canvas,c) create line [expr $x - $gbl(mark_size)] $y\
	[expr $x + $gbl(mark_size)] $y\
	-fill $wk(color,multiple) -tags multiple -width 2
}

# --------------------------------------------------------------------
# Plot multiple marks for all the existent pick marks 
# --------------------------------------------------------------------
proc multiple_plot_all {} {
   # global --------------------
   global gbl wk
   
   # main --------------------
   set dt [expr {$wk(multiple,direc) < 0} ? \
	       [expr - $gbl(obs,depth) * 2.0 / $gbl(water_v)] :\
	       [expr $gbl(obs,depth) * 2.0 / $gbl(water_v)] ]
   plot_all_multiple $dt
}

# --------------------------------------------------------------------
# Flip multiple direction (posterior/prior)
# --------------------------------------------------------------------
proc multiple_flip_direction {} {
   global wk
   set wk(multiple,direc) [expr {$wk(multiple,direc) < 0} ? {1} : {-1}]
   clear_multiple
   multiple_plot_all
}