# --------------------------------------------------------------------
# a window for change pick property
#     (x1, x2, y1, y2 must be a canvas widget coord)
# --------------------------------------------------------------------
proc change_pick_property {w x1 x2 y1 y2 root_x2 root_y2} {
   # global --------------------
   global paste wk change_rect

   # main --------------------
   bind $w <B3-Motion> ""
   bind $w <ButtonRelease-3> ""
   set change_rect(status) 0

   # rectangle position (relative to the root window)
   if { $x1 > $x2 } {
      set wxmin $root_x2
      set wxmax [expr int($root_x2 - ($x2 - $x1))]
   } else {
      set wxmin [expr int($root_x2 - ($x2 - $x1))]
      set wxmax $root_x2
   }
   if { $y1 > $y2 } {
      set wymin [expr $root_y2 - 250]
      set wymax [expr int($root_y2 - ($y2 - $y1))]
   } else {
      set wymin [expr int($root_y2 - ($y2 - $y1)) - 250]
      set wymax $root_y2
   }
   if {$wymin < 0} { set wymin 0 }
   # determine window position
   set wx [expr ($wxmax > [expr $wk(xdimension,width)*0.8]) ? $wxmin : $wxmax]
   set wy [expr ($wymax > [expr $wk(xdimension,height)*0.8]) ? $wymin : $wymax]

   # check pick existence
   if { $paste(canvas,label) == "Normal" } {
      set x1 [coord cx2gx $x1]
      set x2 [coord cx2gx $x2]
      set y1 [coord cy2gt $y1]
      set y2 [coord cy2gt $y2]
   } else {
      set x1 [$w axis invtrans x $x1]
      set x2 [$w axis invtrans x $x2]
      set y1 [$w axis invtrans y $y1]
      set y2 [$w axis invtrans y $y2]
   }
   
   invoke_change_window $x1 $x2 $y1 $y2 $wx $wy
}

# --------------------------------------------------------------------
# Invoke a window for change pick property
# --------------------------------------------------------------------
proc invoke_change_window {x1 x2 y1 y2 wx wy} {
   # global --------------------
   global pick change_rect wk pm paste

   # main --------------------
   # check pick existence
   set plist [pick listup pm $x1 $x2 $y1 $y2 $paste(reduction)]
   if { [llength $plist] < 1 } {
      print_message "No pick mark inside area.\n"
      return
   }

   # invoke property window
   set w .change_pick_property
   catch {destroy $w}
   toplevel $w
   wm title $w "Picks in the region"
   wm geometry $w [format "+%d+%d" $wx $wy]

   # title --------------------
   label $w.title -text "EDIT picks in the REGION"

   # tabset --------------------
   set wk(pickedit,applyto) [set ts $w.tabset]
   tabset $ts -relief flat
   $ts conf -borderwidth 1 -highlightthickness 0 -side top
   $ts insert end "allpick" -text "All"
   $ts insert end "selected" -text "Selected"

   # tabset balloon help
   foreach {tab mes}\
       {allpick "Apply to all picks in the region"\
	    selected "Apply to specified pick in the region"\
	 } {
	    $ts bind $tab <Enter> "balloon \"$mes\" %X %Y"
	    $ts bind $tab <Leave> {cancel_balloon}
	 }
   
   # Apply to all tab --------------------
   set at $ts.alltab
   labelframe $at -text "Apply to ALL picks" -pady 10
   $ts tab conf "allpick" -window $at

   # type
   label $at.label -text "Type"
   picktype_selector_widget $at ptype pick(current_type) pick(range)
   button $at.button -text "Convert"\
       -command "change_pick_type $x1 $x2 $y1 $y2"
   grid $at.label $at.mb(ptype) $at.sb(ptype) - $at.button -sticky nws

   # accuracy
   label $at.label2 -text "Acuracy"
   set bak $pick(default_accuracy)
   spinbox $at.spin -width 7 -textvar pick(default_accuracy) -justify right\
       -values $wk(accuracy,default_list)
   $at.spin set $bak
   label $at.unit -text "second"
   button $at.button2 -text "Set"\
       -command "change_pick_accuracy $x1 $x2 $y1 $y2"
   grid $at.label2 $at.spin - $at.unit $at.button2 -sticky nws
   bind $at.spin <Button-4> "$at.spin invoke buttonup"
   bind $at.spin <Button-5> "$at.spin invoke buttondown"

   # delete picks
   label $at.label3 -text "Delete"
   button $at.eraseb -text "DEL"\
       -command "change_pick_erase $x1 $x2 $y1 $y2 $w"
   grid $at.label3 x x x $at.eraseb -sticky nws

   # selected pick tab --------------------
   set st [frame $ts.seltab]
   $ts tab conf "selected" -window $st

   # apply to ----------
   set apt $st.applyto
   labelframe $apt -text "Target pick type"
   picktype_selector_widget $apt ptype wk(pickedit,apply,type)\
       wk(pickedit,apply,range)
   grid $apt.mb(ptype) $apt.sb(ptype) -sticky nws

   # chage ----------
   set at $st.change
   labelframe $at -text "Operations"

   # type
   label $at.label -text "Type"
   picktype_selector_widget $at ptype pick(current_type) pick(range)
   button $at.button -text "Convert"\
       -command "change_pick_type $x1 $x2 $y1 $y2"
   grid $at.label $at.mb(ptype) $at.sb(ptype) - $at.button -sticky nws

   # accuracy
   label $at.label2 -text "Accuracy"
   set bak $pick(default_accuracy)
   spinbox $at.spin -width 7 -textvar pick(default_accuracy) -justify right\
       -values $wk(accuracy,default_list)
   $at.spin set $bak
   label $at.unit -text "second"
   button $at.button2 -text "Set"\
       -command "change_pick_accuracy $x1 $x2 $y1 $y2"
   grid $at.label2 $at.spin - $at.unit $at.button2 -sticky nws
   bind $at.spin <Button-4> "$at.spin invoke buttonup"
   bind $at.spin <Button-5> "$at.spin invoke buttondown"

   # delete picks
   label $at.label3 -text "Delete"
   button $at.eraseb -text "DEL"\
       -command "change_pick_erase $x1 $x2 $y1 $y2 $w"
   grid $at.label3 x x x $at.eraseb - -sticky nws

   pack $apt $at -side top -fill x -expand 1

   # buttons --------------------
   button_frame $w $w.button

   # pack all --------------------
   pack $w.title $ts $w.button -side top -expand 1 -fill x
   grab $w
}

# --------------------------------------------------------------------
# change pick types
# --------------------------------------------------------------------
proc change_pick_type {x1 x2 t1 t2} {
   # global --------------------
   global pm mark_modification pick paste wk
   
   # main --------------------
   undo_stack
   if { [$wk(pickedit,applyto) get select] == "selected" } {
      set tmp $wk(pickedit,apply,type)
   } else {
      set tmp 0
   }
   set pm [pick region type pm $tmp $x1 $x2 $t1 $t2 \
	   $paste(reduction) $pick(current_type)]
   plot_pick_marks 0
   set mark_modification 1
   print_message "Pick mark type was changed.\n"
}

# --------------------------------------------------------------------
# change pick accuracy
# --------------------------------------------------------------------
proc change_pick_accuracy {x1 x2 t1 t2} {
   # global --------------------
   global pick mark_modification pm paste wk
   
   # main --------------------
   undo_stack
   if { [$wk(pickedit,applyto) get select] == "selected" } {
      set tmp $wk(pickedit,apply,type)
   } else {
      set tmp 0
   }
   set pm [pick region accuracy pm $tmp $x1 $x2 $t1 $t2\
	       $paste(reduction) $pick(default_accuracy)]
   plot_pick_marks 0
   set mark_modification 1
   print_message "Pick accuracy was changed.\n"
}

# --------------------------------------------------------------------
# delete picks inside property region
# --------------------------------------------------------------------
proc change_pick_erase {x1 x2 t1 t2 w} {
   # global --------------------
   global mark_modification pm paste wk
   
   # main --------------------
   undo_stack
   if { [$wk(pickedit,applyto) get select] == "selected" } {
      set status 1
   } else {
      set status 0
   }
   set tmp [expr ($status != 0) ? $wk(pickedit,apply,type) : 0]
   set pm [pick region delete pm $tmp $x1 $x2 $t1 $t2 $paste(reduction)]
   plot_pick_marks 0
   set mark_modification 1
   print_message [format "pick marks were deleted.\n" ]

   if { $status == 0 } {
      destroy $w
   }
}

