# --------------------------------------------------------------------
# drag record section (shift display range)
# --------------------------------------------------------------------
proc drag_section_start {x y} {
   # global --------------------
   global wk

   # main --------------------
   set wk(drag,x) [$wk(w,canvas,c) canvasx $x]
   set wk(drag,y) [$wk(w,canvas,c) canvasy $y]
   set wk(drag,mode) anyway

   # temporary binding ----------
   bind $wk(w,canvas,c) <Meta-B3-Motion> {
      drag_draw_line %x %y
   }
   bind $wk(w,canvas,c) <ButtonRelease-3> {
      # cancel
      drag_clear_line
      drag_clear_drag_binding
   }
   bind $wk(w,canvas,c) <Meta-ButtonRelease-3> {
      # drag!!
      drag_clear_line
      drag_clear_drag_binding
      drag_section %x %y
   }
}

# --------------------------------------------------------------------
# drag record section (shift display range) : only HORIZONTAL shift
# --------------------------------------------------------------------
proc drag_section_horizontal_start {x y} {
   # global --------------------
   global wk

   # main --------------------
   set wk(drag,x) [$wk(w,canvas,c) canvasx $x]
   set wk(drag,y) [$wk(w,canvas,c) canvasy $y]
   set wk(drag,mode) horizontal

   # temporary binding ----------
   bind $wk(w,canvas,c) <Meta-B1-Motion> {
      drag_draw_line %x %y
   }
   bind $wk(w,canvas,c) <ButtonRelease-1> {
      # cancel
      drag_clear_line
      drag_clear_drag_binding
   }
   bind $wk(w,canvas,c) <Meta-ButtonRelease-1> {
      # drag!!
      drag_clear_line
      drag_clear_drag_binding
      drag_section %x %y
   }
}

# --------------------------------------------------------------------
# drag record section (shift display range) : only VERTICAL shift
# --------------------------------------------------------------------
proc drag_section_vertical_start {x y} {
   # global --------------------
   global wk

   # main --------------------
   set wk(drag,x) [$wk(w,canvas,c) canvasx $x]
   set wk(drag,y) [$wk(w,canvas,c) canvasy $y]
   set wk(drag,mode) vertical

   # temporary binding ----------
   bind $wk(w,canvas,c) <Meta-B2-Motion> {
      drag_draw_line %x %y
   }
   bind $wk(w,canvas,c) <ButtonRelease-2> {
      # cancel
      drag_clear_line
      drag_clear_drag_binding
   }
   bind $wk(w,canvas,c) <Meta-ButtonRelease-2> {
      # drag!!
      drag_clear_line
      drag_clear_drag_binding
      drag_section %x %y
   }
}

# --------------------------------------------------------------------
# clear drag line
# --------------------------------------------------------------------
proc drag_clear_line {} {
   # global --------------------
   global wk

   # main --------------------
   catch {$wk(w,canvas,c) delete dragline}
}

# --------------------------------------------------------------------
# draw drag line
# --------------------------------------------------------------------
proc drag_draw_line {x y} {
   # global --------------------
   global wk

   # main --------------------
   set x [$wk(w,canvas,c) canvasx $x]
   set y [$wk(w,canvas,c) canvasy $y]
   # delete old drag line and comments
   drag_clear_line

   # compute dx and dt
   set tmp [drag_comp_dxdt $x $y]
   set dx [lindex $tmp 0]
   set dt [lindex $tmp 1]
   if {$dx >= 0.0} {
      set anc w
   } else {
      set anc e
   }
   if { $wk(drag,mode) == "horizontal" } {
      set y $wk(drag,y)
   }
   if { $wk(drag,mode) == "vertical" } {
      set x $wk(drag,x)
   }

   # draw current drag line
   $wk(w,canvas,c) create line $x $y $wk(drag,x) $wk(drag,y)\
       -width 5 -tag dragline -fill red -dash {5 5} -arrow first

   # draw comments
   $wk(w,canvas,c) create text [expr $x +1] [expr $y +1 ]\
       -anchor $anc -justify left -tag dragline -fill black -font font4\
       -text [format "(dx=%s, dt=%s)" $dx $dt]
   $wk(w,canvas,c) create text $x $y\
       -anchor $anc -justify left -tag dragline -fill red -font font4\
       -text [format "(dx=%s, dt=%s)" $dx $dt]
}

# --------------------------------------------------------------------
# clear drag bindings
# --------------------------------------------------------------------
proc drag_clear_drag_binding {} {
   # global --------------------
   global wk

   # main --------------------
   bind $wk(w,canvas,c) <Meta-ButtonRelease-1> {}
   bind $wk(w,canvas,c) <ButtonRelease-1> {}
   bind $wk(w,canvas,c) <Meta-B1-Motion> {}
   bind $wk(w,canvas,c) <Meta-ButtonRelease-2> {}
   bind $wk(w,canvas,c) <ButtonRelease-2> {}
   bind $wk(w,canvas,c) <Meta-B2-Motion> {}
   bind $wk(w,canvas,c) <Meta-ButtonRelease-3> {}
   bind $wk(w,canvas,c) <ButtonRelease-3> {}
   bind $wk(w,canvas,c) <Meta-B3-Motion> {}
}

# --------------------------------------------------------------------
# compute dx and dt
# --------------------------------------------------------------------
proc drag_comp_dxdt {x y} {
   # global --------------------
   global wk

   # main --------------------
   # coordinates
   set ox [coord cx2gx $wk(drag,x)]
   set ot [coord cy2gt $wk(drag,y)]
   set gx [coord cx2gx $x]
   set gt [coord cy2gt $y]
   if { $wk(drag,mode) == "vertical" } {
      set dx 0.0
   } else {
      set dx [format "%.1f" [expr $gx - $ox]]
   }
   if { $wk(drag,mode) == "horizontal" } {
      set dt 0.0
   } else {
      set dt [format "%.2f" [expr $gt - $ot]]
   }
   return "$dx $dt"
}
# --------------------------------------------------------------------
# drag section
# --------------------------------------------------------------------
proc drag_section {x y} {
   # global --------------------
   global wk paste

   # main --------------------
   # coordinates
   set x [$wk(w,canvas,c) canvasx $x]
   set y [$wk(w,canvas,c) canvasy $y]
   set tmp [drag_comp_dxdt $x $y]
   set dx [lindex $tmp 0]
   set dt [lindex $tmp 1]
   # drag
   set paste(left) [expr $paste(left) + $dx]
   set paste(right) [expr $paste(right) + $dx]
   set paste(top) [expr $paste(top) + $dt]
   set paste(bottom) [expr $paste(bottom) + $dt]
   # draw it
   draw_pasteup
}

# --------------------------------------------------------------------
# shift section to (x, y) direction
# --------------------------------------------------------------------
proc shiftsection {x y} {
   # global --------------------
   global paste

   # main --------------------
   set dx [expr $x * abs( $paste(right) - $paste(left) ) / 2.0 ]
   set dy [expr $y * abs( $paste(bottom) - $paste(top) ) / 4.0 ]
   # shift
   set paste(left)  [expr $paste(left) + $dx]
   set paste(right) [expr $paste(right) + $dx]
   set paste(top) [expr $paste(top) + $dy]
   set paste(bottom) [expr $paste(bottom) + $dy]
   # draw it
   draw_pasteup
}
