# --------------------------------------------------------------------
# Select and load pick data file
# --------------------------------------------------------------------
proc load_pick {} {
   # global --------------------
   global mark_modification file_name gbl

   # main --------------------
   # check mark modification ----------
   if {$mark_modification == 1} {
      set ans [tk_messageBox -type yesno -default yes -icon question -message\
		   "Pick data was modified. Do you really wish to load?"]
      if {$ans == no} { return }
   }
   # select filename ----------
   if { $file_name(pick) != "" } {
      set dir [file dirname $file_name(pick)]
   } else {
      set dir "./"
   }
   set types {
      {"pick files"	{.pick .in .macray .mochi .tt}	}
      {"dat files"	{.dat}	}
      {"All files"		*}
   }
   set fn [tk_getOpenFile -filetypes $types -parent .\
		     -initialdir $dir -defaultextension .pick]
   if { $fn == "" } { return }
   
   # read ----------
   read_pick $fn

   # register to data base, if necessary
   if { $gbl(ds,mode) } {
      data_set_pick_check $fn
   }
   set mark_modeifcation 0
   update_wm_title
}

# --------------------------------------------------------------------
# Merge pick data in pasteup/mochi/rayinvr format
# --------------------------------------------------------------------
proc merge_pick {} {
   # global --------------------
   global wk pm mark_modification file_name

   # main --------------------
   if { [llength $pm] < 1 } {
      # if no picks,
      load_pick
      return
   }
   # determine default directory
   if { $file_name(pick) != "" } {
      set dir [file dirname $file_name(pick)]
   } else {
      set dir ./
   }
   # file selection
   set types {
      {"pick files"	{.pick .in .macray .mochi}	}
      {"dat files"	{.dat}	}
      {"All files"		*}
   }
   set tmpfn [tk_getOpenFile -filetypes $types -parent .\
		     -initialdir $dir -defaultextension .pick]
   if ![string compare $tmpfn ""] {
      print_message "Abort merging.\n"
      return
   }
   # merge pick ------------------------------
   # save existing picks
   set oldpm $pm
   # read new picks
   set pm {}
   set tmp $file_name(pick)
   read_pick $tmpfn
   set file_name(pick) $tmp
   # append old picks
   set pm [concat $oldpm $pm]
   # replot
   plot_pick_marks 0

   set mark_modeifcation 1
}

# --------------------------------------------------------------------
# Save picks
# --------------------------------------------------------------------
proc save_pick {} {
   # global --------------------
   global file_name gbl wk
   
   # main --------------------
   if { $file_name(pick) == "" } {
      return [save_as_pick]
   }
   return [write_pick]
}

# --------------------------------------------------------------------
# Save picks in a new file
# --------------------------------------------------------------------
proc save_as_pick {} {
   # global --------------------
   global file_name gbl wk
   
   # main --------------------
   # select filename
   set wk(tmp) $file_name(pick)
   set status [file_selection wk(tmp) save pick]
   if { $status == "cancel"} {
      print_message "Canceled : save pick data as a new file.\n"
      return "cancel"
   }
   # save
   set file_name(pick) $wk(tmp)
   if { [write_pick] == "ok" } {
      # regist pick file in DB
      if { $gbl(ds,mode) == 1 } {
	 data_set_register_pick $file_name(pick)
      }
   }
   update_wm_title
   return "ok"
}

# --------------------------------------------------------------------
# Read pick data ; update file_name(pick)
# --------------------------------------------------------------------
proc read_pick {fn} {
   # global --------------------
   global pm wk pick file_name mark_modification undopm redopm
   
   # main --------------------
   set wk(pick,format_name) [discriminate_pickfile $fn]
   switch $wk(pick,format_name) {
      "Pasteup" {
	 read_pasteup_pick $fn
	 set wk(pick,format) 0
      }
      "Mochi" {
	 read_mochi_pick $fn
	 set wk(pick,format) 1
      }
      "Rayinvr" {
	 read_rayinvr_pick $fn
	 set wk(pick,format) 2
      }
      "GeoCTII" {
	 read_geoctii_pick $fn
	 set wk(pick,format) 3
      }
      "none" {
	 return
      }
      default {
	 puts stderr "Invalid pick file format"
	 return
      }
   }
   print_message [format "%d points were read from %s (in %s format).\n"\
		      [llength $pm ] $fn $wk(pick,format_name)]
   set file_name(pick) $fn

   # undo
   set undopm {}
   set redopm {}
   set wk(pickedit,undoremain) 0
   set mark_modification 0

   # plot
   plot_pick_marks 0
   update_wm_title
}

# --------------------------------------------------------------------
# write pick data into $file_name(pick)
# --------------------------------------------------------------------
proc write_pick {} {
   # global --------------------
   global file_name wk pm mark_modification

   # main --------------------
   if { [catch { open $file_name(pick) w } fp] == 1 } {
      print_message "Cannot open $file_name(pick) to write.\n"
      return "error"
   }
   close $fp
   switch $wk(pick,format) {
      0 {
	 write_pasteup_pick $file_name(pick)
      }
      1 {
	 write_mochi_pick $file_name(pick)
      }
      2 {
	 write_rayinvr_pick $file_name(pick)
      }
      3 {
	 write_geoctii_pick $file_name(pick)
      }
      default {
	 return "error"
      }
   }
   set mark_modification 0
   print_message [format "%d picks were written in %s in %s format.\n" \
		      [llength $pm] $file_name(pick) $wk(pick,format_name)]
   return "ok"
}

# --------------------------------------------------------------------
# discriminate_pickfile ----------------------------------------------
#            Discriminate pickfile format, whether "macray", "zelt",
#            "mochi" or "geo" format.
# --------------------------------------------------------------------
# file :  pick file name.
# --------------------------------------------------------------------
proc discriminate_pickfile {file} {
   # readable check
   if {[string length $file] < 1} { return none }
   if {![file readable $file]} {
      tk_messageBox -message "Pick file \"$file\" is not readable."\
	  -type ok -icon error
      return none
   }
   # open and read first line
   set fp [open $file]
   gets $fp line
   close $fp
   # check type
   if {[llength $line] == 3} {
      # 3 column format
      return Mochi
   }
   if {[llength $line] == 2} {
      # geoct2 
      return GeoCTII
   }
   if {[lindex $line 3] == 0} {
      # zelt format   (4-th component must be zero)
      return Rayinvr
   } else {
      # macray format (4-th component is distance multiplier, can't be zero)
      return Pasteup
   }
   return none
}

# --------------------------------------------------------------------
# Exec read pick data in mochi format.
# --------------------------------------------------------------------
proc read_mochi_pick {data_file} {
   # global --------------------
   global pm wk gbl
   # main --------------------
   set fd [open $data_file r]
   set gbl(obs,x) 0
   set pm {}
   set i 0
   while {1} {
      gets $fd tmp
      if { [eof $fd] } {
	 break
      }
      if { [llength $tmp] != 3 } {
	 continue
      }
      incr i
      scan $tmp "%s%s%s" px pt sn
      lappend pm "$sn $px $pt 0.02 1"
   }
   catch {close $fd}
}

# --------------------------------------------------------------------
# write pick data in Mochi format
# --------------------------------------------------------------------
proc write_mochi_file {fn} {
   # global --------------------
   global pm
   
   # main --------------------
   set pn [llength $pm]
   set fd [open $fn w]
   puts $fd $pn
   foreach d $pm {
      scan $d "%d%s%s%*s%*d" sn px pt
      puts $fd "$px $pt $sn"
   }
   close $fd
}

# --------------------------------------------------------------------
# Write pick data in pasteup format
# --------------------------------------------------------------------
proc write_pasteup_pick {args} {
   # gloabl --------------------
   global pm gbl
   
   # main --------------------
   set fn [lindex $args 0]

   # decimation ?
   set every [expr { [llength $args] > 1 } ? { [lindex $args 1] } : {1} ]

   # check obs position
   if { $gbl(obs,x) == "" } { set gbl(obs,x) 0.0 }
   if { $gbl(obs,depth) == "" } { set gbl(obs,depth) 0.0 }
   
   # write out
   set fd [open $fn w]
   puts $fd [format "%.3f %.3f 0.0 1.0" $gbl(obs,x) $gbl(obs,depth)]
   if { $every > 1 } {
      # decimation
      set n [llength $pm]
      set i 0
      while { $i < $n } {
	 scan [lindex $pm $i] "%d%f%f%f%d" s x t a m
	 puts $fd [format " %5d %8.3f %7.3f %5.3f %3d" $s $x $t $a $m]
	 incr i $every
      } 
   } else {
      # every pick
      foreach d $pm { 
	 scan $d "%d%f%f%f%d" s x t a m
	 puts $fd [format " %5d %8.3f %7.3f %5.3f %3d" $s $x $t $a $m]
      }
   }
   close $fd
}

# --------------------------------------------------------------------
# read pick in pasteup format
# --------------------------------------------------------------------
proc read_pasteup_pick {data_file} {
   # global
   global pm gbl wk

   # main
   set fd [open $data_file r]
   set tmp [gets $fd]
   set gbl(obs,x) [lindex $tmp 0]
   set depth [lindex $tmp 1]
   if { $depth != 0.0 } {
      set gbl(obs,depth) $depth
   }
   set pm {}
   while { [gets $fd tmp] != -1} {
      if {[llength $tmp] < 4} {
	 continue
      }
      lappend pm $tmp
   }
   close $fd
}

# --------------------------------------------------------------------
# read pick data in rayinvr format.
# --------------------------------------------------------------------
proc read_rayinvr_pick {fn} {
   # global --------------------
   global pm gbl
   # main --------------------
   set fd [open $fn r]
   set tmp [gets $fd]
   set gbl(obs,x) [lindex $tmp 0]
   set i 0
   set pm {}
   while {[gets $fd tmp]} {
      if {[eof $fd] || [llength $tmp] != 4} {break}
      set mark [lindex $tmp 3]
      if { $mark > 0 } {
	 incr i
	 set sn $i
	 set px  [expr [lindex $tmp 0] - $gbl(obs,x)]
	 set pt [lindex $tmp 1]
	 set acc [lindex $tmp 2]
	 lappend pm "$sn $px $pt $acc $mark"
      }
   }
   catch {close $fd}
}

# --------------------------------------------------------------------
# Write pick data in rayinvr format
# --------------------------------------------------------------------
proc write_rayinvr_pick {fn} {
   # gloabl --------------------
   global pm gbl
   
   # main --------------------
   set fd [open $fn w]
   # right-side
   puts $fd [format "%10.3f%10.3f%10.3f%10d" $gbl(obs,x) 1 0.0 0]
   foreach d $pm {
      scan $d "%d%s%s%s%d" sn px pt acc type
      if { $px >= 0.0 } {
	 puts $fd [format "%10.3f%10.3f%10.3f%10d"\
			[expr $gbl(obs,x) + $px] $pt $acc $type]
      }
   }
   # left-side
   puts $fd [format "%10.3f%10.3f%10.3f%10d" $gbl(obs,x) -1 0.0 0]
   foreach d $pm {
      scan $d "%d%s%s%s%d" sn px pt acc type
      if { $px < 0.0 } {
	 puts $fd [format "%10.3f%10.3f%10.3f%10d"\
		       [expr $gbl(obs,x) + $px] $pt $acc $type]
      }
   }
   close $fd
}

# --------------------------------------------------------------------
# read pick in geoctii format
# --------------------------------------------------------------------
proc read_geoctii_pick {fn} {
   # global
   global pm pick gbl

   # main
   set fd [open $fn r]
   gets $fd tmp
   scan $tmp "%d%d" nobs unit
   if { $nobs != 1 } {
      tk_messageBox -type ok -icon warning\
	  -message "Current version cannot read plural sources data."
      close $fd
      return
   }
   switch -- $unit {
      1 {
	 # meter
	 set coef 0.001
      }
      2 {
	 # feet
	 set coef 0.0003048
      }
      default {
	 # km
	 set coef 1.0
      }
   }
   # get shot id and number of picks
   gets $fd tmp
   scan $tmp "%s%s" id pn
   # get shot(OBS) position
   gets $fd tmp
   set gbl(obs,x) [expr [lindex $tmp 0] * $coef]
   set gbl(obs,depth) [expr [lindex $tmp 1] * $coef]
   # read picks
   set pm {}
   for {set i 0} {$i < $pn} {incr i} {
      gets $fd tmp
      # read one pick
      scan $tmp "%d%s%s%s" sn x z pt
      # set OBS depth (depth is in meter)
      set gbl(obs,depth) [expr $z / 1000.0]
      # x-coord --> offset
      set px [expr $x * $coef - $gbl(obs,x)]
      lappend pm "$sn $px $pt $pick(default_accuracy) 1"
   }
   close $fd
}

# --------------------------------------------------------------------
# write pick data in GeoCTII format
# --------------------------------------------------------------------
proc write_geoctii_pick {fn} {
   # gloabl --------------------
   global pm gbl
   
   # main --------------------
   set fd [open $fn w]
   # header of file
   puts $fd "1 1" ; # only 1 OBS, and unit is meter.
   # header for this OBS ----------
   set pn [llength $pm]
   # OBS-id and number of picks
   puts $fd "1 $pn"
   # obs position (x, z)
   puts $fd [format "%f %f"\
		 [expr $gbl(obs,x) * 1000.0]\
		 [expr $gbl(obs,depth) * 1000.0] ]

   # body ----------
   foreach d $pm {
      scan $d "%d%s%s%s%d" sn px pt acc type
      set x [expr ( $px + $gbl(obs,x) ) * 1000.0 ]
      set z 10
      puts $fd [format "%5d %5.4f %5.4f %5.4f" $sn $x $z $pt]
   }
   close $fd
}

# --------------------------------------------------------------------
# read x-position of the site from the pick file
# --------------------------------------------------------------------
proc pick_file_read_xpos {fn} {
   # main --------------------
   switch [discriminate_pickfile $fn] {
      "Pasteup" -
      "Rayinvr" {
	 set fp [open $fn]
	 gets $fp line1
	 close $fp
	 return [lindex $line1 0]
      }
      "GeoCTII" {
	 set fp [open $fn]
	 gets $fp line1
	 gets $fp line2
	 gets $fp line3
	 close $fp
	 scan $line1 "%d%d" nobs unit
	 switch -- $unit {
	    1 {
	       # meter
	       set coef 0.001
	    }
	    2 {
	       # feet
	       set coef 0.0003048
	    }
	    default {
	       # km
	       set coef 1.0
	    }
	 }
	 return [expr [lindex $line3 0] * $coef]
      }
      default {
	 # no x-position data
	 return 0.0
      }
   }
}

# --------------------------------------------------------------------
# Select and load pick data file
# --------------------------------------------------------------------
proc pick_file_select_and_load {op type} {
   load_pick
}
