# --------------------------------------------------------------------
# plot pick label (pick type for each pick groups) 
# --------------------------------------------------------------------
proc pick_label_plot {} {
   # global --------------------
   global pm wk

   # main --------------------
   # erase previous plot
   pick_label_erase

   # determine averaged trace spacing
   set wk(condition) [check_same_rh_condition]
   set headers [getrh range]
   if { [llength $headers] < 10 } { return }
   set dx 0.0
   set prev_x [lindex [lindex $headers 0] 1]
   for {set i 1} {$i <= 5} {incr i} {
      set x [lindex [lindex $headers $i] 1]
      set dx [expr $dx + abs( $x - $prev_x ) ]
      set prev_x $x
   }
   set dx [expr $dx / 5.0]

   # sort pick marks by offset and type
   set pm [pick sort $pm]

   # number of total pick marks
   set pn [llength $pm]

   set newpm {}
   set prev_type -1_
   for {set i 0} { $i < $pn } { incr i } {
      set pick [lindex $pm $i]
      set x [lindex $pick 1]
      set t [lindex $pick 2]
      if { [coord inout $x $t] == 0 } {
	 # outside
	 continue
      }
      set type [lindex $pick 4]
      if { $type != $prev_type } {
	 # complete collecting a pick group 
	 pick_label_plot_type $dx $newpm
	 set newpm {}
	 set prev_type $type
      } else {
	 # new pick group
	 lappend newpm $pick
      }
   }
   pick_label_plot_type $dx $newpm
}

# --------------------------------------------------------------------
# plot pick label for a single type
# --------------------------------------------------------------------
proc pick_label_plot_type {dx plist} {
   # global --------------------
   global paste

   # main --------------------
   set group {}

   set dat [lindex $plist 0]
   set prev_x [lindex $dat 1]
   lappend group $dat
   # grouping ----------
   for { set i 1 } { $i < [llength $plist] } {incr i} {
      set dat [lindex $plist $i]
      set x [lindex $dat 1]
      if { [expr abs($x - $prev_x)] > [expr $paste(plabel,separator) * $dx ]} {
	 # separator
	 pick_label_plot_group $group
	 set group {}
      }
      set prev_x $x
      lappend group $dat
   }
   pick_label_plot_group $group
}

# --------------------------------------------------------------------
# plot pick label for a single group
# --------------------------------------------------------------------
proc pick_label_plot_group {pgroup} {
   # global --------------------
   global paste wk

   # main --------------------
   set n [llength $pgroup]
   if { $n < $paste(plabel,groupsize) } { return }
   
   # middle pick mark
   scan [lindex $pgroup [expr int($n/2) ]] "%*s%f%f%*s%d" x t type

   # plot a string
   set col [lindex $wk(color,list) [expr ($type -1 ) % 20 ]]
   plot_string $wk(w,current) $x $t $type\
       $paste(plabel,anchor) $paste(plabel,font) $col\
       $paste(plabel,shadow) $paste(plabel,shadow_color)
}
# --------------------------------------------------------------------
# erase all the pick labels
# --------------------------------------------------------------------
proc pick_label_erase {} {
   global wk
   clear_string $wk(w,current)
}