# --------------------------------------------------------------------
# Show mouse position
# --------------------------------------------------------------------
proc mouse_position {w mx my} {
   set x [coord cx2gx [$w canvasx $mx]]
   set y [$w canvasy $my]
   print_message [format "%f km   %f sec. (%f sec.)\n"\
		      $x [coord cy2gt $y] [coord cy2tt $x $y]]
}

# --------------------------------------------------------------------
# plot single pick (t is a true time, not a reduced one)
# --------------------------------------------------------------------
proc plot_single_pick {x t acc type} {
   global pick gbl
   if {$gbl(check,showpick)} {
      pick plot $x $t $acc $type
   } else {
      set gbl(check,showpick) 1
      plot_pick_marks 0
   }
   trace_display_pick
}

# --------------------------------------------------------------------
# plot n-th and later picks (if n==0, all the picks will be plotted)
# --------------------------------------------------------------------
proc plot_pick_marks {n} {
   # global --------------------
   global pm pick gbl wk
   
   # main --------------------
   if {!$gbl(check,showpick)} {
      delete_pick_marks
      GraphCanvas::hide_picks $wk(w,canvas).g
      return
   }
   GraphCanvas::show_picks $wk(w,canvas).g

   if { $n == 0 } {
      delete_pick_marks
   }
   pick plotall pm $n
   trace_display_pick
}

# --------------------------------------------------------------------
# Delete i-th pick
# --------------------------------------------------------------------
proc exec_clear_pick {i} {
   global pm
   set pm [lreplace $pm $i $i]
}

# --------------------------------------------------------------------
# draw a crosshair cursor
# --------------------------------------------------------------------
proc crosshair_cursor {w mx my} {
   # global --------------------
   global paste
   
   # main --------------------
   set mx [$w canvasx $mx]
   set my [$w canvasy $my]
   if [coord inout2 [coord cx2gx $mx] [coord cy2gt $my]] {
      print_message "Warning: Out of record section.\n"
      return
   }
   
   # show info ----------
   print_message  [format "%f km   %f sec. (%f sec.)\n"\
		       [coord cx2gx $mx] [coord cy2gt $my]\
		       [coord cy2tt [coord cx2gx $mx] $my]]
   
   # draw a crosshair ----------
   # delete old crosshair
   catch {$w delete crosshair_cursor}
   # horizontal line
   $w create line $paste(left_margin) $my\
       [expr $paste(graph_width)+$paste(left_margin)] $my\
       -width 1 -tag crosshair_cursor -fill green
   # vertical line
   $w create line $mx $paste(top_margin)\
       $mx [expr $paste(graph_height)+$paste(top_margin)]\
       -width 1 -tag crosshair_cursor -fill green
}

# --------------------------------------------------------------------
# erase a crosshair
# --------------------------------------------------------------------
proc clear_crosshair_cursor {w} {
   catch {$w delete crosshair_cursor}
}
# --------------------------------------------------------------------
# forget all the picks (erase from the canvas and memory)
# --------------------------------------------------------------------
proc forget_picks {} {
   # global --------------------
   global mark_modification pm file_name
   
   # main --------------------
   if {$mark_modification != 0 && [llength $pm] > 0} {
      set ans [tk_messageBox -message "Do you want to save Pick data?"\
		   -type yesnocancel]
      if {$ans=="yes"} {
	 if { [save_pick] == "cancel"} { return "cancel" }
      }
      if {$ans=="cancel"} { return "cancel" }
   }
   # exec forget ----------
   # clear picks
   set pm {}
   plot_pick_marks 0
   # forget
   set file_name(pick) ""
   set mark_modification 0
   return ""
}

# --------------------------------------------------------------------
# toggle show/hide uncertainty
# --------------------------------------------------------------------
proc toggle_show_uncertainty {} {
   # global --------------------
   global paste
   set paste(mark,uncertainty) [expr $paste(mark,uncertainty) ? 0 : 1]
   plot_pick_marks 0
}

# --------------------------------------------------------------------
# raise accuracy mark of all picks.
# --------------------------------------------------------------------
proc raise_picks {} {
   # global --------------------
   global wk paste
   
   # Main --------------------
   if { $paste(image,mode) == 0 } { return }
   set bak $paste(image,mode)
   set paste(image,mode) 0
   plot_pick_marks 0
   $wk(w,canvas,c) raise acc
   $wk(w,canvas,c) raise pk
   set paste(image,mode) $bak
}

# --------------------------------------------------------------------
# lower accuracy mark of all picks.
# --------------------------------------------------------------------
proc lower_picks {} {
   # global --------------------
   global wk
  
   # Main --------------------
   $wk(w,canvas,c) delete acc
   eval $wk(w,canvas,g) marker delete [$wk(w,canvas,g) marker names a*]
}

# --------------------------------------------------------------------
# Adjust pick position and trace number
# --------------------------------------------------------------------
proc adjust_picks {} {
   # global --------------------
   global pm mark_modification wk

   # main --------------------
   # Is there any picks?
   if { [llength $pm] < 1 } {
      tk_messageBox -icon warning -title "Warning"\
	  -message "There is no pick.  Adjustment is canceled."
      return
   }
   # confirmation
   set ans [tk_messageBox -icon question -title "Adjust picks?"\
		-message "Horizontal position of all the picks will be moved to the nearest trace, and each shot number will be reassigned.  All right?"\
		-type okcancel -default ok]
   if {$ans=="cancel"} return
   
   # Exec adjustment
   print_message [format "Number of picks is %d (before adjustment) -->"\
		      [llength $pm]]
   set ww [wait_window "now adjusting..."]
   set n [llength $pm]
   for {set i 0} {$i < $n} {incr i} {
      scan [lindex $pm $i] "%d%s%s%s%d" sn px pt acc type
      set shot_info [getrh nearest $px]
      set id [lindex $shot_info 0]
      if { $id < 0 } { 
	 # Clear pick if there is no vicinity trace.
	 exec_clear_pick $i 
	 incr i -1
	 incr n -1
      } else {
	 # Adjust to the nearest trace
	 set sn  $id
	 set px [lindex $shot_info 1]
	 set pm [lreplace $pm $i $i "$sn $px $pt $acc $type"]
      }
   }
   set mark_modification 1
   print_message [format "%d (after)\n" [llength $pm]]
   plot_pick_marks 0
   destroy $ww
}

# --------------------------------------------------------------------
# Remove overlapped_picks 
#         (Only one pick for a pair of shot#, offset, type)
# --------------------------------------------------------------------
proc remove_overlapped_picks {} {
   # global --------------------
   global pm

   # main --------------------
   # Is there any picks?
   set pn [llength $pm]
   if { $pn < 1 } {
      tk_messageBox -icon warning -title "Warning"\
	  -message "There is no pick."
      return
   }
   # confirmation
   set ans [tk_messageBox -icon question -title "Remove Overlapped picks?"\
		-message "Remove overlapped picks.  All right?"\
		-type okcancel -default ok]
   if {$ans=="cancel"} return
   
   # Exec adjustment
   print_message\
       [format "Number of Picks is %d (before remove overlapped) --> " $pn]
   set ww [wait_window "now removing picks..."]

   set pm [lsort -index 0 -integer $pm]
   set i 0
   while {$i < [expr $pn -1] } {
      scan [lindex $pm $i] "%d%s%s%s%d" sn px pt acc type
      set j [expr $i + 1]
      while {$j < $pn} {
	 scan [lindex $pm $j] "%d%s%s%s%d" jsn jpx jpt jacc jtype
	 if {$sn == $jsn} {
	    if { $type == $jtype && $px == $jpx } {
	       exec_clear_pick $j
	       incr pn -1
	    } else {
	       incr j
	    }
	 } else {
	    break
	 }
      }
      set i $j
   }
   set mark_modification 1
   print_message [format "%d (after)\n" [llength $pm]]
   plot_pick_marks 0
   destroy $ww
}

# --------------------------------------------------------------------
# Sort pick marks
# --------------------------------------------------------------------
proc sort_picks {} {
   # global --------------------
   global pm
   
   # main --------------------
   set pm [pick sort $pm]
}

# --------------------------------------------------------------------
# Remap pick x-position by the trace number
# --------------------------------------------------------------------
proc remap_pick_xposition {} {
   # global --------------------
   global pm mark_modification wk

   # main --------------------
   set newpm {}
   foreach d $pm {
      scan $d "%d%s%s%s%d" sn px pt acc type
      set px [getrh shotpos $sn]
      lappend newpm "$sn $px $pt $acc $type"
   }
   set pm $newpm
   # replot
   plot_pick_marks 0
   set mark_modification 1
}

# --------------------------------------------------------------------
# Hide position window
# --------------------------------------------------------------------
proc hide_position_window {} {
   # global --------------------
   global wk

   # Main --------------------
   wm withdraw $wk(balloon,pos)
}

# --------------------------------------------------------------------
# Report pick info
# --------------------------------------------------------------------
proc report_pick_info {gx my rt PX PY} {
   # global --------------------
   global pm gbl paste
   # Main --------------------
   set number [pick nearest pm $gx $rt\
		   [expr double($gbl(mark_size))*($paste(right)-$paste(left))\
			/$paste(graph_width) ]]
   if {$number == -1} {
      print_message "No vicinity shot.\n"
      return
   } else {
      scan [lindex $pm $number] "%d%s%s%s%d" sn px pt acc type
      set sn [expr $sn % 100000]
      set py [coord cy2gt $my]
      
      set px [expr ( $gbl(x_axis,top) == 1 ) ? $gx : [expr $px + $gbl(obs,x)] ]
      print_message\
	  [format "Pick info:%7.3f km, %8.3f sec. (%8.3f sec.),   shot=%6d,   accuracy=%7.3f,   type=%2d\n"\
	       $px $py $pt $sn $acc $type]
      show_position_balloon\
	  [format "Pick:\nx=%.3fkm\nt=%.3fs (%.3fs)\nshot=%d\nacc=%.3f\ntype=%d"\
	       $px $py $pt $sn $acc $type] $PX $PY
   }
}
# --------------------------------------------------------------------
# Report position and closest shot number
# --------------------------------------------------------------------
proc report_position_shot {gx tt rt sn obsn RX RY} {
   # global --------------------
   global gbl
   # Main --------------------
   set x [expr ( $gbl(x_axis,top) == 1 ) ? $gx : [expr $gx + $gbl(obs,x)] ]
   print_message\
       [format\
	    "%10.5f km   %10.5f sec. (%10.5f sec.,  near shot %5d(OBS %d))\n"\
	    $x $rt $tt $sn $obsn]
   show_position_balloon\
       [format "x=%.3f (%.3f) \nt=%.3f(%.3f)\nnear shot %d (OBS%d)" \
	    $gx $x $rt $tt $sn $obsn] $RX $RY
}

# --------------------------------------------------------------------
# Exec Delete closest pick mark (t must be reduced)
# --------------------------------------------------------------------
proc delete_single_pick {x t} {
   # global ------------------------------
   global pm gbl paste mark_modification

   # main ------------------------------
   set n [pick nearest pm $x $t\
	      [expr double($gbl(mark_size))*($paste(right)-$paste(left))\
		   /$paste(graph_width) ]]
   if {$n == -1} {
      print_message "No shot around pick to DELETE.\n"
      return
   }
   scan [lindex $pm $n] "%d%g%g" sn px pt
   print_message\
       [format "A pick was deleted (shot %d, x=%9.3fkm, t=%9.4fs. (%9.4fs.)\n"\
	    $sn $px [coord tt2rt $px $pt] $pt]
   # erace mark from the memory
   undo_stack
   exec_clear_pick $n
   set mark_modification 1
   # replot
   plot_pick_marks 0
}

# --------------------------------------------------------------------
# toggle show picks
# --------------------------------------------------------------------
proc toggle_show_picks {} {
   # global --------------------
   global gbl
   
   # main --------------------
   set gbl(check,showpick) [expr $gbl(check,showpick) ? 0 : 1]
   plot_pick_marks 0
}
