# --------------------------------------------------------------------
# Make a taup pick
# --------------------------------------------------------------------
proc pick_taup_pick {pick_x pick_y} {
   # global --------------------
   global wk taup

   # Main --------------------
   # taup-coordinate (slow=apprent slowness=p)
   set slow [coord cx2gx [$wk(w,canvas,c) canvasx $pick_x]]
   set tau  [coord cy2gt [$wk(w,canvas,c) canvasy $pick_y]]
   if [coord inout2 $slow $tau] {
      return
   }
   # append taup data
   incr taup(no)
   lappend taup(slow) $slow
   lappend taup(tau) $tau
   # plot taup pick
   plot_single_taup $slow $tau
   tausum_inversion
}

# --------------------------------------------------------------------
# Plot a single taup pick
# [arguments]
# slow  : slowness (horizontal coordinate)
# tau   : intercept time (vertical coordinate)
# --------------------------------------------------------------------
proc plot_single_taup {slow tau} {
    # global --------------------
    global wk taup

    # main --------------------
    # convert graph-coodeinate into canvas coordinate
    set x [coord gx2cx $slow]
    set y [coord tt2cy $slow $tau]
    # draw a pick mark
    $wk(w,canvas,c) create arc\
	[expr $x - $taup(pick_size)] [expr $y - $taup(pick_size)]\
	[expr $x + $taup(pick_size)] [expr $y + $taup(pick_size)]\
	-start -1 -extent 359 -fill $taup(color) -outline $taup(color)\
	-tag taup
}

# --------------------------------------------------------------------
# Plot all the taup picks
# --------------------------------------------------------------------
proc plot_taup_picks {} {
    # global --------------------
    global taup

    # main --------------------
    for {set i 0} {$i < $taup(no)} {incr i} {
       plot_single_taup [lindex $taup(slow) $i] [lindex $taup(tau) $i]
    }
}

# --------------------------------------------------------------------
# Delete the closest taup pick
# --------------------------------------------------------------------
proc clear_nearest_taup {pick_x pick_y} {
   # global --------------------
   global wk taup

   # main --------------------
   set cx [$wk(w,canvas,c) canvasx $pick_x]
   set cy [$wk(w,canvas,c) canvasy $pick_y]
   set ind [nearest_taup $cx $cy]

   if {$ind == -1} {
      print_message "No taup pick to DELETE.\n"
      return
   } else {
      print_message\
	  [format "p=%10.5f sec/km, tau=%10.5f sec. tau-p pick was DELETED\n"\
	       [lindex $taup(slow) $ind] [lindex $taup(tau) $ind]]
      erase_all_taup_pick
      exec_delete_taup_pick $ind
      plot_taup_picks
      tausum_inversion
   }
}

# --------------------------------------------------------------------
# Delete single taup pick data
# [argument]
# i   :  index number to delete
# --------------------------------------------------------------------
proc exec_delete_taup_pick {i} {
   # global --------------------
   global taup
   
   # main --------------------
   set taup(slow) [lreplace $taup(slow) $i $i]
   set taup(tau) [lreplace $taup(tau) $i $i]
   set taup(no) [llength $taup(slow)]
}

# --------------------------------------------------------------------
# Show-off all the taup pick mark
# --------------------------------------------------------------------
proc erase_all_taup_pick {} {
   # global --------------------
   global wk
   
   # main --------------------
   $wk(w,canvas,c) delete taup
}

# --------------------------------------------------------------------
# return the taup pick number of the closest taup pick.
#     1. search for closest taup pick
#     2. return -1 if not sufficiently close, otherwise taup pick #
# [arugment]
# x     : horizontal canvas coordinate
# y     : vertical canvas coodinate
# --------------------------------------------------------------------
proc nearest_taup {x y} {
   # global --------------------
   global taup
   
   # main --------------------
   set ind -1
   set min 100000
   for {set i 0} {$i < $taup(no)} {incr i} {
      set cx [coord gx2cx [lindex $taup(slow) $i]]
      set cy [coord tt2cy [lindex $taup(slow) $i] [lindex $taup(tau) $i]]
      set dist [expr hypot($x - $cx,  $y - $cy)]
      if {$min > $dist} {
	 set ind $i
	 set min $dist
      }
   }
   # check min-dist
   if {$min < [expr $taup(pick_size) * 3.0]} {
      return $ind
   }
   return -1
}

