# --------------------------------------------------------------------
# stack undo info
# --------------------------------------------------------------------
# args : if anything is specified, the redopm will not be cleared.
# --------------------------------------------------------------------
proc undo_stack {args} {
   # global --------------------
   global pm undopm redopm gbl wk

   # main --------------------
   if {$gbl(option,maxundo) < 1} {
      return
   }
   # preserve undo info
   lappend undopm $pm
   # undo size adjustment
   if { [llength $undopm] > $gbl(option,maxundo) } {
      set undopm [lrange $undopm end-[expr $gbl(option,maxundo) - 1] end]
   }
   set wk(pickedit,undoremain) [llength $undopm]

   # clear redo
   if { [llength $args] == 0 } {
      set redopm {}
   }
}

# --------------------------------------------------------------------
# pop undo info
# --------------------------------------------------------------------
proc undo_pop {} {
   # global --------------------
   global pm undopm redopm gbl wk

   # main --------------------
   if { !$gbl(check,showpick) } {
      set gbl(check,showpick) 1
      print_message "WARNING: Cannot undo while not showing picks.\n"
      plot_pick_marks 0
      return
   }
   if { [llength $undopm] > 0 } {
      # update redo
      lappend redopm $pm
      # recover
      set pm [lindex $undopm end]
      # update undo info
      set undopm [lrange $undopm 0 end-1]
      # replot
      plot_pick_marks 0
      set mark_modification 1
      print_message "Undo pick.\n"
   } else {
      if { $gbl(option,maxundo) < 1} {
	 print_message "Undo is disabled.\n"
      } else {
	 print_message "No more undo info.\n"
      }
   }
   set wk(pickedit,undoremain) [llength $undopm]
}

# --------------------------------------------------------------------
# pop undo info
# --------------------------------------------------------------------
proc update_maxundo {} {
   # global --------------------
   global undopm gbl redopm

   # main --------------------
   if {$gbl(option,maxundo) < 1} {
      set undopm {}
      set redopm {}
      return
   }
   if {$gbl(option,maxundo) > [llength $undopm]} {
      set undopm [lrange $undopm end-[expr $gbl(option,maxundo) - 1] end]
   }
}

# --------------------------------------------------------------------
# pop redo info
# --------------------------------------------------------------------
proc redo_pop {} {
   # global --------------------
   global pm redopm gbl

   # main --------------------
   if { !$gbl(check,showpick) } {
      set gbl(check,showpick) 1
      print_message "WARNING: Cannot redo while not showing picks.\n"
      plot_pick_marks 0
      return
   }
   if { [llength $redopm] > 0 } {
      # preserve undo info
      undo_stack preserve_redopm
      # recover from redo info
      set pm [lindex $redopm end]
      # update redo info
      set redopm [lrange $redopm 0 end-1]
      # replot
      plot_pick_marks 0
      set mark_modification 1
      print_message "Redo pick.\n"
   } else {
      if { $gbl(option,maxundo) < 1} {
	 print_message "Undo/Redo is disabled.\n"
      } else {
	 print_message "No more redo info.\n"
      }
   }
}

