# --------------------------------------------------------------------
# Show velocity and register it
# --------------------------------------------------------------------
proc vel_line_result {x1 t1 x2 t2 RX RY} {
   # global
   global wk
   
   # velocity
   set dt [expr $t1 - $t2 ]
   if {$dt == 0.0} {
      print_message "ERROR: Infinite velocity.\n"
      return
   }
   set velocity [expr abs(  ($x1 - $x2) / $dt  )]

   # intercept
   set velocity [expr abs($velocity)]
   set intercept [expr $t1 - abs( $x1 / $velocity ) ]

   # show result
   print_message [format "%f km/sec. Slope.  Intercept Time : %f sec.\n"\
		      $velocity $intercept]
   show_position_balloon [format "Slope: %.3f km/sec\nIntercept: %.3f sec"\
			      $velocity $intercept] $RX $RY
   # save velocity data
   lappend wk(vline,data) [list $x1 $x2 $t1 $t2 $velocity $intercept]
   update_velocity_listbox
}

# --------------------------------------------------------------------
# erase the most recent velocity line
# --------------------------------------------------------------------
proc clear_last_vel_line {} {
   # global --------------------
   global wk

   # main --------------------
   set wk(vline,data) [lreplace $wk(vline,data) end end]
   draw_all_vline
   update_velocity_listbox
}
# --------------------------------------------------------------------
# Delete selected velocity line
# --------------------------------------------------------------------
proc delete_selected_vline {} {
   # global ------------------------------
   global wk

   # main ------------------------------
   # get selected items
   set sel [lsort -integer -decreasing [$wk(vline,listbox) curselection]]
   foreach n $sel {
      set wk(vline,data) [lreplace $wk(vline,data) $n $n]
   }
   update_velocity_listbox
   draw_all_vline
}

# --------------------------------------------------------------------
# Update velocity listbox in the window of Edit/Save velocity values
# --------------------------------------------------------------------
proc update_velocity_listbox {} {
   # global ------------------------------
   global wk

   # main ------------------------------
   # Does the widget exist?
   if ![winfo exists .analysis_window] {
      return
   }
   
   # clear
   $wk(vline,listbox) delete 0 end
   # show
   set i 1
   foreach d $wk(vline,data) {
      scan $d "%s %s %s %s %s %s" x1 x2 t1 t2 vel t
      $wk(vline,listbox)\
	  insert end [format "%4d %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f"\
			  $i $x1 $x2 $t1 $t2 $vel $t]
      incr i
   }
}

# -------------------------------------------------------------------
# Exec Save Velocity Value
#    Originally created by Kei Murase@KGE
# -------------------------------------------------------------------
proc exec_save_vline {} {
   # global --------------------
   global gbl wk
   
   # main --------------------
   # enough data?
   if { [llength $wk(vline,data)] < 1} {
      tk_messageBox -type "ok" -message "No data.  Save aborted."\
	  -title "Error" -icon warning
      return
   }
   
   # get filename
   set f_name [tk_getSaveFile\
		   -initialfile "default.vel" -title "Save Velocity"]
   if {$f_name==""} {
      tk_messageBox -type "ok" -message "Save canceled" -title "Canceled"
      return
   }

   # write to the file
   set fp [open $f_name w]
   foreach d $wk(vline,data) {
      scan $d "%s %s %s %s %s %s" x1 x2 t1 t2 vel t
      puts $fp [format "%10.4f %10.4f %10.4f %10.4f %10.4f %10.4f"\
		    $x1 $x2 $t1 $t2 $vel $t]
   }
   close $fp
   
   # clear velocity lines?
   set ans [tk_messageBox -icon question -type "yesno"\
		-message "Clear all the velocity lines?"\
		-title "Clear velocity lines"]

   if {$ans == "yes"} {
      set wk(vline,data) {}
      draw_all_vline
   }
}

# --------------------------------------------------------------------
# Sort velocity lines by "key"
# --------------------------------------------------------------------
proc vline_sort {key} {
   # global ------------------------------
   global wk

   # main ------------------------------
   if {$key == "x1"} {
      set wk(vline,data) [lsort -index 0 -real $wk(vline,data)]
   } elseif {$key == "intercept"} {
      set wk(vline,data) [lsort -index 5 -real $wk(vline,data)]
   } else {
      # invalid key
      return
   }
   # update
   update_velocity_listbox
}