# --------------------------------------------------------------------
# Invoke pick manipulation window
# --------------------------------------------------------------------
proc pick_window {} {
   # Global --------------------------------------------------
   global gbl mark_color macray file_name pick wk paste custom

   # Main --------------------------------------------------
   set pw .pick_window
   if [raise_window $pw] { return }
   toplevel $pw
   wm title $pw "PasteUp: Pick control window"

   # binding ------------------------------
   basic_bindings $pw

   # title frame --------------------
   set tf [title_frame $pw.title_frame "Pasteup: Pick Window"]

   # tabset frame --------------------
   set ts $pw.tabset
   tabset $ts -pagewidth 510 -relief flat
   $ts conf -borderwidth 1 -highlightthickness 0 -side top
   $ts insert end "file" -text "File"
   $ts insert end "synthe" -text "Synthe"
   $ts insert end "edit" -text "Edit"
   $ts insert end "mark" -text "Mark"
   $ts insert end "label" -text "Label"
   $ts insert end "auto" -text "Auto Pick"
   $ts insert end "amp2acc" -text "Amp2Acc"

   # --------------------------------------------------
   # pick file tab ------------------------------------

   set pff $ts.file_frame
   frame $pff
   $ts tab conf "file" -window $pff -fill x

   set ff [labelframe $pff.file -text "Pick file"]
   filename_hscroll_com $ff fn "Filename" file_name(pick) 45 load pick \
       "pick_file_select_and_load"
   bind $ff.entry(fn) <Button-4> {change_pick_file prev 1}
   bind $ff.entry(fn) <Button-5> {change_pick_file next 1}
   make_menu_com $ff fmt "Pick format" wk(pick,format) "update_pick_format" \
       wk(pick,format_name) Pasteup 0 Mochi 1 Rayinvr 2 GeoCTII 3
   set ffbf [button_frame "" $ff.button_frame \
		  "Merge" merge_pick \
		  "Save" save_pick \
		  "Save As" save_as_pick]
   grid $ffbf - - - - -sticky news

   # parameter frame -----
   set pf [labelframe $pff.parameters -text "OBS position"]
   
   # OBS position
   make_entry_right $pf obs_position "Horizontal position" gbl(obs,x)\
       "km  (shot range)"
   make_entry_right $pf obs_depth "Depth" gbl(obs,depth) "km"

   # button frame -----
   set bf [button_frame "" $pff.button_frame "Clear All Picks" forget_picks]

   # pack pick file tab -----
   grid $ff -sticky news -pady 5
   grid $pf -sticky news -pady 5 -ipady 10
   grid $bf -sticky news -pady 5

   # --------------------------------------------------
   # synthetic tab ------------------------------------

   set pf $ts.synthe_frame
   frame $pf
   $ts tab conf "synthe" -window $pf -fill x

   # file frame --------------------
   set fn $pf.filename
   labelframe $fn -text "Synthetic data file" -padx 15
   
   # Filename
   filename_hscroll_frame $fn dat ""\
       macray(filename) 50 load pick
   set fn_bf [button_frame "" $fn.buttons\
		  "Reload" Synthetic::reread "Toggle" Synthetic::toggle]
   grid $fn_bf - -sticky news

   # styles --------------------
   set sty $pf.sytles
   labelframe $sty -text "Synthetic plot styles"

   # plot style
   make_optionMenu_button $sty plotstyle "Plot style" macray(style)\
       "Symbol" "Line" "Line&Symbol"

   # synthetic color mode ----------
   set cm [labelframe $pf.color_mode -text "Synthetic color"]
   set cs [labelframe $pf.color_set -text "Color assignment"]

   make_radio_vertical_com $cm mode macray(color_mode)\
       "update_synthe_color_mode $cs"\
       "1 color" 0 "Auto colors" 1 "Manual colors" 2

   button $cs.button(1color) -text "1-color" -background $macray(color)\
       -activebackground $macray(color)\
       -command "setColor $pw $cs.button(1color) macray(color); plot_pick_marks 0"
   button $cs.button(manual) -text "Manual"\
       -command "synthetic_manual_color_setting"
   grid $cs.button(1color) -sticky news
   grid $cs.button(manual) -sticky news
   update_synthe_color_mode $cs

   # line & symbols --------------------
   set ls [labelframe $pf.line_style -text "Line style"]
   make_optionMenu_button $ls style "Style" macray(line_style)\
       solid dashed dotted
   make_hscale $ls synthe_lwidth "Width" macray(width) 0 3.0 0 0.1 ""
   set ss [labelframe $pf.symbol_style -text "Symbol style"]
   make_optionMenu_button $ss style "Mark" macray(symbol_style)\
       oval cross
   make_hscale $ss synthe_ssize "Size" macray(symbol_size) 0 5.0 0 0.1 ""

   # packing --------------------
   #pack $fn $sty $ls -side top -fill x -expand 1
   grid $fn - -sticky news -pady 3 -padx 5
   grid $cm $cs -sticky news -pady 3 -padx 5
   grid $sty x -sticky news -pady 3 -padx 5
   grid $ls $ss -sticky news -pady 3 -padx 5

   # --------------------------------------------------
   # mark tab -----------------------------------------
   set pf [frame $ts.mark_frame]
   $ts tab conf "mark" -window $pf -fill x

   # Minimum pick accuracy(uncertainty)
   set pft [labelframe $pf.type -text "Pick type and Accuracy"]
   label $pft.acc_label -text "Min/Default acc."
   set bak $pick(default_accuracy)
   set spinw [spinbox $pft.spin -width 7 -textvar pick(default_accuracy)\
		  -justify right -values $wk(accuracy,default_list)]
   $spinw set $bak
   label $pft.unit -text "sec" -anchor w
   grid $pft.acc_label $spinw $pft.unit -sticky news
   bind $spinw <Button-4> "$spinw invoke buttonup"
   bind $spinw <Button-5> "$spinw invoke buttondown"
   
   # pick type selection -----
   label $pft.type_label -text "Pick type" -anchor e
   picktype_selector_widget $pft ptype pick(current_type) pick(range)
   grid $pft.type_label $pft.mb(ptype) $pft.sb(ptype) -sticky news

   # mark style and size -----
   set pfs [labelframe $pf.style -text "Looks"]
   make_optionMenu_com $pfs style "Style" paste(mark,style) update_mark_style\
       splus plus cross diamond circle square
   make_spinbox_com $pfs pick_mark_size "Size" paste(mark,size) 1 75 1 ""\
       update_mark_style
   make_check_label $pfs "Show Uncertainty" Uncertainty\
       paste(mark,uncertainty) "plot_pick_marks 0"

   # packing
   grid $pft -sticky news -ipadx 30 -ipady 5 -pady 10
   grid $pfs -sticky news -ipadx 30 -ipady 5 -pady 10

   # --------------------------------------------------
   # pick editor tab ----------------------------------

   set pf $ts.edit_frame
   frame $pf
   $ts tab conf "edit" -window $pf -fill x

   # undo --------------------
   set uf [labelframe $pf.undo -text "Undo/Redo of pick edit" -pady 5]
   button $uf.undo -command undo_pop -text "undo"
   button $uf.redo -command redo_pop -text "redo"
   grid $uf.undo $uf.redo
   # pack $uf.undo $uf.redo -side top

   set uc [labelframe $pf.undo_config -text "Number of Undo"]
   make_optionMenu_com $uc max  "Maximum undo" gbl(option,maxundo)\
       update_maxundo 0 5 20 100
   make_label $uc remain "Remaining undo" wk(pickedit,undoremain)
   
   # change pick type --------------------
   set cpt $pf.change_pick_type
   labelframe $cpt -text "CONVERT pick type"
   label $cpt.lfrom -text "    From:"
   picktype_selector_widget $cpt from wk(pickedit,from) wk(pickedit,from,range)
   label $cpt.linto -text   "            into:"
   picktype_selector_widget $cpt into wk(pickedit,into) wk(pickedit,into,range)
   button $cpt.exe -text "convert" -command "pickedit_type_conversion"
   grid $cpt.lfrom $cpt.mb(from) $cpt.sb(from)\
       $cpt.linto $cpt.mb(into) $cpt.sb(into)\
       $cpt.exe -sticky news

   # delete selected pick type --------------------
   set ds [labelframe $pf.del_select -text "DELETE selected pick type"]
   picktype_selector_widget $ds select wk(pickedit,del) wk(pickedit,del,range)
   button $ds.exe -text "DEL" -command "pickedit_delete_one_type"
   grid $ds.mb(select) $ds.sb(select) $ds.exe -sticky news

   # overlapped picks --------------------
   set do [labelframe $pf.del_overlap -text "DELETE picks"]
   make_command_button $do pick_rm "Overlapped" "DEL"\
       remove_overlapped_picks

   # sort pick --------------------
   set sp $pf.sort_pick
   set sp [labelframe $pf.sort_pick -text "SORT pick mark"]
   make_command_button $sp pick_sort "By offset and type" "SORT" sort_picks

   # Adjust picks--------------------
   set app $pf.adjust_pick
   labelframe $app -text "ADJUST x-position of all picks"
   make_command_button $app nearest "nearest trace" "ADJUST" adjust_picks
   make_command_button $app sn "shot number" "ADJUST" remap_pick_xposition

   grid $uf $uc -sticky news -pady 3 -padx 5
   grid $cpt - -sticky news -pady 3 -padx 5 -ipady 10
   grid $ds $do -sticky news -pady 3 -padx 5 -ipady 10
   grid $sp $app -sticky news -pady 3 -padx 5 -ipady 5

   # --------------------------------------------------
   # auto pick tab ------------------------------------

   set pf $ts.auto_frame
   frame $pf
   $ts tab conf "auto" -window $pf -fill x

   # algorithm tabset --------------------
   set ats $pf.algorithm_tabset
   tabset $ats -pagewidth 480 -relief flat
   $ats conf -borderwidth 1  -highlightthickness 0 -side top
   $ats insert end "armodel" -text "AR model"
   $ats insert end "correlation" -text "Correlation"

   # AR model --------------------
   set ar $ats.armodel
   frame $ar
   $ats tab conf "armodel" -window $ar -fill x

   # range
   label $ar.label(range) -text "Pick Range(km)"
   entry $ar.entry(min) -width 12 -textvar gbl(pos,min) -justify right
   label $ar.label(to) -text "to"
   entry $ar.entry(max) -width 12 -textvar gbl(pos,max) -justify right
   grid $ar.label(range) $ar.entry(min) $ar.label(to) $ar.entry(max)\
       -sticky news
   # AR-model order
   make_entry_right $ar ar "Max AR-model" gbl(pos,maxorder)\
       "samples"
   make_on_off $ar "Apply SU-filter before autopick" applyfilter\
       gbl(corpick,applyfilter)

   # separator
   label $ar.separator -text ""
   grid $ar.separator -pady 5

   # button
   button $ar.apick -text "AutoPick" -width 12 -command "auto_pick_AR"
   grid $ar.apick - - - -sticky ns

   # correlation --------------------
   set cor $ats.correlation
   frame $cor
   $ats tab conf "correlation" -window $cor -fill x

   # range
   label $cor.label(range) -text "Pick Range(km)"
   entry $cor.entry(min) -width 12 -textvar gbl(pos,min) -justify right
   label $cor.label(to) -text "to"
   entry $cor.entry(max) -width 12 -textvar gbl(pos,max) -justify right
   grid $cor.label(range) $cor.entry(min) $cor.label(to) $cor.entry(max)\
       -sticky news

   # entry widgets / radio buttons
   make_entry_right $cor\
       srcwin "Source signature length" gbl(corpick,srcwin) "second"
   make_entry_right $cor\
       redvel "Reduction velocity" gbl(corpick,redvel) "km/sec"
   make_entry_right $cor\
       winbeg "Beginning of temporal window" gbl(corpick,winbeg) "second"
   make_entry_right $cor\
       winsz "Length of temporal window" gbl(corpick,winsz) "second"
   make_on_off $cor "Calculating water wave arrivals" wtrsw gbl(corpick,wtrsw)
   make_entry_right $cor water_depth "Water Depth" gbl(obs,depth) "(km)"
   make_on_off $cor "Apply SU-filter before autopick" applyfilter\
       gbl(check,sufilter)

   # separator
   label $cor.separator -text ""
   grid $cor.separator -pady 5

   # button
   button $cor.apick -text "AutoPick" -width 12 -command "auto_pick_cor"
   grid $cor.apick - - - -sticky ns

   # pack pick file tab --------------------
   pack $ats -side top

   # --------------------------------------------------
   # pick amp2acc tab -------------------------------

   set pf $ts.amp2acc
   frame $pf
   $ts tab conf "amp2acc" -window $pf -fill x

   # mode ----------
   make_many_radio $pf "S/N determined by" mode paste(amp2acc,mode)\
       "Average amp." 0 "Max-amp." 1

   # period ----------
   make_entry_right $pf tbefore "Before" paste(amp2acc,tbefore) "second"
   make_entry_right $pf tafter "After" paste(amp2acc,tafter) "second"

   # s/n array ----------
   make_long_entry $pf snarray "Array of S/N" paste(amp2acc,sn)

   # error array ----------
   make_long_entry $pf errarray "Array of Accuracy(sec)" paste(amp2acc,acc)

   # exec button ----------
   make_command_button $pf exec "Amp to Accuracy"\
       "apply" amp2acc

   # --------------------------------------------------
   # pick label tab -----------------------------------

   set pl [frame $ts.pick_label]
   $ts tab conf "label" -window $pl -fill x

   # font frame ----------
   set font [labelframe $pl.font -text "Label font"]
   
   # font
   PSFontSelect::make_widget $font font "Font" paste(plabel,font)
   # position(anchor)
   make_optionMenu_button $font anchor "Position(anchor)"\
       paste(plabel,anchor) n s
   
   # shadow
   make_spinbox $font shadow "Shadow gap" paste(plabel,shadow) 0 5 1 pixel
   make_color_button $font color "Shadow color" "" $paste(plabel,shadow_color)\
       paste(plabel,shadow_color)\
       "setColor $pl $font.button(color) paste(plabel,shadow_color)"

   # criteria ----------
   set cf [labelframe $pl.criteria_frame -text "Criteria"]
   make_spinbox $cf separator "Group separation"\
       paste(plabel,separator) 3 20 1 "traces"
   make_spinbox $cf minimum "Minimum group size"\
       paste(plabel,groupsize) 3 50 1 "traces"

   # buttons ----------
   set bf [button_frame "" $pl.buttons\
	       Plot pick_label_plot Erase pick_label_erase]

   # placing ----------
   grid $font -sticky news -pady 5 -ipadx 40 -ipady 10
   grid $cf -sticky news -pady 5 -ipady 10
   grid $bf - -sticky w -pady 5

   # --------------------------------------------------
   # button frame -------------------------------------
   set bf [button_frame $pw $pw.button_frame]

   # packing --------------------
   pack $tf $ts $bf -side top -fill x
}

# --------------------------------------------------------------------
# Make pick type menu
# --------------------------------------------------------------------
# [arguments]
# w     : widget name of menubutton
# start : make pick type menu from $start to $star+$pick(n_color)-1
# var   : variable name of pick type
# --------------------------------------------------------------------
proc mk_pick_type_menu {w start var} {
   # global ----------
   upvar $var current_type
   global pick
   
   # Main ------------------------------
   # update menu button 
   set c [expr ($current_type - 1) % $pick(n_color) + 1]
   $w configure -bg $pick(color,$c) -activebackground $pick(color,$c)
   # delete old menu, if exists
   $w.menu delete 0 end
   # add menus 
   if { [regexp "^wk" $var] } {
      # var is "wk(....)"
      for {set i $start} {$i < [expr $start + $pick(n_color)]} {incr i} {
	 set c [expr ($i-1) % $pick(n_color) + 1]
	 $w.menu add radiobutton -value $i -label [format "%3d" $i]\
	     -variable $var\
	     -background $pick(color,$c) -activebackground $pick(color,$c)\
	     -command "exec_update_pick_type_menu_color $w $c"
      }
   } else {
      # var is "pick(...)"
      for {set i $start} {$i < [expr $start + $pick(n_color)]} {incr i} {
	 set c [expr ($i-1) % $pick(n_color) + 1]
	 $w.menu add radiobutton -value $i -label [format "%3d" $i]\
	     -variable $var\
	     -background $pick(color,$c) -activebackground $pick(color,$c)\
	     -command "update_pick_type_menu_color $c"
      }
   }

   # add menus
   $w.menu entryconfigure 10 -columnbreak 1
}

# --------------------------------------------------------------------
# Manual color setting window for synthetic traveltime data
#    Originally created by Kei Murase@KGE
# --------------------------------------------------------------------
proc synthetic_manual_color_setting {} {
   # global --------------------
   global macray wk custom
   
   # main --------------------
   set wc .pick_window.color
   set wcf $wc.para_frame
   set wcb $wc.button

   # Are there any pick marks? ----------
   if { $macray(n) < 1 } {
      catch {destroy $wc}
      tk_messageBox -type ok -icon warning\
	  -message "There is no synthetic mark"
      return
   }

   # invoke window ----------
   if [raise_window $wc] {
      catch { destroy $wcf $wcb }
      pack forget $wc.title
   } else {
      # create new window
      toplevel $wc
      wm title $wc "PasteUp: Synthetic Colors"
      title_frame $wc.title "Synthetic Colors"
   }

   # Count up pick mark types ----------
   set typelist {}
   for {set i 0} {$i < $macray(n)} {incr i} {
      if { [lsearch $typelist $macray(m,$i)] < 0} {
	 lappend typelist $macray(m,$i)
      }
   }
   set typelist [lsort -integer $typelist]
   
   # Parameter frame ----------------------------
   labelframe $wcf -padx 10 -pady 10
   # make label
   set wlist $wcf.label
   label $wcf.label -text "Type" -relief flat
   foreach type $typelist {
      label $wcf.l($type) -text $type -relief flat
      lappend wlist $wcf.l($type)
   }
   eval grid $wlist -sticky news
   # make buttons
   set wlist $wcf.button
   label $wcf.button -text "Color" -relief flat
   foreach type $typelist {
      if { [array names macray mcolor,$type] == "" } {
	 # initialize color
	 set macray(mcolor,$type) "\#00ff00"
      }
      button $wcf.cb($type) -text $type -bg $macray(mcolor,$type)\
	  -command "setColor $wcf $wcf.cb($type) macray(mcolor,$type)"
      lappend wlist $wcf.cb($type)
   }
   eval grid $wlist -sticky news

   # button frame ------------------------------
   button_frame $wc $wcb \
       "Reset" "array unset macray mcolor,* ; synthetic_manual_color_setting"\
       "Apply" "Synthetic::toggle on"

   # packing
   pack $wc.title -side top -fill x
   pack $wcf -side top -fill x -pady 10
   pack $wcb -side top -fill x
   update
}

# --------------------------------------------------------------------
# Wheel-bindings for pick type change
# --------------------------------------------------------------------
# [argument]
# w    :  widget name of menu button
# direc:  direction of change (next or prev)
# var  :  variable name
# --------------------------------------------------------------------
proc wheel_picktype_change {w direc var} {
   # global ------------------------------
   upvar $var current_type
   global gbl pick wk

   # main ------------------------------
   set i [expr ( $current_type -1 ) % 20]
   switch $direc {
      prev {
	 # previous menu
	 if { $i < 1 } { return }
	 incr current_type -1
      }
      next {
	 # next menu
	 if { $i >= 19 } { return }
	 incr current_type
      }
   }
   # invoke the menu
   $w.menu invoke [expr ( $current_type -1 ) % 20]
   
   set $var $current_type
}

# --------------------------------------------------------------------
# picktype_selector_widget -------------------------------------------
# --------------------------------------------------------------------
# [arguments]
#    w     : parent widget name
#    name  : id inside program (for grid manager)
#    var   : variable name for pick type
#    range : variable name for picktype range
# --------------------------------------------------------------------
# generate two widgets: $w.mb($name) and $w.sb($name)
# --------------------------------------------------------------------
proc picktype_selector_widget {w name var range} {
   # variables --------------------------------------------------
   upvar $var typevar
   upvar $range typerange
   global pick wk

   # main --------------------------------------------------
   # pick type selection menu
   set mb $w.mb($name)
   menubutton $mb -relief raised -bd 2 -width 2 -menu $mb.menu -textvar $var
   menu $mb.menu -tearoff 0
   mk_pick_type_menu $mb [expr 1 + $typerange * $pick(n_color)] $var
   bind $mb <Button-4> "wheel_picktype_change $mb prev $var"
   bind $mb <Button-5> "wheel_picktype_change $mb next $var"

   # range label var name
   if {$range == "pick(range)"} {
      set labelvar pick(range,name)
   } else {
      set labelvar wk(pickedit,$name,range,name)
   }

   # pick type range selection spin box
   set sb $w.sb($name)
   spinbox $sb -values $pick(range,list) -state readonly -relief flat\
       -textvar $labelvar -width 10 -justify right\
       -command "update_pickrange $mb $var $range $labelvar"
   bind $sb <Button-4> "$sb invoke buttondown"
   bind $sb <Button-5> "$sb invoke buttonup"

   # set pick range properly
   set $labelvar [lindex [$sb cget -values] $typerange]

   update_pickrange $mb $var $range $labelvar
}
