# --------------------------------------------------------------------
# auto pick by AR modeling
# --------------------------------------------------------------------
proc auto_pick_AR {} {
   # global --------------------------------------------------
   global gbl wk file_name mark_modification TMPDIR pick

   # main --------------------------------------------------
   
   # many many checks --------------------
   # SU data?
   if {$gbl(data,type) != 2 && $gbl(data,type) < 5} {
      tk_messageBox -message "Auto pick is available only with SU/SEG-Y data."\
	  -type ok -icon warning -parent .
      return
   }
   # CWPROOT?
   if { [check_su_mode] == 0 } {return}
   # have the suautopickar ?
   if { [search_external_command suautopickar] == 0 } {
      tk_messageBox -message "\"suautopickar\" command is required to perform auto-picking by AR model fitting."\
	  -type ok -icon warning -parent .
      return
   }
   
   # input data is readable?
   if { ![file readable $file_name(wave)] } {
      tk_messageBox -message "Cannot read wave-data file."\
	  -type ok -icon error -parent .
      return
   }

   # Already have picks?
   if { [forget_picks] == "cancel" } {
      return
   }

   # Apply auto-pick --------------------

   # apply filter?
   set wavefile $file_name(wave)
   if $gbl(corpick,applyfilter) {
      set wavefile [format "%s/pasteup.sufiltered.%s" $TMPDIR [pid]]
      set com [SUfilter::command]
      if {$com != ""} {
	 set com [make_su_command 0 $com $file_name(wave) $wavefile "" ""]
	 set status [exec_external_command $com "SUFILTER"]
	 if $status {
	    # some error occurred
	    ptus stderr $com
	    file delete -force $wavefile
	    tk_messageBox -icon error -type ok\
		-message "Error occurred during applying SU fileter."
	    return
	 }
      } else {
	 tk_messageBox -icon error -type ok -message "Invalid SU filter."
	 return
      }
   }

   # make pick file
   set tmp [format "%s/pasteup.autopick.%s" $TMPDIR [pid]]
   set com [make_su_command 0\
		[format "| suautopickar maxorder=%d minoffset=%d maxoffset=%d\
                           uncertainty=%f"\
		     $gbl(pos,maxorder) [expr int( $gbl(pos,min)*1000.0)]\
		     [expr int( $gbl(pos,max)*1000.0)]\
		     $pick(default_accuracy)]\
		$wavefile $tmp "" ""]
   set stat [exec_external_command $com "SUAUTOPICKAR"]

   if $stat {
      file delete -force $tmp
      return
   }

   if $gbl(corpick,applyfilter) {
      file delete -force wavefile
   }

   # read it
   read_pasteup_pick $tmp

   # check pick
   check_waterwave_pick

   # remove it
   file delete -force $tmp

   # plot picks
   plot_pick_marks 0
}

# --------------------------------------------------------------------
# position fix by traveltme picks
# --------------------------------------------------------------------
proc position_fix {} {
   # global --------------------------------------------------
   global gbl pm TMPDIR wk file_name

   # main --------------------------------------------------
   # many many checks --------------------
   # SU data?
   if {$gbl(data,type) != 2 && $gbl(data,type) < 5} {
      tk_messageBox -message "Position-fix is available only with SU data."\
	  -type ok -icon warning -parent .
      return
   }
   # CWPROOT?
   if { [check_su_mode] == 0 } {return}
   # have the supick2loc ?
   if { [search_external_command supick2loc] == 0 } {
      tk_messageBox -message "\"supick2loc\" command is required to fix OBS position."\
	  -type ok -icon warning -parent .
      return
   }
   # have the obslocinvgun ?
   if { [search_external_command obslocinvgun3] == 0 } {
      tk_messageBox -message "\"obslocinvgun3\" command is required to fix OBS position using traveltime picks by the inversion."\
	  -type ok -icon warning -parent .
      return
   }
   # input data is readable?
   if { ![file readable $file_name(wave)] } {
      tk_messageBox -message "Cannot read wave-data file."\
	  -type ok -icon error -parent .
      return
   }

   # generate pick file for position fix --------------------
   set tmp [format "%s/pasteup.picks.%s" $TMPDIR [pid]]
   set fp [open $tmp "w"]
   puts $fp "0.0 0.0 0.0 1.0"
   set n 0
   foreach d $pm {
      scan $d "%d%s%s%s%d" sn px pt acc type
      if { $px >= $gbl(pos,min) && $px <= $gbl(pos,max) } {
	 puts $fp [format "%5d %.5f %.5f %.5f" $sn $px $pt $acc]
	 incr n
      }
   }
   # check the number of picks in the range
   close $fp
   if {$n < 4} {
      file delete $tmp
      tk_messageBox -icon error -type ok \
	  -message "To fix OBS position, at least 4 picks are required."
      return
   }
   # generate parameter file --------------------
   set tmp2 [format "%s/pasteup.obslocinvgun.%s" $TMPDIR [pid]]
   if { $gbl(data,type) == 2 } {
      # SU
      set com [format "supick2loc pfile=%s wv=%f dv=%f dx=%f dz=%f dt=%f\
                       limit=%f < %s > %s"\
		   $tmp $gbl(water_v) $gbl(pos,dw) $gbl(pos,dh) $gbl(pos,dd)\
		   $gbl(pos,dt) $gbl(pos,limit)  $file_name(wave) $tmp2]
   } else {
      # SEGY
      set com [format "%s tape=%s endian=%d hfile=/dev/null bfile=/dev/null\
                       | supick2loc pfile=%s wv=%f dv=%f dx=%f limit=%f > %s"\
		   $wk(SU,SEGY2SU) $file_name(wave) $gbl(su,endian)\
		   $tmp $gbl(water_v) $gbl(pos,dw) $gbl(pos,dh)\
		   $gbl(pos,limit) $tmp2]
   }
   set stat [exec_external_command $com "SUPICK2LOC"]
   file delete -force $tmp
   
   # inversion --------------------
   set com [open "| obslocinvgun3 $tmp2 silent"]
   gets $com result
   catch {close $com}
   file delete -force $tmp2

   # check result
   if { $result == "" } {
      tk_messageBox -message "Error: Probably invalid header info."\
	  -type ok -icon error
      return
   }

   # dialog --------------------
   position_dialog $result
}

# --------------------------------------------------------------------
# position dialog
# --------------------------------------------------------------------
proc position_dialog {result} {
   # global --------------------------------------------------
   global gbl wk custom

   # main --------------------------------------------------
   # get results
   scan $result "%s%s%s%s%s%s%s%s%s%s%s%s"\
       new_lon new_lat new_dep newv stc lon lat dep orgv orgstc dist azimuth
   set new_dep [expr int($new_dep)]
   set dep [expr int($dep)]
   
   # show message
   print_message [format "(obslocinv) : orig=(%s,%s,%smeter)\n"\
		      $lon $lat $dep]
   print_message [format "            :  new=(%s,%s,%smeter) vel=%skm/s stc=%ssec shift=%smeter azimuth=%sdegrees\n"\
		      $new_lon $new_lat $new_dep $newv $stc $dist $azimuth]

   # invoke window
   set w .position_window
   catch {destroy $w}
   toplevel $w
   wm title $w "PasteUp: Postion Fix"

   # title frame --------------------
   set tf $w.title_frame
   label $tf -text "Pasteup: Position Fix -- result --" -relief groove -bd 2\
       -font "$wk(font,title) 18 bold"
   
   # position frame --------------------
   set pf $w.position_frame
   frame $pf
   label $pf.leg_lon -text "longitude"\
       -relief groove -bd 1 -bg lavender -width 20
   label $pf.leg_lat -text "latitude"\
       -relief groove -bd 1 -bg lavender -width 20
   label $pf.leg_dep -text " depth(m) " -relief groove -bd 1 -bg lavender
   label $pf.leg_vel -text " velocity(km/s) " -relief groove -bd 1 -bg lavender
   label $pf.label -text "   orig:" -relief groove -bd 1 -bg lavender
   label $pf.lon -text $lon -relief groove -bd 1
   label $pf.lat -text $lat -relief groove -bd 1
   label $pf.dep -text $dep -relief groove -bd 1
   label $pf.vel -text $orgv -relief groove -bd 1
   label $pf.new_label -text "    new:" -relief groove -bd 1 -bg lavender
   label $pf.new_lon -text $new_lon -relief groove -bd 1
   label $pf.new_lat -text $new_lat -relief groove -bd 1
   label $pf.new_dep -text $new_dep -relief groove -bd 1
   label $pf.new_vel -text $newv -relief groove -bd 1
   grid x $pf.leg_lon $pf.leg_lat $pf.leg_dep $pf.leg_vel -sticky news
   grid $pf.label $pf.lon $pf.lat $pf.dep $pf.vel -sticky news
   grid $pf.new_label $pf.new_lon $pf.new_lat $pf.new_dep $pf.new_vel\
       -sticky news
   
   # separator
   label $pf.sep -text ""
   grid $pf.sep

   # static correction/shift/azimuth
   label $pf.clabel -text "Static correction"\
       -relief groove -bd 1 -bg ivory -width 20
   label $pf.stc -text $stc -relief groove -bd 1 -width 20 -anchor e
   label $pf.cunit -text "(second)" -anchor w
   label $pf.slabel -text "Horizontal Shift"\
       -relief groove -bd 1 -bg ivory -width 20
   label $pf.shift -text $dist -relief groove -bd 1 -width 20 -anchor e
   label $pf.sunit -text "(meter)" -anchor w
   label $pf.alabel -text "Azimuth"\
       -relief groove -bd 1 -bg ivory -width 20
   label $pf.azimuth -text $azimuth -relief groove -bd 1 -width 20 -anchor e
   label $pf.aunit -text "(degree)" -anchor w
   grid x $pf.clabel $pf.stc $pf.cunit -sticky news
   grid x $pf.slabel $pf.shift $pf.sunit -sticky news
   grid x $pf.alabel $pf.azimuth $pf.aunit -sticky news

   # button frame --------------------
   set bf $w.button_frame
   frame $bf -relief groove -border 2
   button $bf.accept -text "Apply"\
       -command "su_update_position $new_lon $new_lat $new_dep $stc $newv $w"
   button $bf.dismiss -text " not apply " -command "destroy $w" -width 12
   grid $bf.accept $bf.dismiss -sticky news

   # pack 
   pack $tf -side top -fill x
   pack $pf -side top -fill x -pady 15
   pack $bf -side top -fill x

   grab $w
}

# --------------------------------------------------------------------
# Check waterwave picks
# --------------------------------------------------------------------
proc check_waterwave_pick {} {
   # global --------------------
   global pm gbl
   
   # main --------------------
   set z [expr $gbl(obs,depth)]
   set i 0
   foreach d $pm {
      scan $d "%d%s%s%s%d" sn px pt acc type
      set estimate [expr hypot($px , $gbl(obs,depth) ) / $gbl(water_v)]
      set diff [expr abs( $pt - $estimate )]
      if { $diff > 1.0 } {
	 exec_clear_pick $i
      } else {
	 incr i
      }
   }
}

# --------------------------------------------------------------------
# auto pick by cross-correlation
# --------------------------------------------------------------------
proc auto_pick_cor {} {
   # global --------------------------------------------------
   global gbl wk file_name mark_modification TMPDIR pick pm

   # main --------------------------------------------------
   
   # many many checks --------------------
   # SU data?
   if {$gbl(data,type) != 2 && $gbl(data,type) < 5} {
      tk_messageBox -message "Auto pick is available only with SU data."\
	  -type ok -icon warning -parent .
      return
   }
   # CWPROOT?
   if { [check_su_mode] == 0 } {return}
   # have the sucorpick ?
   if { [search_external_command suautopickar] == 0 } {
      tk_messageBox -message "\"sucorpick\" command is required to perform auto-picking by cross-correlation."\
	  -type ok -icon warning -parent .
      return
   }
   
   # input data is readable?
   if { ![file readable $file_name(wave)] } {
      tk_messageBox -message "Cannot read wave-data file."\
	  -type ok -icon error -parent .
      return
   }

   # have a pick ?
   if { [llength $pm] < 1 } {
      tk_messageBox -type ok -icon error\
	  -message "At least 1 pick mark is required for applying cross-correlation pick."
      return
   }

   # Apply auto-pick --------------------

   # apply filter?
   set wavefile $file_name(wave)
   if $gbl(corpick,applyfilter) {
      set wavefile [format "%s/pasteup.sufiltered.%s" $TMPDIR [pid]]
      set com [SUfilter::command]
      if {$com != ""} {
	 set com [make_su_command 0 $com $file_name(wave) $wavefile "" ""]
	 set status [exec_external_command $com "SUFILTER"]
	 if $status {
	    # some error occurred
	    ptus stderr $com
	    file delete -force $wavefile
	    tk_messageBox -icon error -type ok\
		-message "Error occurred during applying SU fileter."
	    return
	 }
      } else {
	 tk_messageBox -icon error -type ok -message "Invalid SU filter."
	 return
      }
   }

   # save pick data to a temporary file
   undo_stack
   sort_picks
   set tmp_before [format "%s/pasteup.corpick-before.%s" $TMPDIR [pid]]
   write_pasteup_pick $tmp_before

   # make pick file
   set tmp_after [format "%s/pasteup.cor-pick-after.%s" $TMPDIR [pid]]
   set com\
       [make_su_command\
	    [format "| sucorpick pickfile=%s xmin=%f xmax=%f srcwin=%f\
                           redvel=%f winbeg=%f winsz=%f wtrsw=%d wtrdpth=%f"\
		 $tmp_before $gbl(pos,min) $gbl(pos,max) $gbl(corpick,srcwin)\
		 $gbl(corpick,redvel) $gbl(corpick,winbeg)\
		 $gbl(corpick,winsz) $gbl(corpick,wtrsw) $gbl(obs,depth)]\
	    $wavefile $tmp_after]
   set stat [exec_external_command $com "SUCORPICK"]
   
   if $gbl(corpick,applyfilter) {
      file delete -force wavefile
   }

   file delete -force $tmp_before
   if $stat {
      file delete -force $tmp_after
      return
   }

   # read
   read_pasteup_pick $tmp_after

   # remove
   file delete -force $tmp_after

   # plot picks
   plot_pick_marks 0
}
