proc print_pasteup {} {
   # global --------------------
   global print p gbl wk custom PREF

   # Main --------------------
   set pw .print_window
   if [raise_window $pw] { return }
   toplevel $pw
   wm title $pw "PasteUp:Print Window"
   
   # binding ------------------------------
   basic_bindings $pw

   # title frame --------------------
   set tf [title_frame $pw.title_frame "Pasteup: Print Window"]

   # ------------------------------------------------------------
   # Send PostScript to printer / file --------------------------
   
   # to printer --------------------
   set w1 [radio_labelframe $pw print\
	       "Print to a PostScript printer ($PREF(print,system))"\
	       print(file_or_printer) printer]
   if { $PREF(print,system) == "CUPS" } {
      set printers [print_listup_printer]
      eval make_optionMenu_button $w1 printer "Printer" print(printer) $printers
      if { [lsearch $printers $print(printer)] < 0 } {
	 set print(printer) [lindex $printers 0]
      }
   } else {
      make_entry $w1 printer "Printer" print(printer) ""
   }
   
   # to file --------------------
   set w2 [radio_labelframe $pw file "Print to a file"\
	       print(file_or_printer) file]
   # filename
   filename_hscroll_frame $w2 fname "Filename" file_name(postscript) 40 save ps
   # format
   make_optionMenu_com $w2 fmt "File format" print(format)\
       "update_print_format $w2" PostScript PNG JPEG
   # Image density
   make_entry_right $w2 density "Resolution" print(density) "(dpi)"

   update_radio_frames $pw print(file_or_printer)
   update_print_format $w2

   # ------------------------------------------------------------
   # options ----------------------------------------------------

   set pf [labelframe $pw.parameter_frame -text "Options"]

   # color mode
   make_optionMenu_button $pf color_mode "Color mode" print(mode)\
       color gray mono

   # paper size
   make_optionMenu_button $pf paper_size "Paper size" print(size)\
       A4 A3 A2 A1 B5

   # normalize by t or distance
   make_menu_com $pf normalize "Normalized by" print(normalize) ""\
       "X(distance)" h "T(depth)" t
   # paper orientation
   make_menu_com $pf orientation "Paper Orientation" print(orientation) ""\
       Landscape Landscape Portrait Portrait

   # Button frame ------------------------------
   set b [button_frame $pw $pw.button_frame "Print" exec_print_pasteup]

   # Pack ----------
   pack $tf -side top -fill x -expand 1
   pack $w1 $w2 $pf -side top -fill x -expand 1 -padx 10
   pack $b -side top -fill x -expand 1
}

proc print_listup_printer {} {
   # check lpstat command
   if { [catch "exec lpstat"] } {
      tk_messageBox -type warning -type ok -icon warning \
	  -message "Cannot execute \"lpstat\" command of CUPS.  You can't send data to your printer directly."
      return {dummy}
   }
   # listup printer
   set printers {}
   set com [open "| lpstat -v"]
   while { [gets $com line] >= 0} {
      lappend printers [regsub \:\$ [lindex $line 2] ""]
   }
   catch {close $com}
   return $printers
}

proc exec_print_pasteup {} {
   global print
   if {$print(file_or_printer) == "file"} {
      print_to_file
   } else {
      print_directly
   }
}

proc print_directly {} {
   # global
   global print PREF

   # main
   set com [make_print_command ""]
   if { $PREF(print,system) == "CUPS" } {
      set lpr [open "| lp -d $print(printer)" w]
   } else {
      set lpr [open "| lpr -P $printer" w]
   }
   puts $lpr [eval $com]
   close $lpr
   print_message "Pasteup Image was sent to $print(printer).\n"
}

proc print_to_file {} {
   # global
   global file_name fontmap TMPDIR print
   
   # main ------------------------------
   if { [file exists $file_name(postscript)] } {
      set ans [tk_messageBox -type yesnocancel -icon question\
		   -parent . -default cancel\
		   -message "\"$file_name(postscript)\" already exists.  Overwirte it?"]
      if { $ans == "cancel" } {return}
      if { $ans == "no" } {
	 set file_name(postscript) ""
      }
   }

   if {$file_name(postscript) == ""} {
      file_selection file_name(postscript) save $print(format)
      if {$file_name(postscript) == ""} { return }
   }

   if { $print(format) == "PostScript" } {
      # PostScript
      exec_print_to_file $file_name(postscript)
   } else {
      # Image(PNG/JPEG)
      set psfile [format "%s/pasteup.ps.%s" $TMPDIR [pid]]
      exec_print_to_file $psfile
      print_ps2image $print(format) $psfile $file_name(postscript)
      file delete $psfile
   }
   # Output message
   print_message "Pasteup Image was saved in $file_name(postscript).\n"
}

proc exec_print_to_file {fn} {
   if {$fn == ""} { return }
   eval [make_print_command $fn]
}

proc toggle_message_window {} {
   # global --------------------
   global mf wk gbl
   
   # main --------------------
   set state [catch "pack info $mf"]
   if { $state != 0 } {
      # When message frame is not shown
      pack forget $wk(w,canvas)
      pack $gbl(widget,bind_info) $mf -side bottom -fill x
      pack $wk(w,canvas) -side top -fill both -expand yes
   } else {
      # When message frame is shown
      pack forget $gbl(widget,bind_info) $mf 
   }
}

proc print_ps2image {format psfn imagefn} {
   # global --------------------
   global PREF print
   # main --------------------
   # check convert command
   if { ! [file executable $PREF(com,convert)] } {
      tk_messageBox -type ok -icon error -message\
	  "Cannot convert into bitmap image (invalid CONVERT command).  Check the PasteUp Preference."
      return
   }
   set opt [expr {$print(orientation) == "Landscape" } ? {"-rotate 90"} : {""}]
   eval exec $PREF(com,convert) $opt -density $print(density) $psfn $imagefn
}