# --------------------------------------------------------------------
# Execute Ray-Tracing
# --------------------------------------------------------------------
proc raytrace_exec {} {
   # global --------------------------------------------------
   global gbl TMPDIR PREF macray wk

   # main --------------------------------------------------
   # old data time
   set synthetime 0
   set tfn [format "%s/time%s_%s.dat"\
		$wk(raytrace,dir) $gbl(obs,x) $gbl(obs,depth)]
   set rfn [format "%s/rays%s_%s.dat"\
		$wk(raytrace,dir) $gbl(obs,x) $gbl(obs,depth)]
   if { $gbl(option,raytrace,preserve) } {
      if { ![file exists $wk(raytrace,dir)] } { 
	 file mkdir $wk(raytrace,dir)
      }
      if { $gbl(hamen,raypath) == 1 } {
	 if { [file exists $tfn] && [file exists $rfn] } {
	    set synthetime [file mtime $rfn] 
	 }
      } else {
	 if { [file exists $tfn] } {
	    set synthetime [file mtime $tfn] 
	 }
      }
   }

   # model made time
   if { $gbl(vmodel,format) == 0 } {
      # ray86 + gm
      set t1 [file mtime $gbl(vmodel,ray86)]
      set t2 [file mtime $gbl(vmodel,gm)]
      set modeltime [expr ($t1 > $t2) ? $t1 : $t2]
   } else {
      # netCDF
      set modeltime [file mtime $gbl(vmodel,netcdf)]
   }

   # raytrace --------------------------------------------------
   if { $modeltime > $synthetime } {
      # new computation ------------------------------
      # make temporary filename (pick or config file)
      set tmpfn [format "%s/pasteup.hamen.%s" $TMPDIR [pid]]
      if {$gbl(hamen,twopo) == 1} {
	 # two-point ----------
	 write_pasteup_pick $tmpfn $gbl(hamen,rout)
	 set com [raytrace_twopoint_command $tmpfn]
      } else {
	 # shooting to surface ----------
	 raytrace_make_hamencfg $tmpfn
	 # hamen or hamenCDF
	 if {$gbl(vmodel,format) == 0 } {
	    set com [format "%s/hamen %s" $PREF(com,hamenpath) $tmpfn]
	 } else {
	    set com [format "%s/hamenCDF %s" $PREF(com,hamenpath) $tmpfn]
	 }
      }
      set ww [wait_window "Ray Tracing..."]
      exec_external_command_fifo $com "Ray Tracing"
      destroy $ww
      file delete $tmpfn

      # draw synthetic traveltimes
      set macray(filename) $gbl(hamen,filename)
   } else {
      # use preserved data
      file copy -force $tfn $gbl(hamen,filename)
      if { $gbl(hamen,raypath) == 1} {
	 file copy -force $rfn $gbl(hamen,rayfilename)
      }
   }
   
   # plot synthetic traveltimes
   Synthetic::reread

   # draw raypaths, if required
   if {$gbl(hamen,raypath) == 1} {
      set ww [wait_window "Plotting rays..."]
      raytrace_plot_raypaths
      destroy $ww
   }

   # preserve synthetic data 
   if { $gbl(option,raytrace,preserve) } {
      file copy -force $gbl(hamen,filename) $tfn
      if {$gbl(hamen,raypath) == 1} {
	 file copy -force $gbl(hamen,rayfilename) $rfn
      }
   }
   # focus on the canvas window
   focus -force $wk(w,canvas)
}

# --------------------------------------------------------------------
# Execute Two-point ray-tracing 
# --------------------------------------------------------------------
proc raytrace_twopoint_command {pickfn} {
   # global --------------------------------------------------
   global gbl PREF macray
   
   # main --------------------------------------------------
   if {$gbl(vmodel,format) == 0 } {
      # twopo_hamen ------------------------------
      set com [format "%s/twopo_hamen %s %s"\
		   $PREF(com,hamenpath) $gbl(vmodel,ray86) $pickfn]
      # gm model
      if { [file readable $gbl(vmodel,gm)] } {
	 set com [format "%s -g %s -b %d" $com\
		      $gbl(vmodel,gm) $gbl(vmodel,mboundary)]
      }
   } else {
      # twopo_hamenCDF ------------------------------
      set com [format "%s/twopo_hamenCDF %s %s -v"\
		   $PREF(com,hamenpath) $gbl(vmodel,netcdf) $pickfn]
      # ray86 model
      if { [file readable $gbl(vmodel,ray86)] } {
	 set com [format "%s -m %s" $com $gbl(vmodel,ray86)]
      }
   }
   # raypath ?
   if { $gbl(hamen,raypath) == 1 } {
      set com [format "%s -f 2" $com]
   } else {
      set com [format "%s -f 0" $com]
   }
   # options
   return [format "%s -t %s -r %s -c %d -p %s -a %s"\
	       $com $gbl(hamen,filename) $gbl(hamen,rayfilename)\
	       $gbl(hamen,raycode) $gbl(hamen,twopo,type) $gbl(hamen,accuracy)]
}

# --------------------------------------------------------------------
# Make config file for HAMEN
# --------------------------------------------------------------------
proc raytrace_make_hamencfg {cfg} {
   # global --------------------------------------------------
   global gbl

   # main --------------------------------------------------
   set fp [open $cfg "w"]
   # model ----------
   puts $fp "\# Model and source."
   if { $gbl(vmodel,format) == 0 } {
      # ray86 + gm
      if { [file readable $gbl(vmodel,ray86)] == 0 } {
	 tk_messageBox -type ok -icon error\
	     -message "Ray86 file is not readable."
	 return
      }
      puts $fp [format "model=%s" $gbl(vmodel,ray86)]
      if { [file readable $gbl(vmodel,gm)] } {
	 puts $fp [format "gmodel=%s" $gbl(vmodel,gm)]
	 puts $fp [format "mboundary=%d" $gbl(vmodel,mboundary)]
      }
   } else {
      # netCDF (+ ray86)
      if { [file readable $gbl(vmodel,netcdf)] == 0 } {
	 tk_messageBox -type -icon error\
	     -message "netCDF file is not readable."
	 return
      }
      puts $fp [format "cdfmodel=%s" $gbl(vmodel,netcdf)]
      if { [file readable $gbl(vmodel,ray86)] } {
	 puts $fp [format "model=%s" $gbl(vmodel,ray86)]
      }
   }
   puts $fp [format "sx=%s" $gbl(obs,x)]
   puts $fp [format "sz=%s" $gbl(obs,depth)]

   # synthetic ----------
   puts $fp "\# Synthetic configuration."
   puts $fp [format "tfile=%s" $gbl(hamen,filename)]
   puts $fp [format "tout=%d" 2]
   puts $fp [format "raycode=%s" $gbl(hamen,raycode)]
   puts $fp [format "rfile=%s" $gbl(hamen,rayfilename)]
   puts $fp [format "accuracy=%f" $gbl(hamen,accuracy)]
   puts $fp [format "rformat=%d" 1]
   if {$gbl(hamen,raypath) == 1} {
      puts $fp [format "rout=%d" $gbl(hamen,rout)]
   }

   # conditions for computation ----------
   puts $fp "\# conditions for computation"
   puts $fp [format "dx=%s" $gbl(mapping,dx)]
   puts $fp [format "dsx=%s" $gbl(mapping,dx)]
   
   # verboseness
   puts $fp [format "verbose=%s" $gbl(option,verbose)]
   close $fp
}

# --------------------------------------------------------------------
# Invoke raypath window
# --------------------------------------------------------------------
proc raytrace_plot_raypaths {} {
   # global --------------------------------------------------
   global gbl wk

   # main --------------------------------------------------
   VModel::invoke_window

   # plot raypaths ------------------------------
   raytrace_plot_rays $wk(vmodel,widget).graph $gbl(hamen,rayfilename)
}

# --------------------------------------------------------------------
# raytrace_comp_tick_spacing -----------------------------------------
# --------------------------------------------------------------------
proc raytrace_comp_tick_spacing {range} {
   set delta [expr $range / 5.0]
   set width [expr pow(10.0, int(log10($delta)))]
   if { [expr int($range / $width)] > 10 } {
      set width [expr 2.0 * $width]
   }
   if { [expr int($range / $width)] > 10 } {
      set width [expr $width * 2.5]
   }
   return $width
}

# --------------------------------------------------------------------
# raytrace_plot_rays -------------------------------------------------
#           plot rays for raytrace window
# --------------------------------------------------------------------
proc raytrace_plot_rays {g fn} {
   # Global variables --------------------------------------------------
   global gbl file_name
   
   # Main --------------------------------------------------
   eval $g marker delete [$g marker names r*]
   # The file exists ? ----------
   if { ![file readable $fn] } {
      return
   }
   # title
   $g conf -title [file rootname [file tail $file_name(wave)]]

   # Plot ----------
   rays plot $fn $g
}

# --------------------------------------------------------------------
# make_cpt_file ------------------------------------------------------
#           Make a sample CPT file
# --------------------------------------------------------------------
proc make_cpt_file {} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   set fp [open $gbl(vmodel,cptfile) "w"]
   puts $fp "# Sample CPT file.
# Created by pasteup
1.51    255     0       255     2       149     0       255
2       149     0       255     3     42      0       255
3       42      0       255     4       0       63      255
4       0       63      255     5       0       170     255
5       0       170     255     5.5     0       255     234
5.5     0       255     234     6       85      255     0
6       85      255     0       6.5     255     213     0
6.5     255     213     0       7       255     106     0
7       255     106     0       8.5     255     0       0
F       255     0       0
B       255     255     255"
   close $fp
}

# --------------------------------------------------------------------
# Update analysis window after clicking the check button of raypath
# --------------------------------------------------------------------
proc raytrace_update_raypath_checkbutton {w} {
   # global --------------------------------------------------
   global gbl

   # main --------------------------------------------------
   if { $gbl(hamen,twopo,rays) == 0 } {
      # disabled panel
      labelframe_status $w disabled
   } else {
      # enable panel
      labelframe_status $w enable
   }
   
}

# --------------------------------------------------------------------
# Update analysis window after changing raypath computation mode
# --------------------------------------------------------------------
proc raytrace_update_raypath_comp_mode {w} {
   # global --------------------------------------------------
   global gbl

   # main --------------------------------------------------
   if { $gbl(hamen,twopo) == 0 } {
      # disabled panel
      labelframe_status $w disabled
   } else {
      # enable panel
      labelframe_status $w enable
   }
}

# --------------------------------------------------------------------
# Delete preserved synthetic data, if exist.
# --------------------------------------------------------------------
proc raytrace_delete_preserved_data {} {
   # global --------------------------------------------------
   global wk

   # main --------------------------------------------------
   if { [file exists $wk(raytrace,dir)] } {
      file delete -force $wk(raytrace,dir)
      print_message "Synthetic data directories were removed.\n"
   }
}

# --------------------------------------------------------------------
# Delete preserved synthetic data for one site.
# --------------------------------------------------------------------
proc raytrace_delete_this_site {} {
   # global --------------------------------------------------
   global gbl wk

   # main --------------------------------------------------
   if { ! [file exists $wk(raytrace,dir)] } {
      return
   }
   set tfn [format "%s/time%s_%s.dat"\
		$wk(raytrace,dir) $gbl(obs,x) $gbl(obs,depth)]
   set rfn [format "%s/rays%s_%s.dat"\
		$wk(raytrace,dir) $gbl(obs,x) $gbl(obs,depth)]
   set files {}
   foreach fn "$tfn $rfn" {
      if { [file exists $fn] } {
	 file delete $fn
	 lappend files $fn
      }
   }
   if { [llength $files] > 0 } {
      print_message "Synthetic file(s) for this site was removed.\n"
   }
}