#include <stdio.h>
#include <string.h>
#include <math.h>
#include "pasteup.h"

/********************************************************************
Make a trace : read data and time-domain-filtering.
********************************************************************/
int make_data(float *a, float *t_org,
	      FILE *fp, int data_format, int bfmt, int ch,
	      int data_point1, int data_point2,
	      float start_time, float end_time,
	      float sampling, float dt_clock, int filter_type, int causal,
	      double f1, double f2, double f3, double f4)
{
    /********** Variables **********/
    int n;
     
    /*** check range ***/
    end_time = ( (end_time - start_time)*sampling > maxsample) ?
	start_time + (maxsample-1)/sampling : end_time;

    /*** read data ***/
    n = read_data(a, t_org, fp, data_format, bfmt, ch,
		  data_point1, data_point2,
		  start_time, end_time, sampling, dt_clock);
    if (n <= 0) return(0);

    /*** Filter ***/
    filter(a, n, sampling, filter_type, causal, f1, f2, f3, f4);

    return(n);
}

/********************************************************************
Read a trace data
********************************************************************/     
int read_data(float *a, float *t,
	      FILE *fp, int data_format, int bfmt, int ch,
	      int data_point1, int data_point2,
	      float start_time, float end_time, float sampling, float dt_clock)
{
    /********** Variables **********/
    int n;
    
    /********** Function **********/
    int read_SU_data(float *a, float *t, FILE *fp,
		     int data_point1, int data_point2,
		     float start_time, float end_time,
		     float sampling, float dt_clock);
    int read_SEGY_data(float *a, float *t, FILE *fp, int bfmt,
		       int data_point1, int data_point2,
		       float start_time, float end_time,
		       float sampling, float dt_clock);
    int read_kasalab_data(float *a, float *t, FILE *fp, int bfmt,
			  int data_point1, int data_point2,
			  float start_time, float end_time,
			  float sampling, float dt_clock);
    int read_THK_data(float *a, float *t, FILE *fp, int ch, int bfmt,
		      int data_point1, int data_point2,
		      float start_time, float end_time,
		      float sampling, float dt_clock);
    
    /**************************/
    /********** Main **********/
    /**************************/
    
    switch(data_format) {
	case SU_FORMAT:
	    n = read_SU_data(a, t, fp, data_point1, data_point2,
			     start_time, end_time, sampling, dt_clock);
	    break;
	case SEGY_FORMAT:
	case SEGY1_FORMAT:
	    n = read_SEGY_data(a, t, fp, bfmt, data_point1, data_point2,
			       start_time, end_time, sampling, dt_clock);
	    break;
	case KASA_FORMAT:
	    n = read_kasalab_data(a, t, fp, bfmt, data_point1, data_point2,
				  start_time, end_time, sampling, dt_clock);
	    break;
	case THK_FORMAT:
	    n = read_THK_data(a, t, fp, ch, bfmt,data_point1, data_point2,
			      start_time, end_time, sampling, dt_clock);
	    break;
	default:
	    /** Invalid : return no data **/
	    fprintf(stderr, "read_data: Invalid data format to read.\n");
	    return(0);
	    break;
    }
    return(n);
}
