#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "pasteup.h"

/********************************************************************
 Read record header information and return number of traces.
--------------------------------------------------------------------
(Arguments)
 data_format : data format. (Kasahara-lab or SU, and so on)
 data_byte   : size of one sample of waveform.
 rh          : record header filename.
 wave        : waveform data file, binary.
 lon         : OBS longitude.
 lat         : OBS latitude.
 dist_axis   : SU, SEGY: X-axis type, Offset, Tracl, CDP.
               THK:X-Axis direction of distance axis. EAST or NORTH.
 channel     : channle number.
 dat         : (output).
********************************************************************/
int read_header(int data_format, int data_byte, char *rh, char *wave,
		double lon, double lat, int dist_axis, int channel)
{
     /********** Local Variables **********/
     FILE *fp;
     int total_shot;

     /********** Function **********/
     int read_rh(FILE *fp);
     int su_read_rh(FILE *fp, int x);
     int THKheader2rh(FILE *fp_input, int data_byte,
		      double obslon, double obslat, int dist_axis);
     int segy_read_rh(char *f, int data_type, int xaxis);
     int read_CHIBA_rh(FILE *fp);

     /**************************/
     /********** Main **********/
     /**************************/
     switch(data_format) {
	 case SU_FORMAT:
	     fp = open_file(wave, "waveform file(SU-rh)");
	     total_shot = su_read_rh(fp, dist_axis);
	     fclose(fp);
	     break;
	 case SEGY_FORMAT:
	 case SEGY1_FORMAT:
	     total_shot = segy_read_rh(wave, data_byte, dist_axis);
	     break;
	 case KASA_FORMAT:
	     fp = open_file(rh, "record header");
	     total_shot = read_rh(fp);
	     fclose(fp);
	     break;
	 case THK_FORMAT:
	     fp = open_file(wave, "waveform file(THK-rh)");
	     total_shot = THKheader2rh(fp, data_byte, lon, lat, dist_axis);
	     fclose(fp);
	     break;
	 case CHIBA_FORMAT:
	     fp = open_file(rh, "CHIBA record header");
	     total_shot = read_CHIBA_rh(fp);
	     fclose(fp);
	     break;
	 default:
	     total_shot = 0;
	     break;
     }
     return(total_shot);
}
