# --------------------------------------------------------------------
# Save parameters (resource) to a file
# --------------------------------------------------------------------
proc save_resource args {
   # gloabl --------------------
   global wk file_name paste gbl amp filt macray print pick
   
   # main --------------------
   if {[llength $args] > 0} {
      set resource [lindex $args 0]
      set all 1
   } else {
      if { [file isdir $wk(resource_dir)] == 0} {
	 file mkdir $wk(resource_dir)
      }
      set resource $wk(default_resource)
      set all 0
   }
   
   # check current geometry for the canvas window
   set paste(geometry) [lindex [split [wm geometry .cw] "+"] 0]

   # make all lists
   set list ""
   foreach i {file_name paste gbl amp filt macray print} {
      foreach member [lsort [array name $i]] {
	 if {$i == "macray" && !$all} {
	    if { [string match "t,*" $member] ||\
		     [string match "x,*" $member] ||\
		     [string match "m,*" $member] ||\
		     $member == "n" } {
	       continue
	    }
	 }
	 lappend list [format "%s(%s)" $i $member]
      }
   }
   lappend list pick(default_accuracy) pick(current_type)

   # Output to the file ------------------------------
   set fp [open $resource w]
   set t [clock format [clock second] -format "%Y/%m/%d %H:%M:%S"]
   puts $fp "\# This is a parameter file for pasteup."
   puts $fp "\#         Generated at $t"
   foreach i $list {
      eval set val [list [set $i]]
      puts $fp [format "set %s \"%s\"" $i $val]
   }
   close $fp
}

# --------------------------------------------------------------------
# Save as : resource (parameter) file
# --------------------------------------------------------------------
proc save_resource_as {} {
   # main --------------------
   set types {
      {"rc files"		*.rc}
      {"All files"	        *}
   }
   set file [tk_getSaveFile -filetypes $types -title "Save PARAMETER file"]
   if {$file != ""} {
      save_resource $file
   }
}
# --------------------------------------------------------------------
# Load resouce(parameter) file
# --------------------------------------------------------------------
proc load_resource {} {
   # global --------------------
   global paste gbl pick filt print file_name wk custom
   
   # global variable for compatibility
   global amp
    
   # main --------------------
   set types {
      {"rc files"		*.rc}
      {"All files"	        *}
   }
   set file [tk_getOpenFile -filetypes $types -title "Open PARAMETER file"]
   if {$file != ""} {
      set filt(f4) ""   ; # There's no filt(f4) before V1.4
   } else {
      return
   }
   exec_load_resource $file
}
# --------------------------------------------------------------------
# Exec Load resouce(parameter) file
# --------------------------------------------------------------------
proc exec_load_resource {fn} {
   # global --------------------
   global paste gbl pick filt print file_name wk custom

   source $fn
   # old version management : to assure upper-compatibility
   resource_compatibility
   # update data_type label
   set gbl(data,prev_type) $gbl(data,type)
   update_data_type
   # update plotting style
   update_displaystyle_menu
}
# --------------------------------------------------------------------
# resource(parameter) compatibility procedure
# --------------------------------------------------------------------
proc resource_compatibility {} {
   # global --------------------
   global paste gbl

   # main --------------------

   # --------------------------------------------------------------------
   # compatibility check (v1.9 and 1.8)
   set names [array name gbl mapping,*]
   if { [lsearch $names mapping,velformat] >= 0 } {
      set gbl(vmodel,format) $gbl(mapping,velformat)
      unset gbl(mapping,velformat)
   }
   if { [lsearch $names mapping,netcdf] >= 0} {
      set gbl(vmodel,netcdf) $gbl(mapping,netcdf)
      unset gbl(mapping,netcdf)
   }
   foreach nam {"view,aspect" "view,density" "model,contour" "model,cptfile"\
		    "view,ipalette" "view,img"} {
      if [info exist gbl(mapping,$nam)] {
	 unset gbl(mapping,$nam)
      }
   }
   if { [string length $paste(string,unit)] < 2 } {
      set paste(string,unit) "tx"
   }
   if { [string length $paste(fontsize,title)] < 2 } {
      set paste(fontsize,title) Medium
   }
   if { [string length $paste(fontsize,tic)] < 2 } {
      set paste(fontsize,tic) Medium
   }
   if { [string length $gbl(mapping,wsign)] < 3 } {
      set gbl(mapping,wsign) "deeper"
   }
   update_filter
   # check mark style
   if { [regexp "^splus$|^plus$|^cross$|^diamond$|^circle$|^square$"\
	     $paste(mark,style)] == 0 } {
      puts stderr "Invalid mark style, $paste(mark,style)"
      puts stderr "Use \"splus\", instead."
      set paste(mark,style) "splus"
   }
   # gbl(data,type) must be preceed than gbl(data,type,label) in old version
   if { ![info exist gbl(data,type,label)] } {
      switch -- $gbl(data,type) {
	 1 {
	    set gbl(data,type,label) "KASA-lab"
	 }
	 2 {
	    set gbl(data,type,label) "SU"
	 }
	 3 {
	    set gbl(data,type,label) "THK"
	 }
	 4 {
	    set gbl(data,type,label) "CHIBA"
	 }
	 5 {
	    set gbl(data,type,label) "SEGY(IASPEI)"
	 }
	 6 {
	    set gbl(data,type,label) "SEGY(1Ch)"
	 }
      }
   }
   if { $gbl(t_axis) < 0 } { set gbl(t_axis) 0}
}