#ifndef _SEGY_H
#define _SEGY_H 1

/*****
      SEG-Y header information
 *****/

/** Length of header information in bytes **/
#define REEL_HEADER_PART1 3200
#define REEL_HEADER_PART2 400
#define REEL_HEADER 3600      /** Sum of above two size **/
#define TRACE_HEADER_LENGTH 240

/** Reel header part2 : header information locations **/
#define SEGY_JOB_ID 0
#define SEGY_LINE_NO 4
#define SEGY_REEL_NO 8
#define SEGY_TRACE_NO 12       /** short int, trace number per record **/
#define SEGY_SAMPLE_INTERVAL 16/** short int, sampling interval in microsec **/
#define SEGY_TRACE_SAMPLES 20  /** short int, no of samples per trace **/
#define SEGY_DATA_FORMAT 24    /** short int, Data sample format code **/
#define SEGY_VERSION 298       /** short int, SEG-Y format version number **/

/** Trace header information locations **/
#define TRACE_YEAR 156        /** short int, year of start of trace **/

/*****
      Structure definition for SEG-Y data.
 *****/

struct segy_reel_header {
    short int n_trace_gather;/** Number of traces per gather **/
    short int n_trace;       /** Number of data traces in this file **/
    short int sampling;  /** Sampling interval in microsecond **/
    short int version;   /** SEG-Y file format version times 100 **/
    short int n_sample;  /** Number of samples per trace **/
    short int data_format;/** Data sample format code **/
};

/***** SEGY(IASPEI) trace header *****/
struct segy_iaspei_header {
    int tracl;	/* trace sequence number within line (0byte)*/
    int tracr;	/* trace sequence number within tape */
    int fldr;	/* Sequential Shot Number */
    int tracf;	/* Receiver Site Number */
    int ep;	/* Shot Site Number */
    int cdp;	/* CDP ensemble number */
    int cdpt;	/* trace number within CDP ensemble */
    short trid;	/* trace identification code (28byte)*/
    short nvs;	/* number of vertically summed traces */
    short nhs;	/* number of horizontally summed traces */
    short duse;	/* data use */
    int offset;	/* distance from source point to receiver group (36byte)*/
    int gelev;	/* receiver group elevation from sea level */
    int selev;	/* source elevation from sea level */
    int sdepth;	/* source depth (positive) */
    int gdel;	/* datum elevation at receiver group */
    int sdel;	/* datum elevation at source */
    int swdep;	/* water depth at source */
    int gwdep;	/* water depth at receiver group */
    short scalel;	/* scale factor for previous 7 entries (68byte)*/
    short scalco;	/* scale factor for next 4 entries */
    int  sx;	/* X source coordinate (72byte)*/
    int  sy;	/* Y source coordinate */
    int  gx;	/* X group coordinate */
    int  gy;	/* Y group coordinate */
    short counit;	/* coordinate units code (88byte)*/
    short wevel;	/* weathering velocity */
    short swevel;	/* subweathering velocity */
    short sut;	/* uphole time at source */
    short gut;	/* uphole time at receiver group */
    short sstat;	/* source static correction */
    short gstat;	/* group static correction */
    short tstat;	/* total static applied */
    short laga;	/* lag time A */
    short lagb;	/* lag time B */
    unsigned short delrt;	/* delay recording time, time in ms */ 
    short muts;	/* mute time--start */
    short mute;	/* mute time--end */
    unsigned short ns;	/* number of samples in this trace (114byte)*/
    unsigned short dt;	/* sample interval; in micro-seconds */
    short gain;	/* gain type of field instruments code (118byte)*/
    short igc;	/* data in nanometers/sec = (tape data)*(10**igc) */
    short igi;	/* instrument early or initial gain */
    short corr;	/* correlated */
    short sfs;	/* sweep frequency at start */
    short sfe;	/* sweep frequency at end */
    short slen;	/* sweep length in ms */
    short styp;	/* sweep type code */
    short stas;	/* sweep trace length at start in ms */
    short stae;	/* sweep trace length at end in ms */
    short tatyp;	/* taper type: 1=linear, 2=cos^2, 3=other */
    short afilf;	/* alias filter frequency if used */
    short afils;	/* alias filter slope */
    short nofilf;	/* notch filter frequency if used */
    short nofils;	/* notch filter slope */
    short lcf;	/* low cut frequency if used */
    short hcf;	/* high cut frequncy if used */
    short lcs;	/* low cut slope */
    short hcs;	/* high cut slope */
    short year;	/* year data recorded */
    short day;	/* day of year */
    short hour;	/* hour of day (24 hour clock) */
    short minute;	/* minute of hour */
    short sec;	/* second of minute */
    short timbas;	/* time basis code */
    short trwf;	/* trace weighting factor */
    short grnors;	/* geophone group number of roll switch position one */
    short grnofr;	/* geophone group number of trace one within original field record */
    short grnlof;	/* geophone group number of last trace within original field record */
    short gaps;	/* gap size (total number of groups dropped) */
    short overt;	/* Field Line number */
    /** IASPEI **/
    int mst;    /* Microseconds of trace start time */
    short charge;       /* Charge size in kg or airgun size in litres */
    short syer; /* Shot or triger time - year */
    short sday; /* Shot or triger time - Julian day */
    short shour;/* Shot or triger time - hour */
    short smin; /* Shot or triger time - minute */
    short sseco;/* Shot or triger time - second */
    int ssmic;  /* Shot or triger time - microsecond */
    int isin;   /* Override for sample interval */
    short geoazi;       /* Azimuth of geophone orientation axis with
			   respect to true north in minutes of arc */
    short geover;       /* Angle between geophone orientation axis and
			   vertical in minutes of arc */
    int ttrace; /* Static correction (microseconnds) */
    short tapply;       /* Flag to signal that ttrace has been used to
			   modify trace start time */
    short instru;       /* Recording instrument type */
    short cor;  /* Millisecond of timing correction to be added to
		   reported times to get true local or gmt times */
    short azimut;       /* Azimuth of receiver */
    short unass[10];	/* unassigned--NOTE */
};

#endif /* ! _SEGY_H */
