# --------------------------------------------------------------------
# Search SEG-Y file from a specified directory
# Return the first SEG-Y file in alphabetical order.
# --------------------------------------------------------------------
# [Arguments]
# dir : target directory
# --------------------------------------------------------------------
proc segy_search_datafile {dir} {
   set list [glob -nocomplain -dir $dir "*.segy" "*.sgy"]
   set list [lsort [concat $list]]
   if { [llength $list] < 1 } {
      return "./ch1.sgy"
   } else {
      return [lindex $list 0]
   }
}

# --------------------------------------------------------------------
# Identified "little endian?" or "big endian?"
# Return 1 if little endian, otherwise return -1.
# --------------------------------------------------------------------
# [Arguments]
#  fn : SEG-Y filename
# --------------------------------------------------------------------
proc segy_little_or_big {fn} {
   # file size
   set fsize [file size $fn]
   # open SEG-Y file and seek to reel header part2, and read it
   set fp [open $fn]
   seek $fp 3200
   set reel [read $fp 26]        ; # read only first 26 byte
   close $fp
   binary scan $reel i3s7 il sl  ; # read as little endian
   binary scan $reel I3S7 ib sb  ; # read as big endian
   # file size by header
   set lsize [expr 3600 + ( 240 + 4 * [lindex $sl 4] ) * [lindex $sl 0]]
   set bsize [expr 3600 + ( 240 + 4 * [lindex $sb 4] ) * [lindex $sb 0]]
   # Big or little-endian?
   if { [expr abs( $lsize - $fsize ) ] < [expr abs( $bsize - $fsize )] } {
      return 1   ; # little endian
   } else {
      return -1  ; # big endian
   }
}

# --------------------------------------------------------------------
# Identified segy binary format (Reel header 3224-th byte, short int)
# 1...IBM float,   2...long int,   3...short int (not supported)
# --------------------------------------------------------------------
# [Arguments]
#  fn : SEG-Y filename
# --------------------------------------------------------------------
proc segy_binary_format {fn} {
   set fp [open $fn]
   seek $fp 3224
   set dat [read $fp 2]
   binary scan $dat S fmt ; # always read as big-endian
   close $fp
   return $fmt
}

# --------------------------------------------------------------------
# Read single header value from SEGY file (using sugethw)
# --------------------------------------------------------------------
# [argument]
# head  : header name like "gwdep", "ns"
# --------------------------------------------------------------------
proc segy_read_single_header {head} {
   # global ------------------------------
   global wk file_name gbl
   # main ------------------------------
   if [file readable $file_name(wave)] {
      set c [format "| %s tape=%s endian=%d hfile=/dev/null bfile=/dev/null\
                     | sugethw key=%s output=geom"\
		 $wk(SU,SEGY2SU) $file_name(wave) $gbl(su,endian) $head]
      set com [open $c]
      gets $com val
      catch {close $com}
      return [regsub \\s $val ""]
   }
   return 0
}

