#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "pasteup.h"

/********************************************************************
Return a[t] by interpolation
--------------------------------------------------------------------
a[] has n components.
time of a[0] is t_org,  time of a[n-1] is t_org+(n-1)/smp
--------------------------------------------------------------------

Linear interpolation
********************************************************************/
static float interpolate(int n, float *a, float t_org, float smp, float t)
{
    /***** Local Variables *****/
    int i;
    float t1, t2;
    
    /********** Main **********/
    /***** No extrapolation *****/
    if(t < t_org || (float)(n-1)/smp + t_org < t) return(0.0);

    /***** both side of t *****/
    i = (int)((t - t_org)*smp);
    t1 = (float)  (i)/smp + t_org;
    t2 = (float)(i+1)/smp + t_org;

    /***** return *****/
    return( ( (t2 - t)*a[i] + (t - t1)*a[i+1] ) * smp );
}

int make_stacked(int i, int data_format, int bfmt, int ch,
		 float *a, float *t_org, float start_time, float end_time,
		 float rd_vel, int stack,
		 double stack_range, double stack_vel, double stack_time,
		 int filter_type, int causal,
		 double f1, double f2, double f3, double f4, FILE *fp)
{
    /********** local Variables *********/
    int j, k, n;
    int i_start, fold;
    float dt_orig; /** delta-t of the first sample (from the shot time) **/
    float dt;
    float stack_slowness, rv_slowness;

    int n_sub;
    float *a_sub;
    float t_sub;

    /******************** main *******************/

    if(NULL == (a_sub = (float *)malloc(maxsample * sizeof(float))) ){
	fprintf(stderr, "Error: Memory allocation failed, in make_stacked.\n");
	return TCL_ERROR;
    }

    /***** Initial settings *****/
    rv_slowness = (rd_vel > 0.0) ? 1.0/rd_vel : 0.0;

    /***** read the center trace *****/
    n = make_data(a, t_org, fp, data_format, bfmt, ch,
		  gbl_rh[i].start_point, gbl_rh[i].end_point,
		  start_time, end_time, gbl_rh[i].sampling, gbl_rh[i].dsec,
		  filter_type, causal, f1, f2, f3, f4);

    /** too short trace **/
    if (n < 1) return(0);

    /********** stack traces inside range **********/
    fold = 1;
    stack_slowness = (stack_vel != 0.0) ? 1.0/stack_vel : 0.0;
    if(stack == 1) {
	/* hyperbola */
	stack_time = stack_time*stack_time;
	stack_slowness = stack_slowness * stack_slowness;
	dt_orig = sqrt(stack_time
		       +(gbl_rh[i].x*gbl_rh[i].x)
		       *stack_slowness);
    } else {
	dt_orig = 0.0;
    }
    for(j=0;j<total_shot;j++){
	if ( fabs(gbl_rh[i].x - gbl_rh[j].x) <= stack_range
	     && i != j) {
	    /** determine time range **/
	    i_start = (start_time + gbl_rh[j].dsec)*gbl_rh[j].sampling;
	    if(i_start < 0) i_start = 0;
	    /** read a trace **/
	    n_sub = make_data(a_sub, &t_sub, fp, data_format, bfmt, ch,
			      gbl_rh[j].start_point, gbl_rh[j].end_point,
			      start_time, end_time,
			      gbl_rh[j].sampling, gbl_rh[j].dsec,
			      filter_type, causal, f1, f2, f3, f4);
	    /** stack **/
	    if (stack == 1) {
		/* hyperbola */
		dt = sqrt(stack_time + (gbl_rh[j].x*gbl_rh[j].x)
			  * stack_slowness) - dt_orig
		    + (fabs(gbl_rh[i].x) - fabs(gbl_rh[j].x))
		    * rv_slowness;
	    } else {
		/** slant **/
		dt = (fabs(gbl_rh[i].x) - fabs(gbl_rh[j].x))
		    *(rv_slowness - stack_slowness);
	    }
	    for(k=0;k<n;k++){
		a[k] += interpolate(n_sub, a_sub, t_sub, gbl_rh[j].sampling,
				    *t_org + k/gbl_rh[i].sampling + dt);
	    }
	    fold++;
	}
    }
     
    /***** normalize wave-data by the fold number *****/
    if(fold > 1) for(k=0;k<n;k++) a[k] /= (float)fold;
    free(a_sub);
    return(n);
}

