#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

#include "pasteup.h"
#include "su.h"

float su_dist_conv(float org_x, float scale, int counit)
{
    if(counit == 2)/* second of arc */
	return( - org_x / (3600.0 * scale) );
    return(org_x/1000.0);/* meter -> kilomter */
}

int su_read_rh(FILE *fp, int xaxis)
{
    /********** Local Variables **********/
    int i, n;
    struct su_header tr;
    struct tm tt;

    /**************************/
    /********** Main **********/
    /**************************/

    n = 0;
    /***** Read SU trace header for the first trace *****/
    fread(&tr, sizeof(struct su_header), 1, fp);
    while(!feof(fp)) {
	gbl_rh[n].start_point = ftell(fp);
	gbl_rh[n].end_point   = tr.ns;

	gbl_rh[n].hw[RECORD_HEADER_TRACL] = tr.tracl;
	gbl_rh[n].hw[RECORD_HEADER_TRACR] = tr.tracr;
	gbl_rh[n].hw[RECORD_HEADER_FLDR]  = tr.fldr;
	gbl_rh[n].hw[RECORD_HEADER_TRACF] = tr.tracf;
	gbl_rh[n].hw[RECORD_HEADER_EP]    = tr.ep;

	gbl_rh[n].sdepth      = tr.sdepth;
	gbl_rh[n].swdep       = tr.swdep;
	gbl_rh[n].gwdep       = tr.gwdep;
	gbl_rh[n].sampling    = 1.0e6/(float)(tr.dt);

	tt.tm_year = tr.year - 1900;
	tt.tm_mon  = 0;
	tt.tm_mday = tr.day;
	tt.tm_hour = tr.hour;
	tt.tm_min  = tr.minute;
	tt.tm_sec  = tr.sec;
	gbl_rh[n].st = mktime(&tt);
	gbl_rh[n].msec = 0.0;

	gbl_rh[n].counit      = tr.counit;
	gbl_rh[n].scalco      = tr.scalco;
	gbl_rh[n].sx          = tr.sx;
	gbl_rh[n].sy          = tr.sy;
	gbl_rh[n].gx          = tr.gx;
	gbl_rh[n].gy          = tr.gy;

	gbl_rh[n].dsec        =
	    -(float)(tr.delrt + tr.lagb - tr.laga)/1000.0;
	gbl_rh[n].offset      = tr.offset / 1000.0;

	/***** set x-axis parameter *****/
	switch(xaxis) {
	    case OFFSET: /** offset **/
		gbl_rh[n].x    = gbl_rh[n].offset;
		break;
	    case FLDR:
		gbl_rh[n].x = tr.fldr;
		break;
	    case SX:     /** sx     **/
		gbl_rh[n].x = su_dist_conv(tr.sx, tr.scalco, tr.counit);
		break;
	    case SY:     /** sy     **/
		gbl_rh[n].x = su_dist_conv(tr.sy, tr.scalco, tr.counit);
		break;
	    case GX:     /** gx     **/
		gbl_rh[n].x = su_dist_conv(tr.gx, tr.scalco, tr.counit);
		break;
	    case GY:     /** gy     **/
		gbl_rh[n].x = su_dist_conv(tr.gy, tr.scalco, tr.counit);
		break;
	    case TRACF:
		gbl_rh[n].x = tr.tracf;
		break;
	    case TRACL:  /** tracl  **/
		gbl_rh[n].x = tr.tracl;
		break;
	    case CDP:    /** cdp    **/
		gbl_rh[n].x = tr.cdp;
		break;
	}

	/********** seek to next trace **********/
	fseek(fp, 4*gbl_rh[n].end_point, SEEK_CUR);
	n++;

	/***** require re-allocation ? *****/
	if(n >= maxtrace) {
	    maxtrace += 1000;
	    if(realloc_global_variable() == 1) {
		fprintf(stderr, "ERROR: Memory reallocation failed.\n");
		exit(1);
	    }
	}

	/***** read next header *****/
	fread(&tr, sizeof(struct su_header), 1, fp);
    }
    if ( tr.trid == 200 /** when migration image **/ ) {
	for(i=0;i<n;i++) gbl_rh[i].sampling = 1000.0/tr.d1;
    }

    return(n);
}

/*
  --------------------------------------------------------------------
*/

/********************************************************************
Read a SU trace
--------------------------------------------------------------------
a           : an array for amplitude (output)
t           : time of the first sample
fp          : file pointer for the SU file
data_point1 : start point of the trace in the file
data_point2 : end point of the trace in the file
start_time  : cut out start time
end_time    : cut out end time
sampling    : sampling rate (Hz)
dt_clock    : clock modification.
********************************************************************/
int read_SU_data(float *a, float *t, FILE *fp,
		 int data_point1, int data_point2,
		 float start_time, float end_time,
		 float sampling, float dt_clock)
{
    /********** Local Variables **********/
    int i_start, i_end, count, address;

    /**************************/
    /********** Main **********/
    /**************************/

    /** Index of Start time **/
    if ( start_time + dt_clock >= 0.0 ) {
	i_start = (start_time + dt_clock)*sampling;
    } else {
	i_start = 0;
    }
    /** Index of End time **/
    i_end = ceil((end_time + dt_clock)*sampling);
    if (i_end > data_point2) i_end = data_point2;
    count = i_end - i_start;
    if (count < 1) return(0);
    address = data_point1 + i_start*4;

    /***** Read all the data (new implementation) *****/
    fseek(fp, address, SEEK_SET);
    fread(a, 4, count, fp);
    *t = (float)(i_start)/sampling - dt_clock;
    return(count);
}

/*
  --------------------------------------------------------------------
*/

int get_SU_year(char *data_file)
/********************************************************************
 Return the year of observation
********************************************************************/
{
    /********** Local Variables **********/
    struct su_header tr;
    FILE *fp;
    
    /**************************/
    /********** Main **********/
    /**************************/

    fp = open_file(data_file, "Data file in SU format");
    fread(&tr, sizeof(struct su_header), 1, fp);
    fclose(fp);
    return(tr.year);
}
