#ifndef _SU_H
#define _SU_H 1

struct su_header { /* trace header for SU */
    int tracl;	/* trace sequence number within line (0byte)*/
    int tracr;	/* trace sequence number within reel */
    int fldr;	/* field record number */
    int tracf;	/* trace number within field record */
    int ep;	/* energy source point number */
    int cdp;	/* CDP ensemble number */
    int cdpt;	/* trace number within CDP ensemble */
    short trid;	/* trace identification code (28byte)*/
    short nvs;	/* number of vertically summed traces */
    short nhs;	/* number of horizontally summed traces */
    short duse;	/* data use */
    int offset;	/* distance from source point to receiver group (36byte)*/
    int gelev;	/* receiver group elevation from sea level */
    int selev;	/* source elevation from sea level */
    int sdepth;	/* source depth (positive) */
    int gdel;	/* datum elevation at receiver group */
    int sdel;	/* datum elevation at source */
    int swdep;	/* water depth at source */
    int gwdep;	/* water depth at receiver group */
    short scalel;	/* scale factor for previous 7 entries (68byte)*/
    short scalco;	/* scale factor for next 4 entries */
    int  sx;	/* X source coordinate (72byte)*/
    int  sy;	/* Y source coordinate */
    int  gx;	/* X group coordinate */
    int  gy;	/* Y group coordinate */
    short counit;	/* coordinate units code (88byte)*/
    short wevel;	/* weathering velocity */
    short swevel;	/* subweathering velocity */
    short sut;	/* uphole time at source */
    short gut;	/* uphole time at receiver group */
    short sstat;	/* source static correction */
    short gstat;	/* group static correction */
    short tstat;	/* total static applied */
    short laga;	/* lag time A */
    short lagb;	/* lag time B */
#ifdef UNSIGNED_DELRT
    unsigned short delrt;/* delay recording time, time in ms */ 
#else
    short delrt;         /* delay recording time, time in ms */ 
#endif
    short muts;	/* mute time--start */
    short mute;	/* mute time--end */
    unsigned short ns;	/* number of samples in this trace (114byte)*/
    unsigned short dt;	/* sample interval; in micro-seconds */
    short gain;	/* gain type of field instruments code (118byte)*/
    short igc;	/* instrument gain constant */
    short igi;	/* instrument early or initial gain */
    short corr;	/* correlated */
    short sfs;	/* sweep frequency at start */
    short sfe;	/* sweep frequency at end */
    short slen;	/* sweep length in ms */
    short styp;	/* sweep type code */
    short stas;	/* sweep trace length at start in ms */
    short stae;	/* sweep trace length at end in ms */
    short tatyp;	/* taper type: 1=linear, 2=cos^2, 3=other */
    short afilf;	/* alias filter frequency if used */
    short afils;	/* alias filter slope */
    short nofilf;	/* notch filter frequency if used */
    short nofils;	/* notch filter slope */
    short lcf;	/* low cut frequency if used */
    short hcf;	/* high cut frequncy if used */
    short lcs;	/* low cut slope */
    short hcs;	/* high cut slope */
    short year;	/* year data recorded */
    short day;	/* day of year */
    short hour;	/* hour of day (24 hour clock) */
    short minute;	/* minute of hour */
    short sec;	/* second of minute */
    short timbas;	/* time basis code */
    short trwf;	/* trace weighting factor */
    short grnors;	/* geophone group number of roll switch position one */
    short grnofr;	/* geophone group number of trace one within original field record */
    short grnlof;	/* geophone group number of last trace within original field record */
    short gaps;	/* gap size (total number of groups dropped) */
    short otrav;	/* overtravel taper code */

    /***** for SU  *****/
    float d1;	/* sample spacing for non-seismic data (180byte)*/
    float f1;	/* first sample location for non-seismic data */
    float d2;	/* sample spacing between traces */
    float f2;	/* first trace location */
    float ungpow;/* negative of power used for dynamic range compression */
    float unscale;/* reciprocal of scaling factor to normalize range */
    int ntr; 	/* number of traces (204byte)*/
    short mark;	/* mark selected traces (208byte)*/
    short shortpad; /* alignment padding */
    short unass[14];	/* unassigned--NOTE */
    /* the end of 240 byte */
};
#endif  /* !_SU_H */
