#
# Description about variables
# 
# gbl(su,mod)     :  all the values of all the listing modules
#
# wk(SU,mod,ind)  :  the index number of current selected module
#                    in the gbl(su,mod)
# wk(SU,mod,info) :  module infomation (name, expalnation, options)
# wk(SU,opt,name) :  current selected option name
# gbl(su,mod,DB)  :  DataBase for su module lists
#

# --------------------------------------------------------------------
# SU filter ----------------------------------------------------------
# --------------------------------------------------------------------

namespace eval SUfilter {
   namespace export window

   # --------------------------------------------------------------------
   # invoke SU filter setting window
   # --------------------------------------------------------------------
   proc window {} {
      # Global variable --------------------------------------------------
      global gbl wk custom

      # Main --------------------------------------------------
      # CWPROOT?
      if { [check_su_mode] == 0 } {return}
      set w $wk(w,sufilter)
      if [raise_window $w] { return }
      toplevel $w
      wm title $w "PasteUp: SU Filter Window"
      set wk(SU,mod,ind) -1
      
      # binding ------------------------------
      basic_bindings $w
      
      # menu bar ------------------------------
      set mf [menu $w.menu_frame]

      # Module List menu ----------
      set m [menu $mf.module -tearoff 0]
      $mf add cascade -label "ModuleList" -menu $m -underline 0
      
      # module list manager
      set m2 [menu $m.restore -tearoff 0]
      $m add command -label "Module list manager" -command "SUfilter::DBmanager"
      $m add command -label "Store current module list into DB" \
	  -command "SUfilter::DBstore"

      # Load module-list
      $m add separator
      $m add command -label "Load module list from file..." \
	  -command "SUfilter::load $w"
      $m add command -label "Save module list to file..." \
	  -command "SUfilter::save"
      $m add command -label "Clear module list..." \
	  -command "SUfilter::new"
      $m add separator
      set m2 [menu $m.samples -tearoff 0]
      $m add cascade -label "Insert SAMPLE module list..." -menu $m2
      $m2 add command -label "Deconvolution(Predictive Error Filter)"\
	  -command "SUfilter::insert_modules decon"
      $m2 add command -label "STACK (Phase Weight Stack ; adjacent traces)"\
	  -command "SUfilter::insert_modules stack"
      $m2 add command -label "Panel Analysis(PEF Decon)"\
	  -command "SUfilter::insert_modules deconpanel"
      $m2 add command -label "Tau-P conversion (offset > 0)"\
	  -command "SUfilter::insert_modules taup_right"
      $m2 add command -label "Tau-P conversion (offset < 0)"\
	  -command "SUfilter::insert_modules taup_left"

      # add modules ----------
      set m [menu $mf.addmodule -tearoff 0]
      $mf add cascade -label "AddModule" -menu $m -underline 0

      set section 0
      foreach i $wk(SU,module_list) {
	 set name [lindex $i 0]
	 if {$name == "-"} {
	    # new section
	    incr section
	    set m2 $m.$section
	    $m add cascade -label [lindex $i 1] -menu $m2
	    menu $m2 -tearoff 0
	 } else {
	    # add a module
	    if {$section > 0} {
	       $m2 add command -label [format "%-15s%s" $name [lindex $i 1]]\
		   -command "SUfilter::add_module $name"\
		   -font normal_fixed
	    }
	 }
      }

      # option menu ----------
      set m [menu $mf.optionmenu -tearoff 0]
      $mf add cascade -label "Option" -menu $m -underline 0

      # endian
      set m2 $m.endian
      menu $m2 -tearoff 0
      $m add cascade -label "CPU Endian" -menu $m2
      $m2 add radio -label "Little Endian(IA32/IA64/Alpha...)"\
	  -value 0 -var gbl(su,endian)
      $m2 add radio -label "Big Endian(Sparc/PowerPC...)"\
	  -value 1 -var gbl(su,endian)

      # help ----------
      set m [menu $mf.help -tearoff 0]
      $mf add cascade -label "Help" -menu $m -underline 0

      $m add command -label "Header words list"\
	  -command "SUfilter::show_headerwords"

      $w conf -menu $mf

      # parameter frame ----------------------------------
      set pf [frame $w.para_frame]

      # module flow ----------
      set left $pf.left
      frame $left
      # auto windowing?
      checkbutton $left.autowindow -text "Auto-offset-window"\
	  -va gbl(su,autowindow)
      # listbox 
      listbox $left.list -width 20 -height 20 -yscrollcom "$left.scr set"\
	  -font normal_fixed -selectback darkseagreen1 -selectfo black
      scrollbar $left.scr -width 10 -command "$left.list yview" -relief ridge
      # automatic processing?
      set af $left.autoframe
      frame $af
      checkbutton $af.checkscale -text "Auto-scaling" -va gbl(su,autoscale)
      entry $af.entryscale -width 6 -textva gbl(su,scaleperc) -justify right
      label $af.unitscale -text "%"
      checkbutton $af.checkrange -text "Auto-range  " -va gbl(su,autorange)
      entry $af.entryrange -width 6 -textva gbl(su,rangedx) -justify right
      label $af.unitrange -text "km"
      grid $af.checkscale $af.entryscale $af.unitscale -sticky news
      grid $af.checkrange $af.entryrange $af.unitrange -sticky news

      # table (packing)
      table $left $left.autowindow 0,0 -fill x -cspan 2
      table $left $left.list 1,0 -fill y
      table $left $left.scr 1,1 -fill y
      table $left $af 2,0 -fill x -cspan 2
      set wk(SU,mlistbox) $left.list

      # popup menu
      set m .sufilter_module_popup
      if {![winfo exist $m]} {
	 menu $m -tearoff 0
	 $m add command -label "Enable/Disable" -comm "SUfilter::toggle_module"
	 $m add command -label "Delete" -command "SUfilter::delete_module"
	 $m add command -label "Show help" -command "SUfilter::show_sudoc"
      }
      # binding
      bind $left.list <Double-Button-1> "SUfilter::toggle_module"
      bind $left.list <Button-3> "SUfilter::module_popupmenu $m %X %Y"
      bind $left.list <<ListboxSelect>> "SUfilter::module_edit"
      bind $left.list <Control-Button-1> "SUfilter::raise_module"
      bind $left.list <Control-Button-3> "SUfilter::lower_module"
      bind $left.list <Button-4> "SUfilter::raise_module"
      bind $left.list <Button-5> "SUfilter::lower_module"

      # insert module list
      update_module_listbox

      # module options ------------------------------
      set r $pf.right
      frame $r -relief groove -bd 2
      label $r.title -text "module parameter :"
      listbox $r.list -width 80 -height 20 -yscrollcom "$r.scr set"\
	  -font normal_fixed -selectback lightblue1 -selectfore black
      scrollbar $r.scr -width 10 -relief ridge -command "$r.list yview"
      label $r.opt_label -text "option:" -width 18 -anchor e
      entry $r.opt_val -width 32
      label $r.def_label -text "default:" -width 18 -anchor e
      label $r.def_val -text "" -width 32 -anchor w -relief sunken
      table $r $r.title 0,0 -cspan 3 -fill x
      table $r $r.list 1,0 -cspan 2 -fill both
      table $r $r.scr 1,2 -fill y
      table $r $r.opt_label 2,0 -fill x
      table $r $r.opt_val 2,1 -cspan 2 -fill x
      table $r $r.def_label 3,0 -fill x
      table $r $r.def_val 3,1 -cspan 2 -fill x

      set wk(SU,optwin) $r

      # binding
      bind $r.list <<ListboxSelect>> "SUfilter::option_edit"
      bind $r.opt_val <Return> "SUfilter::option_next"
      bind $r.opt_val <KeyRelease> "SUfilter::update_current_option"
      bind $r.opt_val <Down> "SUfilter::option_next"
      bind $r.opt_val <Control-n> "SUfilter::option_next"
      bind $r.opt_val <Up> "SUfilter::option_prev"
      bind $r.opt_val <Control-p> "SUfilter::option_prev"
      focus $r.opt_val

      # packing ------------------------------
      pack $left -side left
      pack $r -side right -fill x -expand yes

      # --------------------------------------------------
      # Button frame -------------------------------------
      set bf [button_frame $w $w.button_frame "Diagnosis" SUfilter::diagnosis \
		 "Save filtered SU" SUfilter::save_filtered_su]

      # --------------------------------------------------
      # packing ------------------------------------------
      pack $pf $bf -side top -fill x -expand 1
   }

   # --------------------------------------------------------------------
   # update module listbox
   # --------------------------------------------------------------------
   proc update_module_listbox {} {
      # global ------------------------------
      global wk gbl
      # main ------------------------------
      # erase it
      $wk(SU,mlistbox) delete 0 end
      # insert all the moudles
      foreach i $gbl(su,mod) {
	 $wk(SU,mlistbox) insert end [lindex $i 0]
	 if {[lindex $i 1] == 1} {
	    # enabled
	    $wk(SU,mlistbox) itemconf end -fg black -selectfore black
	 } else {
	    # disabled
	    $wk(SU,mlistbox) itemconf end -fg gray80 -selectfore gray80
	 }
      }
   }

   # --------------------------------------------------------------------
   # update module listbox
   # --------------------------------------------------------------------
   proc clear_option_listbox {} {
      # global ------------------------------
      global wk

      # main ------------------------------
      $wk(SU,optwin).list delete 0 end
      set wk(SU,mod,ind) -1
      set wk(SU,opt,name) ""
   }

   # --------------------------------------------------------------------
   # Add a module to a su-filter list
   # --------------------------------------------------------------------
   # [argument]
   # mod  : module name
   # --------------------------------------------------------------------
   proc add_module {mod} {
      # global ------------------------------
      global wk gbl
      # main ------------------------------
      # set ind [$wk(SU,mlistbox) curselection]
      if { $wk(SU,mod,ind) < 0 } { 
	 # append last
	 set ind end
	 # data format is a list in a form of {modname on/off options}.
	 lappend gbl(su,mod) [list $mod 1 ""]
      } else {
	 # after the current selection
	 set ind [expr $wk(SU,mod,ind) + 1]
	 # data format is a list in a form of {modname on/off options}.
	 set gbl(su,mod) [linsert $gbl(su,mod) $ind [list $mod 1 ""]]
      }
      $wk(SU,mlistbox) insert $ind $mod
      update_module_listbox
      $wk(SU,mlistbox) selection set $ind
      module_edit
   }

   # --------------------------------------------------------------------
   # Show a popup menu if having a selection in the module list.
   # --------------------------------------------------------------------
   # [arguments]
   #  m   : menu
   #  x   : root x-coordinate
   #  y   : root y-coordinate
   # --------------------------------------------------------------------
   proc module_popupmenu {m x y} {
      # global ------------------------------
      global wk
      # main ------------------------------
      if { [$wk(SU,mlistbox) curselection] == "" && $wk(SU,mod,ind) < 0 } {
	 return
      }
      tk_popup $m $x $y
   }

   # --------------------------------------------------------------------
   # Toggle Enable/Disable the module in the module list
   # --------------------------------------------------------------------
   proc toggle_module {} {
      # global ------------------------------
      global wk gbl
      # main ------------------------------
      set ind [$wk(SU,mlistbox) curselection]
      if {$ind == ""} {
	 if { $wk(SU,mod,ind) < 0 } {return}
	 set ind $wk(SU,mod,ind)
      }

      # toggle
      set dat [lindex $gbl(su,mod) $ind]
      set state [lindex $dat 1]
      if {$state == 0} {
	 # enabling
	 set gbl(su,mod) [lreplace $gbl(su,mod) $ind $ind [lreplace $dat 1 1 1]]
	 $wk(SU,mlistbox) itemconf $ind -fg black -selectfore black
      } else {
	 # disabling
	 set gbl(su,mod) [lreplace $gbl(su,mod) $ind $ind [lreplace $dat 1 1 0]]
	 $wk(SU,mlistbox) itemconf $ind -fg gray80 -selectfore gray80
      }
   }

   # --------------------------------------------------------------------
   # Delete selected modules from list
   # --------------------------------------------------------------------
   proc delete_module {} {
      # global ------------------------------
      global wk gbl
      # main ------------------------------
      set ind [$wk(SU,mlistbox) curselection]
      if {$ind == ""} {
	 if { $wk(SU,mod,ind) < 0 } {return}
	 set ind $wk(SU,mod,ind)
      }
      # delete
      $wk(SU,mlistbox) delete $ind
      set gbl(su,mod) [lreplace $gbl(su,mod) $ind $ind]
      # clear the option window
      clear_option_listbox
   }

   # --------------------------------------------------------------------
   # Make a option listbox for a selected module
   # --------------------------------------------------------------------
   proc module_edit {} {
      # global ------------------------------
      global wk gbl custom

      # main ------------------------------
      # have a selection?
      set ind [$wk(SU,mlistbox) curselection]
      if {$ind == ""} { return }

      # delete all options of previously selected moudle
      $wk(SU,optwin).list delete 0 end

      # set module name background
      for {set i 0} {$i < [llength $gbl(su,mod)]} {incr i} {
	 $wk(SU,mlistbox) itemconf $i -back $custom(*Entry.Background)
      }
      $wk(SU,mlistbox) itemconf $ind -back darkseagreen1
      
      # make a listbox
      set dat [lindex $gbl(su,mod) $ind]
      set name [lindex $dat 0]
      set opt [lindex $dat 2]
      make_option_listbox $name $opt

      # current selection info
      set wk(SU,mod,ind) $ind
      set wk(SU,mod,info) [get_modinfo $name]
      set wk(SU,opt,name) ""
      set wk(SU,opt,ind) -1

      # update selected module name
      set exp [lindex $dat 1]
      $wk(SU,optwin).title conf\
	  -text [format "%s : %s" $name [lindex $wk(SU,mod,info) 1]]

      # initialize option selection
      $wk(SU,optwin).list selection set 0
      option_edit
   }

   # --------------------------------------------------------------------
   # Return all the information of the specified module
   # --------------------------------------------------------------------
   # [arguments]
   #  mod_name : module name
   # --------------------------------------------------------------------
   proc get_modinfo {mod_name} {
      global wk

      # main --------------------------------------------------
      # get module option lists
      set i [lsearch -regexp $wk(SU,module_list) ^$mod_name]
      if {$i < 0} {
	 puts stderr "sufilter: module isn't found, $mod_name"
	 return ""
      }
      return [lindex $wk(SU,module_list) $i]
   }

   # --------------------------------------------------------------------
   # Make listbox of options for a selected module
   # --------------------------------------------------------------------
   # [arguments]
   #  mod_name : module name which is selected
   #  opt      : list of options which have been given
   # --------------------------------------------------------------------
   proc make_option_listbox {mod_name opt} {
      global wk

      # main --------------------------------------------------
      # get module option lists
      set i [lsearch -regexp $wk(SU,module_list) ^$mod_name]
      if {$i < 0} {
	 puts stderr "sufilter: module isn't found, $mod_name"
	 return
      }
      # insert lists
      set minfo [lindex $wk(SU,module_list) $i]
      set olist [lindex $minfo 2]
      foreach i $olist {
	 set name [lindex $i 0]
	 set exp [lindex $i 1]
	 set val [option_get_value $name $opt]
	 $wk(SU,optwin).list insert end\
	     [format "%-15s %s" [format "%s=%s" $name $val] $exp]
      }
      return
   }

   # --------------------------------------------------------------------
   # gets option values from "options"
   # --------------------------------------------------------------------
   # [arguments]
   #  name    : option name (such as "min")
   #  options : option and value list (such as "min=1 max=100")
   # --------------------------------------------------------------------
   proc option_get_value {name options} {
      global wk

      # main --------------------------------------------------
      foreach i $options {
	 if [regexp (\\w+)=(\\S+) $i dum key val] {
	    if {$key == $name} { return $val }
	 }
      }
      return ""
   }

   # --------------------------------------------------------------------
   # Edit current selected option
   # --------------------------------------------------------------------
   proc option_edit {} {
      # global ------------------------------
      global wk gbl custom

      # main ------------------------------
      # module is selected?
      if { $wk(SU,mod,ind) < 0 } {return}

      # have a selection?
      set ind [$wk(SU,optwin).list curselection]
      if {$ind != ""} {
	 set wk(SU,opt,ind) $ind
      }

      # clear entry widget
      $wk(SU,optwin).opt_val delete 0 end

      # number of options of the current module
      set n [llength [lindex $wk(SU,mod,info) 2]]

      # set background of selected option
      for {set i 0} {$i < $n} {incr i} {
	 $wk(SU,optwin).list itemconf $i -back $custom(*Entry.Background)
      }
      $wk(SU,optwin).list itemconf $wk(SU,opt,ind) -back lightblue1

      # update widgets related to input options
      set coption [lindex [lindex $wk(SU,mod,info) 2] $wk(SU,opt,ind)]
      set wk(SU,opt,name) [lindex $coption 0]
      set exp [lindex $coption 1]
      set def [lindex $coption 2]
      $wk(SU,optwin).opt_label conf -text "$wk(SU,opt,name)="
      $wk(SU,optwin).def_label conf -text "default value="
      $wk(SU,optwin).def_val conf -text $def

      # insert value (if exists)
      set given_opt [lindex [lindex $gbl(su,mod) $wk(SU,mod,ind)] 2]
      $wk(SU,optwin).opt_val insert 0\
	  [option_get_value $wk(SU,opt,name) $given_opt]
   }

   # --------------------------------------------------------------------
   # Update current selected option
   # --------------------------------------------------------------------
   proc update_current_option {} {
      # global ------------------------------
      global wk gbl

      # main ------------------------------
      # get the contents of entry widget
      set val [$wk(SU,optwin).opt_val get]
      # remove leading/trailing spaces
      set val [regsub \\s+$ [regsub \\s+ $val ""] ""]
      
      # register the value
      remove_option $wk(SU,mod,ind) $wk(SU,opt,name)
      if {$val != ""} {
	 append_option $wk(SU,mod,ind) "$wk(SU,opt,name)=$val"
      }
      # update option-listbox
      set exp [lindex [lindex [lindex $wk(SU,mod,info) 2] $wk(SU,opt,ind)] 1]
      $wk(SU,optwin).list delete $wk(SU,opt,ind)
      $wk(SU,optwin).list insert $wk(SU,opt,ind)\
	  [format "%-15s %s" [format "%s=%s" $wk(SU,opt,name) $val] $exp]
      $wk(SU,optwin).list itemconf $wk(SU,opt,ind) -back lightblue1
   }

   # --------------------------------------------------------------------
   # Remove specified option from gbl(su,mod)
   #  ind      : index number of module 
   #  opt_name : option name
   # --------------------------------------------------------------------
   proc remove_option {ind opt_name} {
      # global ------------------------------
      global wk gbl

      # main ------------------------------
      # get module info
      set data [lindex $gbl(su,mod) $ind]
      set name [lindex $data 0]
      set state [lindex $data 1]
      set dat [lindex $data 2]
      
      # search option
      set i [lsearch -regexp $dat ^$opt_name=]
      if {$i < 0} {return}
      # remove it!
      set dat [lreplace $dat $i $i]
      
      # update 
      set gbl(su,mod) [lreplace $gbl(su,mod) $ind $ind\
			   [list $name $state $dat]]
   }

   # --------------------------------------------------------------------
   # Append specified option into gbl(su,mod)
   #  ind  : index number of module 
   #  data : option name and values, like "min=100"
   # --------------------------------------------------------------------
   proc append_option {ind data} {
      # global ------------------------------
      global wk gbl

      # main ------------------------------
      set cdata [lindex $gbl(su,mod) $ind]
      set name [lindex $cdata 0]
      set state [lindex $cdata 1]
      set optlist [lindex $cdata 2]
      
      # append
      lappend optlist $data
      # update 
      set gbl(su,mod) [lreplace $gbl(su,mod) $ind $ind\
			   [list $name $state $optlist]]
   }

   # --------------------------------------------------------------------
   # Select next option
   # --------------------------------------------------------------------
   proc option_next {} {
      # global ------------------------------
      global wk 

      # main ------------------------------
      set total_num [llength [lindex $wk(SU,mod,info) 2]]
      if { $wk(SU,opt,name) != "" && $wk(SU,opt,ind) >= 0 &&\
	       $total_num > [expr $wk(SU,opt,ind) +1] } {
	 # clear selection
	 $wk(SU,optwin).list selection clear 0 end
	 # select next option
	 incr wk(SU,opt,ind)
	 # update
	 option_edit
	 return
      }
      SUfilter::update_current_option
   }

   # --------------------------------------------------------------------
   # Select previous option
   # --------------------------------------------------------------------
   proc option_prev {} {
      # global ------------------------------
      global wk 

      # main ------------------------------
      if { $wk(SU,opt,name) != "" && $wk(SU,opt,ind) > 0 } {
	 # clear selection
	 $wk(SU,optwin).list selection clear 0 end
	 # select previous optin
	 incr wk(SU,opt,ind) -1
	 # update
	 option_edit
	 return
      }
      SUfilter::update_current_option
      $wk(SU,optwin).list selection set $wk(SU,opt,ind)
   }

   # --------------------------------------------------------------------
   # Generate sufilter command
   # --------------------------------------------------------------------
   proc command {} {
      # global ------------------------------
      global wk gbl paste file_name

      # main ------------------------------
      # CWPROOT?
      if { [check_su_mode] == 0 } {return ""}
      if {$gbl(data,type) != 2 && $gbl(data,type) < 5} { return "" }

      # make command ------------------------------
      set com ""
      # windowing?
      if { $gbl(su,autowindow) && $gbl(su,autorange) == 0 } {
	 set com [format "| suwind key=offset min=%d max=%d "\
		      [expr int( $paste(left) * 1000.0 )]\
		      [expr int( $paste(right) * 1000.0 ) ]]
      }
      # commands
      foreach i $gbl(su,mod) {
	 set mod_name [lindex $i 0]
	 set state    [lindex $i 1]
	 set opt_list [lindex $i 2]
	 if $state {
	    if { $mod_name == "susort" || $mod_name == "sufifosort" } {
	       # abnormal module (odd option usage)
	       set keys [string trimleft $opt_list keys=]
	       regsub -all -- \\+ $keys " +" keys
	       regsub -all -- - $keys " -" keys
	       set com [format "%s | %s %s" $com $mod_name $keys]
	    } elseif { [lsearch "supickstack sutmixfilter sutracekill" \
			    $mod_name] >= 0} {
	       # handling pfile option
	       if { [lsearch $opt_list pfile] < 0 } {
		  # make working pick file
		  write_pasteup_pick $wk(fn,pick)
		  lappend opt_list [format "pfile=%s" $wk(fn,pick)]
	       }
	       set com [format "%s | %s %s" $com $mod_name $opt_list]
	    } else {
	       # normal module
	       set com [format "%s | %s %s" $com $mod_name $opt_list]
	    }
	 }
      }
      return $com
   }

   # --------------------------------------------------------------------
   # load sufilter module-list
   # --------------------------------------------------------------------
   proc load {w} {
      # global ------------------------------
      global gbl

      # main ------------------------------
      # file selection
      set types {
	 {"filter files"   {.filter} }
	 {"dat files"      {.dat}    }
	 {"All files"               *}
      }
      set loadfn [tk_getOpenFile -filetypes $types -parent .\
		      -defaultextension .filter]

      exec_load $loadfn

      update_module_listbox
      clear_option_listbox
   }

   # --------------------------------------------------------------------
   # exec load sufilter module-list
   # --------------------------------------------------------------------
   proc exec_load {fn} {
      # global ------------------------------
      global gbl

      # main ------------------------------
      if { [file exist $fn] } {
	 # delete current module list
	 set gbl(su,mod) {}
	 # load file
	 source $fn
	 # restart sufilter window
      }
   }

   # --------------------------------------------------------------------
   # save sufilter module-list
   # --------------------------------------------------------------------
   proc save {} {
      # global ------------------------------
      global gbl

      # main ------------------------------
      # file selection
      set types {
	 {"filter files"   {.filter} }
	 {"dat files"      {.dat}    }
	 {"All files"               *}
      }
      set savefn [tk_getSaveFile -filetypes $types]
      if { $savefn == "" } {
	 return
      }
      # save parameters
      set fp [open $savefn w]
      set t [clock format [clock second] -format "%Y/%m/%d %H:%M:%S"]
      puts $fp "\# This is a module-list file for pasteup sufilter."
      puts $fp "\#         Generated at $t"
      puts $fp [format "set %s \"%s\"" gbl(su,mod) $gbl(su,mod)]
      close $fp
      print_message "sufilter: module list was written in $savefn.\n"
   }

   # --------------------------------------------------------------------
   # Clear current module-list to make a new module list
   # --------------------------------------------------------------------
   proc new {} {
      # global ------------------------------
      global gbl

      # main ------------------------------
      # delete current module list
      set gbl(su,mod) {}
      # restart sufilter window
      update_module_listbox
      clear_option_listbox
   }

   # --------------------------------------------------------------------
   # raise current module
   # --------------------------------------------------------------------
   proc raise_module {} {
      # global ------------------------------
      global wk gbl

      # main ------------------------------
      # have a selection?
      set ind $wk(SU,mod,ind)
      if { $ind == -1 || $ind < 1} { return }

      # raise ----------
      # get selected module info
      set cur [lindex $gbl(su,mod) $ind]
      # remove it from data
      set gbl(su,mod) [lreplace $gbl(su,mod) $ind $ind]
      # insert the module info 
      set ind [expr $ind - 1]
      set gbl(su,mod) [linsert $gbl(su,mod) $ind $cur]
      
      # update listbox ----------
      update_module_listbox
      $wk(SU,mlistbox) selection set $ind
      set wk(SU,mod,ind) $ind
   }

   # --------------------------------------------------------------------
   # lower current module
   # --------------------------------------------------------------------
   proc lower_module {} {
      # global ------------------------------
      global wk gbl

      # main ------------------------------
      # have a selection?
      set ind $wk(SU,mod,ind)
      set n [llength $gbl(su,mod)]
      if { $ind == -1 || $ind >= [expr $n -1]} { return }

      # raise ----------
      # get selected module info
      set cur [lindex $gbl(su,mod) $ind]
      # remove it from data
      set gbl(su,mod) [lreplace $gbl(su,mod) $ind $ind]
      # insert the module info 
      set ind [expr $ind + 1]
      set gbl(su,mod) [linsert $gbl(su,mod) $ind $cur]
      
      # update listbox ----------
      update_module_listbox
      $wk(SU,mlistbox) selection set $ind
      set wk(SU,mod,ind) $ind
   }

   # --------------------------------------------------------------------
   # Invoke help window
   # --------------------------------------------------------------------
   #  w  : top level widget name
   # --------------------------------------------------------------------
   proc invoke_help_window {w} {
      global wk custom

      # main --------------------------------------------------
      if [raise_window $w] { return }
      # make a new window
      toplevel $w
      wm title $w "PasteUP: SU selfdoc"
      title_frame $w.l "SU selfdoc : "
      text $w.t -width 80 -height 30 -yscrollcom "$w.s set" -bg gray95
      scrollbar $w.s -width 15 -command "$w.t yview"
      button $w.b -text " dismiss " -command "destroy $w" -width 12\
	  -bg $custom(DismissButtonBackground)
      grid $w.l - -sticky news
      grid $w.t $w.s -sticky news
      grid $w.b - -sticky nes -padx 10 -pady 10
   }

   # --------------------------------------------------------------------
   # Show sudoc help message
   # --------------------------------------------------------------------
   proc show_sudoc {} {
      # global ------------------------------
      global wk gbl TMPDIR

      # main ------------------------------
      set ind $wk(SU,mod,ind)
      set modname [lindex [lindex $gbl(su,mod) $ind] 0]
      
      # CWPROOT?
      if { [check_su_mode] == 0 } {return}

      # suhelp window ----------
      set w .suhelp_window
      invoke_help_window $w
      
      # update title ----------
      $w.l conf -text "SU selfdoc : $modname"

      # insert documents into text-widget ----------
      # clear
      $w.t delete 0.0 end

      # get documents
      set stderr_file [format "%s/pasteup.stderr.%s" $TMPDIR [pid]]
      catch {eval exec "$modname 2> $stderr_file"}
      
      # insert it
      set fp [open $stderr_file]
      while { [gets $fp line] >= 0} {
	 $w.t insert end "$line\n"
      }
      close $fp
      
      # remove temporary file
      file delete -force $stderr_file
   }

   # --------------------------------------------------------------------
   # Show sukeyword -o.
   # --------------------------------------------------------------------
   proc show_headerwords {} {
      global wk TMPDIR env

      # CWPROOT?
      if { [check_su_mode] == 0 } {return}

      # suhelp window ----------
      set w .suhelp_window
      invoke_help_window $w
      
      # update title ----------
      $w.l conf -text "SU selfdoc : Header words list"

      # insert documents into text-widget ----------
      # clear
      $w.t delete 0.0 end

      # get header words list
      set stderr_file [format "%s/pasteup.stderr.%s" $TMPDIR [pid]]
      set env(PAGER) more
      set c [open "| sukeyword -o"]
      set doc {}
      while { [gets $c line] >= 0 } {
	 lappend doc $line
      }
      close $c
      # format ----------
      # search for "typedef struct" 
      set start_ind [expr [lsearch $doc "typedef struct*"]]
      # search for "\} segy;"
      set end_ind [expr [lsearch $doc "*segy;"] -1]
      set doc [lrange $doc $start_ind $end_ind]
      # insert documents ----------
      foreach line $doc {
	 if [regexp \\S $line] {
	    $w.t insert end "$line\n"
	 }
      }
   }

   # --------------------------------------------------------------------
   # Apply sufilter
   # --------------------------------------------------------------------
   # Return the original filename if valid, or 0 if invalid
   # --------------------------------------------------------------------
   proc apply {} {
      global gbl wk file_name TMPDIR env

      # main --------------------------------------------------
      if {$wk(SU,mode) != 1 || $gbl(check,sufilter) != 1 ||\
	      ( $gbl(data,type) != 2 && $gbl(data,type) < 5 )} { return 0}
      # temporary file for STDERR
      set stderr_file [format "%s/pasteup.stderr.%s" $TMPDIR [pid]]
      if [file exists $stderr_file] {
	 file delete -force $stderr_file
      }
      set ww [wait_window "filtering..."]
      print_message [format "Apply SU filter (%s) ---> "\
			 [clock format [clock seconds] -format %T] ]
      # make filter command ----------
      set com [command]
      set orgname 0
      # apply filter ----------
      if {$com != ""} {
	 set orgname $file_name(wave)
	 set file_name(wave) [format "%s/pasteup.%s.su" $TMPDIR [pid]]
	 set com [make_su_command 0 $com $orgname $file_name(wave) "" ""]
	 # exec filter -----
	 set status [exec_external_command $com "SUFILTER"]
	 print_message [format "(%s)\n"\
			    [clock format [clock seconds] -format %T] ]
	 # change su mode
	 set gbl(data,type) 2
	 set gbl(data,byte) 4
	 if $status {
	    # some error occurred
	    puts stderr $com
	    file delete -force $file_name(wave)
	    set file_name(wave) $orgname
	    destroy $ww
	    return -1
	 } else {
	    # automatic process?
	    if $gbl(su,autorange) {
	       set tmp $gbl(xtic_int)
	       set gbl(xtic_int) $gbl(su,rangedx)
	       auto_offset_range ""
	       set gbl(xtic_int) $tmp
	    }
	    if $gbl(su,autoscale) {
	       set tmp $gbl(option,scaling_factor)
	       set gbl(option,scaling_factor) $gbl(su,scaleperc)
	       auto_scale_factor
	       set gbl(option,scaling_factor) $tmp
	    }
	 }
	 file delete -force $wk(fn,pick)
      } else {
	 print_message "Error\n"
      }
      destroy $ww
      return $orgname
   }

   # --------------------------------------------------------------------
   # Convert SU file into SEGY file
   # --------------------------------------------------------------------
   proc segy_write {su segy} {
      # global ------------------------------
      global gbl wk TMPDIR

      # main ------------------------------
      # temporary files
      set hfile [format "%s/pasteup.%s.header" $TMPDIR [pid]]
      set bfile [format "%s/pasteup.%s.binary" $TMPDIR [pid]]

      # make segy headers ----------
      set com [format "segyhdrs hfile=%s bfile=%s < %s" $hfile $bfile $su]
      set status [exec_external_command $com "SEGYHDRS"]
      if { $status == 0 } {
	 # convert su into segy ----------
	 set com [format "%s tape=%s endian=%d hfile=%s bfile=%s < %s"\
		      $wk(SU,SU2SEGY) $segy $gbl(su,endian) $hfile $bfile $su]
	 set status [exec_external_command $com "SU2SEGY"]
      }
      file delete -force $hfile
      file delete -force $bfile
   }

   # --------------------------------------------------------------------
   # Insert sample module list 
   # --------------------------------------------------------------------
   proc insert_modules {name} {
      # global ------------------------------
      global gbl wk

      # main ------------------------------
      # insert position
      set ind $wk(SU,mod,ind)

      # get module list to insert
      switch -- $name {
	 decon {
	    set mod {\
			 {sucfilter 1 {}}\
			 {supef 1 {minlag=0.025 maxlag=0.2}} \
			 {sugain 1 {agc=1 wagc=3}}\
		      }
	 }
	 stack {
	    set mod {\
			 {sucfilter 1 {}}\
			 {suppstack 1 {}}\
			 {supws 1 {}}\
			 {susetoffset 1 {}}\
		      }
	 }
	 deconpanel {
	    set mod {\
			 {suwind 1 {key=offset min=20000 max=30000}}\
			 {sufifosort 1 keys=offset}\
			 {sucfilter 1 {}}\
			 {sudeconpanel 1 {gaps=0.015,0.030,0.050 lags=0.2,0.4,0.6 rv=6}}\
			 {sugain 1 {agc=1 wagc=3}}\
		      }
	 }
	 taup_right {
	    set mod {  \
			   {suwind 1 {key=offset min=0 max=10000}} \
			   {sufifosort 1 keys=+offset} \
			   {sucfilter 1 {f=3,20,25 roffset=1}} \
			   {supef 1 maxlag=0.2} \
			   {suvdtaup 1 {max=10 min=1}} \
			}
	 }
	 taup_left {
	    set mod {  \
			   {suwind 1 {key=offset min=-10000 max=0}} \
			   {sufifosort 1 keys=+offset} \
			   {sucfilter 1 {f=3,20,25 roffset=1}} \
			   {supef 1 maxlag=0.2} \
			   {suvdtaup 1 {max=-1 min=-10}} \
			}
	 }
	 default {
	    puts stderr "Unknown module list to insert."
	    puts stderr "         $name"
	    set mod {}
	 }
      }
      # insert
      set left [lrange $gbl(su,mod) 0 $ind]
      set right [lrange $gbl(su,mod) [expr $ind + 1] end]
      set gbl(su,mod) [concat $left $mod $right]

      # restart sufilter window
      update_module_listbox
      clear_option_listbox
   }

   # --------------------------------------------------------------------
   # Confirm whether overwrite module list $n
   # --------------------------------------------------------------------
   proc confirm_overwrite {n} {
      global ans gbl
      # invoke window
      set w .sufilter_confirm
      toplevel $w
      wm title $w "Overwrite module list?"
      # title frame
      set tf [frame $w.title_frame]
      label $tf.icon -bitmap question
      label $tf.text -justify left -text\
	  "Overwirte the existing module list?\nThe existing list is,"
      pack $tf.icon $tf.text -side left

      # module list frame
      set mf [label $w.modules -just left -relief ridge -bd 2\
		  -text [listup_module $n]]

      # button frame
      set bf [frame $w.button_frame]
      button $bf.overwrite -text "Overwrite" -command "set ans 1" -width 10
      button $bf.cancel -text "cancel" -command "set ans 0" -width 10
      pack $bf.overwrite $bf.cancel -side left -fill x -expand 1

      # packing
      pack $tf $mf $bf -side top -fill x

      # return with value
      grab $w
      set ans -1
      tkwait variable ans
      destroy $w
      return $ans
   }

   # --------------------------------------------------------------------
   # module list manager
   # --------------------------------------------------------------------
   proc DBmanager {} {
      global custom wk gbl
      set w .modlist_manager
      if [raise_window $w] { return }
      toplevel $w
      wm title $w "PasteUp: SU module list DB manager"
      
      # title frame --------------------
      set tf [title_frame $w.title_frame "Pasteup: SU module list DB manager"]
      
      # binding ------------------------------
      basic_bindings $w

      # menu frame --------------------
      set mf [frame $w.menu_frame -relief raised -bd 1]

      # db menu
      set db [menubutton $mf.db_button -text "DataBase" -relief flat]
      set m [menu $db.menu -tearoff 0]
      $db conf -menu $m 
      $m add command -label "Save DataBase as ..." \
	  -command "SUfilter::DBmanager_save_as"
      $m add command -label "Load DataBase ..." \
	  -command "SUfilter::DBmanager_load"
      $m add command -label "Clear DataBase ..." \
	  -command "SUfilter::DBmanager_clear"

      # sort menu
      set sb [menubutton $mf.sort_button -text "Sort" -relief flat]
      set m [menu $sb.menu -tearoff 0]
      $sb conf -menu $m 
      $m add command -label "Sort by name (description)" \
	  -command "SUfilter::DBmanager_sort_by_name"
      $m add command -label "Sort by date (create/modification)" \
	  -command "SUfilter::DBmanager_sort_by_date"
      $m add separator
      $m add command -label "Reverse order" \
	  -command "SUfilter::DBmanager_sort_reverse"
      pack $db $sb -side left

      # parameter frame --------------------
      set pf [frame $w.param_frame]

      # fail safe
      set wk(w,sudb,description) ""

      # existing module list
      DBmanager_windowparts $pf

      # binding
      bind $wk(w,sudb,modlist) <Double-Button-1> "SUfilter::DBmanager_select"

      # button frame --------------------
      set bf [button_frame $w $w.button_frame \
		  "Use" "SUfilter::DBmanager_select" \
		  "Delete" "SUfilter::DBmanager_delete" \
		  "Rename" "SUfilter::DBmanager_rename"]
      pack $tf $mf $pf $bf -side top -fill x
   }

   # --------------------------------------------------------------------
   # window parts for module list manager (existing module list)
   # --------------------------------------------------------------------
   proc DBmanager_windowparts {pf} {
      global custom wk gbl
      
      # module list
      set pfl [labelframe $pf.left -text "Module list name (description)"]
      set wk(w,sudb,modlist) [listbox $pfl.lb -width 50 -height 20 \
				  -xscrollcommand "$pfl.sh set" \
				  -yscrollcommand "$pfl.sv set" \
				  -selectback darkseagreen1 -selectfo black \
				  -font normal_fixed]
      scrollbar $pfl.sh -width 10 -orient horizontal -command "$pfl.lb xview"
      scrollbar $pfl.sv -width 10 -orient vertical -command "$pfl.lb yview"
      grid $pfl.lb $pfl.sv -sticky news
      grid $pfl.sh x -sticky ew
      bind $pfl.lb <<ListboxSelect>> "SUfilter::DBstore_lbselect %W"

      # listup
      DBmanager_listup

      # module contents
      set pfr [labelframe $pf.right -text "Contents"]
      set wk(w,sudb,contents) [listbox $pfr.lb -width 20 -height 20 \
				   -yscrollcommand "$pfr.sv set" \
				   -font normal_fixed -state disabled \
				   -relief flat -bg $custom(*Background) \
				   -disabledforeground black]
      scrollbar $pfr.sv -width 10 -orient vertical -command "$pfr.lb yview"
      grid $pfr.lb $pfr.sv -sticky news

      pack $pfl $pfr -side left -fill both -padx 3 -ipadx 3 -pady 3
   }

   # --------------------------------------------------------------------
   # listup all the module lists in DB
   # --------------------------------------------------------------------
   proc DBmanager_listup {} {
      global gbl wk
      $wk(w,sudb,modlist) delete 0 end
      foreach i $gbl(su,mod,DB) {
	 set dsc [lindex $i 0]
	 $wk(w,sudb,modlist) insert end $dsc
      }
   }

   # --------------------------------------------------------------------
   # restore a module list
   # --------------------------------------------------------------------
   proc DBmanager_select {} {
      global gbl wk
      set ind [$wk(w,sudb,modlist) curselection]
      if { $ind == "" } {
	 tk_messageBox -type ok -icon error \
	     -message "Select a module list name first."
	 return
      }
      set gbl(su,mod) [lindex [lindex $gbl(su,mod,DB) $ind] 1]
      if { [ winfo exists $wk(w,sufilter) ] } {
	 update_module_listbox
	 clear_option_listbox
      }
   }

   # --------------------------------------------------------------------
   # clear all the DB
   # --------------------------------------------------------------------
   proc DBmanager_clear {} {
      global gbl wk
      if { [llength $gbl(su,mod,DB)] > 0 } { 
	 set ans [tk_messageBox -type okcancel -default cancel -icon warning \
		      -message "Are you sure to clear your DataBase?"]
	 if { $ans == "cancel" } { return }
	 set gbl(su,mod,DB) {}
	 DBmanager_listup
      }
   }

   # --------------------------------------------------------------------
   # Save DB
   # --------------------------------------------------------------------
   proc DBmanager_save_as {} {
      global gbl wk
      if { [llength $gbl(su,mod,DB)] <= 0 } { 
	 # no module list
	 return
      }
      set types { 
	 {"SU module list DB" {.sudb}}
	 {"All files" *}
      }
      if { [file isdir $wk(sudb_dir)] == 0} {
	 file mkdir $wk(sudb_dir)
      }
      set fn [tk_getSaveFile -defaultextension .sudb -filetypes $types \
		  -initialdir "$wk(resource_dir)/sudb/" \
		  -title "SU module list DataBase"]
      if { $fn == "" } {
	 return
      }
      # save
      set fp [open $fn w]
      set t [clock format [clock second] -format "%Y/%m/%d %H:%M:%S"]
      puts $fp "\# This is a parameter file for pasteup."
      puts $fp "\#         Generated at $t"
      puts $fp [format "set gbl(su,mod,DB) \"%s\"" $gbl(su,mod,DB)]
      close $fp
   }

   # --------------------------------------------------------------------
   # Save DB
   # --------------------------------------------------------------------
   proc DBmanager_load {} {
      global gbl wk
      # check
      if { [llength $gbl(su,mod,DB)] > 0 } { 
	 set ans [tk_messageBox -type okcancel -default cancel -icon question \
		      -message "If you load a SU modeled DataBase, your current DataBase will be lost.  Are you really going to load?"]
	 if { $ans == "cancel" } { return }
      }
      # file
      set types { 
	 {"SU module list DB" {.sudb}}
	 {"All files" *}
      }
      set fn [tk_getOpenFile -defaultextension .sudb -filetypes $types \
		  -initialdir "$wk(resource_dir)/sudb/" \
		  -title "SU module list DataBase"]
      if {$fn == ""} { return }
      # load
      source $fn
      DBmanager_listup
   }
   # --------------------------------------------------------------------
   # DB : Sort by name
   # --------------------------------------------------------------------
   proc DBmanager_sort_by_name {} {
      global gbl
      
      set gbl(su,mod,DB) [lsort -index 0 -dictionary $gbl(su,mod,DB)]
      DBmanager_listup
   }

   # --------------------------------------------------------------------
   # DB : Sort by date
   # --------------------------------------------------------------------
   proc DBmanager_sort_by_date {} {
      global gbl date
      
      set gbl(su,mod,DB) [lsort -index 2 -decreasing -integer $gbl(su,mod,DB)]
      DBmanager_listup
   }

   # --------------------------------------------------------------------
   # DB : reverse module list order
   # --------------------------------------------------------------------
   proc DBmanager_sort_reverse {} {
      global gbl date
      
      set tmp {}
      set n [expr [llength $gbl(su,mod,DB)] - 1]
      for {set i $n} {$i >= 0} {incr i -1} {
	 lappend tmp [lindex $gbl(su,mod,DB) $i]
      }
      set gbl(su,mod,DB) $tmp
      DBmanager_listup
   }

   # --------------------------------------------------------------------
   # delete a module list from DB
   # --------------------------------------------------------------------
   proc DBmanager_delete {} {
      global gbl wk
      set ind [$wk(w,sudb,modlist) curselection]
      if { $ind == "" } {
	 tk_messageBox -type ok -icon error \
	     -message "Select a module list name first."
	 return
      }
      set ans [tk_messageBox -type okcancel -default cancel -icon question \
		   -message "Delete the selected module list?"]
      if { $ans == "cancel" } {
	 return
      }
      set gbl(su,mod,DB) [lreplace $gbl(su,mod,DB) $ind $ind]
      DBmanager_listup
   }

   # --------------------------------------------------------------------
   # Rename a module list in DB
   # --------------------------------------------------------------------
   proc DBmanager_rename {} {
      global gbl wk
      # check
      set ind [$wk(w,sudb,modlist) curselection]
      if { $ind == "" } {
	 tk_messageBox -type ok -icon error \
	     -message "Select a module list name first."
	 return
      }
      set data [lindex $gbl(su,mod,DB) $ind]
      
      # calc geometry of window
      set top [winfo toplevel $wk(w,sudb,modlist)]
      set x [expr [winfo rootx $top] + 15]
      set y [expr [winfo rooty $top] + [winfo height $top]/2 - 15]

      # create window
      set w [toplevel .sudb_rename]
      wm title $w "PasteUp: SU filter DB : Rename"
      wm geometry $w [format "+%s+%s" $x $y]
      label $w.l -text "Module name (description)"
      entry $w.e -width 50
      grid $w.l $w.e -sticky news
      button $w.ok -text "update" -width 7 \
	  -command "SUfilter::DBmanager_rename_exec $ind $w"
      button $w.cancel -text "cancel" -width 7 -command "destroy $w"
      grid $w.ok $w.cancel -sticky ns

      $w.e insert 0 [lindex $data 0]
      update
      grab $w
   }

   # --------------------------------------------------------------------
   # Rename a module list in DB
   # --------------------------------------------------------------------
   proc DBmanager_rename_exec {ind w} {
      global gbl wk

      set newname [$w.e get]
      set dsc [regsub -all {\s+} $newname " "]
      if { [regsub -all {\s} $dsc ""] == "" } {
	 # null string
	 return
      }

      # check same name?
      foreach i $gbl(su,mod,DB) {
	 set d [lindex $i 0]
	 if { $dsc == $d } {
	    tk_messageBox -icon warning -type ok \
		-message "A module list with the same name exists.  The module name is NOT changed."
	    return
	 }
      }

      # replace
      set olddata [lindex $gbl(su,mod,DB) $ind]
      set newdata [lreplace $olddata 0 0 $dsc]
      set gbl(su,mod,DB) [lreplace $gbl(su,mod,DB) $ind $ind $newdata]
      DBmanager_listup
      destroy $w
   }

   # --------------------------------------------------------------------
   # window for storing a module list into DB
   # --------------------------------------------------------------------
   proc DBstore {} {
      global custom wk gbl
      set w .modlist_manager
      catch {destroy $w}
      toplevel $w
      wm title $w "PasteUp: SU module list DB : Store "
      
      # title frame --------------------
      set tf [title_frame $w.title_frame "Pasteup: SU module list DB : Store "]
      
      # parameter frame --------------------
      set pf [frame $w.param_frame]
      
      # new description
      set pf1 [frame $pf.description]
      label $pf1.l -text "Name (description) for new list"
      set wk(w,sudb,description) [entry $pf1.e -width 50]
      grid $pf1.l $pf1.e -sticky news

      # existing module list
      set pf2 [labelframe $pf.existing -text "Existing module list"]
      DBmanager_windowparts $pf2

      pack $pf1 $pf2 -side top -fill x -pady 5 -padx 3

      # button frame --------------------
      set bf [button_frame $w $w.button_frame "Store" \
		  "SUfilter::DBstore_exec $w"]

      pack $tf $pf $bf -side top -fill x
      grab $w
   }

   # --------------------------------------------------------------------
   # binding proc for listbox in the store window
   # --------------------------------------------------------------------
   proc DBstore_lbselect {w} {
      global wk gbl
      # clear contents
      $wk(w,sudb,contents) conf -state normal
      $wk(w,sudb,contents) delete 0 end
      # get contents
      set data [lindex $gbl(su,mod,DB) [$w curselection]]
      # update
      foreach i [lindex $data 1] {
	 if { [lindex $i 1] } {
	    $wk(w,sudb,contents) insert end [lindex $i 0]
	 } else {
	    $wk(w,sudb,contents) insert end [format " (%s)" [lindex $i 0]]
	 }
      }
      $wk(w,sudb,contents) conf -state disabled
      # update description, if possible
      if { [winfo exists $wk(w,sudb,description)] } {
	 $wk(w,sudb,description) delete 0 end
	 $wk(w,sudb,description) insert 0 [lindex $data 0]
      }
   }

   # --------------------------------------------------------------------
   # Store a module list into DB
   # --------------------------------------------------------------------
   proc DBstore_exec {w} {
      global wk gbl

      # check description
      set dsc [regsub -all {\s+} [$wk(w,sudb,description) get] " "]
      if { [regsub -all {\s} $dsc ""] == "" } {
	 # null string
	 return
      }
      set ind 0
      foreach i $gbl(su,mod,DB) {
	 set d [lindex $i 0]
	 if { $dsc == $d } {
	    # overwrite ?
	    set ans [tk_messageBox -icon question -type yesno -default no \
			 -message "Overwrite existing module list?" ]
	    if {$ans == "no"} {
	       return
	    }
	    break
	 }
	 incr ind
      }
      if { $ind < [llength $gbl(su,mod,DB)] } {
	 set gbl(su,mod,DB) [lreplace $gbl(su,mod,DB) $ind $ind]
      }

      # append 
      lappend gbl(su,mod,DB) [list $dsc $gbl(su,mod) [clock seconds]]

      # quit window and invoke manager
      destroy $w
      DBmanager
   }

   # --------------------------------------------------------------------
   # diagnosis for SU flow
   # --------------------------------------------------------------------
   proc diagnosis {} {
      global gbl wk file_name

      # invoke window
      set w .sufilter_diagnosis
      toplevel $w
      wm title $w "PasteUp: SU-Filter Diagnosis"

      # title frame
      set tf [title_frame $w.title_frame "Pasteup: SU-Filter Diagnosis"]

      # text frame
      set mf [frame $w.text_frame]
      set t [text $mf.text -yscrollcommand "$mf.vscroll set" -relief flat]
      scrollbar $mf.vscroll -orient vertical -command "$mf.text yview" -width 10
      pack $mf.text $mf.vscroll -fill y -expand yes -side left

      # button frame
      set bf [button_frame $w $w.button_frame]
      pack $tf $mf $bf -side top -fill x -expand 1

      proc dputs {w m} {
	 if { $m == "" } {
	    $w insert end "\n"
	 } else {
	    $w insert end $m
	 }
      }

      proc dcheck {t c} {
	 dputs $t "\t ** $c "
	 if [search_external_command $c] {
	    dputs $t "is found\n"
	    return 0
	 } else {
	    dputs $t "is NOT found\n"
	    return 1
	 }
      }

      # mandatory commands check
      dputs $t "* Check mandatory command\n\n"
      set notfound 0
      foreach c {suwind} {
	 incr notfound [dcheck $t $c]
      }
      dputs $t ""

      # check command existence (in your flow)
      dputs $t "* Check commands in your flow\n\n"
      foreach i $gbl(su,mod) {
	 set mod_name [lindex $i 0]
	 set state    [lindex $i 1]
	 set opt_list [lindex $i 2]
	 if $state {
	    incr notfound [dcheck $t $mod_name]
	 }
      }
      dputs $t ""
      if { $notfound > 0 } {
	 dputs $t "* results\n\n"
	 dputs $t "\t ERROR : $notfound command(s) are not found.\n\n"
	 dputs $t "\t\t ** remedy\n"
	 dputs $t "\t\t - check your command search path.\n"
	 dputs $t "\t\t - install requested command(s).\n"
	 dputs $t "\t\t - use alternative SU command(s).\n"
	 grab $w
	 return
      }

      # check input file condition
      dputs $t "* Check your input data\n\n"

      dputs $t "\t ** check trace length\n\n"
      set com [make_su_command 0 "| suwind" $file_name(wave) \
		   /dev/null /dev/null /dev/null]
      set stat [exec_external_command $com "SU Filter Diagnosis"]
      if { $stat == 1 } { 
	 dputs $t "\t WARNING : trace length is probably not constant.\n\n"
	 return
      }

      # OK.
      dputs $t "No errors and no suspicious points have been found.\n"
	       
   }

   # --------------------------------------------------------------------
   # Save a new SU file after applying SUFILTER
   # --------------------------------------------------------------------
   proc save_filtered_su {} {
      global gbl wk file_name

      # SU/SEG-Y?
      if {$gbl(data,type) != 2 && $gbl(data,type) < 5} { 
	 tk_messageBox -icon warning -type ok -message \
	     "You cannot apply sufilter to the current data file."
	 return 
      }
      set w .createsu_window
      if [raise_window $w] { return }
      toplevel $w
      wm title $w "PasteUP: Save a new SU file after applying SUFILTER"
      
      # title frame --------------------
      set titlef [title_frame $w.title_frame \
		      "Pasteup: Save as a new SU file after applying SUFILTER"]

      # parameter frame ------------------------------
      set pf $w.parameter_frame
      frame $pf

      # output filename ----------
      set dir [file dirname $file_name(wave)]
      set fn [file rootname [file tail $file_name(wave)]]
      set wk(conv,output_fn) [format "%s/%s_filtered.su" $dir $fn]
      filename_hscroll_frame $pf output_filename "A new SU file" \
	  wk(conv,output_fn) 60 save su

      # Button frame ----------------------------------------
      set bf [button_frame $w $w.button_frame "Create" \
		  "SUfilter::exec_create_after_filter"]
   
      # Packing ----------
      pack $titlef $pf $bf -side top -fill x -expand 1
   }

   # --------------------------------------------------------------------
   # Exec create a new SU file after applying SUFILTER
   # --------------------------------------------------------------------
   proc exec_create_after_filter {} {
      global gbl wk file_name

      # Main --------------------
      # check overwrite?
      if { [file exists $wk(conv,output_fn)] } {
	 set ans [tk_messageBox -icon question -type okcancel \
		      -message "The output file already exists.  Are you sure to overwite it?"]
	 if { $ans == "cancel"} {return}
      }
      
      # execution
      set orgfn [apply]
      if { $orgfn == 0 } {
	 tk_messageBox -icon error -type ok -message \
	     "An error occurred during applying the SUFILTER."
	 return
      }
      file rename -force $file_name(wave) $wk(conv,output_fn)
      set file_name(wave) $orgfn
      
      # fin
      update
      destroy .createsu_window
   }
}