# --------------------------------------------------------------------
#
#
#                        Synthetic traveltimes
#
#
# --------------------------------------------------------------------

namespace eval Synthetic {

   # -----------------------------------------------------------------
   # Plot synthetics
   # -----------------------------------------------------------------
   proc plot {} {
      global macray wk

      # main --------------------
      if { $macray(mode) != "on" } { return }
      delete_synthetic $wk(w,current)
      plot_synthetic $wk(w,current)
   }

   # -----------------------------------------------------------------
   # load synthetic data
   # -----------------------------------------------------------------
   proc load {} {
      # global --------------------
      global macray wk
      
      # main --------------------
      if { $macray(filename) != "" } {
	 set dir [file dirname $macray(filename)]
      } else {
	 set dir ./
      }
      
      # file selection
      set wk(tmp) ""
      file_selection wk(tmp) load pick
      if { $wk(tmp) == "" } { 
	 print_message "Abort reading.\n"
	 return 
      }
      set macray(filename) $wk(tmp)
      read $macray(filename)
   }
   
   # -----------------------------------------------------------------
   # Re-Read synthetics from file
   # -----------------------------------------------------------------
   proc reread {} {
      # Global --------------------
      global macray
      
      # Main --------------------
      if [file readable $macray(filename)] {
	 read $macray(filename)
      } else {
	 load
      }
   }

   # -----------------------------------------------------------------
   # Read synthetics from file
   # -----------------------------------------------------------------
   proc read {fn} {
      # gloabl --------------------
      global macray
      
      # main --------------------
      set fmt [discriminate_pickfile $fn]
      switch $fmt {
	 "Pasteup" {
	    set fd [open $fn r]
	    set tmp [gets $fd]
	    set xoffset [lindex $tmp 0]
	    # read
	    set i 0
	    while {1} {
	       set tmp [gets $fd]
	       if {[llength $tmp] < 4} {
		  break;
	       }
	       scan $tmp "%*s%s%s%*s%s" macray(x,$i) macray(t,$i) macray(m,$i)
	       incr i
	    }
	    catch {close $fd}
	    set macray(n) $i
	 }
	 "Rayinvr" {
	    set fd [open $fn r]
	    set tmp [gets $fd]
	    set xoffset [lindex $tmp 0]
	    # read
	    set i 0
	    while {[gets $fd tmp]} {
	       if {[eof $fd] || [llength $tmp] != 4} {break}
	       set mark [lindex $tmp 3]
	       if { $mark > 0 } {
		  set macray(x,$i) [expr [lindex $tmp 0] - $xoffset]
		  set macray(t,$i) [lindex $tmp 1]
		  set macray(m,$i) $mark
		  incr i
	       }
	    }
	    catch {close $fd}
	    set macray(n) $i
	 }
	 "Mochi" {
	    set fd [open $fn r]
	    set tmp [gets $fd]
	    # read
	    set i 0
	    while {[gets $fd tmp]} {
	       if {[eof $fd] || [llength $tmp] != 3} {break}
	       set macray(x,$i) [expr [lindex $tmp 0] - $xoffset]
	       set macray(t,$i) [lindex $tmp 1]
	       set macray(m,$i) 1
	       incr i
	    }
	    catch {close $fd}
	    set macray(n) $i
	 }
	 "GeoCTII" {
	    set fd [open $fn r]
	    set tmp [gets $fd]
	    scan $tmp "%d%d" nobs unit
	    if { $nobs != 1 } {
	       tk_messageBox -type ok -icon warning\
		   -message "Current version cannot read plural sources data in Geoct2 format."
	       close $fd
	       return
	    }
	    switch -- $unit {
	       1 {
		  # meter
		  set coef 0.001
	       }
	       2 {
		  # feet
		  set coef 0.0003048
	       }
	       default {
		  # km
		  set coef 1.0
	       }
	    }
	    set tmp [gets $fd]
	    set macray(n) [lindex $tmp 1]
	    set tmp [gets $fd]
	    set xoffset [expr [lindex $tmp 0] * $coef]
	    # read
	    for {set i 0} {$i < $macray(n)} {incr i} {
	       gets $fd tmp
	       scan $tmp "%*s%s%*s%s" x macray(t,$i)
	       set macray(x,$i) [expr $x * $coef - $xoffset]
	       set macray(m,$i) 1
	    }
	    catch {close $fd}
	 }
	 default {
	    return
	 }
      }
      print_message [format\
			 "%d points were read from  %s (in %s format)\n"\
			 $macray(n) $fn $fmt]
      set macray(mode) on
      set macray(button_message) "Invisible"
      plot
   }

   # -----------------------------------------------------------------
   # Toggle show synthetics
   # -----------------------------------------------------------------
   proc toggle {args} {
      # global --------------------
      global wk macray
      
      # main --------------------
      if { [llength $args] > 0 } {set macray(mode) "off"}
      
      if { $macray(mode) == "off" } {
	 # if invisible, i.e., macray(mode) is off
	 if {$macray(n) < 1} {
	    reread
	 }
	 set macray(mode) "on"
	 plot
	 set macray(button_message) "Invisible"
	 print_message "Synthetic Traveltimes were displayed.\n"
      } else {
	 # if visible, i.e. macray(mode) is on
	 set macray(mode) "off"
	 set macray(button_message) "Visible"
	 delete_synthetic $wk(w,current)
	 print_message "Synthetic Traveltimes were hidden.\n"
      }
   }
   
   # -----------------------------------------------------------------
   # Determine synthetic symbol color
   # -----------------------------------------------------------------
   proc color {type} {
      # global --------------------
      global macray wk
      
      # main --------------------
      switch $macray(color_mode) {
	 "0" {
	    # 1 color mode
	    return $macray(color)
	 }
	 "1" {
	    # Colors by type
	    return [lindex $wk(color,list) [expr ($type-1) % 20]]
	 }
	 "2" {
	    # manual setting 
	    if { [array names macray mcolor,$type] != "" } {
	       return $macray(mcolor,$type)
	    } else {
	       return $macray(color)
	    }
	 }
      }
   }
}

