#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <math.h>

#include "pasteup.h"

/** module variable **/
int total_channel;

/********************************************************************
Read THK data format
--------------------------------------------------------------------
إإå֥åեޥå
header[0]    trace length [sec]
header[1]    sampling frequency
header[2]    flag (0 ... old format, 1 ... new format)
header[3]    number of data channels
shot_inf[0]  shot id number
shot_inf[1]  shot time(year)
shot_inf[2]  shot time(month)
shot_inf[3]  shot time(day)
shot_inf[4]  shot time(hour)
shot_inf[5]  shot time(min)
shot_inf[6]  shot time(sec)
shot_inf[7]  shot time(msec)
shot_inf[8]  shot position(lat, deg)
shot_inf[9]  shot position(lat, 1/10000deg)
shot_inf[10] shot position(lon, deg) 
shot_inf[11] shot position(lon, 1/10000deg)
shot_inf[12] shot position(elevation) [m]
shot_inf[13] shot static correction [msec]
gain[0]      amp gain for ch.1 in dB
gain[1]      amp gain for ch.1 in dB
gain[2]      amp gain for ch.1 in dB
gain[3]      amp gain for ch.1 in dB
--------------------------------------------------------------------
إǡѡrecord_header ¤Τ
int shot_no;        åֹ  100000ߴ¬ֹ
int month;          åȻ()
int day;            åȻ()
int hour;           åȻ()
int min;            åȻ(ʬ)
int start_point;    ǡեǡϥݥ
int end_point;      ȥ졼Υץ
int water_depth;    åȰ֤ο忼
double sec;         åȻ
double dsec;        åȻ֤ȤΤ()
double distance;    åȤȣϣ£ӤοʿΥ
double sampling;    ץ󥰥졼
********************************************************************/
int THKheader2rh(FILE *fp_input, int bfmt,
		 double obslon, double obslat, int dist_axis)
{
    /********** Local Variables **********/
    int i, n, eof_stat, data_byte;
    short int header[4], shot_inf[14], gain[4];
    double lat, lon;
    struct tm tt;
    
    /********** Local Function **********/
    int thk_read_short(FILE *fp, int *end_of_file);
    double distcal(double lat, double lon,
		   double obslat, double obslon, int sgn);
    
    /**********************************************/
    /******************** Main ********************/
    /**********************************************/
    
    data_byte = (bfmt == FMT_4BYTE_FLOAT) ? 4 : bfmt;

    n = 0; /* number of shots **/
    while(1) {
	/********** إåɤ߹ **********/
	for(i=0;i<4;i++){
	    header[i] = thk_read_short(fp_input, &eof_stat);
	  }
	if (eof_stat == ON) break;
	/********** shot_inf ɤ߹ **********/
	for(i=0;i<14;i++){
	    shot_inf[i] = thk_read_short(fp_input, &eof_stat);
	}
	if (eof_stat == ON) break;
	/********** ٷ٤׻ **********/
	lat = shot_inf[8] + shot_inf[9]/10000.0;
	lon = shot_inf[10] + shot_inf[11]/10000.0;
	/********** gain ɤ߹ **********/
	for(i=0;i<4;i++){
	    gain[i] = thk_read_short(fp_input, &eof_stat);
	  }
	if (eof_stat == ON) break;
	/********** ǡƬ˰ư **********/
	fseek(fp_input, header[3]*header[1]*data_byte - 44, SEEK_CUR);
	
	/********** Ѵ  **********/
	gbl_rh[n].hw[RECORD_HEADER_TRACL]
	    = gbl_rh[n].hw[RECORD_HEADER_TRACR]
	    = gbl_rh[n].hw[RECORD_HEADER_FLDR]
	    = gbl_rh[n].hw[RECORD_HEADER_TRACF]
	    = shot_inf[0];
	gbl_rh[n].hw[RECORD_HEADER_EP]         = 0;
	tt.tm_year = shot_inf[1];
	tt.tm_mon  = shot_inf[2] -1;
	tt.tm_mday = shot_inf[3];
	tt.tm_hour = shot_inf[4];
	tt.tm_min  = shot_inf[5];
	tt.tm_sec  = shot_inf[6];
	gbl_rh[n].st = mktime(&tt);	

	gbl_rh[n].msec        = shot_inf[7];
	gbl_rh[n].dsec        = 0.0;
	gbl_rh[n].sampling    = header[1];
	gbl_rh[n].start_point = ftell(fp_input);
	gbl_rh[n].end_point   = header[0] * header[1]; /* Number of samples */
	gbl_rh[n].swdep       = 0;
	total_channel = header[3];
	
	/** ɸ׻ **/
	lat = shot_inf[8] + shot_inf[9]/10000.0;
	lon = shot_inf[10] + shot_inf[11]/10000.0;
	
	gbl_rh[n].x    = distcal(lat, lon, obslat, obslon, dist_axis);
	/********** ȷǡɤФ **********/
	fseek(fp_input, header[0]*header[1]*header[3]*data_byte, SEEK_CUR);
	n++;

	/***** require re-allocation ? *****/
	if(n >= maxtrace) {
	    maxtrace += 1000;
	    if(realloc_global_variable() == 1) {
		fprintf(stderr, "ERROR: Memory reallocation failed.\n");
		exit(1);
	    }
	}
    }
    return(n);
}

/*
  --------------------------------------------------------------------
*/

int thk_read_short(FILE *fp, int *end_of_file)
/********************************************************************
ե fp  2byte ǡɤ߹
********************************************************************/
{
    /***** ɽѿ *****/
    char buf[3];

    /***** ᥤ *****/
    *end_of_file = OFF;
    buf[0] = getc(fp);
    buf[1] = getc(fp);
    if(feof(fp)) {
	*end_of_file = ON;
	return(0);
    }
    return( ((buf[0]<<24) >>16) | (buf[1]&0x000000ff) );
}

/*
  --------------------------------------------------------------------
*/

/********************************************************************
Read waveform from the Tohoku-Univ format
--------------------------------------------------------------------
[arguments]
a           : amplitude array (for output)
t           : time of the first sample
fp          : input file
ch          : channel
bfmt        : data binary formmat (2:2byte int, 3:3byte int, 4:4byte int)
data_point1 : trace start point
data_point2 : trace end point
start_time  : cut out time (start)
end_time    : cut out time (end)
sampling    : sampling rate (Hz)
dt_clock    : clock error (sec)
********************************************************************/
int read_THK_data(float *a, float *t, FILE *fp, int ch, int bfmt,
		  int data_point1, int data_point2,
		  float start_time, float end_time,
		  float sampling, float dt_clock)
{
    /********** Local Variables **********/
    register int i, j;
    int i_start, i_end, count, address, val, data_byte;
    char buf[20];

    /********** Local Function **********/
    float mkval(char *buf, int num, int bfmt);

    /**************************/
    /********** Main **********/
    /**************************/

    data_byte = (bfmt == FMT_4BYTE_FLOAT) ? 4 : bfmt;
    ch = ch -1;     /* the first channel is 0 */

    /***** calc addresses *****/
    /** start address **/
    if ( start_time + dt_clock >= 0.0 ) {
	i_start = (start_time + dt_clock)*sampling;
    } else {
	i_start = 0;
    }
    /** end address **/
    i_end = ceil((end_time + dt_clock)*sampling);
    if (i_end > data_point2)
	i_end = data_point2;
    count = i_end - i_start;
    if (count < 1) return(0);
    address = data_point1 + i_start*data_byte*total_channel;

    /***** read *****/
    fseek(fp, address, SEEK_SET);
    switch(bfmt) {
	case FMT_2BYTE_INT:
	    *t = (float)i_start/sampling - dt_clock;
	    for(i=0;i<count;i++) {
		fread(buf, sizeof(char), total_channel*data_byte, fp);
		a[i] = mkval(buf, ch, data_byte);
	    }
	    break;
	case FMT_4BYTE_INT:
	    *t = (float)i_start/sampling - dt_clock;
	    for(i=0;i<count;i++) {
		fread(buf, sizeof(char), total_channel*data_byte, fp);
		j = ch*data_byte;
		val = buf[j]<<24 | ( (buf[j+1]<<16)&0x00ff0000) |
		    ((buf[j+2]<<8)&0x0000ff00)|(buf[j+3]&0x000000ff) ;
		a[i] = (float)val;
	    }
	    break;
    }
    return(count);
}

/*
  --------------------------------------------------------------------
*/

int get_THK_year(char *data_file)
/********************************************************************
 Return the year of observation
********************************************************************/
{
    /********** Local Variables **********/
    FILE *fp;
    int i, eof_stat;
    short int header[4], shot_inf[14];

    /********** Local Function **********/
    FILE *open_file(char *filename, char *file_type);
    int thk_read_short(FILE *fp, int *end_of_file);

    /**************************/
    /********** Main **********/
    /**************************/

    fp = open_file(data_file, "Data file in THK format");

    /********** Read header **********/
    for(i=0;i<4;i++){
	header[i] = thk_read_short(fp, &eof_stat);
    }
     /********** read shot_inf **********/
     for(i=0;i<14;i++){
	 shot_inf[i] = thk_read_short(fp, &eof_stat);
     }
     if (eof_stat == ON) {
	 fprintf(stderr, "Too short data file in get_THK_year.\n");
	 return(-1);
     }
     return((int)shot_inf[1]);
}
