# --------------------------------------------------------------------
# Invoke trace window
# --------------------------------------------------------------------
proc trace_window {} {
   # global --------------------
   global gbl wk paste pick custom
   
   # ------------------------------------------------------------
   # main -------------------------------------------------------

   # Invoke --------------------
   set w $wk(w,trace)
   if [winfo exists $w] {
      # already exist
      raise $w
      return
   } else {
      # invoke 
      set wk(trace,from) $paste(top)
      set wk(trace,to)   $paste(bottom)
      set wk(trace,amp)  1.0
   }
   toplevel $w
   wm resizable $w 1 1
   wm minsize $w 350 350
   wm title $w "Pasteup: Trace window."
   wm iconname $w "Pasteup: Trace window."
   
   # Title frame --------------------
   set tf [title_frame $w.title_frame "Pasteup: Trace Window"]
   
   # parameter frame --------------------
   set pf [frame $w.parameter_frame]
   # station, shot number and distance.
   set pfi $pf.trace_info_frame
   frame $pfi
   label $pfi.lstat -text "Station:"
   label $pfi.stat -text "" -width 8 -justify left
   label $pfi.lshot -text "Shot:"
   label $pfi.shot -text "" -width 8 -justify left
   label $pfi.ldist -text "Offset:"
   label $pfi.dist -text "" -width 8 -justify left
   
   set wk(trace,widget,station) $pfi.stat
   set wk(trace,widget,shot) $pfi.shot
   set wk(trace,widget,dist) $pfi.dist
   
   pack $pfi.lstat $pfi.stat $pfi.lshot $pfi.shot \
       $pfi.ldist  $pfi.dist -side left

   # left, right button/show pick, clip, use cache
   set pfb [frame $pf.button_frame]
   button $pfb.left -image left_icon -command "display_trace -1"\
       -relief flat
   bind $pfb.left <Enter> {balloon "Show left trace" %X %Y}
   button $pfb.right -image right_icon -command "display_trace +1"\
       -relief flat
   bind $pfb.right <Enter> {balloon "Show right trace" %X %Y}
   checkbutton $pfb.showpick -text "Show pick" -var paste(trace,showpick) \
       -command "display_trace 0"
   checkbutton $pfb.clip -text "Clip" -var paste(trace,clip)\
       -command "display_trace 0"
   checkbutton $pfb.usecache -text "Use Cache" -var paste(trace,usecache)\
       -command "display_trace 0"

   pack $pfb.left $pfb.right -side left
   pack $pfb.usecache $pfb.clip $pfb.showpick -side right

   # packing
   pack $pfi $pfb -side top -anchor w -fill x

   foreach b {left right} {
      bind $pfb.$b <Leave> {cancel_balloon}
   }
   
   # Area for waveform --------------------
   set wf $w.wave_frame
   frame $wf -relief groove -bd 2 -takefocus 1

   bind $wf <Enter> "focus $wf"
   bind $wf <Right> "display_trace +1"
   bind $wf <Control-f> "display_trace +1"
   bind $wf <Left> "display_trace -1"
   bind $wf <Control-b> "display_trace -1"
   bind $wf <Control-u> {undo_pop}
   bind $wf <Control-z> {undo_pop}
   bind $wf <Up> "trace_time_shift +10"
   bind $wf <Control-p> "trace_time_shift +10"
   bind $wf <Down> "trace_time_shift -10"
   bind $wf <Control-n> "trace_time_shift -10"
   bind $wf <plus> {trace_change_amp 1}
   bind $wf <minus> {trace_change_amp -1}

   # area for waveform -----
   canvas $wf.section -width $wk(trace,width) -height $wk(trace,height) \
       -bg white
   bind $wf.section <Button-2> {trace_pick_start %x %y}
   bind $wf.section <Button-3> {trace_pick_clear %x %y}
   bind $wf.section <Button-4> {trace_time_shift 10}
   bind $wf.section <Button-5> {trace_time_shift -10}
   bind $wf.section <Motion> {trace_cursor_position %x %y}
   set wk(trace,widget,canvas) $wf.section

   # Time-axis manipulate panel -----
   frame $wf.panel
   set widgets ""
   foreach {i j k} {up2 50 "up 50" up 20 "up 20"\
			down -20 "down 20" down2 -50 "down 50"} {
      button $wf.panel.$i -image $i\_arrow -relief flat\
	  -command "trace_time_shift $j"
      bind $wf.panel.$i <Enter> "balloon {Schroll $k%%} %X %Y"
      bind $wf.panel.$i <Leave> {cancel_balloon}
   }
   pack $wf.panel.up2 $wf.panel.up -side top
   pack $wf.panel.down2 $wf.panel.down -side bottom
   pack $wf.section -side left -fill both -expand 1
   pack $wf.panel -side right -fill y
   bind $wf.panel <Button-4> {trace_time_shift 10}
   bind $wf.panel <Button-5> {trace_time_shift -10}

   # Console --------------------
   set cf [frame $w.console -relief groove -bd 2]
   
   # cursor position
   label $cf.position -text "(amp=, t=)"
   grid $cf.position - - - -sticky nws
   set wk(trace,widget,position) $cf.position

   # Scale factor panel -----
   label $cf.lamp -text "Scaling factor" -anchor e
   entry $cf.amp -width 16 -textvar wk(trace,amp) -just right
   button $cf.amp_up -image magnify -command "trace_change_amp 1"
   button $cf.amp_down -image demagnify -command "trace_change_amp -1"
   button $cf.amp_auto -text "auto" -command "trace_auto_scale"
   bind $cf.amp <Return> {trace_draw_graph 0}
   grid $cf.lamp $cf.amp - x x $cf.amp_up $cf.amp_down $cf.amp_auto -sticky news

   # Time-axis panel -----
   label $cf.ltime -text "Time range" -anchor e
   entry $cf.from -width 8 -textvar wk(trace,from) -just right
   label $cf.lmiddle -text "   to   " -width 8
   entry $cf.to -width 8 -textvar wk(trace,to) -just right
   label $cf.unit -text "sec."
   button $cf.mag -image magnify -command "trace_change_time 1"
   button $cf.demag -image demagnify -command "trace_change_time -1"
   bind $cf.from <Return> {display_trace 0}
   bind $cf.to <Return> {display_trace 0}
   grid $cf.ltime $cf.from $cf.lmiddle $cf.to $cf.unit $cf.mag $cf.demag \
       -sticky news

   bind $w <w> {trace_change_time 1}
   bind $w <s> {trace_change_time -1}

   # aux line panel -----
   checkbutton $cf.laux -text "Aux. line" -anchor e \
       -var paste(trace,check_aux) -command "trace_aux_line"
   entry $cf.eaux -width 8 -textvar paste(trace,aux) -just right
   grid $cf.laux $cf.eaux -sticky news

   # Button frame --------------------
   set bf [button_frame $w $w.button_frame "Save as PS" trace_save_postscript]

   # packing --------------------
   pack $tf $pf -side top -fill x
   pack $bf $cf -side bottom -fill x
   pack $wf -side top -fill both -expand 1

   bind $w <Control-w> "destroy $w"

   # fore reconfigure window size
   update
   bind $w <Configure> "trace_reconf_window"
}
# --------------------------------------------------------------------
# Show a single trace in the trace window
# --------------------------------------------------------------------
# [arguments]
# mode : trace selection mode
#        0:closest, n:n-th right, -n:n-th left
# --------------------------------------------------------------------
proc display_trace {mode} {
   global wk

   if { ![winfo exists $wk(w,trace)]} {
      return
   }

   if [trace_check_parameter] {
      put_error "Invalid specification (trace window)."
      return
   }

   # Show a trace --------------------
   set status [trace_draw_graph $mode]

   # aux line
   trace_aux_line
}

# --------------------------------------------------------------------
# re-configure trace window (change size, position etc)
# --------------------------------------------------------------------
proc trace_reconf_window {} {
   global wk

   # resize canvas widget
   set w [winfo width $wk(trace,widget,canvas)]
   set h [winfo height $wk(trace,widget,canvas)]
   if { $w == $wk(trace,width) && $h == $wk(trace,height) } {return}
   set wk(trace,width) $w
   set wk(trace,height) $h

   # redraw
   exec_update_trace_window_size
   trace_coordinate
   trace_draw_graph 0
}

# --------------------------------------------------------------------
# trace_draw_frame
# --------------------------------------------------------------------
proc trace_draw_graph {mode} {
   # global --------------------
   global wk paste
   
   # ------------------------------------------------------------
   # main -------------------------------------------------------
   
   # read info and show trace
   set info [gettrace $wk(trace,current,dist) $mode]
   if { [llength $info] != 3 } {
      return
   }
   set wk(trace,current,stat) [lindex $info 0]
   set wk(trace,current,shot) [lindex $info 1]
   set wk(trace,current,dist) [lindex $info 2]
   
   # frame
   trace_draw_frame

   # picks
   if { $paste(trace,showpick) } {
      trace_display_pick
   }

   # update info --------------------
   # station name
   $wk(trace,widget,station) conf -text $wk(trace,current,stat)
   
   # shot number
   $wk(trace,widget,shot) conf -text $wk(trace,current,shot)
   
   # offset distance
   $wk(trace,widget,dist) conf -text [format "%.4f" $wk(trace,current,dist)]
}

# --------------------------------------------------------------------
# plot aux. line in trace window
# --------------------------------------------------------------------
proc trace_aux_line {} {
   # global --------------------
   global wk paste
   
   # ------------------------------------------------------------
   # main -------------------------------------------------------
   $wk(trace,widget,canvas) delete aux
   if { $paste(trace,check_aux) == 0 || $paste(trace,aux) <= 0.0 } { return }
   set xright [expr $wk(trace,coord,x_zero) + $paste(trace,aux)*$wk(trace,amp)]
   if { $xright >= $wk(trace,coord,x_right) } {return}
   set xleft [expr $wk(trace,coord,x_zero) - $paste(trace,aux)*$wk(trace,amp)]

   foreach x "$xright $xleft" {
      $wk(trace,widget,canvas) create line $x $wk(trace,coord,y_max) \
	  $x $wk(trace,coord,y_min) -width 1 -tags aux -fill $wk(color,auxline)
   }
}

# --------------------------------------------------------------------
# Check parameters for showing trace
# --------------------------------------------------------------------
proc trace_check_parameter {} {
   # global --------------------
   global gbl file_name paste wk

   # ------------------------------------------------------------
   # main -------------------------------------------------------

   # exist input files?
   if {[file readable $file_name(wave)] == 0} {
      put_error "Cannot find wave-from data file. Aborted."
      return 1
   }
   if {$gbl(data,type) == 1 && [file readable $file_name(rh)] == 0} {
      put_error "Cannot find Record Header file. Aborted."
      return 1
   }
   
   # check time axis
   if {$wk(trace,from) > $wk(trace,to)} {
      set tmp $wk(trace,from)
      set wk(trace,from) $wk(trace,to)
      set wk(trace,to) $tmp
   }
   if {$wk(trace,from) >= $wk(trace,to) ||\
	   [string length $wk(trace,from)] < 1} {
      put_error "Invalid time range in trace window."
      return 1
   }
   # compute coefficients
   set wk(trace,t_coef) [expr $wk(trace,graph_height)/\
			     ( $wk(trace,to) - $wk(trace,from) ) ]

   return 0
}

# --------------------------------------------------------------------
# Compute trace related coordinates
# --------------------------------------------------------------------
proc trace_coordinate {} {
   # gloabl --------------------
   global wk

   # ------------------------------------------------------------
   # main -------------------------------------------------------

   set t trace
   set tc trace,coord

   set wk($tc,graph_amp) [expr $wk(trace,graph_width) / 2 ]
   set wk($tc,x_zero) [expr $wk($t,left_margin) + $wk($tc,graph_amp)]
   set wk($tc,x_left) [expr $wk($t,left_margin)]
   set wk($tc,x_right) [expr $wk($t,left_margin) + $wk(trace,graph_width)]
   set wk($tc,y_min) [expr $wk($t,top_margin)]
   set wk($tc,y_max) [expr $wk($tc,y_min) + $wk(trace,graph_height)]
}

# --------------------------------------------------------------------
# Draw trace window frame
# --------------------------------------------------------------------
proc trace_draw_frame {} {
   # global --------------------
   global wk

   # ------------------------------------------------------------
   # main -------------------------------------------------------

   set t trace
   set c coord
   set w $wk($t,widget,canvas)
   
   # show region --------------------
   $w create rectangle $wk($t,$c,x_left) $wk($t,$c,y_min)\
       $wk($t,$c,x_right) $wk($t,$c,y_max)

   # zero level --------------------
   $w create line $wk($t,$c,x_zero) $wk($t,$c,y_min)\
       $wk($t,$c,x_zero) $wk($t,$c,y_max) -dash {4 4}

   # tick --------------------
   # amplitude -----
   set amp [expr $wk($t,$c,graph_amp)/$wk($t,amp)]
   set damp [trace_comp_amp_interval $amp]
   trace_plot_xtick 0.0
   for {set a $damp} {$a <= $amp} {set a [expr $a + $damp]} {
      trace_plot_xtick $a
      trace_plot_xtick [expr -$a]
   }

   # time -----
   set period [expr abs($wk($t,from) - $wk($t,to))]
   set dt [trace_comp_t_interval $period]
   for {set j $wk($t,from)} {$j <= $wk($t,to)} {set j [expr $j + $dt]} {
      trace_plot_ttick $j
   }
}

# --------------------------------------------------------------------
# trace_plot_xtic
#      plot tick mark and label at amplitude a
# --------------------------------------------------------------------
# a : amplitude
# --------------------------------------------------------------------
proc trace_plot_xtick {a} {
   # gloabl --------------------
   global wk
   
   # ------------------------------------------------------------
   # main -------------------------------------------------------
    
   # coordinate --------------------
   set x [expr $wk(trace,coord,x_zero) + $a*$wk(trace,amp)]
   
   # tick mark --------------------
   $wk(trace,widget,canvas) create line $x $wk(trace,coord,y_min)\
       $x [expr $wk(trace,coord,y_min) - $wk(trace,tick_length)]\
       -width 0.5 -tag trace_frame
   
   $wk(trace,widget,canvas) create line $x $wk(trace,coord,y_max)\
       $x [expr $wk(trace,coord,y_max) + $wk(trace,tick_length) + 1]\
       -width 0.5 -tag trace_frame
   
   # tick label --------------------
   $wk(trace,widget,canvas) create text\
       $x [expr ($wk(trace,top_margin) - $wk(trace,tick_length)) / 2.0]\
       -anchor c -text [format "%g" $a] -font font3 -tag trace_frame
}
# --------------------------------------------------------------------
# trace_plot_ttic
#      Plot tick mark and tick label
# --------------------------------------------------------------------
# j : time
# --------------------------------------------------------------------
proc trace_plot_ttick {j} {
   # global --------------------
   global wk
   
   # ------------------------------------------------------------
   # main -------------------------------------------------------
   
   # coordinate --------------------
   set y [trace_time2y $j]
   
   # tick mark --------------------
   $wk(trace,widget,canvas) create line\
       $wk(trace,coord,x_left) $y\
       [expr $wk(trace,coord,x_left) - $wk(trace,tick_length)] $y\
       -width 0.5 -tag trace_frame
   
   $wk(trace,widget,canvas) create line\
       $wk(trace,coord,x_right) $y\
       [expr $wk(trace,coord,x_right) + $wk(trace,tick_length) + 1] $y\
       -width 0.5 -tag trace_frame
   
   # tick label --------------------
   $wk(trace,widget,canvas) create text\
       [expr $wk(trace,left_margin) - $wk(trace,tick_length) -2] $y\
       -anchor e -text [format "%g" $j] -font font3 -tag trace_frame
}

# --------------------------------------------------------------------
# trace_time2y
#    Convert time j into canvas coordinate y
# --------------------------------------------------------------------
# j  : time
# --------------------------------------------------------------------
proc trace_time2y {j} {
   # global --------------------
   global wk gbl
   
   # ------------------------------------------------------------
   # main -------------------------------------------------------
   if {$gbl(t_axis) > 0} {
      # t-axis downward
      return [expr $wk(trace,coord,y_min)\
		  + ( $j - $wk(trace,from) ) * $wk(trace,t_coef)]
   } 
   # t-axis upward
   return [expr $wk(trace,coord,y_max)\
	       - ( $j - $wk(trace,from) ) * $wk(trace,t_coef)]
}

# --------------------------------------------------------------------
# trace_y2time
#    Convert canvas-y into (non-reduced) time
# --------------------------------------------------------------------
# y  : canvas y
# --------------------------------------------------------------------
proc trace_y2time {y} {
   # gloabl --------------------
   global wk gbl paste

   # ------------------------------------------------------------
   # main -------------------------------------------------------
   set y [expr $y * 1.0]
   if {$gbl(t_axis) > 0} {
      # t-axis downward
      set t [expr ($y - $wk(trace,coord,y_min)) / $wk(trace,t_coef)\
		 + $wk(trace,from)]
   } else {
      # t-axis upward
      set t [expr ($wk(trace,coord,y_max) - $y) / $wk(trace,t_coef)\
		 + $wk(trace,from)]
   }
   # un-reduce
   if {$paste(reduction)!=0.0} {
      return [expr $t + abs($wk(trace,current,dist))/$paste(reduction)]
   }
   return $t
}

# --------------------------------------------------------------------
# Convert canvas-x into amplitude
# --------------------------------------------------------------------
proc trace_x2amp {cx} {
   global wk 
   return [expr ($cx - $wk(trace,coord,x_zero)) / $wk(trace,amp)]
}

# --------------------------------------------------------------------
# trace_comp_amp_interval
#    Compute amp-tick interval from the maximum amplitude
# --------------------------------------------------------------------
# amp : max amplitude
# --------------------------------------------------------------------
proc trace_comp_amp_interval {amp} {
   # global --------------------
   global wk
   
   # ------------------------------------------------------------
   # main -------------------------------------------------------
   
   # number of tick marks in oneside is 2 or 3
   set order [expr pow(10, floor( log10($amp) ) )]
   set n [expr int( $amp/$order )]
   switch -regexp [expr $n%10] {
      [01] {
	 return [expr $order/2.0]
      }
      [23] {
	 return $order
      }
      [456] {
	 return [expr $order * 2.0]
      }
      [789] {
	 return [expr $order * 2.5]
      }
   }
   return 0
}
# --------------------------------------------------------------------
# trace_comp_t_interval
#    Compute appropriate t-tic interval from show period
# --------------------------------------------------------------------
# period : show period
# --------------------------------------------------------------------
proc trace_comp_t_interval {period} {
   # global --------------------
   global wk
   
   # ------------------------------------------------------------
   # main -------------------------------------------------------
   
   # number of tick marks is 5 to 10.
   set order [expr pow(10, floor( log10($period) ))]
   set n [expr int( $period/$order )]
   if {$n < 5} {
      set order [expr $order/10.0]
      set n [expr int( $period/$order )]
   }
   if {$n > 20} {
      set order [expr $order*2.5]
      set n [expr int( $period/$order )]
   }
   if {$n > 10} {
      set order [expr $order*2.0]
      set n [expr int( $period/$order )]
   }
   return [expr $order]
}

# --------------------------------------------------------------------
# trace_change_time
#    zoom (flag==1) or zoom out (flag==-1)
# --------------------------------------------------------------------
proc trace_change_time {flag} {
   # global --------------------
   global wk
   
   # main -------------------------------------------------------
   set range [expr $wk(trace,to) - $wk(trace,from)]
   if { $range < 0.0 } { return }
   if {$flag > 0} {
      # zoom
      if { $range > 10.0 } {
	 set range [expr int($range) - $range/10]
      } elseif { $range > 3.0 } {
	 set range [expr int($range) - 1]
      } elseif { $range > 1.0 } {
	 set range [expr int($range) - 0.5]
      } elseif { $range > 0.3 } {
	 set range [expr int($range*10.0)/10.0 - 0.1]
      } elseif { $range > 0.05 } {
	 set range [expr int($range*100.0)/100.0 - 0.05]
      } else {
	 set range 0.01
      }
      set wk(trace,to) [expr $wk(trace,from) + $range]
   } else {
      # zoom out
      if { $range < 0.01} {
	 set range 0.01
      } elseif { $range < 0.1 } {
	 set range [expr int($range*100)/100.0 + 0.01]
      } elseif { $range < 1.0 } {
	 set range [expr int($range*10)/10.0 + 0.1]
      } elseif { $range < 3.0 } {
	 set range [expr int($range*10)/10.0 + 0.5]
      } elseif { $range < 10.0 } {
	 set range [expr int($range) + 1.0]
      } else {
	 set range [expr int($range) + int($range/10)]
      }
      set wk(trace,to) [expr $wk(trace,from) + $range]
   }
   display_trace 0
}

# --------------------------------------------------------------------
# trace_change_amp
#    Amplify (flag==1) or Attenuate (flag==-1)
# --------------------------------------------------------------------
proc trace_change_amp {flag} {
   # global --------------------
   global wk
   
   # ------------------------------------------------------------
   # main -------------------------------------------------------

   if {$flag > 0} {
      set wk(trace,amp) [expr $wk(trace,amp) * 2.0]
   } else {
      set wk(trace,amp) [expr $wk(trace,amp) / 2.0]
   }
   
   trace_draw_graph 0
   trace_aux_line
}

# --------------------------------------------------------------------
# trace_time_shift
# --------------------------------------------------------------------
# s : shift direction and quantity (percentile)
# --------------------------------------------------------------------
proc trace_time_shift {s} {
   # global --------------------
   global wk gbl

   # ------------------------------------------------------------
   # main -------------------------------------------------------
   if {$gbl(t_axis) > 0} {
      # in case of downward time axis
      set s [expr - $s]
   }
   set period [expr $wk(trace,to) - $wk(trace,from)]
   foreach i {to from} {
      set wk(trace,$i) [expr $wk(trace,$i) + $period*$s/100.0]
   }
   display_trace 0
}

# --------------------------------------------------------------------
# trace_display_pick  : show pick mark
# --------------------------------------------------------------------
proc trace_display_pick {} {
   # global --------------------
   global wk pm paste pick
   
   # ------------------------------------------------------------
   # body -------------------------------------------------------
   
   set t trace
   set c coord
   
   # Exist trace window ?
   if {![winfo exist .trace_window] ||  $wk($t,current,dist) == ""} {
      return
   }
   # delete
   $wk($t,widget,canvas) delete trace_pick_accuracy trace_pick
   
   foreach d $pm {
      scan $d "%d%s%s%s%d" sn px pt acc type
      if {[expr abs($px - $wk($t,current,dist)) < 0.001] } {
	 # reduce
	 if {$paste(reduction) > 0.0} {
	    set pt [expr $pt - abs($wk($t,current,dist))/$paste(reduction)]
	 }
	 # accuracy
	 set t1 [expr $pt - $acc]
	 set t2 [expr $pt + $acc]
	 if { $t1 > $wk($t,to) || $t2 < $wk($t,from) } {
	    continue
	 }
	 if { $t1 < $wk($t,from) } { set t1 $wk($t,from) }
	 if { $t2 > $wk($t,to) } { set t2 $wk($t,to) }
	 $wk($t,widget,canvas) create rectangle\
	     $wk($t,$c,x_left) [trace_time2y $t1]\
	     $wk($t,$c,x_right) [trace_time2y $t2]\
	     -fill gray -tags trace_pick_accuracy -width 0 -outline {}
	 # mark
	 if { ($pt - $wk($t,from))*($wk($t,to) - $pt) >= 0.0 } {
	    $wk($t,widget,canvas) create line\
		$wk($t,$c,x_left) [trace_time2y $pt]\
		$wk($t,$c,x_right) [trace_time2y $pt]\
		-tags trace_pick -width 1 -fill $pick(color,$type)
	 }
      }
   }
   $wk($t,widget,canvas) lower trace_pick_accuracy
}

# --------------------------------------------------------------------
# show current position
# --------------------------------------------------------------------
proc trace_cursor_position {x y} {
   # global --------------------
   global wk
   
   # main --------------------
   set t [trace_y2time $y]
   set amp [trace_x2amp $x]
   $wk(trace,widget,position) conf -text [format "(amp=%9g,  t=%5g)" $amp $t]
}

# --------------------------------------------------------------------
# start pick in the trace window
# --------------------------------------------------------------------
proc trace_pick_start {pick_x pick_y} {
   # global --------------------
   global wk
   
   # main --------------------
   # save coordinates in the array
   set wk(trace,pick,status) on
   set wk(trace,pick,start_y) $pick_y
   
   # detemine bindings
   bind $wk(trace,widget,canvas) <B2-Motion> {
      trace_pick_motion %x %y
   }
   bind $wk(trace,widget,canvas) <ButtonRelease-2> {
      trace_pick_end %x %y
   }
}

# --------------------------------------------------------------------
# during pick...
# --------------------------------------------------------------------
proc trace_pick_motion {x y} {
   # global --------------------
   global wk pick
   
   # main --------------------
   if {[string compare $wk(trace,pick,status) "on"] != 0} {
      bind $wk(trace,widget,canvas) <B2-Motion> ""
      bind $wk(trace,widget,canvas) <ButtonRelease-2> ""
      return
   }
   catch {$wk(trace,widget,canvas) delete temp_mark}
   # pick region
   $wk(trace,widget,canvas) create rectangle\
       $wk(trace,coord,x_left) $wk(trace,pick,start_y)\
       $wk(trace,coord,x_right) $y\
       -fill gray -tags temp_mark -width 0
   $wk(trace,widget,canvas) lower temp_mark
   set cy [expr ($wk(trace,pick,start_y) + $y)/2.0]
   $wk(trace,widget,canvas) create line\
       $wk(trace,coord,x_left) $cy $wk(trace,coord,x_right) $cy\
       -fill $pick(color,$pick(current_type)) -tags temp_mark -width 1
}
# --------------------------------------------------------------------
# pick end in the trace window
# --------------------------------------------------------------------
proc trace_pick_end {mx my} {
   # global --------------------
   global wk pm mark_modification pick paste
   
   # main --------------------
   # delete temporary marks
   $wk(trace,widget,canvas) delete temp_mark
   bind $wk(trace,widget,canvas) <B2-Motion> ""
   bind $wk(trace,widget,canvas) <ButtonRelease-2> ""
   
   # pick time
   set t0 [trace_y2time [expr ($my + $wk(trace,pick,start_y))/2.0]]
   set t1 [trace_y2time $wk(trace,pick,start_y)]
   set t2 [trace_y2time $my]
   set acc [expr abs($t1 - $t2) / 2.0]
   if { $acc < $pick(default_accuracy) } {
      set acc $pick(default_accuracy)
   }
   
   # pick data update
   undo_stack
   set sn $wk(trace,current,shot)
   set px $wk(trace,current,dist) 
   lappend pm "$sn $px $t0 $acc $pick(current_type)"
   set mark_modification 1
   
   # draw mark in the canvas
   set y1 [coord tt2cy $px [expr $t0 + $acc] ]
   set y2 [coord tt2cy $px [expr $t0 - $acc] ]
   pick plot $px $t0 $acc $pick(current_type)
   $wk(w,canvas,c) lower acc
   
   # draw mark in the trace window
   set paste(trace,showpick)
   trace_display_pick
   
   # show info in the message window
   print_message [format "Shot #%4d : Location(%fkm, %fsec), Accuracy %fsec, Pick Type%d\n"\
		      $sn $px $t0 $acc $pick(current_type)]
}
# --------------------------------------------------------------------
# Erase pick in the trace window
# --------------------------------------------------------------------
proc trace_pick_clear {mx my} {
   # global --------------------
   global wk paste pick pm gbl mark_modification
   
   # main --------------------
   set n [trace_nearest_pick $my]
   if {$n == -1} {
      print_message "No shot around pick to DELETE.\n"
      return
   }
   scan [lindex $pm $n] "%d%s%s%s%d" sn px pt acc type
   print_message [format "Pick was deleted(%10.5f km, %10.5f sec., shot %d)\n"\
	$px $pt $sn]
   undo_stack
   exec_clear_pick $n
   plot_pick_marks 0
   set mark_modification 1
}

# --------------------------------------------------------------------
# trace_nearest_pick
#    search nearest pick (y-range)
# --------------------------------------------------------------------
proc trace_nearest_pick {y} {
   # global --------------------
   global wk pm paste
   
   # ------------------------------------------------------------
   # main -------------------------------------------------------
   
   set t trace
   set c coord
   
   set min_id -1
   set min_dist 1000 ;  # must be larger than the threshold
   set i 0
   foreach d $pm {
      scan $d "%d%s%s%s%d" sn px pt acc type
      if {[expr abs($px - $wk(trace,current,dist)) < 0.001] } {
	 # reduce
	 if {$paste(reduction)!=0.0} {
	    set dt [expr abs($wk(trace,current,dist))/$paste(reduction)]
	 } else {
	    set dt 0.0
	 }
	 set t0 [expr $pt - $dt]
	 set dist [expr abs($y - [trace_time2y $t0])]
	 if { $dist < 10 } { # threshold is 10 pixel
	    set min_dist $dist
	    set min_id $i
	 }
      }
      incr i
   }
   return $min_id
}

# --------------------------------------------------------------------
# save trace-graph as a PostScript file
# --------------------------------------------------------------------
proc trace_save_postscript {} {
   # global --------------------
   global wk
   
   # ------------------------------------------------------------
   # main -------------------------------------------------------
   if { [file_selection wk(trace,psfile) save ps] == "cancel" } {
      print_message "Cancel : save trace-graph.\n"
      return
   }

   $wk(trace,widget,canvas) postscript -fontmap fontmap -file $wk(trace,psfile)
   print_message "\"$wk(trace,psfile)\" was created.\n"
}

# --------------------------------------------------------------------
# set scaling factor automaticaly
# --------------------------------------------------------------------
proc trace_auto_scale {} {
   global wk

   # read trace
   set info [gettrace $wk(trace,current,dist) 9999]
   if { [llength $info] != 4 } {
      return
   }
   set maxamp [lindex $info 3]

   # calc scaling factor
   if { $maxamp <= 0.0 } { return }
   set saturate_amp [expr $wk(trace,graph_width) / $maxamp]
   set wk(trace,amp) [expr pow(10, floor( log10($saturate_amp) ))]

   # update trace
   trace_draw_graph 0
   trace_aux_line
}