# --------------------------------------------------------------------
# update "mode" variable
# --------------------------------------------------------------------
proc update_mode {} {
   # global
   global gbl paste wk
   
   # main
   switch -glob $gbl(mode_label) {
      "Wiggle" {
	 set gbl(mode) $wk(DRAW_MODE,WIGGLE_MODE)
	 set gbl(line_width) 1
	 set paste(image,mode) $gbl(option,default_style_va)
	 set name "Wiggle"
      }
      "VA" {
	 set gbl(mode) $wk(DRAW_MODE,VA_MODE)
	 set gbl(line_width) 1
	 set paste(image,mode) $gbl(option,default_style_va)
	 set name "VA"
      }
      "VA(R&B)" {
	 set gbl(mode) $wk(DRAW_MODE,VA_REDBLUE_MODE)
	 set gbl(line_width) 1
	 set paste(image,mode) $gbl(option,default_style_va)
	 set name "VA"
      }
      "VA(w/o wiggle)" {
	 set gbl(mode) $wk(DRAW_MODE,VA_WITHOUT_WIGGLE)
	 set gbl(line_width) 1
	 set paste(image,mode) $gbl(option,default_style_va)
	 set name "VA"
      }
      "VA(w/o wig RB)" {
	 set gbl(mode) $wk(DRAW_MODE,VA_WO_WIGGLE_RB)
	 set gbl(line_width) 1
	 set paste(image,mode) $gbl(option,default_style_va)
	 set name "VA"
      }
      "VD" {
	 set paste(image,mode) $gbl(option,default_style_vd)
	 set gbl(mode) $wk(DRAW_MODE,VD)
	 update_vdcolor
	 set name "VD"
      }
      "Wiggle on VD" {
	 set gbl(mode) $wk(DRAW_MODE,WIGGLE+VD)
	 if { $paste(image,mode) == 0 } { set paste(image,mode) 2 }
	 set name "Wiggle+VD"
      }
      "VA on VD" {
	 set gbl(mode) $wk(DRAW_MODE,VA+VD)
	 if { $paste(image,mode) == 0 } { set paste(image,mode) 2 }
	 set name "VA+VD"
      }
      default {
	 puts stderr "Invalid mode specification..."
      }
   }
   switch -- $paste(image,mode) {
      0 {
	 set paste(image,mode,label) "Vector"
      }
      1 {
	 set paste(image,mode,label) "Image"
      }
      2 {
	 set paste(image,mode,label) "Interpolate"
      }
   }
   update_image_mode
   update_displaystyle_menu $name
   binding
   # update widgets ----------
   # clip
   set wlist [lsearch -regexp -inline -all\
		  [winfo children $wk(w,console,misc)] amp_saturate]
   set stat [expr { $gbl(mode) == 10 } ? {"disabled"} : {"normal"}]
   foreach w $wlist { $w conf -state $stat }
   # va related
   set wlist [lsearch -regexp -inline -all\
		  [winfo children $wk(w,console,misc)] va_threshold]
   set stat [expr {$gbl(mode) == 10 || $gbl(mode) == 1 || $gbl(mode) == 110} ?\
		 {"disabled"} : {"normal"}]
   foreach w $wlist { $w conf -state $stat }
   # wiggle/va/vd amplitude ratio
   set wlist [lsearch -regexp -inline -all [winfo children $wk(w,console,amp)]\
		  ratio]
   set stat [expr { $gbl(mode) < 100 } ? {"disabled"} : {"normal"}]
   foreach w $wlist { $w conf -state $stat }
   # vd example
   set stat [expr { $gbl(mode) < 10 } ? {"disabled"} : {"normal"}]
   $wk(w,console).frame(plot).plot.mb(vdcolor) conf -state $stat
}
# --------------------------------------------------------------------
# update vd color mode
# --------------------------------------------------------------------
proc update_vdcolor {} {
   global gbl wk
   # main ------------------------------
   # update menu
   set w  $wk(w,console).frame(plot).plot.mb(vdcolor)
   $w conf -image dsicon($gbl(vdcolor)) -relief raised -bd 2 -width 10\
       -menu $w.menu
   # update VD-icon
   dsicon($wk(DRAW_MODE,VD)) copy dsicon($gbl(vdcolor))
   # update superimposed-icon
   dsicon($wk(DRAW_MODE,WIGGLE+VD)) copy dsicon($gbl(vdcolor))
   dsicon($wk(DRAW_MODE,WIGGLE+VD)) copy dsicon($wk(DRAW_MODE,WIGGLE_MODE))\
       -composit overlay
   dsicon($wk(DRAW_MODE,VA+VD)) copy dsicon($gbl(vdcolor))
   dsicon($wk(DRAW_MODE,VA+VD)) copy dsicon($wk(DRAW_MODE,VA_MODE))\
       -composit overlay

   # update others
   update_displaystyle_menu
}
# --------------------------------------------------------------------
# update "display style" menu
# --------------------------------------------------------------------
proc update_displaystyle_menu {args} {
   global gbl wk
   # main ------------------------------
   set w  $wk(w,console).frame(plot).plot.mb(disp_style)
   set name [expr { [llength $args] > 0 } ? { $args } : { [$w cget -text] } ]
   if { $gbl(mode) == 10 } {
      $w conf -relief raised -bd 2 -width 15 -menu $w.menu -text $name -image ""
   } else {
      $w conf -image dsicon($gbl(mode)) -relief raised -bd 2 -width 15\
	  -menu $w.menu -compound left -text $name
   }
   # update VD-icon
   $w.menu entryconfigure 5 -image dsicon($gbl(vdcolor))
}
# --------------------------------------------------------------------
# update "data type(format)"
# --------------------------------------------------------------------
proc update_data_type {} {
   # global
   global gbl wk file_name
   
   # main ------------------------------
   switch $gbl(data,type,label) {
      "KASA-lab" {
	 # KASA-lab
	 set gbl(data,type) 1
	 console_data_frame 1 1 0 0 0
	 set wk(data,bytelist)\
	     {2 "2byte integer" 3 "3byte integer" 5 "4byte float"}
	 set fwave ./ch1.orig
	 set frh ./rh.dat
      }
      "SU" {
	 # SU
	 set gbl(data,type) 2
	 console_data_frame 1 0 1 0 0
	 set wk(data,bytelist) {14 "4byte float"}
	 set fwave [su_search_datafile "."]
      }
      "THK" {
	 # THK
	 set gbl(data,type) 3
	 console_data_frame 1 0 0 1 1
	 set wk(data,bytelist) {2 "2byte integer" 4 "4byte integer"}
	 set fwave [thk_search_datafile "."]
      }
      "CHIBA" {
	 # CHIBA
	 set gbl(data,type) 4
	 console_data_frame 1 1 0 1 0
	 set wk(data,bytelist) {2 "2byte integer"}
	 set fwave [chiba_search_datafile "."]
	 set frh [chiba_search_rhfile "."]
      }
      "SEGY(1Ch)" {
	 # SEGY(1Ch)
	 set gbl(data,type) 6
	 console_data_frame 1 0 1 0 0
	 set wk(data,bytelist) {4 "4byte integer" \
				    -4 "4byte integer(swap)" \
				    15 "4byte IBM float" \
				    -15 "4byte IBM float(swap)" \
				    3 "3byte integer" \
				    -3 "3byte integer(swap)" \
				 }
      }
      default {
	 tk_messageBox -icon error -type ok\
	     -message "Unknown data type \"$gbl(data,type,label)\""
	 return
      }
   }
   # update "data byte"
   set tmp {}
   foreach {i j} $wk(data,bytelist) {
      lappend tmp $i
   }
   set i [lsearch $tmp $gbl(data,byte)] 
   if {  $i < 0 } {
      set gbl(data,byte) [lindex $wk(data,bytelist) 0]
      set i 0
   }
   set wk(data,byte,label) [lindex $wk(data,bytelist) [expr $i*2 + 1]]
   # update "data byte menu"
   set m $wk(data,byte,widget)
   $m delete 0 end
   foreach {i j} $wk(data,bytelist) {
      $m add radiobutton -value $i -label $j -variable gbl(data,byte)\
	  -command "set wk(data,byte,label) \"$j\""
   }
   # update filename 
   if { $gbl(data,prev_type) >= 0 && $gbl(data,type) != $gbl(data,prev_type)} {
      if [info exist fwave] { set file_name(wave) $fwave }
      if [info exist frh] { set file_name(rh) $frh }
   }
   if {$gbl(data,type) != $gbl(data,prev_type)} {
      set gbl(data,prev_type) $gbl(data,type)
   }
   # update SU filter
   enabling_su_filter_frame
}

# --------------------------------------------------------------------
# update gbl(image,mode) variable
# --------------------------------------------------------------------
proc update_image_mode {} {
   # global
   global paste gbl wk
   
   # main
   switch -- $paste(image,mode,label) {
      "Vector" {
	 set paste(image,mode) 0
      }
      "Image" {
	 set paste(image,mode) 1
      }
      "Interpolate" {
	 set paste(image,mode) 2
	 if { $gbl(mode) < 10 } {
	    set gbl(mode) $wk(DRAW_MODE,VD)
	    set gbl(mode_label) VD
	    update_mode
	 }
      }
   }
   if {$paste(image,mode) == 2 && $gbl(mode) < 100} {
      foreach w [winfo children $wk(w,console,decim)] {
	 $w conf -state disabled
      }
      $wk(w,console).frame(plot).plot.conf(mode) conf -state normal
   } else {
      foreach w [winfo children $wk(w,console,decim)] {
	 $w conf -state normal
      }
      $wk(w,console).frame(plot).plot.conf(mode) conf -state disabled
   }
   set wlist [lsearch -regexp -inline -all\
		  [winfo children $wk(w,console,misc)] line_width]
   if {$paste(image,mode) == 2 && $gbl(mode) < 100 } {
      foreach w $wlist {$w conf -state disabled}
   } else {
      foreach w $wlist {$w conf -state normal}
   }
}

# --------------------------------------------------------------------
# Update paste(amp,correct) 
# --------------------------------------------------------------------
proc update_amp_correct {} {
   # global 
   global paste gbl wk
   
   # Main
   if { $gbl(check,amp) == 0 } {
      set paste(amp,correct) 0
   } else {
      switch [string tolower $paste(amp,correct_label)] {
	 "geometric" {
	    set paste(amp,correct_label) "Geometric"
	    set paste(amp,correct) 3
	 }
	 "relative" {
	    set paste(amp,correct_label) "Relative"
	    set paste(amp,correct) 2
	 }
	 "agc" {
	    set paste(amp,correct_label) "AGC"
	    set paste(amp,correct) 1
	 }
	 default {
	    tk_messageBox -icon error -title "Invalid amplitude correction" \
		-message "$paste(amp,correct_label) is not valid amplitude correction mode"\
		-type ok
	    set paste(amp,correct_label) "AGC"
	    set paste(amp,correct) 1
	    set gbl(check,amp) 0
	 }
      }
   }

   if {$paste(amp,correct) == 0} {
      $wk(w,console,amp).amp_mode conf -state disabled
   } else {
      $wk(w,console,amp).amp_mode conf -state normal
   }
   if {$paste(amp,correct) == 1} { ; # AGC mode
      $wk(w,console,amp).entry(agc) conf -state normal
      $wk(w,console,amp).label(agc) conf -state normal
   } else {
      $wk(w,console,amp).entry(agc) conf -state disabled
      $wk(w,console,amp).label(agc) conf -state disabled
   }
   if {$paste(amp,correct) == 3} { ; # Geometric
      $wk(w,console,amp).geo_conf conf -state normal
   } else {
      $wk(w,console,amp).geo_conf conf -state disabled
   }
}

# --------------------------------------------------------------------
# Update paper size : ps_page_short, ps_page_long 
# --------------------------------------------------------------------
proc update_paper_size {} {
   # Global variables
   global ps_page_long ps_page_short print

   # Main
   switch -glob $print(size) {
      "A4" {
	 set ps_page_short 21
	 set ps_page_long 29.7
      }
      "A3" {
	 set ps_page_short 29.7
	 set ps_page_long 42
      }
      "A2" {
	 set ps_page_short 42
	 set ps_page_long 59.4
      }
      "A1" {
	 set ps_page_short 59.4
	 set ps_page_long 84
      }
      "B5" {
	 set ps_page_short 17.6
	 set ps_page_long 25
      }
      default {
	 puts stderr "Invalid paper size..."
      }
   }
}
# --------------------------------------------------------------------
# update canvas window size
# --------------------------------------------------------------------
proc update_canvas_size {} {
   exec_update_canvas_size
   exec_update_trace_window_size
   draw_options
}
# --------------------------------------------------------------------
# update canvas window size
# --------------------------------------------------------------------
proc update_figure_title {} {
   global paste
   set paste(label,title) [make_title]
}
# --------------------------------------------------------------------
# update print format (postscript/png/jpeg)
# --------------------------------------------------------------------
proc update_print_format {w} {
   global print file_name
   $w.entry(density) conf -state normal
   switch -- $print(format) {
      PostScript {
	 set file_name(postscript) [change_suffix $file_name(postscript) .ps]
	 $w.entry(density) conf -state disabled
      }
      PNG {
	 set file_name(postscript) [change_suffix $file_name(postscript) .png]
      }
      JPEG {
	 set file_name(postscript) [change_suffix $file_name(postscript) .jpg]
      }
   }
}
# --------------------------------------------------------------------
# update_canvas_size_variables
# --------------------------------------------------------------------
proc update_canvas_size_variables {} {
   # global variables --------------------
   global paste fontmap
   
   # criterion
   # [vertical]  : spare 30% blank for each strings
   # [horizontal]: spare 10% blank for each strings

   # Main --------------------
   # sub plot
   if { $paste(sub,check) } {
      if { $paste(sub,position) == "top" } {
	 set subtop [expr $paste(sub,height) + $paste(tic_size) * 2]
	 set subbot 0
      } else {
	 set subtop 0
	 set subbot [expr $paste(sub,height) + $paste(tic_size) * 2]
      }
   } else {
      set subtop 0
      set subbot 0
   }

   # font size
   set s [tk scaling]
   set s1 [expr ceil( [lindex $fontmap($paste(fn,title)) end] / $s ) ]
   set s2 [expr ceil( [lindex $fontmap($paste(fn,axis)) end] / $s ) ]
   set s3 [expr ceil( [lindex $fontmap($paste(fn,tick)) end] / $s ) ]
   set ts $paste(tic_size)
   
   set paste(top_margin)\
       [expr $paste(check,toplabel) ?\
	    {int( 1.3*($s1 + $s2 + $s3) + $ts + $subtop )} :\
	    {int(1.3*($s1 + $s3 + $ts) + $subtop )} ]
   set paste(left_margin)\
       [expr { $paste(sub,check) } ?\
	    int(1.1*($s3*5) + $ts ) :\
	    int(1.1*($s3*3) + $ts ) ]
   set BM [expr $paste(check,bottomlabel) ?\
	       {int( 1.3*($s1 + $s2 + $s3) + $ts + $subbot )} :\
	       {int( 1.3*($s1 + $s3) + $ts + $subbot )} ]
   set RM [expr 1.1*($s3*2) + $ts]
   # Canvas sizes
   set paste(canvas_height)\
       [expr $paste(top_margin) + $paste(graph_height) + $BM]
   set paste(canvas_width)\
       [expr $paste(left_margin) + $paste(graph_width) + $RM]
   # Font sizes
   set paste(title_font_height) [expr $s1 * 1.3]
   set paste(label_font_height) [expr $s2 * 1.3]
   set paste(label_font_width) [expr $s2 * 1.1]
   set paste(tick_font_height) [expr $s3 * 1.3]
   set paste(tick_font_width) [expr $s3 * 1.1]
}
# --------------------------------------------------------------------
# Resize canvas size
# --------------------------------------------------------------------
proc exec_update_canvas_size {} {
   # global variables --------------------
   global paste wk
   
   # Main --------------------
   update_canvas_size_variables

   # update canvas scrollbar behaviors
   $wk(w,canvas,c) conf\
       -scrollregion "0 0 $paste(canvas_width) $paste(canvas_height)"
}
# --------------------------------------------------------------------
# update trace window size
# --------------------------------------------------------------------
proc exec_update_trace_window_size {} {
   # global variables --------------------
   global wk
   
   # Main --------------------
   set wk(trace,graph_width) [expr $wk(trace,width) \
				  - $wk(trace,left_margin) \
				  - $wk(trace,right_margin) \
				  - 16] ; # scroll bar width is 16
   set wk(trace,graph_height) [expr $wk(trace,height) \
				   - $wk(trace,top_margin) \
				   - $wk(trace,bot_margin)]
}
# --------------------------------------------------------------------
# update_console_window 
#        toggle on/off frames in the console window
# --------------------------------------------------------------------
proc update_console_window {} {
   # global variable ------------------------------
   global gbl wk
   
   # main ------------------------------
   set args ""
   foreach {i} {data range plot filter} {
      pack forget $wk(w,console).title($i) $wk(w,console).frame($i)
      if {$gbl(console,$i) == 1} {
	 set args [format "%s %s %s" $args $wk(w,console).title($i)\
		       $wk(w,console).frame($i)]
	 $wk(w,console).title($i).b conf -image close_icon
      } else {
	 set args [format "%s %s" $args $wk(w,console).title($i)]
	 $wk(w,console).title($i).b conf -image down_arrow
      }
   }
   eval pack $args -side top -fill x
}

# --------------------------------------------------------------------
# Filter configuration
# --------------------------------------------------------------------
proc update_filter {} {
   # Global variables ----------
   global filt gbl wk
   
   # Main ------------------------------
   set filt(type) [lsearch $wk(filter,list) $filt(type_label)]
   if { $filt(type) == 0 } {
      # No-filter
      set filt(type) 100
   }
   if { $filt(type) < 0 } {
      # if invalid filter, set No-filter
      puts stdout "Invalid filter type."
      set filt(type) 100
      set filt(type_label) [lindex $wk(filter,list) $filt(type)]
   }
}

# --------------------------------------------------------------------
# Update pick range
# --------------------------------------------------------------------
# [arguments]
# w        : widget name of pick type selection menubutton
# var      : variable name of pick type
# rangevar : variable name of pick type range
# labelvar : variable name of pick type range label
# --------------------------------------------------------------------
proc update_pickrange {w var rangevar labelvar} {
   # global ----------
   upvar $var typevar
   upvar $rangevar typerange
   upvar $labelvar typelabel
   global pick wk
   
   # Main ------------------------------
   if { $var == "pick(current_type)" } {
      set wl {
	 .cw.binding_frame.left.down.mb(ptype)
	 .pick_window.tabset.mark_frame.mb(ptype)
	 .change_pick_property.tabset.alltab.mb(ptype)
	 .change_pick_property.tabset.seltab.change.mb(ptype)
      }
   } else {
      set wl $w
   }
   set first [lindex $typelabel 1]
   set $rangevar [expr ($first -1) / $pick(n_color)]
   if { [expr $typevar % $pick(n_color)] == 0 } {
      set $var [expr 20 + $typerange * $pick(n_color)]
   } else {
      set $var [expr $typevar % $pick(n_color) + $typerange * $pick(n_color)]
   }

   foreach w $wl {
      if { [winfo exist $w] } {
	 mk_pick_type_menu $w $first $var
      }
   }
}

# --------------------------------------------------------------------
# Update x-axis
# --------------------------------------------------------------------
proc update_xaxis {} {
   # Global variables
   global gbl paste

   # Main
   set label [string tolower $gbl(data,xaxis_label)]
   switch -- $label {
      "offset" {
	 set gbl(data,xaxis) 0
      }
      "tracl" {
	 set gbl(data,xaxis) 1
	 xaxis_not_offset
      }
      "cdp" {
	 set gbl(data,xaxis) 2
	 xaxis_not_offset
	 set label CDP
      }
      "sx" {
	 set gbl(data,xaxis) 3
	 xaxis_not_offset
      }
      "sy" {
	 set gbl(data,xaxis) 4
	 xaxis_not_offset
      }
      "gx" {
	 set gbl(data,xaxis) 5
	 xaxis_not_offset
      }
      "gy" {
	 set gbl(data,xaxis) 6
	 xaxis_not_offset
      }
      "fldr" {
	 set gbl(data,xaxis) 7
	 xaxis_not_offset
      }
      "tracf" {
	 set gbl(data,xaxis) 8
	 xaxis_not_offset
      }
      default { # otherwise --> offset
	 set gbl(data,xaxis) 0
	 set label offset
      }
   }
   set gbl(data,xaxis_label) $label
}

# --------------------------------------------------------------------
# Process for not distance xaxis
# --------------------------------------------------------------------
proc xaxis_not_offset {} {
   # Global variables
   global gbl paste

   # Main ------------------------------
   # No geometric amp-correction
   if { $paste(amp,correct_label) == "Geometric"} {
      set gbl(check,amp) 0
      set paste(amp,correct_label) "AGC"
      update_amp_correct
   }
   # No reduction
   set paste(reduction) 0
}

# --------------------------------------------------------------------
# update pick type menu background color
# --------------------------------------------------------------------
proc update_pick_type_menu_color {col} {
   # global
   global pick wk

   # main
   foreach w "\.pick_window.tabset.mark_frame.type.mb(ptype)\
      $wk(widget,binding).mb(ptype)\
      .change_pick_property.tabset.alltab.mb(ptype)\
      .change_pick_property.tabset.seltab.change.mb(ptype)" {
      exec_update_pick_type_menu_color $w $col
   }
}

# --------------------------------------------------------------------
# update pick type menu background color
# --------------------------------------------------------------------
proc exec_update_pick_type_menu_color {w col} {
   # global
   global pick

   # main
   if { [winfo exist $w] } {
      $w configure -bg $pick(color,$col) -activeback $pick(color,$col)
   }
}

# --------------------------------------------------------------------
# update toggle synthetic traveltime color mode
# --------------------------------------------------------------------
# w : parent widget name
# --------------------------------------------------------------------
proc update_synthe_color_mode {w} {
   # global 
   global macray
   
   # main
   if { $macray(color_mode) == 0 } {
      # use specified color
      $w.button(1color) conf -state active
   } else {
      # others
      $w.button(1color) conf -state disabled
   }
   if { $macray(color_mode) == 2 } {
      # manual color
      $w.button(manual) conf -state active
   } else {
      # others
      $w.button(manual) conf -state disabled
   }
}

# --------------------------------------------------------------------
# update water depth of tausum graph 
# --------------------------------------------------------------------
proc update_tausum_depth {} {
   global gbl wk

   $wk(w,analysis).tabset.taup.graph.vzgraph element conf vzseafloor \
       -xdata "0 100" -ydata "$gbl(obs,depth) $gbl(obs,depth)"
}

# --------------------------------------------------------------------
# update wm title of the canvas window
# --------------------------------------------------------------------
proc update_wm_title {} {
   # global --------------------
   global file_name wk

   # main
   set title "PasteUp"
   if { [llength $file_name(wave)] > 0 } {
      set title [format "%s : %s" $title\
		     [lindex [split $file_name(wave) "/"] end]]
   }
   if { [llength $file_name(pick)] > 0 } {
      set title [format "%s : %s" $title\
		     [lindex [split $file_name(pick) "/"] end]]
   }

   # update
   wm title .cw $title
}

# --------------------------------------------------------------------
# update canvas mode (Normal/Graph)
# --------------------------------------------------------------------
proc update_canvas_mode {} {
   global paste wk
   if [namespace exist GraphCanvas] {
      namespace forget GraphCanvas::*
   }
   if [namespace exist NormalCanvas] {
      namespace forget NormalCanvas::*
   }
   if { $paste(canvas,label) == "Normal" } {
      namespace import NormalCanvas::*
      set paste(canvas,mode) 1
      set wk(w,current) $wk(w,canvas,c)
   } else {
      namespace import GraphCanvas::*
      set paste(canvas,mode) 2
      set wk(w,current) $wk(w,canvas,g)
   }
   setup_canvas
}
# --------------------------------------------------------------------
# Update listbox selection
# --------------------------------------------------------------------
proc update_listbox_selection {lb} {
   global custom
   for {set i 0} {$i < [$lb size]} { incr i} {
      $lb itemconf $i -background $custom(*Entry.Background)\
	  -foreground $custom(*Foreground)
   }
   $lb itemconf [$lb cursel] -background $custom(*selectBackground)\
       -foreground $custom(*selectForeground)
}
# --------------------------------------------------------------------
# Update stack mode
# --------------------------------------------------------------------
proc update_stack_mode {pf bf} {
   global paste
   set stat [expr {$paste(stack,mode) == "hyperbola"} ?\
		 {"normal"} : {"disabled"} ]
   $pf.label(time) conf -state $stat
   $pf.entry(time) conf -state $stat
   $bf.1 conf -state $stat
}
# --------------------------------------------------------------------
# Update data set mode
# --------------------------------------------------------------------
proc update_data_set_mode {} {
   global gbl
   update_data_type
   data_set_window_enable_disable
}
# --------------------------------------------------------------------
# Refer synthetics or not in Database
# --------------------------------------------------------------------
proc update_data_set_synthetic {} {
   # Global variable
   global gbl wk
   
   # Main --------------------
   set w [winfo parent $wk(w,ds,syn)]
   if { $gbl(ds,opt,syn) } {
      labelframe_status $w normal
   } else {
      labelframe_status $w disabled
   }
}
# --------------------------------------------------------------------
# Update pick mark
# --------------------------------------------------------------------
proc update_mark_style {} {
   # Global variable
   global paste wk
   
   # Main --------------------
   if {$paste(canvas,mode) == 1} {
      # NormalCanvas
      plot_pick_marks 0
   }
   # GraphCanvas (update always)
   for {set i 0} {$i < 20} {incr i} {
      $wk(w,canvas,g) elem conf pick$i -symbol $paste(mark,style)\
	  -pixels $paste(mark,size)
   }
}
# --------------------------------------------------------------------
# Update environment variables
#
# mode;  1:update PREF using env,  0:update env using PREF
# --------------------------------------------------------------------
proc update_env_variable {mode} {
   # Global variable
   global env PREF
   
   # Main --------------------
   set env(LC_ALL) C
   foreach n {OMP_NUM_THREADS CWPROOT} {
      if $mode {
	 if { [lsearch [array get env] $n] >= 0} {
	    set PREF(env,$n) $env($n)
	 }
      } else {
	 if { $PREF(env,$n) != "" } {
	    set env($n) $PREF(env,$n)
	 }
      }
   }
}
# --------------------------------------------------------------------
# update PATH environment variable
# --------------------------------------------------------------------
proc update_PATH {} {
   # Global variable
   global env PREF
   
   # Main --------------------
   # defun
   proc add_PATH {p} {
      global env
      if { [lsearch [split $env(PATH) ":"] $p] < 0 } {
	 set env(PATH) [format "%s:%s" $p $env(PATH)]
      }
   }
   # CWPROOT
   if { [lsearch [array get env] CWPROOT] >= 0 && \
	    [file isdirectory $env(CWPROOT)] == 1 } {
      add_PATH [format "%s/bin" $env(CWPROOT)]
   }
   # PREF
   foreach n {supath1 supath2} {
      if { $PREF(com,$n) != "" && [file isdirectory $PREF(com,$n)] } {
	 add_PATH $PREF(com,$n)
      }
   }
}

# --------------------------------------------------------------------
# Update pick file format
# --------------------------------------------------------------------
proc update_pick_format {} {
   # Global variable
   global wk
   
   # Main --------------------
   switch $wk(pick,format) {
      0 {
	 set wk(pick,format_name) Pasteup
      }
      1 {
	 set wk(pick,format_name) Mochi
      }
      2 {
	 set wk(pick,format_name) Rayinvr
      }
      3 {
	 set wk(pick,format_name) GeoCTII
      }
   }
}