#include <stdio.h>

#define OVER   1
#define UNDER  0

/********************************************************************
Does the straight line segment (x1,y1)-(x2,y2) cross the threshold level?
Return the x-coordinate of the intersection, otherwith return -1.0
********************************************************************/
static float check_cross(float x1, float y1, float x2, float y2,
			 float level)
{
    if ( (y1 - level)*(level - y2) > 0.0 ){
	return(x1 + (x2 - x1)*(level - y1)/(y2 - y1));
    } else {
	return(-1.0);
    }
}

/********************************************************************
Compute coordinates points crossing threshold level
--------------------------------------------------------------------
arguments
  n      : dimension size of ag and tg
  ag     : amplitude array
  tg     : time array
  va     : (output) Array of crossing times
  level  : threshold level of amplitdue
********************************************************************/
int search_threshold(int n, float *ag, float *tg, float *va, float level)
{
    /***** Local Variables *****/
    int i, k;
    char amp_sign;
    float tmp;
    
    /*************** Main ***************/
    
    /***** Initialize *****/
    k = 0;
    amp_sign = -1;
    if(ag[0] >= level){
	va[k++] = tg[0];
	amp_sign = 1;
    }
    
    /***** Search threshold *****/
    for(i=0;i<n-1;i++){
	tmp = check_cross(tg[i], ag[i], tg[i+1], ag[i+1], level);
	if ( (ag[i+1] - level)*amp_sign < 0.0 && tmp > 0.0) {
	    /** Crossing Threshold **/
	    va[k++] = tmp;
	    amp_sign *= -1;
	}
    }
    if (k%2 == 0) return(k);      /** even **/
    va[k++] = tg[n-1]; return(k); /** odd **/
}

/********************************************************************
Compute coordinates points crossing threshold level (minus side)
--------------------------------------------------------------------
arguments
  n      : dimension size of ag and tg
  ag     : amplitude array
  tg     : time array
  va     : (output) Array of crossing times
  level  : threshold level of amplitdue
********************************************************************/
int search_threshold_minus(int n, float *ag, float *tg, float *va,
			   float level)
{
    /***** Local Variables *****/
    int i, k;
    char amp_sign;
    float tmp;
    
    /*************** Main ***************/
    
    /***** Initialize *****/
    k = 0;
    amp_sign = +1;
    if(ag[0] <= level){
	va[k++] = tg[0];
	amp_sign = -1;
    }
    
    /***** Search Threshold *****/
    for(i=0;i<n-1;i++){
	tmp = check_cross(tg[i], ag[i], tg[i+1], ag[i+1], level);
	if ( (ag[i+1] - level)*amp_sign < 0.0 && tmp > 0.0) {
	    /** Crossing threshold **/
	    va[k++] = tmp;
	    amp_sign *= -1;
	}
    }
    if (k%2 == 0) return(k);      /** even **/
    va[k++] = tg[n-1]; return(k); /** odd **/
}

/********************************************************************
Return amplitude at time t
********************************************************************/
float interpolate_amp(int n, float t, float *ag, float *tg) 
{
    /***** local Variables *****/
    int k;
    
    /*************** Main ***************/
    k = (t - tg[0])/(tg[1] - tg[0]);
    if(k>=n-1) return(ag[n-1]);
    if(k<0) return(ag[0]);
    return( ag[k] + (ag[k+1] - ag[k])*(t - tg[k])
	    /(tg[k+1] - tg[k]) );
}
