# --------------------------------------------------------------------
#
#
#                        Velocity Model related module
#
#
# --------------------------------------------------------------------

namespace eval VModel {

   # -----------------------------------------------------------------
   # setup window (subcontractor of invoke_window)
   # -----------------------------------------------------------------
   proc setup_window {w} {
      # global --------------------------------------------------
      global gbl TMPDIR wk custom
      
      # main --------------------------------------------------
      set g $w.graph
      
      # top level widget ------------------------------
      toplevel $w
      wm title $w "PasteUp: Velocity Model"
      wm resizable $w 0 0
      
      # graph widget ------------------------------
      graph $g
      $g yaxis conf -descending 1
      $g y2axis conf -descending 1
      $g crosshairs conf -color green
      $g conf -plotpadx 0 -plotpady 0 -plotborderwidth 0 -borderwidth 0\
	  -topmargin 50 -bottommargin 25 -leftmargin 50 -rightmargin 50
      Blt_Crosshairs $g
      Blt_ZoomStack $g
      
      # buttons ------------------------------
      set bf [button_frame $w $w.button_frame\
		  property "VModel::property_window $g"\
		  "save PS" "VModel::save_postscript $g vmodel"]
      
      # binding
      basic_bindings $w

      # packing ------------------------------
      pack $g -side top -fill both -expand yes
      pack $bf -side bottom -fill x
      show_position $g vmodel
      set wk(vmodel,filetime) -1
   }

   # -----------------------------------------------------------------
   # Invoke velocity model window
   # -----------------------------------------------------------------
   proc invoke_window {} {
      # global --------------------------------------------------
      global gbl TMPDIR wk
      
      # main --------------------------------------------------
      # invoke window
      set w $wk(vmodel,widget)
      set g $w.graph
      if [raise_window $w] {
	 # nothing
      } else {
	 # new window
	 setup_window $w 
      }
      # delete previous ----------
      eval $g marker delete [$g marker name]
      update 
      # model file in netCDF format ----------
      set modelfn  [format "%s/pasteup.%s.grd" $TMPDIR [pid]]
      if {$gbl(vmodel,format) == 0 } {
	 # ray86 + gm
	 file stat $gbl(maping,ray86) fnstat
      } else {
	 # netcdf
	 file stat $gbl(vmodel,netcdf) fnstat
      }
      
      # Read velocity model info and make velocity model image ----------
      if { $fnstat(ctime) > $wk(vmodel,filetime) } {
	 # make velocity model image
	 create_image
	 set wk(vmodel,filetime) $fnstat(ctime)
      }
      scan $wk(vmodel,coords) "%f%f%f%f" xmin zmin xmax zmax
      
      # set up graph area ----------
      set gz [expr $gbl(vmodel,width) * $wk(vmodel,height_coef) + 75]
      set gx [expr $gbl(vmodel,width) + 100]
      $g conf -height $gz -width $gx
      
      # graph area
      set xstep [raytrace_comp_tick_spacing [expr $xmax - $xmin]]
      set zstep [raytrace_comp_tick_spacing [expr $zmax - $zmin]]
      
      $g axis conf x -min $xmin -max $xmax -step $xstep
      $g axis conf y -min $zmin -max $zmax -step $zstep
      $g axis conf x2 -min $xmin -max $xmax -step $xstep -hide false
      $g axis conf y2 -min $zmin -max $zmax -step $zstep -hide false
      
      # plot background image ----------
      $g marker create image -image vmodel -name vmodel -under 1\
	  -coord $wk(vmodel,coords)
      
      # boundary ----------
      if { $gbl(vmodel,view,plotxy) } {
	 mapping_plot_model_boundary $g
      }
      
      # change title
      $g conf -title [file rootname [file tail $gbl(vmodel,netcdf)]]
      
      # update window size ----------
      set pos [string trimleft [wm geometry $w] 1234567890x]
      wm geometry $w [format "%dx%d%s" $gx [expr int($gz + 40)] $pos]
   }

   # -----------------------------------------------------------------
   # Create velocity model image 
   # -----------------------------------------------------------------
   proc create_image {} {
      # global --------------------------------------------------
      global gbl TMPDIR wk PREF
      
      # main --------------------------------------------------
      
      # make netCDF file, if necessary----------
      if {$gbl(vmodel,format) == 0 } {
	 # ray86 + gm
	 set netCDF [format "%s/pasteup.vmodel.%s.grd" $TMPDIR [pid]]
	 set com [format "%s -b %d -d %s %s %s %s" $PREF(com,model2cdf)\
		      $gbl(vmodel,mboundary) $gbl(mapping,dx)\
		      $gbl(vmodel,ray86) $gbl(vmodel,gm) $netCDF]
	 exec_external_command_fifo $com "model2cdf"
	 set tmpnetCDF $netCDF
      } else {
	 # netcdf
	 set netCDF $gbl(vmodel,netcdf)
	 set tmpnetCDF ""
      }
      
      # read netCDF file ----------
      set f [open "| grdinfo -C $netCDF"]
      scan [gets $f] "%*s%f%f%f%f" xmin xmax zmin zmax
      close $f
      set wk(vmodel,coords) "$xmin $zmin $xmax $zmax"
      # aspect ratio
      set wk(vmodel,height_coef) [expr $gbl(vmodel,aspect) * ($zmax - $zmin)\
				      / ($xmax - $xmin)]
      
      # color palette ----------
      if { $gbl(vmodel,cptfile) == "" } {
	 set gbl(vmodel,cptfile) "./vel.cpt"
      }
      if { ! [file exist $gbl(vmodel,cptfile)] } {
	 make_cpt_file
	 tk_messageBox -icon warning -type ok\
	     -message "Couldn't find CPT file for velocity structure image.  I made a one for you.  Check it out."
      }
      
      # make velocity model image ----------
      set com {}
      set imgfn  [format "%s/pasteup.vmodel.%s.jpg" $TMPDIR [pid]]
      set psfn  [format "%s/pasteup.vmodel.%s.ps" $TMPDIR [pid]]
      lappend com "gmtset PAPER_MEDIA a4+"
      
      # aspect
      set w 20
      set h [expr - $w * $wk(vmodel,height_coef)]
      # make external commands
      if { $gbl(vmodel,contour) > 0.0 } {
	 lappend com [format "grdimage %s -C%s -JX%g/%g -R%f/%f/%f/%f -P -K > %s"\
			  $netCDF $gbl(vmodel,cptfile) $w $h\
			  $xmin $xmax $zmin $zmax $psfn]
	 lappend com [format "grdcontour %s -JX%g/%g -R%g/%g/%g/%g -C%g -A%g -P -O >>%s"\
			  $netCDF $w $h $xmin $xmax $zmin $zmax\
			  $gbl(vmodel,contour) $gbl(vmodel,contour) $psfn]
      } else {
      lappend com [format "grdimage %s -C%s -JX%g/%g -R%f/%f/%f/%f -P > %s"\
		       $netCDF $gbl(vmodel,cptfile) $w $h\
		       $xmin $xmax $zmin $zmax $psfn]
      }
      lappend com [format "%s -density %g %s %s"\
		       $PREF(com,convert) $gbl(vmodel,density) $psfn $imgfn]
      
      # exec external command
      exec_shell_script $com "Make velocity model"
      
      image create photo vmodel -file $imgfn
      file delete -force $psfn $imgfn $tmpnetCDF
   }

   # -----------------------------------------------------------------
   # Property window
   # -----------------------------------------------------------------
   proc property_window {g} {
      # Global --------------------------------------------------
      global gbl wk
      
      # main --------------------------------------------------
      set w .vmodel_property
      toplevel $w
      wm title $w "Pasteup : Properties for the velocity model"
      
      # title frame --------------------
      set tf [title_frame $w.title_frame\
		  "Pasteup: Properties for the velocity model"]
      
      # parameter frame ------------------------------
      set pf [frame $w.parameters]
      
      # filenames
      labelframe $pf.fn -text "Files" -pady 3 -padx 5
      filename_hscroll_frame $pf.fn cptfile "CPT" gbl(vmodel,cptfile) 45 load cpt
      filename_hscroll_frame $pf.fn xydata "Layer boundary"\
	  gbl(mapping,view,xydata) 45 load "dat"
      
      # image dimension
      labelframe $pf.isize -text "Image dimension" -pady 3 -padx 5
      make_entry_right $pf.isize asp "Aspect ratio (height/width)"\
	  gbl(vmodel,aspect) ""
      make_entry_right $pf.isize w "Width of the plot area"\
	  gbl(vmodel,width) "pixel"
      make_entry_right $pf.isize density "Image resolution"\
	  gbl(vmodel,density) "dpi"
      
      # plot or not
      labelframe $pf.plot -text "Plot" -pady 3 -padx 5
      make_checkbutton $pf.plot "Layer boundary" plotxy gbl(vmodel,view,plotxy)
      set c [make_checkbutton $pf.plot "Cursor position"\
		 cursr gbl(vmodel,show_position)]
      $c conf -command "VModel::show_position $g vmodel"
      
      # velocity model parameters
      set vm [labelframe $pf.vmodel -text "Options for velocity model" -pady 10]
      make_entry_right $vm cint "Contour interval" gbl(vmodel,contour) "km/sec."
      make_color_button $vm xycolor "Color of layer boundary" ""\
	  $gbl(mapping,view,xycolor) gbl(mapping,view,xycolor)\
	  "setColor $vm $vm.button(xycolor) gbl(mapping,view,xycolor)"
      
      # Rays
      labelframe $pf.rays -text "Rays" -pady 3 -padx 5
      make_color_button $pf.rays raycolor "Ray color" "" $gbl(raytrace,raycolor)\
	  gbl(raytrace,raycolor)\
	  "setColor $pf $pf.button(raycolor) gbl(raytrace,raycolor)"
      grid $pf.fn - -sticky news -padx 5
      grid $pf.isize $pf.plot -sticky news -padx 5
      grid $vm $pf.rays -sticky news -padx 5
      
      # button frame ------------------------------
      set bf [button_frame $w $w.button_frame apply VModel::update_image]
      
      # packing ------------------------------
      pack $tf $pf $bf -side top -fill x
      grab $w
   }

   # -----------------------------------------------------------------
   # Update image
   # -----------------------------------------------------------------
   proc update_image {} {
      global wk
      set wk(vmodel,filetime) -1
      if { [lsearch [$wk(vmodel,widget).graph marker name] r0] } {
	 # You have rays
	 raytrace_plot_raypaths
      } else {
	 invoke_window
      }
   }

   # -----------------------------------------------------------------
   # Save velocity image as a postscript file
   # -----------------------------------------------------------------
   # g    : graph widget
   # mode : "vmodel" or "mapping"
   # -----------------------------------------------------------------
   proc save_postscript {g mode} {
      # Global --------------------------------------------------
      global wk
      
      # main --------------------------------------------------
      # select filename
      set wk($mode,psfile) ""
      file_selection wk($mode,psfile) save ps
      if {$wk($mode,psfile) == ""} { return }
      # save
      $g postscript output $wk($mode,psfile) -paperh 20.9c -paperw 29.7c\
	  -center 1 -landscape 1 -maxpect 1
   }

   # -----------------------------------------------------------------
   # binding to show position in the vmodel, mapping window
   # -----------------------------------------------------------------
   # g    : graph widget
   # mode : "vmodel" or "mapping"
   # -----------------------------------------------------------------
   proc show_position {g mode} {
      # Global --------------------------------------------------
      global gbl
      
      # main --------------------------------------------------
      if {$gbl($mode,show_position)} {
	 # show
	 bind $g <Any-Motion> "VModel::position $g %x %y %X %Y"
	 bind $g <Leave> "hide_position_window"
      } else {
	 # hide
	 bind $g <Any-Motion> ""
      }
      hide_position_window
   }

   # -----------------------------------------------------------------
   # binding to show position in the vmodel and mapping window
   # -----------------------------------------------------------------
   proc position {g x y RX RY} {
      scan [$g invtransform $x $y] "%f%f" gx gy
      show_position_balloon [format "(%.1f, %.1f)" $gx $gy] $RX $RY
   }
}