# --------------------------------------------------------------------
# Make entry widget set
# --------------------------------------------------------------------
#    w     : paraent widget name.
#    name  : Name of parameters (Identifier in this script)
#    label : Labels to display.
#    val   : Variable name in this script.
#    unit  : Unit.
# --------------------------------------------------------------------
proc make_entry {w name label var unit} {
   label $w.label($name) -text $label -anchor e
   if { $var == "" } {
      entry $w.entry($name) -width 12
   } else {
      entry $w.entry($name) -width 12 -textvariable $var
   }
   label $w.unit($name) -text $unit -anchor w
   grid $w.label($name) $w.entry($name) $w.unit($name) -in $w -sticky news
   return $w.entry($name)
}
# --------------------------------------------------------------------
# Make entry widget set (wider version)
# --------------------------------------------------------------------
#    w     : parent widget
#    name  : identifier 
#    label : text to show
#    var   : variable name
# --------------------------------------------------------------------
proc make_long_entry {w name label var} {
   label $w.label($name) -text $label -anchor e
   entry $w.entry($name) -width 50 -textvariable $var
   grid $w.label($name) $w.entry($name) - - - -in $w -sticky news
}
# --------------------------------------------------------------------
# Make right-justified entry widget sets
# --------------------------------------------------------------------
#    w     : parent widget name
#    name  : identifier in this application
#    label : text to show
#    var   : variable name
#    unit  : unit
# --------------------------------------------------------------------
proc make_entry_right {w name label var unit} {
   label $w.label($name) -text $label -anchor e
   entry $w.entry($name) -width 12 -textvariable $var -justify right
   label $w.unit($name) -text $unit -anchor w
   grid $w.label($name) $w.entry($name) $w.unit($name) -in $w -sticky news
}
# --------------------------------------------------------------------
# Make right-justified entry widget sets; leftedge widgets is "null"
# --------------------------------------------------------------------
#    w     : parent widget name
#    name  : identifier in this application
#    label : text to show
#    var   : variable name
#    unit  : unit
# --------------------------------------------------------------------
proc make_entry_right_ch {w name label var unit} {
   label $w.label($name) -text $label -anchor e
   entry $w.entry($name) -width 12 -textvariable $var -justify right
   label $w.unit($name) -text $unit -anchor w
   grid x $w.label($name) $w.entry($name) $w.unit($name) -in $w -sticky news
}
# --------------------------------------------------------------------
# Make entry frame to set parameter with a command button
# --------------------------------------------------------------------
#    w     : paraent widget name.
#    label : Labels to display.
#    id    : Name of parameters (Identifier in this script)
#    val   : Variable name in this script.
#    blabel: label for the command button
#    com   : command to execute
# --------------------------------------------------------------------
proc make_entry_com {w label id var blabel com} {
    label $w.label($id) -text $label -anchor e
    entry $w.entry($id) -width 12 -textvariable $var -justify right
    button $w.but($id) -text $blabel -command $com
    grid $w.label($id) $w.entry($id) $w.but($id)\
	-in $w -sticky news
}
# --------------------------------------------------------------------
# Make label to show info
# --------------------------------------------------------------------
#    w     : paraent widget name.
#    name  : Name of parameters (Identifier in this script)
#    label : Labels to display.
#    val   : Variable name in this script.
# --------------------------------------------------------------------
proc make_label {w name label var} {
    # main --------------------------------------------------
    label $w.label($name) -text $label -anchor e
    entry $w.entry($name) -width 12 -textvariable $var\
	-relief flat -state disabled -disabledfore black
    grid $w.label($name) $w.entry($name) -in $w -sticky news
}
# --------------------------------------------------------------------
# Make checkbutton widget set
# --------------------------------------------------------------------
#    w     : parent widget
#    label : text to show
#    name  : identifier
#    var   : variable name
# --------------------------------------------------------------------
proc make_checkbutton {w label name var} {
   checkbutton $w.check($name) -var $var -text $label
   grid $w.check($name) -sticky nws
   return $w.check($name)
}
# --------------------------------------------------------------------
# Make checkbutton plus label widget
# --------------------------------------------------------------------
#    w     : parent widget
#    label : text to show
#    id    : identifier
#    var   : variable name
#    com   : command to bind the checkbutton
# --------------------------------------------------------------------
proc make_check_label {w label id var com} {
   checkbutton $w.check($id) -var $var -command $com
   label $w.label($id) -text $label
   grid $w.check($id) $w.label($id) - - -sticky nws
   grid conf $w.check($id) -sticky nes
   return $w.check($id)
}
# --------------------------------------------------------------------
# Make checkbutton widget and command button
# --------------------------------------------------------------------
#    w     : parent widget
#    label : text to show
#    name  : identifier
#    var   : variable name
#    blabel: label for command button 
#    com   : command
# --------------------------------------------------------------------
proc make_checkbutton_com {w label name var blabel com} {
   global gbl
   checkbutton $w.check($name) -var $var -text $label\
       -command "widgets_update $var $w.but($name)"
   button $w.but($name) -text $blabel -command $com
   grid $w.check($name) $w.but($name) -sticky nws
   grid conf $w.but($name) -sticky news
   widgets_update $var $w.but($name)
   return $w.check($name)
}

# --------------------------------------------------------------------
# Make checkbutton plus entry widget
# --------------------------------------------------------------------
#    w     : parent widget
#    label : text to show
#    id    : identifier
#    vc    : variable name for the check button
#    ve    : variable name for the entry
# --------------------------------------------------------------------
proc make_check_entry {w label id vc ve} {
   global paste gbl
   proc update_check_entry {w v} {
      global paste gbl
      upvar $v var
      if {$var} {
	 $w conf -state normal
      } else {
	 $w conf -state disabled
      }
   }
   checkbutton $w.check($id) -var $vc -text $label\
       -command "update_check_entry $w.entry($id) $vc"
   entry $w.entry($id) -textvar $ve -width 30
   grid $w.check($id) $w.entry($id) -sticky nws
   grid conf $w.entry($id) -sticky news
   update_check_entry $w.entry($id) $vc
   return $w.check($id)
}

# --------------------------------------------------------------------
# Make on/off widget
# --------------------------------------------------------------------
#    w     : parent widget
#    label : text to show
#    name  : identifier
#    var   : variable name
# --------------------------------------------------------------------
proc make_on_off {w label name var} {
   label $w.label($name) -text $label -anchor e
   radiobutton $w.on($name) -text "on" -variable $var -value 1 -anchor w
   radiobutton $w.off($name) -text "off" -variable $var -value 0 -anchor w
   grid $w.label($name) $w.on($name) $w.off($name) -in $w -sticky news
}

# --------------------------------------------------------------------
# Make radio button widget (multiple choice widget)
# --------------------------------------------------------------------
#    w     : parent widget name
#    label : text to show
#    name  : identifier
#    var   : variable name
#   (variable length arguments)
#    l1    : label
#    v1    : value
# --------------------------------------------------------------------
proc make_many_radio {w label name var args} {
    # Variable length arguments --------------------
    set i 0
    foreach {j k} $args {
	set l($i) $j
	set v($i) $k	
	incr i
    }
    label $w.label($name) -text $label -anchor e
    set winlist $w.label($name)
    for {set j 0} {$j < $i} {incr j} {
	radiobutton $w.radio($name,$j) -text $l($j) -anchor w\
		-variable $var -value $v($j) 
	lappend winlist $w.radio($name,$j)
    }
    eval {grid} $winlist -sticky news
}

# --------------------------------------------------------------------
# Make radio button widget : vertical version
# --------------------------------------------------------------------
#    w     : parent widget name
#    name  : identifier
#    var   : variable name
#   (variable length arguments)
#    l1    : label
#    v1    : value
# --------------------------------------------------------------------
proc make_radio_vertical {w name var args} {
   # Variable length arguments --------------------
   set i 0
   foreach {j k} $args {
      set l($i) $j
      set v($i) $k	
      incr i
   }
   set winlist $w.label($name)
   for {set j 0} {$j < $i} {incr j} {
      radiobutton $w.radio($name,$j) -text $l($j) -anchor w\
	  -variable $var -value $v($j) -justify left
      grid $w.radio($name,$j) -sticky nesw
   }
}

# --------------------------------------------------------------------
# Make many radiobutton widget set
# --------------------------------------------------------------------
#    w     : parent widget
#    label : text to show
#    name  : identifier
#    var   : variable name for the radiobutton
#    com   : command
#   (below: variable length args )
#    l1    : text for the 1-st button
#    v1    : value for the 1-st button
# --------------------------------------------------------------------
proc make_many_radio_com {w label name var com args} {
   # Variable length arguments --------------------
   set i 0
   foreach {j k} $args {
      set l($i) $j
      set v($i) $k	
      incr i
   }
   label $w.label($name) -text $label -anchor e
   set winlist $w.label($name)
   for {set j 0} {$j < $i} {incr j} {
      radiobutton $w.radio($name,$j) -text $l($j) -anchor w\
	  -variable $var -value $v($j) -command $com
      lappend winlist $w.radio($name,$j)
   }
   eval {grid} $winlist -sticky news
}
# --------------------------------------------------------------------
# Vertical radio button widget set with command
# --------------------------------------------------------------------
#    w     : parent widget
#    name  : identifier
#    var   : variable name
#    com   : command bind to the radio buttons
# variable length arguments
#    l1    : label
#    v1    : value
# --------------------------------------------------------------------
proc make_radio_vertical_com {w name var com args} {
   # Variable length arguments --------------------
   set i 0
   foreach {j k} $args {
      set l($i) $j
      set v($i) $k	
      incr i
   }
   for {set j 0} {$j < $i} {incr j} {
      radiobutton $w.radio($name,$j) -text $l($j) -anchor w\
	  -variable $var -value $v($j) -command $com
      grid $w.radio($name,$j) -sticky news
   }
}

# --------------------------------------------------------------------
# Make a color selector widget
# --------------------------------------------------------------------
#    w     : parent widget
#    name  : identifier
#    label : text to show
#    blabel: text to show on the button; can be ""
#    initc : initial color for the button
#    var   : variable name
#    com   : commond bind to the button
# --------------------------------------------------------------------
proc make_color_button {w name label blabel initc var com} {
   label $w.label($name) -text $label -anchor e
   button $w.button($name) -text $blabel -background $initc\
       -activebackground $initc -command $com
   grid $w.label($name) $w.button($name) -in $w -sticky news
}

# --------------------------------------------------------------------
# Make a commond button with label
# --------------------------------------------------------------------
#    w     : parent widget
#    name  : identifier
#    label : text to show
#    blabel: text on the button (can be "")
#    com   : command to bind for button
# --------------------------------------------------------------------
proc make_command_button {w name label blabel com} {
    label $w.label($name) -text $label -anchor e
    button $w.button($name) -text $blabel -command $com
    grid $w.label($name) $w.button($name) -in $w -sticky news
}

# --------------------------------------------------------------------
# Make several commond button
# --------------------------------------------------------------------
#    w     : parent widget
#    name  : identifier
#    label : text to show
#    l1    : label on the 1-st button
#    c1    : command for the 1-st button
#    l2    : label on the 2-nd button
#    c2    : command for the 2-nd button
#     :
# --------------------------------------------------------------------
proc make_many_command_button {w name label args} {
   # variable length arguments
   label $w.label($name) -text $label -anchor e
   set wl $w.label($name)
   set i 0
   foreach {j k} $args {
      set l $j
      set c $k	
      button $w.button($name,$i) -text $l -command $c
      lappend wl $w.button($name,$i)
      incr i
   }
   eval {grid} $wl {-in $w -sticky news}
}

# --------------------------------------------------------------------
# Menu button widget set
# --------------------------------------------------------------------
#    w     : parent widget name
#    name  : identifier
#    label : text to show
#    var   : variable name
# variable length arguments:
#    value1, value2, ...
# --------------------------------------------------------------------
proc make_optionMenu_button {w name label var args} {
   label $w.label($name) -text $label -anchor e
   eval {tk_optionMenu $w.button($name) $var} $args
   grid $w.label($name) $w.button($name) -in $w -sticky news
}

# --------------------------------------------------------------------
# Menu button widget set ; label can be different from value
# --------------------------------------------------------------------
#    w     : parent widget name
#    name  : identifier
#    label : text to show
#    var   : variable name
#    com   : command
# variable length arguments:
#    label1, value1, label2, value2, ...
# --------------------------------------------------------------------
proc make_optionMenu_value {w name label var com args} {
   # explanation ----------
   label $w.label($name) -text $label -anchor e
   # divide args ----------
   set labels {}
   set vals {}
   foreach {i j} $args {
      lappend labels $i
      lappend vals $j
   }
   # optionMenu
   set mw [eval {tk_optionMenu $w.button($name) $var} $labels]

   # assign command
   for {set i 0} {$i < [llength $labels]} {incr i} {
      $mw entryconf $i -value [lindex $vals $i] -label [lindex $labels $i]\
	  -command $com
   }
   grid $w.label($name) $w.button($name) - - -in $w -sticky news
   return $mw
}

# --------------------------------------------------------------------
# Menu button widget set with command
# --------------------------------------------------------------------
#    w     : parent widget name
#    name  : identifier
#    label : text to show
#    var   : variable name
#    com   : commands
# variable length arguments:
#    value1, value2, ...
# --------------------------------------------------------------------
proc make_optionMenu_com {w name label var com args} {
   label $w.label($name) -text $label -anchor e
   eval {pasteup_optionMenu_com $w.button($name) $var $com} $args
   grid $w.label($name) $w.button($name) -in $w -sticky news
}

# --------------------------------------------------------------------
# an extension of tk_optionMenu (enabling command)
# --------------------------------------------------------------------
#    w     : widget name for the optionMenu
#    var   : variable name
#    com   : commands
# variable length arguments:
#    value1, value2, ...
# --------------------------------------------------------------------
proc pasteup_optionMenu_com {w var com args} {
   eval set mw [eval tk_optionMenu $w $var $args]
   # assign command
   for {set i 0} {$i < [llength $args]} {incr i} {
      $mw entryconf $i -command $com
   }
   return $mw
}
# --------------------------------------------------------------------
# Make filename frame with entry and ref button
# --------------------------------------------------------------------
# w           : parent widget
# id          : identifier 
# text        : label to show
# var         : variable name for filename
# operation   : "load" or "save"
# type        : default extension indicator
# --------------------------------------------------------------------
proc filename_frame {w id text var operation type} {
    label $w.label($id) -text $text -width 18 -anchor e
    entry $w.entry($id) -textvariable $var -width 60
    button $w.button($id) -text "Ref..."\
	-command "file_selection $var $operation $type"
    grid $w.label($id) $w.entry($id) - - - $w.button($id) -sticky news
}

# --------------------------------------------------------------------
# Filename combo widgets : with h-scrollbar
# --------------------------------------------------------------------
# w           : parent widget name
# id          : id name in this application
# text        : label to show
# width       : width of entry widget
# var         : variable name for file
# operation   : "load", "save" or "dir"
# type        : default extension indicator
# --------------------------------------------------------------------
proc filename_hscroll_frame {w id text var width operation type} {
   # procedure for ref button
   # label, filename entry and "ref" button
   label $w.label($id) -text $text -anchor e
   entry $w.entry($id) -textvariable $var -width $width\
       -xscrollcommand "$w.hscroll($id) set"
   if { $type == "" } {
      set type "any"
   }
   button $w.button($id) -text "Ref..."\
       -command "file_selection $var $operation $type"
   grid $w.label($id) $w.entry($id) - - - $w.button($id) -sticky news
   bind $w.entry($id) <Button-4> "$w.entry($id) xview scroll -3 unit"
   bind $w.entry($id) <Button-5> "$w.entry($id) xview scroll +3 unit"
   # scroll bar
   scrollbar $w.hscroll($id) -width 8 -orient horiz -relief flat -bd 2\
       -command "$w.entry($id) xview"
   grid x $w.hscroll($id) - - - x -sticky news
   # view
   $w.entry($id) xview end
}

# --------------------------------------------------------------------
# Filename combo widgets : with h-scrollbar and command-binding
# --------------------------------------------------------------------
# w           : parent widget name
# id          : id name in this application
# text        : label to show
# width       : width of entry widget
# var         : variable name for file
# operation   : "load", "save" or "dir"
# type        : default extension indicator
# com         : command
# --------------------------------------------------------------------
proc filename_hscroll_com {w id text var width operation type com} {
   # label, filename entry and "ref" button
   set col [option get . foreground Foreground]
   label $w.label($id) -text $text -anchor e
   entry $w.entry($id) -textvariable $var -width $width -state disabled\
       -xscrollcommand "$w.hscroll($id) set" -disabledfore $col
   button $w.button($id) -text "Ref..." -command "$com $operation $type"
   grid $w.label($id) $w.entry($id) - - - $w.button($id) -sticky news
   bind $w.entry($id) <Button-4> "$w.entry($id) xview scroll -3 unit"
   bind $w.entry($id) <Button-5> "$w.entry($id) xview scroll +3 unit"
   # scroll bar
   scrollbar $w.hscroll($id) -width 8 -orient horiz -relief flat -bd 2\
       -command "$w.entry($id) xview"
   grid x $w.hscroll($id) - - - x -sticky news
   # view
   $w.entry($id) xview end
}

# --------------------------------------------------------------------
# Make horizontal scale widget set
# --------------------------------------------------------------------
#    w     : parent widget
#    name  : identifier in this application
#    label : label to show
#    val   : variable name
#    min   : minimum for the scale
#    max   : maxmum for the scale
#    tic   : tic interval
#    res   : resolution for the scale
#    unit  : unit
# --------------------------------------------------------------------
proc make_hscale {w name label val min max tic res unit} {
    label $w.label($name) -text $label -anchor e
    scale $w.scale($name) -orient horizontal -width 12 -variable $val\
	-from $min -to $max -tickinterval $tic -resolution $res
    label $w.unit($name) -text $unit -anchor w
    grid $w.label($name) $w.scale($name) $w.unit($name) -in $w -sticky news
}
# --------------------------------------------------------------------
# Make spinbox widget set
# --------------------------------------------------------------------
#    w     : parent widget name
#    id    : id inside program (for grid manager)
#    label : explanation
#    var   : variable name
#    min   : minimum value
#    max   : maximum value
#    inc   : increment
#    unit  : unit
# --------------------------------------------------------------------
proc make_spinbox {w id label var min max inc unit} {
   # main --------------------------------------------------
   label $w.label($id) -text $label -anchor e
   set s [spinbox $w.spinbox($id) -from $min -to $max -width 12\
	      -increment $inc -textvariable $var -justify right]
   label $w.unit($id) -text $unit -anchor w
   grid $w.label($id) $s $w.unit($id) -in $w -sticky news
   bind $s <Button-4> "$s invoke buttonup"
   bind $s <Button-5> "$s invoke buttondown"
}
# --------------------------------------------------------------------
# Make spinbox widget set with binding a command
# --------------------------------------------------------------------
#    w     : parent widget name
#    id    : id inside program (for grid manager)
#    label : explanation
#    var   : variable name
#    min   : minimum value
#    max   : maximum value
#    inc   : increment
#    unit  : unit
#    com   : command to bind
# --------------------------------------------------------------------
proc make_spinbox_com {w id label var min max inc unit com} {
   # main --------------------------------------------------
   label $w.label($id) -text $label -anchor e
   set s [spinbox $w.spinbox($id) -from $min -to $max -width 12\
	      -increment $inc -textvariable $var -justify right -com $com]
   label $w.unit($id) -text $unit -anchor w
   grid $w.label($id) $s $w.unit($id) -in $w -sticky news
   bind $s <Button-4> "$s invoke buttonup"
   bind $s <Button-5> "$s invoke buttondown"
}
# --------------------------------------------------------------------
# Select Color Button
# --------------------------------------------------------------------
#    w          : parent widget name
#    but        : button widget path name
#    color_name : color array name
# --------------------------------------------------------------------
proc setColor {w but color_name} {
   # global
   global p
   global $color_name
   upvar $color_name color_variable
   
   # main
   grab $w
   set initialColor $color_variable
   set color_variable\
       [tk_chooseColor -title "Select Color" -parent $w \
	    -initialcolor $initialColor]
   if [string compare $color_variable ""] {
      $but config -backg $color_variable -activebackg $color_variable
      set return_val OK
   } else {
      set color_variable $initialColor
      set return_val CANCEL
   }
   grab release $w
   return $return_val
}

# --------------------------------------------------------------------
# button_frame -------------------------------------------------------
#    Create button frame for several window
# --------------------------------------------------------------------
# w     : toplevel widget name (to dismiss), if $w=="", then no dismiss button
# f     : button frame widget name
# args  : "label" and "command"
# --------------------------------------------------------------------
proc button_frame {w f args} {
   # global --------------------------------------------------
   global custom
   
   # main --------------------------------------------------
   frame $f

   # command button(s) (left side) ----------
   set i 1
   set coms {}
   foreach {l c} $args {
      button $f.$i -text $l -command $c -width 12 -bd 2
      lappend coms $f.$i
      incr i
   }
   if { $coms != {} } {
      eval pack $coms -side left -padx 10 -pady 5
   }

   # dismiss button (right side) ----------
   if { $w != "" } {
      button $f.dismiss -text "dismiss" -command "destroy $w" -width 12 -bd 2\
	  -bg $custom(DismissButtonBackground)
      pack $f.dismiss -side right -padx 10 -pady 5
   }
   return $f
}

# --------------------------------------------------------------------
# title_frame -------------------------------------------------------
#    Create title frame for several window
# --------------------------------------------------------------------
# w  : widget name for title frame
# l  : labels to show
# --------------------------------------------------------------------
proc title_frame {w l} {
   # global --------------------------------------------------
   global custom
   
   # main --------------------------------------------------
   proc shift_window {w X Y} {
      set tw [winfo toplevel $w]
      scan [split [wm geometry $tw] "+x"] "%d%d" w h
      wm geometry $tw [format "+%d+%d" [expr $X-$w/2] [expr $Y-25]]
   }
   label $w -text $l -font $custom(WindowTitleFont) -relief groove -bd 3 \
       -bg $custom(ConsoleTitleBackground) -fg $custom(ConsoleTitleForeground)
   bind $w <B1-Motion> "  shift_window $w %X %Y"
   return $w
}

# --------------------------------------------------------------------
# update widgets status depending on the variable
# --------------------------------------------------------------------
# var    : variable name (=0:off, =1:on)
# args   : widgets (one or more)
# --------------------------------------------------------------------
proc widgets_update {var args} {
   global gbl wk
   upvar $var flag
   if { [llength $args] < 1 } { return }
   if { $flag } {
      # on
      foreach w $args {
	 $w conf -state normal
      }
   } else {
      # off
      foreach w $args {
	 $w conf -state disabled
      }
   }
}

namespace eval PSFontSelect {
   # --------------------------------------------------------------------
   # make_font_selection ------------------------------------------------
   #    Make font selection widget set
   # --------------------------------------------------------------------
   # w   : parent widget name
   # id  : identifier in the program code
   # l   : text to show (explanation)
   # var : variable name of font
   # --------------------------------------------------------------------
   proc make_widget {w id l var} {
      upvar $var fn
      label $w.title($id) -text "$l :"
      set pw [label $w.sample($id) -text "ABCabc-0.123" -font $fn]
      button $w.button($id) -text "set"\
	  -command "[namespace current]::select_window $pw {$l} $var $id"
      grid $w.title($id) $w.sample($id) $w.button($id) -sticky news
      grid conf $w.title($id) -sticky nes
      grid conf $w.button($id) -sticky ew
   }
   
   proc select_window {pw l var id} {
      # global ------------------------------
      global wk fontmap
      upvar $var fn
      
      # invoke window ------------------------------
      set w .font_selection
      catch {destroy $w}
      toplevel $w
      wm title $w "PasteUp:Font selection"
      # title frame
      set tf [title_frame $w.title_frame "Pasteup: Font selection"]
      # button frame
      set bf [button_frame $w $w.button_frame "apply"\
		  "[namespace current]::apply $pw $w $var"]
      
      # font selection frame
      set ff [labelframe $w.font_select -text "$l font" -padx 5 -pady 10]
      listbox $ff.family -height 6 -yscrollcom "$ff.fscroll set"\
	  -export false -width 20
      scrollbar $ff.fscroll -width 10 -command "$ff.family yview"
      listbox $ff.size -height 6 -width 3 -yscrollcom "$ff.sscroll set" -export 0
      scrollbar $ff.sscroll -width 10 -command "$ff.size yview"
      
      grid $ff.family $ff.fscroll $ff.size $ff.sscroll -sticky news
      # sample
      label $ff.sample -text "ABCabc-0.123"
      grid $ff.sample - - - -sticky news
      
      # packing
      pack $tf $ff $bf -side top -fill x
      
      # make list ------------------------------
      set fl [font families]
      foreach f $wk(font,families) {
	 if { [lsearch -exact $fl [string tolower $f]] < 0} { continue }
	 $ff.family insert end $f
      }
      # for size; determine pixel size from PS-font size
      set pixel 0
      set pssize [lindex $fontmap($fn) end]
      set c [tk scaling]
      foreach sz $wk(font,sizes) {
	 $ff.size insert end $sz
	 if { [expr int($c * $sz)] == $pssize } {
	    set pixel $sz
	 }
      }
      # init value ------------------------------
      $ff.family selection set\
	  [lsearch $wk(font,families) [lrange $fontmap($fn) 0 end-1]]
      $ff.size selection set [lsearch $wk(font,sizes) $pixel]
      
      # see selection
      foreach widget {family size} { 
	 set selindex [$ff.$widget cursel]
	 if { $selindex != "" } { $ff.$widget see $selindex }
      }
      
      # bindings ------------------------------
      foreach widget {family size} {
	 bind $ff.$widget <<ListboxSelect>> "
	 update_listbox_selection $ff.$widget
	 [namespace current]::update_sample $ff
         "
      }
      
      update_sample $ff
      grab $w
   }
   proc update_sample {w} {
      set fam [$w.family get [$w.family cursel]]
      set size [$w.size get [$w.size cursel]]
      $w.sample conf -font [format "{%s} %s" $fam $size]
   }
   proc apply {pw w var} {
      # global
      global wk fontmap paste

      # resolve font name
      set ff $w.font_select
      set fam [$ff.family get [$ff.family cursel]]
      set size [expr int( [tk scaling] * [$ff.size get [$ff.size cursel]] )]
      foreach f [array names fontmap] {
	 if { [lrange $fontmap($f) 0 end-1] == $fam\
		  && [lindex $fontmap($f) end] == $size } {
	    $pw conf -font $f
	    set $var $f
	    break
	 }
      }
      destroy $w
   }
}
# --------------------------------------------------------------------
# gf_optionMenu ------------------------------------------------------
#            Make an option Menu widget set
# --------------------------------------------------------------------
#    w     : widget path for this widget set
#    var   : variable name
#    com   : command
#   (variable length arguments)
#    l1    : label
#    v1    : value
# --------------------------------------------------------------------
proc gf_optionMenu {w var com args} {
   upvar #0 $var value
   # Does tvar exist?
   if { [expr [llength $args] % 2] == 1 } {
      set tvar [lindex $args 0]
      set args [lreplace $args 0 0]
   } else {
      set tvar ""
   }
   # determine length 
   set len 8
   foreach {i j} $args {
      set tmp [string length $i]
      set len [expr {$tmp > $len} ? $tmp : $len ]
   }
   # create menu
   menubutton $w -indicatoron 1 -menu $w.menu -direction flush\
       -relief raised -bd 2 -highlightthickness 2 -anchor c -width $len
   if { $tvar != "" } {
      $w conf -textvar $tvar
      proc gf_optionMenu_local {w com args} {
	 eval $com
	 $w conf -text $args
      }
   } else {
      proc gf_optionMenu_local {w com args} {
	 return
      }
   }
   menu $w.menu -tearoff 0
   foreach {i j} $args {
      $w.menu add radiobutton -label $i -variable $var -value $j\
	  -command "gf_optionMenu_local $w \"$com\" $i"
      if {$j == $value} {
	 $w conf -text $i
      }
   }
   return $w.menu
}
# --------------------------------------------------------------------
# make_menu_com ------------------------------------------------------
#            Make menu with command binding
# --------------------------------------------------------------------
#    w     : parent widget name
#    id    : id
#    label : explanation to show
#    var   : variable name
#    com   : command
#   (variable length arguments)
#    tvar  : text (label) variable name (can be skip)
#    l1    : label
#    v1    : value
# --------------------------------------------------------------------
proc make_menu_com {w id label var com args} {
   label $w.label($id) -text $label -anchor e
   eval {gf_optionMenu $w.mb($id) $var $com} "$args"
   grid $w.label($id) $w.mb($id) -sticky news
   return $w.mb($id).menu
}
# --------------------------------------------------------------------
# make_menu_plus_button ----------------------------------------------
#            Make menu + command button
# --------------------------------------------------------------------
#    w     : parent widget name
#    id    : id
#    label : explanation to show
#    var   : variable name
#    com   : command for menu button
#    blabel: label for command button
#    bcom  : command for button
#   (variable length arguments)
#    l1    : label
#    v1    : value
# --------------------------------------------------------------------
proc make_menu_plus_button {w id label var com blabel bcom args} {
   label $w.label($id) -text $label -anchor w
   eval {gf_optionMenu $w.mb($id) $var $com} $args
   button $w.button($id) -text $blabel -command $bcom
   grid $w.label($id) $w.mb($id) $w.button($id) -sticky news
}

# --------------------------------------------------------------------
# enable/disable labelframe and its descendant
# --------------------------------------------------------------------
proc labelframe_status {w newstatus} {
   foreach cw [winfo children $w] {
      # current status
      catch {$cw cget -state} state
      if {$state == "disabled" || $state == "normal" } {
	 $cw conf -state $newstatus
      }
      # frame?
      if [string match -nocase frame [winfo class $cw] ] {
	 labelframe_status $cw $newstatus
      }
      # labelframe?
      if [string match -nocase labelframe [winfo class $cw] ] {
	 # disabled
	 if { $newstatus == "disabled" } {
	    labelframe_status $cw $newstatus
	    continue
	 }
	 # normal
	 set lw [$cw cget -labelwidget]
	 if { $lw == "" } {
	    labelframe_status $cw "normal"
	    continue
	 }
	 set lwclass [winfo class $lw]
	 if { "Radiobutton" == $lwclass } {
	    set varname [$lw cget -variable]
	    set onvalue [$lw cget -value]
	    upvar #0 $varname value
	    if { $value == $onvalue } {
	       labelframe_status $cw "normal"
	    } else {
	       labelframe_status $cw "disabled"
	    }
	 } elseif { "Checkbutton" == $lwclass } {
	    set varname [$lw cget -variable]
	    set onvalue [$lw cget -onvalue]
	    upvar #0 $varname value
	    if { $value == $onvalue } {
	       labelframe_status $cw "normal"
	    } else {
	       labelframe_status $cw "disabled"
	    }
	 }
      }
   }
}

# --------------------------------------------------------------------
# enable/disable labelframe and its descendant ; checkbutton type
# --------------------------------------------------------------------
proc labelframe_checkbutton_status {w varname} {
   upvar $varname val
   # main --------------------------------------------------
   proc update_status {w newstatus} {
      foreach cw [winfo children $w] {
	 set cls [winfo class $cw]
	 if {  $cls == "Frame" || $cls == "Labelframe" } {
	    update_status $cw $newstatus
	 }
	 catch {$cw cget -state} state
	 if {$state == "disabled" || $state == "normal" } {
	    $cw conf -state $newstatus
	 }
      }
   }
   if { $val == 0 } {
      set newstatus "disabled"
   } else {
      set newstatus normal
   }
   update_status $w $newstatus
}
# --------------------------------------------------------------------
# Update radio_frame and radio_labelframes
# --------------------------------------------------------------------
proc update_radio_frames {w varName} {
   upvar #0 $varName value
   foreach {cw} [winfo children $w] {
      if { [regexp \.radio\(.*\)$ $cw] } {
	 set var [$cw cget -var]
	 if { $var != $varName } { continue }
	 regsub ".radio" $cw ".lframe" lf
	 if { ![winfo exists $lf] } { continue }
	 set val [$cw cget -val]
	 if { $val == $value} {
	    labelframe_status $lf normal
	 } else {
	    labelframe_status $lf disabled
	 }
      }
   }
}
# --------------------------------------------------------------------
# radio_frame --------------------------------------------------------
#    radiobutton with use of radio_labelframe widget sets
# --------------------------------------------------------------------
# w     : parent widget
# id    : identifier of this labelframe
# text  : label to show
# var   : variable name for radiobutton
# val   : value of the variable
# --------------------------------------------------------------------
proc radio_frame {w id text var val} {
   set f [radiobutton $w.radio($id) -text $text -var $var -value $val\
	      -anchor w -command "update_radio_frames $w $var"]
   return $f
}
# --------------------------------------------------------------------
# radio_labelframe ---------------------------------------------------
#    labelframe with radiobutton
# --------------------------------------------------------------------
# w     : parent widget
# id    : identifier of this labelframe
# text  : label to show
# var   : variable name for radiobutton
# val   : value of the variable
# --------------------------------------------------------------------
proc radio_labelframe {w id text var val} {
   # create widgets
   set f [labelframe $w.lframe($id)]
   radiobutton $w.radio($id) -text $text -var $var -value $val\
       -command "update_radio_frames $w $var"
   $f conf -labelwidget $w.radio($id)
   return $f
}