# --------------------------------------------------------------------
# start to select zoom region
# --------------------------------------------------------------------
proc zoom_start {x y} {
   # global --------------------
   global wk
   
   # main --------------------
   if { [string compare $wk(zoom,status) "on"] == 0 } {
      return
   }
   set wk(zoom,status) on
   # save the coordinates
   set wk(zoom,x1) [$wk(w,canvas,c) canvasx $x]
   set wk(zoom,y1) [$wk(w,canvas,c) canvasy $y]
   # add bind
   bind $wk(w,canvas,c) <Button1-Motion> "zoom_motion %x %y"
   bind $wk(w,canvas,c) <ButtonRelease-1> "zoom_end %x %y"
}
# --------------------------------------------------------------------
# during select region for zoom
# --------------------------------------------------------------------
proc zoom_motion {x y} {
   # global --------------------
   global wk
   
   # main --------------------
   if {[string compare $wk(zoom,status) "on"] != 0} {
      return;
   }
   # show the temporary region
   catch {$wk(w,canvas,c) delete temp_mark}
   $wk(w,canvas,c) create rectangle $wk(zoom,x1) $wk(zoom,y1) \
       [$wk(w,canvas,c) canvasx $x] [$wk(w,canvas,c) canvasy $y]\
       -outline red -tags temp_mark -width 2
}
# --------------------------------------------------------------------
# finish selecting zoom region
# --------------------------------------------------------------------
proc zoom_end {x y} {
   # global --------------------
   global wk
   
   # main --------------------
   # finish point
   set wk(zoom,x2) [$wk(w,canvas,c) canvasx $x]
   set wk(zoom,y2) [$wk(w,canvas,c) canvasy $y]
    
   # show the region
   $wk(w,canvas,c) delete temp_mark zoom_mark
   $wk(w,canvas,c) create rectangle $wk(zoom,x1) $wk(zoom,y1) $wk(zoom,x2) $wk(zoom,y2)\
       -outline green -tags zoom_mark -width 2
   
   # convert window-coord into graph-coord 
   set x1 [coord cx2gx $wk(zoom,x1)]
   set x2 [coord cx2gx $wk(zoom,x2)]
   set t1 [coord cy2gt $wk(zoom,y1)]
   set t2 [coord cy2gt $wk(zoom,y2)]
   
   # off zooming mode
   set wk(zoom,status) "off"
   bind $wk(w,canvas,c) <Button1-Motion> ""
   bind $wk(w,canvas,c) <ButtonRelease-1> ""
   
   # confirmation dialog
   set ans [tk_dialog .zoom_or_not "Zoom or not?"\
		"Zoom and redraw record secion?"\
		question 0 Yes No]
   if {$ans==0} {
      draw_pasteup
   }
   $wk(w,canvas,c) delete zoom_mark
}
# --------------------------------------------------------------------
# Change display range (zooming)
# --------------------------------------------------------------------
proc change_range_by_zoom {} {
   # global variables
   global wk paste
   
   # main ------------------------------
   # widget coordinates -> graph coordinates ----------
   set x1 [coord cx2gx $wk(zoom,x1)]
   set x2 [coord cx2gx $wk(zoom,x2)]
   set t1 [coord cy2gt $wk(zoom,y1)]
   set t2 [coord cy2gt $wk(zoom,y2)]
   
   # Adjust number of digits ----------
   set dx [expr $x2 - $x1]
   set dt [expr $t2 - $t1]
   if {$dx == 0.0 || $dt == 0.0} { 
      put_error "Region is too narrow."
      return 1
   }
   set format [make_digit_format $dx]
   set paste(left) [format $format $x1]
   set paste(right) [format $format $x2]
   
   set format [make_digit_format $dt]
   set paste(top) [format $format $t1]
   set paste(bottom) [format $format $t2]
   
   # Check axis
   switch [check_coordinates] {
      [12] {put_error "Invalid range."; return}
   }
   # Add zoom stack
   incr wk(zoom,count)
   foreach i {left right top bottom} {
      set wk(zoom,$i,$wk(zoom,count)) $paste($i) 
   }
   return 0
}

# --------------------------------------------------------------------
# Make format specifier for showing "x", 
#                           to be the number of significant figures 2
# --------------------------------------------------------------------
proc make_digit_format {x} {
   set order [expr int(log10(abs($x))) - 2]
   if {$order >= 0} {
      return "%.0f"
   } else {
      return [format "%%.%df" [expr abs($order)]]
   }
}

# --------------------------------------------------------------------
# zoom out
# --------------------------------------------------------------------
proc zoom_out {} {
    # global variables
    global wk paste

    # main ------------------------------
    # zoom out is possible ?
    if {$wk(zoom,count) < 1} {
	print_message "Warning: Cannot zoom out any more.\n"
	return
    }
    # reconfirm zoom out
    if { [recon_zoomout] == "cancel"} {
       return
    }

    # recorvery previous coordinates.
    incr wk(zoom,count) -1
    foreach i {left right top bottom} {
	set paste($i) $wk(zoom,$i,$wk(zoom,count))
    }

    # redraw!
    set wk(zoom,status) "out"
    draw_pasteup
    set wk(zoom,status) "off"
}
# --------------------------------------------------------------------
# Clear zoom stack
# --------------------------------------------------------------------
proc clear_zoom_stack {} {
    # global variable
    global wk paste

    # main ------------------------------
    set wk(zoom,count) 0
    foreach i {left right top bottom} {
	set wk(zoom,$i,$wk(zoom,count)) $paste($i)
    }
}

# -------------------------------------------------------------------
# Re-confirming zoom out 
#  Originally created by Kei Murase @ KGE
#  Modified by G. FUJIE.
# -------------------------------------------------------------------
proc recon_zoomout {} {
   global wk paste

   # main --------------------------------------------------
   # initial setting
   set comment "Zoom Out or not?"
   set boxtype "okcancel"
   
   # trace number after zoom out ----------
   set c [expr $wk(zoom,count) -1]
   set right $paste(right)
   set left $paste(left)
   set paste(right) $wk(zoom,right,$c)
   set paste(left) $wk(zoom,left,$c)
   set wk(condition) [check_same_rh_condition]
   set n [llength [getrh range]]
   set paste(right) $right
   set paste(left) $left
   
   # Large or not. (time_range>10sec or trace_number > 100)
   if { [expr $paste(graph_width) / $n] < 7 \
	    || [expr $wk(zoom,bottom,$c) - $wk(zoom,top,$c)] > 10 } {
      if {$paste(image,mode)==0} {
	 set comment "Many data for plotting in 'Vector' mode.
                      \n\nChange mode into 'Image'?"
	 set boxtype "yesnocancel"
      }
   }

   # confirmation
   set ans [tk_messageBox -title "Zoom Out" -type $boxtype -message $comment\
	       -icon question]

   if {$ans != "cancel"} {
      # change drawing mode into "image" or "interpolate"
      if {$ans == "no"} {
	 return ok
      }
      if {$paste(image,mode) == 0} {
	 set paste(image,mode) 1
      }
      return ok
   }
   return cancel
}
